Package: mixtools
Version: 2.0.0.1
Date: 2022-12-04
Title: Tools for Analyzing Finite Mixture Models
Authors@R: c(person("Derek", "Young", role = c("aut", "cre"),
	email = "derek.young@uky.edu", comment = c(ORCID = "0000-0002-3048-3803")),
	person("Tatiana", "Benaglia", role = "aut"),
	person("Didier", "Chauveau", role = "aut"),
	person("David", "Hunter", role = "aut"),
	person("Kedai", "Cheng", role = "aut"),
	person("Ryan", "Elmore", role = "ctb"),
	person("Thomas", "Hettmansperger", role = "ctb"),
	person("Hoben", "Thomas", role = "ctb"),
	person("Fengjuan", "Xuan", role = "ctb"))
Depends: R (>= 4.0.0)
Imports: kernlab, MASS, plotly, scales, segmented, stats, survival
URL: https://github.com/dsy109/mixtools
Description: Analyzes finite mixture models for various parametric and semiparametric settings.  This includes mixtures of parametric distributions (normal, multivariate normal, multinomial, gamma), various Reliability Mixture Models (RMMs), mixtures-of-regressions settings (linear regression, logistic regression, Poisson regression, linear regression with changepoints, predictor-dependent mixing proportions, random effects regressions, hierarchical mixtures-of-experts), and tools for selecting the number of components (bootstrapping the likelihood ratio test statistic, mixturegrams, and model selection criteria).  Bayesian estimation of mixtures-of-linear-regressions models is available as well as a novel data depth method for obtaining credible bands.  This package is based upon work supported by the National Science Foundation under Grant No. SES-0518772 and the Chan Zuckerberg Initiative: Essential Open Source Software for Science (Grant No. 2020-255193).
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2025-03-08 07:39:45 UTC; hornik
Author: Derek Young [aut, cre] (<https://orcid.org/0000-0002-3048-3803>),
  Tatiana Benaglia [aut],
  Didier Chauveau [aut],
  David Hunter [aut],
  Kedai Cheng [aut],
  Ryan Elmore [ctb],
  Thomas Hettmansperger [ctb],
  Hoben Thomas [ctb],
  Fengjuan Xuan [ctb]
Maintainer: Derek Young <derek.young@uky.edu>
Repository: CRAN
Date/Publication: 2025-03-08 08:58:50 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 16:34:20 UTC; unix
Archs: mixtools.so.dSYM

useDynLib(mixtools, .registration = TRUE, .fixes="C_")

# Export all names
export("aug.x")
export("boot.comp")
export("boot.se")
export("compCDF")
export("dexpmixt")
export("ddirichlet")
export("density.npEM")
export("density.spEM")
export("depth")
export("dmvnorm")
export("ellipse")
export("expRMM_EM")
export("flaremix.init")
export("flaremixEM")
export("gammamix.init")
export("gammamixEM")
export("hmeEM")
export("ise.npEM")
export("lambda")
export("lambda.pert")
export("ldmult")
export("logdmvnorm")
export("logisregmix.init")
export("logisregmixEM")
export("makemultdata")
export("matsqrt")
export("mixturegram")
export("multmix.init")
export("multmixEM")
export("multmixmodel.sel")
export("mvnormalmix.init")
export("mvnormalmixEM")
export("mvnpEM")
export("normalmix.init")
export("normalmixEM")
export("normalmixEM2comp")
export("normalmixMMlc")
export("normmix.sim")
export("normmixrm.sim")
export("npEM")
export("npEMindrep")
export("npEMindrepbw")
export("npMSL")
export("parse.constraints")
export("perm")
export("plot.mixEM")
export("plot.mixMCMC")
export("plot.mvnpEM")
export("plot.npEM")
export("plot.spEM")
export("plot.spEMN01")
export("plotexpRMM")
export("plotFDR")
export("plotly_compCDF")
export("plotly_ellipse")
export("plotly_expRMM")
export("plotly_FDR")
export("plotly_ise.npEM")
export("plotly_mixEM")
export("plotly_mixMCMC")
export("plotly_mixturegram")
export("plotly_npEM")
export("plotly_post.beta")
export("plotly_seq.npEM")
export("plotly_spEMN01")
export("plotly_spRMM")
export("plotly_weibullRMM")
export("plotseq.npEM")
export("plotseq")
export("plotspRMM")
export("plotweibullRMM")
export("poisregmix.init")
export("poisregmixEM")
export("post.beta")
export("print.mvnpEM")
export("print.npEM")
export("print.summary.mvnpEM")
export("print.summary.npEM")
export("regcr")
export("regmix.init")
export("regmix.lambda.init")
export("regmix.mixed.init")
export("regmixEM")
export("regmixEM.lambda")
export("regmixEM.loc")
export("regmixEM.mixed")
export("regmixMH")
export("regmixmodel.sel")
export("repnormmix.init")
export("repnormmixEM")
export("repnormmixmodel.sel")
export("rexpmix")
export("rlnormscalemix")
export("rmvnorm")
export("rmvnormmix")
export("rnormmix")
export("rweibullmix")
export("segregmix.init")
export("segregmixEM")
export("spEM")
export("spEMsymloc")
export("spEMsymlocN01")
export("spregmix")
export("spRMM_SEM")
export("summary.mixEM")
export("summary.mvnpEM")
export("summary.npEM")
export("summary.spRMM")
export("tauequivnormalmixEM")
export("test.equality")
export("test.equality.mixed")
export("try.flare")
export("weibullRMM_SEM")
export("wIQR")
export("wkde")
export("wquantile")

# Import all packages listed as Imports or Depends
import(
  plotly,
  segmented,
  survival
)
import(MASS, except=c(select))
import(stats, except=c(filter))

# S3 methods
S3method(density, npEM)
S3method(density, spEM)
S3method(plot, mixEM)
S3method(plot, mixMCMC)
S3method(plot, mvnpEM)
S3method(plot, npEM)
S3method(plot, spEM)
S3method(plot, spEMN01)
S3method(plotseq, npEM)
S3method(print, mvnpEM)
S3method(print, npEM)
S3method(print, summary.mvnpEM)
S3method(print, summary.npEM)
S3method(summary, mixEM)
S3method(summary, mvnpEM)
S3method(summary, npEM)
S3method(summary, spRMM)

# Import from
importFrom("kernlab", "kpca", "pcv", "Q")
importFrom("grDevices", "chull", "adjustcolor", "colors")
importFrom("graphics", "abline", "barplot", "hist", 
  "layout.show", "legend", "lines", "par", "plot", "points",
  "segments", "title", "axis", "polygon", "rect")
importFrom("scales", "hue_pal")
importFrom("utils", "packageDescription", "tail")

segregmix.init <- function (y, x, lambda = NULL, beta = NULL, s = NULL, k = 2, seg.Z, 
psi, psi.locs = NULL) 
{
	n <- length(y)
	p <- ncol(x)
	psi.counts <- apply(psi>0,1,sum)
if(is.null(lambda)|is.null(beta)|is.null(s)|is.null(psi.locs)){
	if (is.null(psi.locs)) {
		psi.locs = vector("list",k)
		psi.locs = lapply(1:k, function(i) if(psi.counts[i]>0) vector("list",psi.counts[i]) else NULL)
		for(i in 1:k){
			if(!is.null(psi.locs[[i]])){
				temp.locs <- which(psi[i,]>0)
				temp.labs=NULL
				for(j in 1:length(temp.locs)){
					psi.locs[[i]][[j]]=sort(runif(psi[i,temp.locs[j]],as.numeric(quantile(x[,temp.locs[j]],.05)),as.numeric(quantile(x[,temp.locs[j]],.95))))
					temp.labs=c(temp.labs,colnames(x)[temp.locs[j]])
				}
			names(psi.locs[[i]])=temp.labs
			}
		}
	} else k = length(psi.locs)
	xnam <- colnames(x)
	fmla <- as.formula(paste("y ~ ", paste(xnam, collapse= "+")))
	TEMP.lm <- lm(fmla,data=x)
	EM.out <- regmixEM(TEMP.lm$res,TEMP.lm$fit,k=k,epsilon=1e-2)
	posts = apply(EM.out$post,1,which.max)
	if (is.null(lambda)) {
		lambda = EM.out$lambda
		if(length(unique(posts))!=k) posts=rep(1:k,n)[1:n]
	} else k = length(lambda)
	A = round(lambda * n)
	while (min(A) <= 4) {
		lambda = runif(k)
		lambda = lambda/sum(lambda)
		A = round(lambda * n)
	}
	w = cbind(y, x)
	w.bin = list()
	for (j in 1:k) {
		w.bin[[j]] <- w[posts==j, ]
	}
	all.w.bin=vector("list",gamma(k+1))
	all.inds=perm(k,k)
	all.X.aug=all.w.bin
	all.lm.out=all.w.bin
	avg.res=NULL
	for(j in 1:length(all.w.bin)){
		all.w.bin[[j]]=w.bin[all.inds[j,]]
		X.aug <- lapply(1:k, function(i) cbind(1,aug.x(w.bin[[all.inds[j,i]]][,-1],unlist(psi.locs[[i]]),psi[i,],delta=NULL)))
		sapply(X.aug,dim)
		lm.out <- lapply(1:k, function(i) lm(w.bin[[all.inds[j,i]]][, 1] ~ X.aug[[i]][,-1]))
		all.X.aug[[j]]=X.aug
		all.lm.out[[j]]=lm.out
		avg.res=c(avg.res,mean(as.vector(unlist(lapply(1:k,function(t) lm.out[[t]]$res)))^2))
	}
	IND=which.min(avg.res)
	w.bin=all.w.bin[[IND]]
	X.aug=all.X.aug[[IND]]
	lm.out=all.lm.out[[IND]]
	s.hyp = lapply(lm.out, anova)
	s.hyp = as.vector(sqrt(sapply(1:k, function(i) tail(s.hyp[[i]]$Mean,1))))
	s.hyp[(s.hyp <= 0) | (is.na(s.hyp) == 1)] = 1
	if (is.null(s)) {
		s = 1/rexp(k, rate = s.hyp)
	} else k = length(s)
	if (is.null(beta)) {
	x.x <- lapply(1:k,function(i) try(solve(t(X.aug[[i]]) %*% X.aug[[i]]),silent=TRUE))
	test <- sum(sapply(1:k, function(i) class(x.x[[i]])[1]=="try-error"))
	if(test>0) stop("Lapack Routine Error")
	beta.hyp = lapply(lm.out,coef) # matrix(sapply(lm.out, coef), ncol = k)
	beta = vector("list",k)
	for (j in 1:k) {
		beta[[j]] = rnorm(length(beta.hyp[[j]]),mean=as.vector(beta.hyp[[j]]),
			sd = (s.hyp[j] * sqrt(diag(x.x[[j]]))))
	}
	} else k = length(beta)
} else{
		for(i in 1:k){
			if(!is.null(psi.locs[[i]])){
				temp.locs <- which(psi[i,]>0)
				temp.labs=NULL
				for(j in 1:length(temp.locs)){
					temp.labs=c(temp.labs,colnames(x)[temp.locs[j]])
				}
			names(psi.locs[[i]])=temp.labs
			}
		}
}
	list(lambda = lambda, beta = beta, s = s, k = k, psi.locs = psi.locs)
}

test.equality.mixed=function (y, x = NULL, w = NULL, arb.R = TRUE, arb.sigma = FALSE, 
    lambda = NULL, mu = NULL, sigma = NULL, R = NULL, alpha = NULL, 
    ...) 
{
    if (arb.R == arb.sigma) 
        stop("Change either 'arb.R' or 'arb.sigma'!")
    v = 1
    while (v == 1) {
        if (arb.R) {
            H0 = regmixEM.mixed(y = y, x = x, w = w, arb.R = TRUE, 
                arb.sigma = FALSE, lambda = lambda, sigma = sigma, 
                mu = mu, R = R, alpha = alpha, ...)
            p = nrow(H0$posterior.beta[[1]])
            k = length(H0$lambda)
            if (is.null(w)) 
                alpha = NULL
            else alpha = H0$alpha

#		n.i=sapply(y,length)
#		N=length(y)
#		tmp=apply(H1$posterior.z*n.i,2,sum)
#		common.sig=sum(tmp/N*H1$sigma)



#            H1 = regmixEM.mixed(y = y, x = x, w = w, arb.R = TRUE, 
#                arb.sigma = TRUE, lambda = H0$lambda, sigma = rep(H0$sigma,k), 
#                mu = H0$mu, R = H0$R, alpha = alpha, ...)

            H1 = regmixEM.mixed(y = y, x = x, w = w, arb.R = TRUE, 
                arb.sigma = TRUE, lambda = H0$lambda, sigma = NULL, 
                mu = H0$mu, R = H0$R, alpha = alpha, ...)

            D = 2 * (H1$loglik - H0$loglik)
            df = k - 1
            alpha = 1 - pchisq(D, df = df)
        }
        else {
            H0 = regmixEM.mixed(y = y, x = x, w = w, arb.R = FALSE, 
                arb.sigma = TRUE, lambda = lambda, sigma = sigma, 
                mu = mu, R = R, alpha = alpha, ...)
            p = nrow(H0$posterior.beta[[1]])
            k = length(H0$lambda)
            if (is.null(w)) 
                alpha = NULL
            else alpha = H0$alpha

#		N=length(y)
#		tmp=(apply(H1$posterior.z,2,sum))/N
#		common.R=matrix(0,ncol=p,nrow=p)
#		for(i in 1:length(H1$lambda)){
#			common.R=common.R+tmp[i]*H1$R[[i]]
#		}


#            H1 = regmixEM.mixed(y = y, x = x, w = w, arb.R = TRUE, 
#                arb.sigma = TRUE, lambda = H0$lambda, sigma = H0$sigma, 
#                mu = H0$mu, R = lapply(1:k,function(i) H0$R), alpha = alpha, ...)
            H1 = regmixEM.mixed(y = y, x = x, w = w, arb.R = TRUE, 
                arb.sigma = TRUE, lambda = H0$lambda, sigma = H0$sigma, 
                mu = H0$mu, R = NULL, alpha = alpha, ...)


            D = 2 * (H1$loglik - H0$loglik)
            df = p * (p + 1) * (k - 1)/2
            alpha = 1 - pchisq(D, df = df)
        }
        if (D < 0) {
            v = 1
	    lambda=NULL
	    sigma=NULL
	    mu=NULL
	    R=NULL
	    alpha=NULL}
        else v = 2
    }
    a = list(chi.sq = D, df = df, p.value = alpha)
    a
}

normalmix.init <- function (x, lambda = NULL, mu = NULL, s = NULL, k = 2, arbmean = TRUE, 
                            arbvar = TRUE){
  if (!is.null(s)) {
    arbvar <- (length(s) > 1)
    if (arbvar) 
      k <- length(s)
  }
  if (!is.null(mu)) {
    arbmean <- (length(mu) > 1)
    if (arbmean) {
      k <- length(mu)
      if (!is.null(s) && length(s) > 1 && k != length(s)) {
        stop("mu and sigma are each of length >1 but not of the same length.")
      }
    }
  }
  if (!arbmean && !arbvar) {
    stop("arbmean and arbvar cannot both be FALSE")
  }
  n = length(x)
  x = sort(x)
  x.bin = list()
  for (j in 1:k) {
    x.bin[[j]] <- x[max(1, floor((j - 1) * n/k)):ceiling(j * 
                                                           n/k)]
  }
  if (is.null(s)) {
    s.hyp = as.vector(sapply(x.bin, sd))
    if (any(s.hyp == 0)) 
      s.hyp[which(s.hyp == 0)] = runif(sum(s.hyp == 0), 
                                       0, sd(x))
    if (arbvar) {
      s = 1/rexp(k, rate = s.hyp)
    }
    else {
      s = 1/rexp(1, rate = mean(s.hyp))
    }
  }
  if (is.null(mu)) {
    mu.hyp <- as.vector(sapply(x.bin, mean))
    if (arbmean) {
      mu = rnorm(k, mean = mu.hyp, sd = s)
    }
    else {
      mu = rnorm(1, mean = mean(mu.hyp), sd = mean(s))
    }
  }
  if (is.null(lambda)) {
    cond = TRUE
    while(cond){
      lambda = runif(k)
      lambda = lambda/sum(lambda)
      if(min(lambda)<0.05) cond=TRUE else cond=FALSE
    }
  }
  else {
    lambda <- rep(lambda, length.out = k)
    lambda <- lambda/sum(lambda)
  }
  list(lambda = lambda, mu = mu, s = s, k = k, arbvar = arbvar, 
       arbmean = arbmean)
}

regmix.init<-function (y, x, lambda = NULL, beta = NULL, s = NULL, k = 2, 
    addintercept = TRUE, arbmean = TRUE, arbvar = TRUE) 
{
    x <- as.matrix(x)
    n <- length(y)
    p <- ncol(x)
    if (addintercept) {
        x = x[, -1]
    }
    else x = x
    if (is.null(lambda)) {
        lambda = runif(k)
        lambda = lambda/sum(lambda)
    }
    else k = length(lambda)
    A = round(lambda * n)
	while(min(A)<4){
	       lambda = runif(k)
             lambda = lambda/sum(lambda)
		 A = round(lambda * n)
	}
    A[k] = n - sum(A[1:(k - 1)])
    B = c(0, cumsum(A))
    w = cbind(y, x)
    w = w[order(w[, 1]), ]
    w.bin = list()
    for (j in 1:k) {
        w.bin[[j]] <- w[(B[j] + 1):B[j + 1], ]
    }
    if (addintercept) {
        lm.out <- lapply(1:k, function(i) lm(w.bin[[i]][, 1] ~ 
            w.bin[[i]][, 2:p]))
    }
    else lm.out <- lapply(1:k, function(i) lm(w.bin[[i]][, 1] ~ 
        w.bin[[i]][, 2:(p + 1)] - 1))

        s.hyp = lapply(lm.out, anova)
        s.hyp = as.vector(sqrt(sapply(1:k, function(i) s.hyp[[i]]$Mean[length(s.hyp[[i]]$Mean)])))
        s.hyp[(s.hyp<=0)|(is.na(s.hyp)==1)]=1
    if (is.null(s)) {
        if (arbvar) {
            s = 1/rexp(k, rate = s.hyp)
        }
        else {
            s.hyp = mean(s.hyp)
            s = 1/rexp(1, rate = s.hyp)
        }
    }
    if (is.null(s) == FALSE && arbvar == TRUE) {
        k = length(s)
    }
    if (is.null(beta)) {
	  x.x <- solve(t(x)%*%x)
        beta.hyp = matrix(sapply(lm.out, coef), ncol = k)
        beta = matrix(0, nrow = p, ncol = k)
        for (j in 1:k) {
            beta[, j] = rnorm(p, mean = as.vector(beta.hyp[, 
#                j]), sd = (s[arbvar * j + (1 - arbvar)]*sqrt(diag(x.x))) ) 
                j]), sd = (s.hyp[arbvar * j + (1 - arbvar)]*sqrt(diag(x.x))) ) 
        }
        if (arbmean == FALSE) {
            beta = apply(beta, 1, mean)
        }
    }
    if (is.null(beta) == FALSE && arbmean == TRUE) {
        k = ncol(beta)
    }
    list(lambda = lambda, beta = beta, s = s, k = k)
}

gammamix.init <- function (x, lambda = NULL, alpha = NULL, beta = NULL, k = 2) 
{
  n <- length(x)
  if (is.null(lambda)) {
    cond = TRUE
    while(cond){
      lambda = runif(k)
      lambda = lambda/sum(lambda)
      if(min(lambda)<0.05) cond=TRUE else cond=FALSE
    }
  }
  else k = length(lambda)
  if (k == 1) {
    x.bar = mean(x)
    x2.bar = mean(x^2)
  }
  else {
    x.sort = sort(x)
    ind = floor(n * cumsum(lambda))
    x.part = list()
    x.part[[1]] = x.sort[1:(ind[1] + 1)]
    for (j in 2:k) {
      x.part[[j]] = x.sort[ind[j - 1]:ind[j]]
    }
    x.bar = sapply(x.part, mean)
    x2.bar = sapply(lapply(x.part, "^", 2), mean)
  }
  if (is.null(alpha)) {
    alpha = x.bar^2/(x2.bar - x.bar^2)
  }
  if (is.null(beta)) {
    beta = (x2.bar - x.bar^2)/x.bar
  }
  list(lambda = lambda, alpha = alpha, beta = beta, k = k)
}

parse.constraints <- function(constr, k=2, allsame=FALSE) {
  if (!is.null(constr)) {
    alpha <- category <- rep(NA, k)
    u <- sum(is.na(constr)) # Number of unconstrained means
    category[is.na(constr)] <- 1:u
    numcon <- as.numeric(constr) # All non-numbers become NA here.
    category[!is.na(numcon)] <- 0  # These means will remain fixed constants
    alpha[!is.na(numcon)] <- numcon[!is.na(numcon)]  
    w <- which(is.na(numcon) & !is.na(constr)) # w indexes only character constraints
    if (length(w)>0) {  # Process all the character (equality) constraints
      constr <- as.character(constr)
      alpha[w] <- constr[w]
      nc <- nchar(constr)[w] # nc:  number of characters (in each constraint)
      ss <- substr(constr[w], nc, nc) # extract final character
      category[w] <- u + match(ss, unique(ss)) # make categories for final characters
      alpha[w] <- substr(alpha[w], 1, nc-1) # Now extract number before final character
      alpha[w][alpha[w]=="-"] <- "-1"
      alpha[w][alpha[w]==""] <- "1"
    }
    alpha <- as.numeric(alpha) # Entire alpha vector should be numeric or NA now.
  } else {
    alpha <- rep(1,k)
    if(allsame) 
      category <- alpha
    else 
      category <- 1:k
  }
  list(category=category, alpha=alpha)
}

.onAttach <- function(lib, pkg){
  info <- packageDescription("mixtools")
  packageStartupMessage(
    paste('mixtools package, version ', info$Version, ', Released ', info$Date, '\n',
          'This package is based upon work supported by the National Science ',
          'Foundation under Grant No. SES-0518772.\n', sep="")
 )
}

regmix.mixed.init <- function(y,x,w=NULL,sigma=NULL,arb.sigma=TRUE,
	alpha=NULL,lambda=NULL,mu=NULL,R=NULL,arb.R=TRUE,k=2,mixed=FALSE,
	addintercept.fixed=FALSE,addintercept.random=TRUE){

    N <- length(y)
    n <- sapply(y, length)
    p <- ncol(x[[1]])

    if (addintercept.random) {
        x = lapply(1:N, function(i) as.matrix(x[[i]][,-1]))
    } else x=x

    if(is.null(w)==FALSE && sum(sapply(w,sum))!=0){
    q <- ncol(w[[1]])
    }

    if (mixed == TRUE && is.null(alpha)==TRUE) {
        if (addintercept.fixed) {
            w.1 = list()
		w.1 = lapply(1:N,function(i) w[[i]][,-1])
		lm.out = lapply(1:N,function(i) lm(y[[i]]~w.1[[i]]))
		alpha.hyp = apply(matrix(sapply(lm.out,coef),ncol=N),1,mean)
		sd.hyp = lapply(lm.out,anova)
		sd.hyp = mean(as.vector(sqrt(sapply(1:N,function(i) sd.hyp[[i]]$Mean[length(sd.hyp[[i]]$Mean)]))))
		alpha = rnorm(q, mean=alpha.hyp, sd=sd.hyp)
        } else {
            w.1 = w
		lm.out = lapply(1:N,function(i) lm(y[[i]]~w.1[[i]]-1))
		alpha.hyp = apply(matrix(sapply(lm.out,coef),ncol=N),1,mean)
		sd.hyp = lapply(lm.out,anova)
		sd.hyp = mean(as.vector(sqrt(sapply(1:N,function(i) sd.hyp[[i]]$Mean[length(sd.hyp[[i]]$Mean)]))))
		alpha = rnorm(q, mean=alpha.hyp, sd=sd.hyp)
	  }
    }
    if(mixed==FALSE) {
        alpha = 0
    }

y.x = lapply(1:N, function(i) cbind(y[[i]],x[[i]]))
a = order(sapply(1:N, function(i) mean(y[[i]])))
y.x = lapply(1:N,function(i) y.x[[a[i]]])

	y.x.bin.list = list()
	y.x.bin = list()
	for(j in 1:k){
		y.x.bin.list[[j]] <- y.x[max(1,floor((j-1)*N/k)):ceiling(j*N/k)]
		y.x.2 <- NULL
		for(i in 1:length(y.x.bin.list[[j]])){
		y.x.2 <- rbind(y.x.2,y.x.bin.list[[j]][[i]])
		}
		y.x.bin[[j]] <- y.x.2
	}


if(addintercept.random){
lm.out <- lapply(1:k, function(i) lm(y.x.bin[[i]][,1]~y.x.bin[[i]][,2:p]))
lm.out.beta <- lapply(1:k, function(j) lapply(1:length(y.x.bin.list[[j]]), function(i) lm(y.x.bin.list[[j]][[i]][,1]~y.x.bin.list[[j]][[i]][,2:p])))
beta <- lapply(1:k,function(j) matrix(sapply(lm.out.beta[[j]],coef),nrow=p))
} else {
lm.out <- lapply(1:k, function(i) lm(y.x.bin[[i]][,1]~y.x.bin[[i]][,2:(p+1)]-1))
lm.out.beta <- lapply(1:k, function(j) lapply(1:length(y.x.bin.list[[j]]), function(i) lm(y.x.bin.list[[j]][[i]][,1]~y.x.bin.list[[j]][[i]][,2:(p+1)]-1)))
beta <- lapply(1:k,function(j) matrix(sapply(lm.out.beta[[j]],coef),nrow=p))
}
	if(is.null(sigma)) {
		sigma.hyp = lapply(lm.out,anova)
		sigma.hyp = as.vector(sqrt(sapply(1:k,function(i) sigma.hyp[[i]]$Mean[length(sigma.hyp[[i]]$Mean)])))
		if(arb.sigma) {
			sigma=1/rexp(k,rate=sigma.hyp)
		} else {
			sigma.hyp=mean(sigma.hyp)
			sigma=1/rexp(1,rate=sigma.hyp)
		}
	}

	if(is.null(sigma)==FALSE && arb.sigma==TRUE) {
		k=length(sigma)
	}

	if(is.null(R)) {
		if(arb.R) {
			R.hyp = lapply(1:k,function(i) (apply(beta[[i]],1,var))^-1)
			R=lapply(1:k,function(i) diag(1/rexp(p,rate=R.hyp[[i]]),p))
		} else {
			R.hyp = apply(matrix(sapply(1:k,function(i) (apply(beta[[i]],1,var))^-1),ncol=k),2,mean)
			R = diag(1/rexp(p,rate=sigma.hyp),p)
	}
}
	if(is.null(R)==FALSE && arb.R==TRUE) {
		k=length(R)
	}


    if (is.null(mu)) {
        mu.hyp = lapply(1:k,function(i) apply(beta[[i]],1,mean))	  
	  mu = matrix(ncol=k,nrow=p)
		if(arb.R==TRUE){
		  for(j in 1:k){
		  mu[,j] = rmvnorm(1,mu=as.vector(mu.hyp[[j]]),sigma=R[[j]])
			} 
		} else {
		  for(j in 1:k){
		  mu[,j] = rmvnorm(1,mu=as.vector(mu.hyp[[j]]),sigma=R)
		} 
    }
} else k = ncol(mu)


    if (is.null(lambda)) {
        lambda = runif(k)
        lambda = lambda/sum(lambda)
    } else k = length(lambda)

list(sigma=sigma,alpha=alpha,lambda=lambda,mu=mu,R=R,k=k)

}

logisregmix.init <- function(y, x, N, lambda = NULL, beta = NULL, k = 2){
  x <- as.matrix(x)
  n <- length(y)
  if (is.null(lambda)) {
    cond = TRUE
    while(cond){
      lambda = runif(k)
      lambda = lambda/sum(lambda)
      if(min(lambda)<0.05) cond=TRUE else cond=FALSE
    }
  } else k = length(lambda)
  p <- ncol(x)
  w = cbind(y, N, x)
  w = w[order(w[, 1]), ]
  w.bin = list()
  for (j in 1:k) {
    w.bin[[j]] <- w[max(1, floor((j - 1) * n/k)):ceiling(j * 
                                                           n/k), ]
  }
  if (is.null(beta)) {
    beta.hyp = matrix(sapply(1:k, function(j) glm.fit(w.bin[[j]][, 
                                                                 3:(p + 2)], w.bin[[j]][, 1:2], family = binomial())$coef), 
                      ncol = k)
    sd.hyp = apply(beta.hyp, 1, sd)
    beta = matrix(0, p, k)
    for (j in 1:k) {
      beta[, j] = rnorm(p, mean = as.vector(beta.hyp[, 
                                                     j]), sd = sd.hyp)
    }
  }
  else k = ncol(beta)
  list(lambda = lambda, beta = beta, k = k)
}

# Method for "density" generic:  Takes an spEM object and returns
# a corresponding KDE for the appropriate component and block, evaluated
# at the given set of points.  
# Does not use the FFT like the density.default function does; still
# quite fast, but not optimized for speed
density.spEM <- function (x, u = NULL, component=1, block = 1, 
                          scale = FALSE, ...) 
{  
  m <- NCOL(x$posteriors)
  r <- NCOL(x$data)
  n <- NROW(x$data)
  if (is.null(blockid <- x$blockid)) {
    coords <- 1
    u2 <- rep(1, r)
  } 
  else {
    u2 <- match(x$blockid, unique(x$blockid)) # convert blockid to integers 1, 2, ...
    coords <- blockid == block
    if (!any(coords)) 
      stop("Illegal value of block argument")
  }
  stackedx <- x$data[rep(1:n,m),]  
  cs <- colSums(x$posteriors)
  z.tmp <- sweep(x$posteriors, 2, cs, "/")
  z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case
  wts <- rep(as.vector(z.tmp),r)
  mu <- matrix(x$muhat, nrow=m)
  sigma <- matrix(x$sigmahat, nrow=m)
  scaledx <- as.vector((stackedx - mu[rep(1:m, each=n), u2])/
                       sigma[rep(1:m, each=n), u2])
  bw <- x$bandwidth
  if (is.null(u)) {
    xx <- as.vector(as.matrix(x$data)[, coords])
    u = seq(min(xx) - 4 * bw, max(xx) + 4 * bw, len = 250)
  }
  # This part isn't used for now:
  if (!is.null(x$symmetric) && x$symmetric) {
    d <- wkde(x=scaledx, u=(u-mu[component, block])/sigma[component, block], 
              w=wts, bw=bw, sym=TRUE) / sigma[component, block]
  }
  else {
    d <- wkde(x=scaledx, u=(u-mu[component, block])/sigma[component, block], 
              w=wts, bw=bw) / sigma[component, block]
  }
  if (scale) 
    d <- d * x$lambdahat[component]
  structure(list(x = u, y = d, bw = bw, n = n, call = match.call(), 
                 data.name = deparse(substitute(x)), has.na = FALSE), 
             class = "density")
}

# Simulate from a normal mixture.  New version simply calls rmvnormmix.

# normmix.sim is here for backwards compatibility
rnormmix <- normmix.sim <- function(n,lambda=1,mu=0,sigma=1) {
  if (NCOL(mu)>1 || NCOL(sigma)>1)
    stop ("Use the rmvnormmix function instead.")
  as.vector(rmvnormmix(n,lambda,mu,sigma))
}

## Use an ECM algorithm (in the sense of Meng and Rubin, Biometrika 1993)
## to search for a local maximum of the likelihood surface for a 
## univariate finite mixture of normals with possible equality
## constraints on the mean and stdev parameters.
normalmixEM <-
function (x, lambda = NULL, mu = NULL, sigma = NULL, k = 2, 
          mean.constr = NULL, sd.constr = NULL,
          epsilon = 1e-08, maxit = 1000, maxrestarts=20, 
          verb = FALSE, fast=FALSE, ECM = FALSE,
          arbmean = TRUE, arbvar = TRUE) {
  warn <- options(warn=-1) # Turn off warnings
  x <- as.vector(x)
  tmp <- normalmix.init(x = x, lambda = lambda, mu = mu, s = sigma, 
                        k = k, arbmean = arbmean, arbvar = arbvar)
  lambda <- tmp$lambda 
  mu <- tmp$mu 
  sigma <- tmp$s
  k <- tmp$k
  arbvar <- tmp$arbvar 
  arbmean <- tmp$arbmean
  if (fast==TRUE && k==2 && arbmean==TRUE) {
    a <- normalmixEM2comp (x, lambda=lambda[1], mu=mu, sigsqrd=sigma^2, 
                           eps=epsilon, maxit=maxit, verb=verb)
  } else {
    z <- parse.constraints(mean.constr, k=k, allsame=!arbmean)
    meancat <- z$category; meanalpha <- z$alpha
    z <- parse.constraints(sd.constr, k=k, allsame=!arbvar)
    sdcat <- z$category; sdalpha <- z$alpha
    ECM <- ECM || any(meancat != 1:k) || any(sdcat != 1)
    n <- length(x)
    notdone <- TRUE
    restarts <- 0
    while(notdone) {
      # Initialize everything
      notdone <- FALSE
      tmp <- normalmix.init(x = x, lambda = lambda, mu = mu, s = sigma, 
                            k = k, arbmean = arbmean, arbvar = arbvar)
      lambda <- tmp$lambda
      mu <- tmp$mu
      k <- tmp$k
      sigma <- tmp$s
      var <- sigma^2
      diff <- epsilon+1
      iter <- 0
      postprobs <- matrix(nrow = n, ncol = k)
      mu <- rep(mu, k)[1:k]
      sigma <- rep(sigma,k)[1:k]
      # Initialization E-step here:
      z <- .C(C_normpost, as.integer(n), as.integer(k),
              as.double(x), as.double(mu), 
              as.double(sigma), as.double(lambda),
              res2 = double(n*k), double(3*k), post = double(n*k),
              loglik = double(1), PACKAGE = "mixtools")
      postprobs <- matrix(z$post, nrow=n)
      res <- matrix(z$res2, nrow=n)
      ll <- obsloglik <- z$loglik
      while (diff > epsilon && iter < maxit) {
        # ECM loop, 1st M-step: condition on sigma, update lambda and mu
        lambda <- colMeans(postprobs)
        mu[meancat==0] <- meanalpha[meancat==0]
        if (max(meancat)>0) {
          for(i in 1:max(meancat)) {
            w <- which(meancat==i)
            if (length(w)==1) {
              mu[w] <- sum(postprobs[,w]*x) / (n*lambda[w])
            } else {
              tmp <- t(postprobs[,w])*(meanalpha[w]/sigma[w]^2)
              mu[w] <- meanalpha[w] * sum(t(tmp)*x) / sum(tmp*meanalpha[w])
            }
          }
        }

        if (ECM) {  # If ECM==FALSE, then this is a true EM algorithm and
                    # so we omit the E-step between the mu and sigma updates
          # E-step number one:
          z <- .C(C_normpost, as.integer(n), as.integer(k),
                  as.double(x), as.double(mu), 
                  as.double(sigma), as.double(lambda),
                  res2 = double(n*k), double(3*k), post = double(n*k),
                  loglik = double(1), PACKAGE = "mixtools")
          postprobs <- matrix(z$post, nrow=n)
          res <- matrix(z$res2, nrow=n)

        # ECM loop, 2nd M-step: condition on mu, update lambda and sigma
          lambda <- colMeans(postprobs) # Redundant if ECM==FALSE
        }
        sigma[sdcat==0] <- sdalpha[sdcat==0]
        if (max(sdcat)>0) {
          for(i in 1:max(sdcat)) {
            w <- which(sdcat==i)
            if (length(w)==1) {
              sigma[w] <- sqrt(sum(postprobs[,w]*res[,w]) / (n*lambda[w]))
            } else {
              tmp <- t(postprobs[,w]) / sdalpha[w]
              sigma[w] <- sdalpha[w] * sqrt(sum(t(tmp) * res[,w])/ (n * sum(lambda[w])))
            }
          }
          if(any(sigma < 1e-08)) {
            notdone <- TRUE
            cat("One of the variances is going to zero; ",
                "trying new starting values.\n")
            restarts <- restarts + 1
            lambda <- mu <- sigma <- NULL
            if(restarts>maxrestarts) { stop("Too many tries!") }
            break
          }
        }
        
        # E-step number two:
        z <- .C(C_normpost, as.integer(n), as.integer(k),
                as.double(x), as.double(mu), 
                as.double(sigma), as.double(lambda),
                res2 = double(n*k), double(3*k), post = double(n*k),
                loglik = double(1), PACKAGE = "mixtools")
        postprobs <- matrix(z$post, nrow=n)
        res <- matrix(z$res2, nrow=n)
        newobsloglik <- z$loglik
        diff <- newobsloglik - obsloglik
        obsloglik <- newobsloglik
        ll <- c(ll, obsloglik)
        iter <- iter + 1
        if (verb) {
          cat("iteration =", iter, " log-lik diff =", diff, " log-lik =", 
              obsloglik, "\n")
          print(rbind(lambda, mu, sigma))
        }
      }
    }
    if (iter == maxit) {
      cat("WARNING! NOT CONVERGENT!", "\n")
    }
    cat("number of iterations=", iter, "\n")
    if(arbmean == FALSE){
      scale.order = order(sigma)
      sigma.min = min(sigma)
      postprobs = postprobs[,scale.order]
      colnames(postprobs) <- c(paste("comp", ".", 1:k, sep = ""))
      a=list(x=x, lambda = lambda[scale.order], mu = mu, sigma = sigma.min, 
             scale = sigma[scale.order]/sigma.min, loglik = obsloglik, 
             posterior = postprobs, all.loglik=ll, restarts=restarts, 
             ft="normalmixEM")
    } else {
      colnames(postprobs) <- c(paste("comp", ".", 1:k, sep = ""))
      a=list(x=x, lambda = lambda, mu = mu, sigma = sigma, loglik = obsloglik, 
             posterior = postprobs, all.loglik=ll, restarts=restarts, 
             ft="normalmixEM")
    }
  }
  class(a) = "mixEM"
  options(warn) # Reset warnings to original value
  a
}

wkde <- function(x, u=x, w=rep(1, length(x)), bw=bw.nrd0(as.vector(x)), sym=FALSE) {
  if (sym) {
    return((wkde(x, u, w, bw) + wkde(x, -u, w, bw))/2)
  }
  Km <- exp(outer(x/bw, u/bw, function(a,b) -(a-b)^2/2))
	normw <- matrix(w/sum(w), nrow=1)
  as.vector(normw %*% Km) / (bw * sqrt(2*pi))
}

mvnormalmixEM = function (x, lambda = NULL, mu = NULL, sigma = NULL, k = 2, arbmean = TRUE, arbvar = TRUE, 
    epsilon = 1e-08, maxit = 10000, verb = FALSE) 
{
    if(arbmean == FALSE && arbvar == FALSE){
    stop(paste("Must change constraints on mu and/or sigma!","\n"))
    }
    x <- as.matrix(x)
    n <- nrow(x)
    p <- ncol(x)
    tmp <- mvnormalmix.init(x = x, lambda = lambda, mu = mu, 
        sigma = sigma, k = k, arbmean=arbmean, arbvar = arbvar)
    lambda <- tmp$lambda
    mu<-tmp$mu
    sigma <- tmp$sigma
    k = tmp$k
    diff <- 1
    iter <- 0
    if (arbmean==FALSE){
        comp <- lapply(1:k, function(i) lambda[i] * dmvnorm(x, 
            mu, sigma[[i]]))
	} else{
    if (arbvar==FALSE) {
        comp <- lapply(1:k, function(i) lambda[i] * dmvnorm(x, 
            mu[[i]], sigma))
    }
    else comp <- lapply(1:k, function(i) lambda[i] * dmvnorm(x, 
        mu[[i]], sigma[[i]]))
    }
    comp <- sapply(comp, cbind)
    compsum <- apply(comp, 1, sum)
    obsloglik <- sum(log(compsum))
    ll <- obsloglik
	restarts <- 0
    while (diff > epsilon & iter < maxit) {
        if (arbvar) {
            z = matrix(nrow = n, ncol = k)
            for (i in 1:n) {
                for (j in 1:k) {
                  z.denom = c()
                  for (m in 1:k) {
                    z.denom = c(z.denom, lambda[m]/lambda[j] * 
                      (det(sigma[[j]])/det(sigma[[m]]))^(0.5) * 
                      exp(-0.5 * ((x[i, ] - mu[[m]]) %*% solve(sigma[[m]]) %*% 
                        t(t(x[i, ] - mu[[m]])) - (x[i, ] - mu[[j]]) %*% 
                        solve(sigma[[j]]) %*% t(t(x[i, ] - mu[[j]])))))
                  }
                  z[i, j] = 1/sum(z.denom)
                }
            }
	z = z/apply(z,1,sum)
#	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)
            sing <- sum(is.nan(z))
            lambda.new <- apply(z, 2, mean)
            if (sum(lambda.new < 1e-08)>0 || is.na(sum(lambda.new))) {
                sing <- 1
            }
            else {
            if(arbmean==FALSE) {
                  mu.new <- lapply(1:k, function(j) sapply(1:p, 
                  function(i) apply(z * x[, i], 2, sum))[j, ])
            mu.new <- apply(sapply(mu.new,as.vector),1,sum)/n
            mu.new <- lapply(1:k, function(j) mu.new)
            } else{
                mu.new <- lapply(1:k, function(j) sapply(1:p, 
                  function(i) apply(z * x[, i], 2, sum))[j, ]/sum(z[, 
                  j]))
            }
                sigma.new <- lapply(1:k, function(j) matrix(apply(sapply(1:n, 
                  function(i) z[i, j] * (x[i, ] - mu.new[[j]]) %*% 
                    t(x[i, ] - mu.new[[j]])), 1, sum), p, p)/sum(z[, 
                  j]))
                lambda <- lambda.new
                mu <- mu.new
                sigma <- sigma.new
                comp <- lapply(1:k, function(i) lambda[i] * dmvnorm(x, 
                  mu[[i]], sigma[[i]]))
                comp <- sapply(comp, cbind)
                compsum <- apply(comp, 1, sum)
                newobsloglik <- sum(log(compsum))
            }
        }
        else {
            z = matrix(nrow = n, ncol = k)
            sigma.inv = solve(sigma)
            for (i in 1:n) {
                for (j in 1:k) {
                  z.denom = c()
                  for (m in 1:k) {
                    z.denom = c(z.denom, lambda[m]/lambda[j] * 
                      (det(sigma.inv)/det(sigma.inv))^(0.5) * 
                      exp(-0.5 * ((x[i, ] - mu[[m]]) %*% sigma.inv %*% 
                        t(t(x[i, ] - mu[[m]])) - (x[i, ] - mu[[j]]) %*% 
                        sigma.inv %*% t(t(x[i, ] - mu[[j]])))))
                  }
                  z[i, j] = 1/sum(z.denom)
                }
            }
#	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)
	z = z/apply(z,1,sum)

            sing <- sum(is.nan(z))
            lambda.new <- apply(z, 2, mean)
            if (sum(lambda.new < 1e-08)>0 || is.na(sum(lambda.new))) {
                sing <- 1
            }
            else {
            if(arbmean==FALSE) {
                  mu.new <- lapply(1:k, function(j) sapply(1:p, 
                  function(i) apply(z * x[, i], 2, sum))[j, ])
            mu.new <- apply(sapply(mu.new,as.vector),1,sum)/n
            mu.new <- lapply(1:k, function(j) mu.new)
            } else{
                mu.new <- lapply(1:k, function(j) sapply(1:p, 
                  function(i) apply(z * x[, i], 2, sum))[j, ]/sum(z[, 
                  j]))
            }
                temp.sig <- lapply(1:k, function(j) matrix(apply(sapply(1:n, 
                  function(i) z[i, j] * (x[i, ] - mu.new[[j]]) %*% 
                    t(x[i, ] - mu.new[[j]])), 1, sum), p, p))
                sigma.new <- matrix(apply(sapply(temp.sig, as.vector), 
                  1, sum), p, p)/n
                lambda <- lambda.new
                mu <- mu.new
                sigma <- sigma.new
                comp <- lapply(1:k, function(i) lambda[i] * dmvnorm(x, 
                  mu[[i]], sigma))
                comp <- sapply(comp, cbind)
                compsum <- apply(comp, 1, sum)
                newobsloglik <- sum(log(compsum))
            }
        }
        if (sing > 0 || is.na(newobsloglik) || abs(newobsloglik) == Inf){# || sum(z) != n) {
            cat("Need new starting values due to singularity...", 
                "\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
            tmp <- mvnormalmix.init(x = x, k = k, arbmean=arbmean, arbvar = arbvar)
            lambda <- tmp$lambda
            mu <- tmp$mu
            sigma <- tmp$sigma
            k = tmp$k
            diff <- 1
            iter <- 0
            if (arbvar) {
                comp <- lapply(1:k, function(i) lambda[i] * dmvnorm(x, 
                  mu[[i]], sigma[[i]]))
            }
            else comp <- lapply(1:k, function(i) lambda[i] * 
                dmvnorm(x, mu[[i]], sigma))
            comp <- sapply(comp, cbind)
            compsum <- apply(comp, 1, sum)
            obsloglik <- sum(log(compsum))
        ll <- obsloglik
        }
        else {
            diff <- newobsloglik - obsloglik
            obsloglik <- newobsloglik
        ll <- c(ll, obsloglik)
            iter <- iter + 1
            if (verb) {
                cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                  obsloglik, "\n")
            }
        }
    }
    if(arbmean==FALSE) {
    mu = mu[[1]]
    }
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
    colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    cat("number of iterations=", iter, "\n")
    a=list(x=x, lambda = lambda, mu = mu, sigma = sigma, 
        loglik = obsloglik, posterior = z, all.loglik=ll, restarts=restarts, ft="mvnormalmixEM")
    class(a) = "mixEM"
    a
}

makemultdata = function (..., cuts) 
{
    temp = sapply(list(...), length)
    m = max(temp)
    g <- length(cuts) + 1
    cuts <- sort(cuts)
    if (sum(temp != m) > 0) {
        full.data <- function(x, maxm) {
            if (!missing(x)) {
                if (length(x) < maxm) {
                  x <- c(x, NA * rep(1, maxm - length(x)))
                }
            }
            else {
                x <- numeric(0)
            }
            x
        }
        x = sapply(list(...), full.data, maxm = m)
    }
    else {
        if (sapply(list(...), is.matrix) == 1 | sapply(list(...), is.data.frame) == 1) {
            x = t(...)
        }
        else x = cbind(...)
    }
    cutfunc <- function(x, lcut, ucut) {
        x <- na.omit(x)
        sum((x <= ucut) * (x > lcut))
    }
    n <- ncol(x)
    y <- matrix(0, g, n)
    y[1, ] <- apply(x, 2, cutfunc, ucut = cuts[1], lcut = -Inf)
    y[g, ] <- apply(x, 2, cutfunc, ucut = Inf, lcut = cuts[g - 
        1])
    if (g > 2) {
        for (i in 2:(g - 1)) {
            y[i, ] <- apply(x, 2, cutfunc, ucut = cuts[i], lcut = cuts[i - 
                1])
        }
    }
    list(x = t(x), y = t(y))
}

plotly_spEM <- plotly_npEM <- function(
  x, blocks = NULL, hist=TRUE, addlegend=TRUE,
  scale = TRUE, title=NULL, breaks="Sturges", 
  dens.col = NULL, newplot=TRUE, ylim = NULL ,
  col.hist = "#1f77b4",
  width = 3, title.x = 0.5 , title.y = 0.95, title.size = 15,
  xlab = "X" , xlab.size = 15 , xtick.size = 15,
  ylab = "Density" , ylab.size = 15 , ytick.size = 15,
  legend.text = "Posteriors",
  legend.text.size = 15,
  legend.size = 15
  ){
  r <- NCOL(x$data)
  m <- NCOL(x$posteriors)
  if(is.null(dens.col)){
    dens.col <- hue_pal()(m)
  }
  if (length(dens.col) > m){
    print(paste("Please specify",m,"colors in 'dens.col'."))
  }
  blockid <- x$blockid
  if (is.null(blocks)) {
    if(!is.null(blockid)) {
      blocks <- 1:max(blockid)
    } else {
      blocks <- blockid <- 1:r
    }
  }
  ylim.orig <- ylim
  out <- list(x=list(), y=list())
  if (!newplot) {
    hist <- FALSE
  }
  ############################
  plot.all <- plot_ly()%>%
    plotly::layout(
      legend = list(title=list(text=legend.text,
                               font=list(size=legend.text.size)),
                    font = list(size=legend.size)),
      title = list(text = "Densities for Different Posteriors",
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for(i in 1:length(blocks)){
    coords <- blockid == blocks[i]
    ylim <- ylim.orig
    if (is.null(title)) {
      if (r>1) {
        tt <- paste(which(coords), collapse=",")
        tt <- paste("Coordinate", ifelse(sum(coords)>1, "s ", " "), tt, sep="")
      } else {
        tt <- "Density Curves"
      }
    } else {
      tt <- rep(title,length(blocks))[i]
    }
    dx <- dy <- NULL
    for (j in 1:m) {
      d <- density(x, component=j, block=blocks[i], scale=scale)
      dx <- cbind(dx, d$x)
      dy <- cbind(dy, d$y)
    }
    xx <- as.vector(as.matrix(x$data)[,coords])
    if (is.null(ylim)) {
      ylim=range(dy)
      if (hist) {
        ylim[2] <- max(ylim[2], hist(xx, breaks=breaks, plot=FALSE)$density)
      }
    }
    if (newplot){
      plot.new <- plot_ly()%>%
        plotly::layout(
          legend = list(title=list(text=legend.text,
                                   font=list(size=legend.text.size)),
                        font = list(size=legend.size)),
          title = list(text = tt,
                       x = title.x,
                       y = title.y,
                       font = list(size=title.size)),
          xaxis = list(title = list(text = xlab,
                                    font = list(size = xlab.size)),
                       tickfont = list(size = xtick.size)
          ),
          yaxis = list(title = list(text = ylab,
                                    font = list(size = ylab.size)),
                       tickfont = list(size = ytick.size)
          )
        )
      for (j in 1:m){
        plot.new <- plot.new %>%
          add_trace(x=dx[,j] , 
                    y=dy[,j] , type = 'scatter' , mode = 'lines',
                    line = list(width = width , color = dens.col[j]),
                    name = paste("Posterior",j), showlegend = addlegend)
      }
      if (hist){
        plot.new <- plot.new%>%
          add_trace(x=xx ,
                    type = 'histogram', histnorm = "probability density",
                    name = 'Data' , showlegend = FALSE,
                    marker = list(color = col.hist,
                                  line = list(color = col.hist))
          )%>%
          plotly::layout(bargap = 0.01)
      }
      print(plot.new)
    } else {
      if (i > 1){addlegend <- FALSE}
      for (j in 1:m){
        plot.all <- plot.all %>%
          add_trace(x=dx[,j] , 
                    y=dy[,j] , type = 'scatter' , mode = 'lines',
                    line = list(width = width , color = dens.col[j]),
                    name = paste("Posterior",j), showlegend = addlegend)
      }
    }
  }
  if (!newplot){print(plot.all)}
}

regmixmodel.sel = function (x, y, w=NULL, k = 2, type = c("fixed", "random", "mixed"), ...) 
{
    aic <- NULL
    bic <- NULL
    caic <- NULL
    icl <- NULL
    type <- match.arg(type)
        AIC <- function(emout, p) {
            emout$loglik - (p - 1)
        }
        BIC <- function(emout, p, n) {
            emout$loglik - log(n) * (p - 1)/2
        }
        CAIC <- function(emout, p, n) {
            emout$loglik - (log(n) + 1) * (p - 1)/2
        }
        ICL <- function(emout, p, n) {
            BIC(emout, p, n) - sum(emout$lambda * log(emout$lambda))
        }

    if (type == "fixed") {
	p <- function(emout) {length(emout$beta) + length(emout$sigma) + (length(emout$scale)-1)*is.null(emout$scale) +length(emout$lambda)}
	n <- length(y)
        for (i in 1:k) {
            if (i == 1) {
                a <- glm(y ~ x)
                beta <- matrix(a$coef, ncol = 1)
                loglik <- log(prod(dnorm(y, mean = beta[1, ] + 
                  as.matrix(x) %*% beta[2:nrow(beta), ], sd = sd(a$res))))
                emout <- list(beta = beta, sigma = sd(a$res), 
                  lambda = 1, loglik = loglik)
            }
            else emout <- regmixEM(y, x, k = i, ...)
		P = p(emout)
            aic[i] <- AIC(emout, p=P)
            bic[i] <- BIC(emout, p=P, n=n)
            caic[i] <- CAIC(emout, p=P, n=n)
            icl[i] <- ICL(emout, p=P, n=n)
        }
    }
    else if (type == "random") {
	p <- function(emout) {
		if(is.list(emout$R)){ 
			m<-nrow(emout$R[[1]])
			v<-length(emout$R)
		} else {
		m<-nrow(emout$R)
		v<-1}
length(emout$mu) +(v*m*(m+1)/2)+ length(emout$sigma)+  length(emout$lambda)}
	n <- sum(sapply(y,length))
        for (j in 1:k) {
            if (j == 1) {
                a <- lapply(1:length(y), function(i) glm(y[[i]] ~ 
                  x[[i]]))
                aa <- sapply(1:length(a), function(i) as.vector(a[[i]]$res))
                b <- sapply(1:length(a), function(i) as.matrix(a[[i]]$coef))
                mu <- apply(b, 1, mean)
                R <- cov(t(b))
                a.res <- NULL
                for (i in 1:length(aa)) {
                  a.res <- c(a.res, aa[[i]])
                }
                sd.a <- sd(a.res)
                loglik <- sapply(1:length(y), function(i) dmvnorm(as.vector(y[[i]]), 
                  b[1, i] + as.matrix(x[[i]]) %*% b[2:nrow(b), 
                    i], sd.a^2 * diag(length(y[[i]]))))
                loglik <- log(prod(loglik))
                emout <- list(mu = mu, R = R, sigma = sd.a, lambda = 1, 
                  loglik = loglik)
            }
            else emout <- regmixEM.mixed(y, x, k = j, ...)
		P = p(emout)
            aic[j] <- AIC(emout, p=P)
            bic[j] <- BIC(emout, p=P, n=n)
            caic[j] <- CAIC(emout, p=P, n=n)
            icl[j] <- ICL(emout, p=P, n=n)
        }
    }
    else if (type == "mixed") {
	p <- function(emout) {
		if(is.list(emout$R)){ 
			m<-nrow(emout$R[[1]])
			v<-length(emout$R)
		} else {
		m<-nrow(emout$R)
		v<-1}
length(emout$alpha)+ length(emout$mu) +(v*m*(m+1)/2)+ length(emout$sigma)+  length(emout$lambda)}
	n <- sum(sapply(y,length))
        for (i in 1:k) {
            if (i == 1) {
                a <- lapply(1:length(y), function(i) glm(y[[i]] ~ 
                  x[[i]] + w[[i]]))
                aa <- sapply(1:length(a), function(i) as.vector(a[[i]]$res))
                alpha <- sapply(1:length(a), function(i) matrix(a[[i]]$coef[(ncol(x[[i]]) + 
                  2):(ncol(x[[i]]) + 1 + ncol(w[[i]]))]))
                alpha <- apply(as.matrix(alpha), 1, mean)
                b <- sapply(1:length(a), function(i) matrix(a[[i]]$coef[1:(ncol(x[[i]]) + 
                  1)]))
                mu <- apply(b, 1, mean)
                R <- cov(t(b))
                a.res <- NULL
                for (i in 1:length(aa)) {
                  a.res <- c(a.res, aa[[i]])
                }
                sd.a <- sd(a.res)
                loglik <- sapply(1:length(y), function(i) dmvnorm(as.vector(y[[i]]), 
                  b[1, i] + as.matrix(x[[i]]) %*% b[2:nrow(b), 
                    i], sd.a^2 * diag(length(y[[i]]))))
                loglik <- log(prod(loglik))
                emout <- list(mu = mu, R = R, alpha = alpha, 
                  sigma = sd.a, lambda = 1, loglik = loglik)
            }
            else emout <- regmixEM.mixed(y, x, k = i, ...)
		P = p(emout)
            aic[i] <- AIC(emout, p=P)
            bic[i] <- BIC(emout, p=P, n=n)
            caic[i] <- CAIC(emout, p=P, n=n)
            icl[i] <- ICL(emout, p=P, n=n)
        }
    }
    out = rbind(aic, bic, caic, icl)
    Winner = apply(out, 1, function(x) (1:length(x))[x == max(x)])
    colnames(out) = 1:k
    rownames(out) = c("AIC", "BIC", "CAIC", "ICL")
    cbind(out, Winner)
}

multmix.init <- function(y, lambda = NULL, theta = NULL, k = 2){

n <- nrow(y)
p <- ncol(y)

    if (is.null(theta)) {
        theta = matrix(runif(p * k), k, p)
        theta = theta/apply(theta, 1, sum)
    }
    else k = nrow(theta)
    if (is.null(lambda)) {
        lambda = runif(k)
        lambda = lambda/sum(lambda)
    } else k = length(lambda)

list(lambda=lambda, theta=theta, k=k)


}

lambda=function (z, x, xi, h = NULL, kernel = c("Gaussian", "Beta", 
    "Triangle", "Cosinus", "Optcosinus"), g = 0) 
{
  if (is.null(h)) {
    cat("WARNING! BANDWIDTH MUST BE SPECIFIED!", "\n")
  }
  X.mat=cbind(1,(xi-x))
  kernel <- match.arg(kernel)
  inwindow <- (abs((xi - x)/h) <= 1)
  if (kernel == "Gaussian") {
    W=(kern.G(x, xi, h) * inwindow)
  }
  else if (kernel == "Beta") {
    W=(kern.B(x, xi, h, g) * inwindow)
  }
  else if (kernel == "Triangle") {
    W=(kern.T(x, xi, h) * inwindow)
  }
  else if (kernel == "Cosinus") {
    W=(kern.C(x, xi, h) * inwindow)
  }
  else if (kernel == "Optcosinus") {
    W=(kern.O(x, xi, h) * inwindow)
  }
  A=try(solve(t(X.mat)%*%(W*X.mat)), silent=TRUE)
  if(class(A)[1]=="try-error") {
    A=ginv(t(X.mat)%*%(W*X.mat))
  }
  beta.x=A%*%t(X.mat)%*%(W*z)
  beta.x
}

repnormmixEM = function (x, lambda = NULL, mu = NULL, sigma = NULL, k = 2, arbmean = TRUE, arbvar = TRUE, 
    epsilon = 1e-08, maxit = 10000, verb = FALSE) 
{
    if(arbmean == FALSE && arbvar == FALSE){
   stop(paste("Must change constraints on mu and/or sigma!","\n"))
    }
    s = sigma
    n <- ncol(x)
    m <- nrow(x)
    tmp <- repnormmix.init(x = x, lambda = lambda, mu = mu, s = s, 
        k = k, arbmean = arbmean, arbvar = arbvar)
    lambda <- tmp$lambda
    mu <- tmp$mu
    s <- tmp$s
    s.2 <- s^2
    k <- tmp$k
    diff <- 1
    iter <- 0
    res <- lapply(1:k, function(i) (x - mu[i * arbmean + (1 - arbmean)])^2)
    comp <- lapply(1:k, function(i) lambda[i] * (2 * pi * s.2[i * 
        arbvar + (1 - arbvar)])^(-m/2) * exp(-0.5 * apply(res[[i]], 
        2, sum)/s.2[i * arbvar + (1 - arbvar)]))
    obsloglik <- sum(log(apply(sapply(comp, rbind), 1, sum)))
    ll <- obsloglik
	restarts <- 0
    while (diff > epsilon & iter < maxit) {
        xmu <- lapply(1:k, function(i) apply(res[[i]], 2, sum))
        z = matrix(nrow = n, ncol = k)
        for (i in 1:n) {
            for (j in 1:k) {
                z.denom = c()
                for (h in 1:k) {
                  z.denom = c(z.denom, lambda[h]/lambda[j] * 
                    (s[j * arbvar + (1 - arbvar)]/s[h * arbvar + 
                      (1 - arbvar)])^m * exp(-0.5 * ((1/s[h * 
                    arbvar + (1 - arbvar)]^2) * xmu[[h]][i] - 
                    (1/s[j * arbvar + (1 - arbvar)]^2) * xmu[[j]][i])))
                }
                z[i, j] = 1/sum(z.denom)
            }
        }
	z = z/apply(z,1,sum)
#	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)

        lambda.new <- apply(z, 2, mean)
        if (sum(lambda.new < 1e-08)>0 || is.na(sum(lambda.new))) {
            sing <- 1
        }
        else {
        if(arbmean){
            mu.new <- sapply(1:k, function(i) sum(t(z[, i] * 
                t(x))))/(m * apply(z, 2, sum))
        } else {
            mu.new <- sum(sapply(1:k, function(i) sum(t(z[, i] * 
                t(x)))))/(m * n)
        }
            res <- lapply(1:k, function(i) (x - mu.new[i * arbmean + (1 - arbmean)])^2)
            if (arbvar) {
                s.new <- sqrt(sapply(1:k, function(i) sum(t(z[, 
                  i] * t(res[[i]]))))/(m * apply(z, 2, sum)))
            }
            else s.new <- sqrt(sum(sapply(1:k, function(i) sum(t(z[, 
                i] * t(res[[i]])))))/(m * n))
            lambda <- lambda.new
            mu <- mu.new
            s <- s.new
            s.2 <- s^2
            sing <- sum(is.nan(z))
            comp <- lapply(1:k, function(i) lambda[i] * (2 * 
                pi * s.2[i * arbvar + (1 - arbvar)])^(-m/2) * 
                exp(-0.5 * apply(res[[i]], 2, sum)/s.2[i * arbvar + 
                  (1 - arbvar)]))
            newobsloglik <- sum(log(apply(sapply(comp, rbind), 
                1, sum)))
        }
        if (sing > 0 || is.na(newobsloglik) || abs(newobsloglik) == Inf ){#|| sum(z) != n) {
            cat("Need new starting values due to singularity...", 
                "\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
            tmp <- repnormmix.init(x = x, k = k, arbmean = arbmean, arbvar = arbvar)
            lambda <- tmp$lambda
            mu <- tmp$mu
            s <- tmp$s
            s.2 <- s^2
            k <- tmp$k
            diff <- 1
            iter <- 0
            res <- lapply(1:k, function(i) (x - mu[i * arbmean + (1 - arbmean)])^2)
            comp <- lapply(1:k, function(i) lambda[i] * (2 * 
                pi * s.2[i * arbvar + (1 - arbvar)])^(-m/2) * 
                exp(-0.5 * apply(res[[i]], 2, sum)/s.2[i * arbvar + 
                  (1 - arbvar)]))
            obsloglik <- sum(log(apply(sapply(comp, rbind), 1, 
                sum)))
        ll <- obsloglik
        }
        else {
            diff <- newobsloglik - obsloglik
            obsloglik <- newobsloglik
        ll <- c(ll, obsloglik)
            iter <- iter + 1
            if (verb) {
                cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                  obsloglik, "\n")
            }
        }
    }
    scale.order = order(s)
    sigma.min = min(s)
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
    cat("number of iterations=", iter, "\n")
    if(arbmean == FALSE){
    z = z[,scale.order]
colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=as.data.frame(x), lambda = lambda[scale.order], mu = mu, sigma = sigma.min, scale = s[scale.order]/sigma.min, loglik = obsloglik, 
        posterior = z[,scale.order], all.loglik=ll, restarts = restarts, ft="repnormmixEM")
    class(a) = "mixEM"
    a
    } else {
colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=as.data.frame(x), lambda = lambda, mu = mu, sigma = s, loglik = obsloglik, 
        posterior = z, all.loglik=ll, restarts = restarts, ft="repnormmixEM")
    class(a) = "mixEM"
    a
    }
}

print.npEM <- function(x, ...) {
  n <- NROW(x$data)
  r <- NCOL(x$data)  
  m <- length(x$lambdahat)
  cat(paste("Observations:", n, "\n"))
  cat(paste("Coordinates per observation:", r, "\n"))
  cat(paste("Mixture components:", m, "\n"))
  if (r>1) {
    B <- max(x$blockid)
    cat(paste("Blocks (of conditionally iid coordinates):", B, "\n\n"))
  }
  dp = match(c("data","posteriors", "lambda", "mu"), names(x), nomatch=0)
  print.default(structure(x[-dp], class=class(x)), ...)
  invisible(x)
}

## EM-like algorithm for a nonparametric univariate mixture model with
## symmetric components from a location family
spEMsymloc <- function(x, mu0, bw = bw.nrd0(x), h=bw, eps = 1e-8, maxiter=100,
                        stochastic = FALSE, verbose = FALSE){
  bw <- h # h is alternative bandwidth argument, for backward compatibility
  n <- length(x)
  if (length(mu0)>1) m <- length(mu0) # mu0=centers
  	else m <- mu0 # when mu0=number of clusters
  z.hat <- matrix(0, nrow=n, ncol=m)
  fkernel <- matrix(0, nrow=n, ncol=m)
  tt0 <- proc.time()
  lambda <- rep(1/m, m)
  kmeans <- kmeans(x, mu0)
  for(j in 1:m) {
    z.hat[kmeans$cluster==j, j] <- 1
  }
  iter <- 0
  if (stochastic) {
    sumpost <- matrix(0, n, m)
  }
  finished <- FALSE
  lambda <- mu <- matrix(0,maxiter,m)
  while (!finished) {
  #while (max(abs(change)) > eps & iter < maxiter) {
    iter <- iter + 1
    t0 <- proc.time()

    ## M-Step
    lambda[iter,] <- colMeans(z.hat) 
    mu[iter,] <- apply(sweep(z.hat, 1, x, "*"), 2, mean)/lambda[iter,]

    ## density estimation step
    if(stochastic){
      z <- t(apply(z.hat, 1, function(prob) rmultinom(1, 1, prob)))
      ans <- .C(C_KDEsymloc, n=as.integer(n), m=as.integer(m),
                mu=as.double(mu[iter,]), x=as.double(x), bw=as.double(bw),
                z=as.double(z), f = double(n*m),
                PACKAGE="mixtools")
    } else {
      ans <- .C(C_KDEsymloc, n=as.integer(n), m=as.integer(m),
                mu=as.double(mu[iter,]), x=as.double(x), bw=as.double(bw),
                z=as.double(z.hat), f = double(n*m),                     
                PACKAGE="mixtools")
    }

    fkernel <- matrix(ans$f, ncol=m)
    lambda.f <- sweep(fkernel, 2, lambda[iter,], "*")

    ## E-step (for next iteration)
    z.hat <- lambda.f/rowSums(lambda.f)    
    finished <- iter >= maxiter
    if (stochastic) {
      sumpost <- sumpost + z.hat
    } else if (iter>1) { # This convergence criterion is too simplistic:
      change <- c(lambda[iter,] - lambda[iter-1,], mu[iter,]-mu[iter-1,])
      finished <- finished | (max(abs(change)) < eps)
    }
    if (verbose) {
      t1 <- proc.time()
      cat("iteration ", iter, "  lambda ", round(lambda[iter,], 4), 
          "  mu ", round(mu[iter,], 4))
      cat(" time", (t1 - t0)[3], "\n")
    }
  }
  if (verbose) {
    tt1 <- proc.time()
    cat("lambda ", round(lambda[iter,], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")
  }
  if(stochastic) {
    return(structure(list(data=x, posteriors=sumpost/iter, lambda=lambda,
                          bandwidth=bw, lambdahat=colMeans(lambda), 
                        mu = mu, muhat = colMeans(mu), symmetric=TRUE),
              class="npEM"))
  } else {
    return(structure(list(data=x, posteriors=z.hat, lambda=lambda[1:iter,],
                          bandwidth=bw, lambdahat=lambda[iter,], 
                          mu = mu[1:iter,], muhat = mu[iter,], symmetric=TRUE),
                        class="npEM"))
  } 
}

###########################################################################
# log density function of multinomial count vector y with parameter theta #
###########################################################################

ldmult<-function(y,theta){
  if (any(is.nan(theta)) || any(theta==0)) {
#      stop ("The theta parameter cannot have a zero component.")
     out = -Inf
   }
   else {
     if (length(y)==length(theta)+1)
       theta=c(theta,1-sum(theta))
     out=lgamma(1+sum(y)) - sum(lgamma(1+y)) + sum(y*log(theta))
   }
   out
}

compCDF <- function(data, weights, 
                    x=seq(min(data, na.rm=TRUE), max(data, na.rm=TRUE), len=250), 
                    comp=1:NCOL(weights), makeplot=TRUE, ...) {
  if (NROW(weights) != NROW(data)) {
    stop("data and weights arguments must have same number of rows")
  }
    
  # First, normalize the weights so the sum of each column is 1/NCOL(data)
  weights <- t(t(weights) / (NCOL(data) * colSums(weights)))
  # Next, give a binomial count for each row of the data and for each x
  f <- function(row, cutpt) colSums(outer(row, cutpt, "<="), na.rm = TRUE)
  bc <- apply(data, 1, f, x)
  # bc is a length(x) by n matrix; each column should be multiplied by
  # the appropriate weight(s) and then the rows summed to give the 
  # unnormalized cdf estimates. This is just a matrix product.
  cdfs <- bc %*% weights[,comp,drop=FALSE]
  
  if(makeplot) {
    plot(range(x), 0:1, type="n", ...)
    for (i in 1:length(comp)) {
      lines(x, cdfs[,comp[i]], lty=i, ...)
    }
  }
  t(cdfs)
}

multmixmodel.sel <- function (y, comps = NULL, ...) 
{
    if (class(y)[1]=="list" && !is.null(y$y)) {
      y <- y$y
    }
    n = dim(y)[1]
    p = dim(y)[2]
    m = min(apply(y, 1, sum))
#    m = unique(apply(y, 1, sum))
#    if (length(m) > 1) {
#        stop("Each row of y must have same total number of observations")
#    }
    max.allowed.comp = floor((m + 1)/2)
    if (is.null(comps)) 
        comps = 1:max.allowed.comp
    if (max(comps) > max.allowed.comp) {
        stop(paste("No more than", max.allowed.comp, "components allowed",
                   "with", m, "multinomial trials"))
    }
    aic = NULL
    bic = NULL
    caic = NULL
    icl = NULL
    ll = NULL
    theta = matrix(0, 0, p)
    lambda = NULL
    for (k in sort(comps)) {
#        cat("Testing", k, "components:  ")
#        newrows = k - nrow(theta)
        tmp <- multmix.init(y, k = k)
        theta <- tmp$theta
        lambda <- tmp$lambda
      if (k!=1){
      em = multmixEM(y, lambda = lambda, theta = theta, k = k, ...)
        loglik = em$loglik
        lambda = em$lambda
        theta = em$theta
#        cat(em$iter, "iterations.\n")
      } else loglik = sum(log(exp(apply(y,1,ldmult,theta=theta))))
        aic = c(aic, loglik - (p * k - 1))
        bic = c(bic, loglik - log(n) * (p * k - 1)/2)
    caic = c(caic, loglik - (log(n) + 1) * (p * k - 1)/2)
    if (k==1) {
    icl = c(icl, loglik - log(n) * (p * k - 1)/2)
    } else icl = c(icl, loglik - log(n) * (p * k - 1)/2 - sum(lambda * log(lambda)))
        ll = c(ll, loglik)
    }
    out = rbind(aic, bic, caic, icl, ll)
#    Winner = apply(out, 1, function(x) (1:length(x))[x == 
#        max(x)])
    win = apply(out, 1, which.max)
    rownames(out) = c("AIC", "BIC", "CAIC", "ICL", "Loglik")
    colnames(out) = sort(comps)
    Winner = as.numeric(colnames(out)[win])
    cbind(out, Winner)
}

plotly_compCDF <- function(data, 
                           weights, 
                           x=seq(min(data, na.rm=TRUE), max(data, na.rm=TRUE), len=250), 
                           comp=1:NCOL(weights), 
                           makeplot=TRUE,
                           cex = 3,
                           width = 3,
                           legend.text = "Composition",
                           legend.text.size = 15,
                           legend.size = 15,
                           title = "Empirical CDF",
                           title.x = 0.5,
                           title.y = 0.95,
                           title.size = 15,
                           xlab = "Data",
                           xlab.size = 15,
                           xtick.size = 15,
                           ylab = "Probability",
                           ylab.size = 15,
                           ytick.size = 15,
                           col.comp = NULL){
  if (NROW(weights) != NROW(data)) {
    stop("data and weights arguments must have same number of rows")
  }
  if (is.null(col.comp)){
    col.comp <- hue_pal()(length(comp))
  }
  if (length(col.comp) != length(comp)){
    print(paste("Please specify",length(comp),"colors in 'col.comp'."))
  }
  # First, normalize the weights so the sum of each column is 1/NCOL(data)
  weights <- t(t(weights) / (NCOL(data) * colSums(weights)))
  # Next, give a binomial count for each row of the data and for each x
  f <- function(row, cutpt) colSums(outer(row, cutpt, "<="), na.rm = TRUE)
  bc <- apply(data, 1, f, x)
  # bc is a length(x) by n matrix; each column should be multiplied by
  # the appropriate weight(s) and then the rows summed to give the 
  # unnormalized cdf estimates. This is just a matrix product.
  cdfs <- bc %*% weights[,comp,drop=FALSE]
  
  if(makeplot) {
    plot <- plot_ly()
    for (i in 1:length(comp)) {
      plot <- add_trace(plot,
                        plot,
                        x=x , y=cdfs[,comp[i]] , type = 'scatter' , mode = 'lines+markers',
                        marker = list(size = cex , color = col.comp[i]),
                        line = list(width = width , color = col.comp[i]),
                        name = comp[i] , showlegend = TRUE) %>%
        plotly::layout(
          legend = list(title=list(text=legend.text,
                                   font=list(size=legend.text.size)),
                        font = list(size=legend.size)),
          title = list(text = title,
                       x = title.x,
                       y = title.y,
                       font = list(size=title.size)),
          xaxis = list(title = list(text = xlab,
                                    font = list(size = xlab.size)),
                       tickfont = list(size = xtick.size)
          ),
          yaxis = list(title = list(text = ylab,
                                    font = list(size = ylab.size)),
                       tickfont = list(size = ytick.size),
                       range = c(0 , 1)
          )
        )
    }
    print(plot)
  }
  invisible(t(cdfs))
}

regmixEM.lambda = function (y, x, lambda = NULL, beta = NULL, sigma = NULL,
  k = 2, addintercept = TRUE, arbmean = TRUE, arbvar = TRUE, epsilon = 1e-08, 
  maxit = 10000, verb = FALSE) {
  if (arbmean == FALSE && arbvar == FALSE) {
    stop(paste("Must change constraints on beta and/or sigma!", 
               "\n"))
  }
  s = sigma
  if (addintercept) {
    x = cbind(1, x)
  }
  n <- length(y)
  p <- ncol(x)
  tmp <- regmix.lambda.init(y = y, x = x, lambda = lambda, beta = beta, 
                            s = s, k = k, addintercept = addintercept,
                            arbmean = arbmean, arbvar = arbvar)
  lambda <- tmp$lambda
  beta <- tmp$beta
  s <- tmp$s
  k <- tmp$k
  diff <- 1
  iter <- 0
  xbeta <- x %*% beta
  res2 <- (y - xbeta)^2
  if (arbmean == FALSE) {
    res2 <- sapply(1:k, function(i) res2)
  }
  comp <- t(t(lambda)/sqrt(2 * pi * s^2)) * exp(-t(t(res2)/(2 * s^2)))
  obsloglik <- sum(log(apply(comp, 1, sum)))
  ll <- obsloglik
  while (diff > epsilon && iter < 1) {
    V=as.double(sweep(lambda, 2, s+rep(0,k), "/"))
    W=as.double(sweep(res2, 2, 2*(s+rep(0,k))^2, "/"))
    z <- matrix(.C(C_newz, as.integer(n), as.integer(k), V=V, W=W,
                   newz=double(n*k), PACKAGE = "mixtools")$newz, ncol=k)
	z = z/apply(z,1,sum)    
    if (addintercept) {
      lm.out <- lapply(1:k, function(i) lm(y ~ x[,-1], weights = z[, i]))
    }
    else lm.out <- lapply(1:k, function(i) lm(y ~ x - 
                                              1, weights = z[, i]))
    beta.new <- sapply(lm.out, coef)
    if (arbmean == FALSE) {
      beta.new <- apply(t(apply(z, 2, sum) * t(beta.new)), 
                        1, sum)/n
    }
    xbeta.new <- x %*% beta.new
    res2 <- (y - xbeta.new)^2
    if (arbmean == FALSE) {
      res2 <- sapply(1:k, function(i) res2)
    }
    if (arbvar) {
      s.new <- sqrt(sapply(1:k, function(i)
                           sum(z[,i] * (res2[, i]))/sum(z[, i])))
    }
    else s.new <- sqrt(sum(z * res2)/n)
    beta <- beta.new
    xbeta <- x %*% beta
    s <- s.new
    sing <- sum(s < 1e-08)
    comp <- lapply(1:k, function(i)
                   lambda[,i] * dnorm(y, xbeta[, i * arbmean +
                                               (1 - arbmean)],
                                      s[i * arbvar + 
                                        (1 - arbvar)]))
    comp <- sapply(comp, cbind)
    compsum <- apply(comp, 1, sum)
    newobsloglik <- sum(log(compsum))
    if (newobsloglik < obsloglik || sing > 0 || abs(newobsloglik) == 
        Inf || is.nan(newobsloglik)){# || sum(z) != n) {
      cat("Need new starting values due to singularity...", 
          "\n")
      tmp <- regmix.lambda.init(y = y, x = x, k = k,
                                addintercept = addintercept, 
                                arbmean = arbmean, arbvar = arbvar)
      lambda <- tmp$lambda
      beta <- tmp$beta
      s <- tmp$s
      k <- tmp$k
      diff <- 1
      iter <- 0
      xbeta <- x %*% beta
      res2 <- (y - xbeta)^2
      if (arbmean == FALSE) {
        res2 <- sapply(1:k, function(i) res2)
      }
      comp <- t(t(lambda)/sqrt(2 * pi * s^2)) * exp(-t(t(res2)/(2 * 
                                                                s^2)))
      obsloglik <- sum(log(apply(comp, 1, sum)))
      ll <- obsloglik
    }
    else {
      diff <- newobsloglik - obsloglik
      obsloglik <- newobsloglik
      ll <- c(ll, obsloglik)
      iter <- iter + 1
    }
  }
  scale.order = order(s)
  sigma.min = min(s)
  if (arbmean == FALSE) {
    z = z[, scale.order]
    names(beta) <- c(paste("beta", ".", 0:(p - 1), sep = ""))
    colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a = list(x = x, y = y, lambda = lambda[,scale.order], 
      beta = beta, sigma = sigma.min, scale = s[scale.order]/sigma.min, 
      loglik = obsloglik, posterior = z[, scale.order], 
      all.loglik = ll, ft="regmixEM.lambda")
    class(a) = "mixEM"
    a
  }
  else {
    rownames(beta) <- c(paste("beta", ".", 0:(p - 1), sep = ""))
    colnames(beta) <- c(paste("comp", ".", 1:k, sep = ""))
    colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a = list(x = x, y = y, lambda = lambda, beta = beta, 
      sigma = s, loglik = obsloglik, posterior = z, all.loglik = ll,
      ft="regmixEM.lambda")
    class(a) = "mixEM"
    a
  }
}

## EM-like algorithm for a nonparametric mixture model with
## independent repeated measures - some ID, some not
npEMindrep <- # npEMindrep is an alias (only for backward compatibility)
  npEM <- function(x, mu0, blockid = 1:ncol(x),
                   bw=bw.nrd0(as.vector(as.matrix(x))), samebw = TRUE,
                   h=bw, eps=1e-8,
                   maxiter=500, stochastic = FALSE, verb = TRUE){
    bw <- h # h is alternative bandwidth argument, for backward compatibility
    x <- as.matrix(x)
    n <- nrow(x)      # number of subjects
    r <- ncol(x)      # number of measurements in each subject
    u <- match(blockid, unique(blockid))
    if (is.matrix(mu0)) 
      m <- dim(mu0)[1]  # mu0=centers
    else 
      m <- mu0  # mu0=number of clusters
    if(!samebw && !is.matrix(bw)) { 
      bw <- matrix(bw, nrow=max(u), ncol=m)
    }
    z.hat <- matrix(0, nrow = n, ncol = m)
    tt0 <-  proc.time() # for total time
    ## Initial Values
    if(m == 1) z.hat <- matrix(1, nrow = n, ncol = m)
    else{
      kmeans <- kmeans(x, mu0)
      for(j in 1:m)
        z.hat[kmeans$cluster==j, j] <- 1
    }
    iter <- 0
    if (stochastic) {
      sumpost <- matrix(0, n, m)
    }
    finished <- FALSE
    lambda <- matrix(0, nrow = maxiter, ncol = m)
    loglik <- NULL
    orderx <- xx <- list()
    for(k in 1:max(u)) {
      xx[[k]] <- as.vector(x[, u==k])
      if (!samebw) {
        orderx[[k]] = order(xx[[k]]) # only needed for IQR calculation for bw
      }
    }
    ### Cfunction <- ifelse(samebw, "KDErepeated", "KDErepeatedbw")
    
    while (!finished) {
      iter <- iter + 1
      bw.old <- bw
      t0 <- proc.time()
      
      ## Note:  Enter loop assuming E-step is done -- i.e., z.hat in place    
      ## M-Step
      lambda[iter, ] <- colMeans(z.hat)
      
      ## density estimation step
      if (stochastic) {
        z <- t(apply(z.hat, 1, function(prob) rmultinom(1, 1, prob)))
        cs <- colSums(z)
        z.tmp <- sweep(z, 2, cs, "/")
        z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case
      }
      else {
        cs <- colSums(z.hat)
        z.tmp <- sweep(z.hat, 2, cs, "/")
        z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case
      }
      fkernel <- matrix(1, n, m)
      h <- bw # This is for samebw == TRUE
      for (k in 1:max(u)) {
        r2 <- sum(u == k)	# block size
        if (!samebw) {
          wts <- apply(z.tmp, 2, function(z) rep(z/r2, r2))
          variances <- colSums(wts * outer(xx[[k]], colSums(wts * xx[[k]]), '-')^2)
          iqr <- apply(as.matrix(wts[orderx[[k]],]), 2, wIQR, xx[[k]][orderx[[k]]], 
                       already.sorted=TRUE, already.normalized=TRUE)
          h <- bw[k, ] <- 0.9 * pmin(sqrt(variances), iqr/1.34) * 
            pmax(1,r2*n*lambda[iter, ])^(-1/5) 
          # Note:  Doesn't allow "sample size" < 1.
          #      browser()
        }
        if(samebw){
          ans <- .C(C_KDErepeated, n = as.integer(n), m = as.integer(m), 
                    r = as.integer(r2), x = as.double(x[,u==k]),
                    h = as.double(h), z = as.double(z.tmp), f = double(n*m), 
                    PACKAGE="mixtools")
        } else{
          ans <- .C(C_KDErepeatedbw, n = as.integer(n), m = as.integer(m), 
                    r = as.integer(r2), x = as.double(x[,u==k]),
                    h = as.double(h), z = as.double(z.tmp), f = double(n*m), 
                    PACKAGE="mixtools")
        }
        fkernel <- fkernel * matrix(ans$f, ncol = m)
      }
      lambda.f <- sweep(fkernel, 2, lambda[iter, ], "*")
      
      ## E-step (for next iteration)
      z.hat <- lambda.f/rowSums(lambda.f)
      
      loglik <- c(loglik,sum(log(rowSums(lambda.f)))) # log-likelihood
      finished <- iter >= maxiter
      if (stochastic) {
        sumpost <- sumpost + z.hat
      } 
      else if (iter > 1) { # This convergence criterion may be too simplistic:
        maxchange <- max(abs(lambda[iter,] - lambda[iter-1,]))
        if (!samebw) 
          maxchange <- max(maxchange, max(abs(bw.old - bw)))
        finished <- finished | (maxchange < eps)
      }
      if (verb) {
        t1 <- proc.time()
        cat("iteration", iter, "  lambda ", round(lambda[iter, ], 4))
        cat(" time", (t1 - t0)[3], "\n")
      }
    }
    if (!samebw) {
      rownames(bw) <- paste("block", 1:max(u))
      colnames(bw) <- paste("component", 1:m)
    }
    if (verb) {
      tt1 <- proc.time()
      cat("lambda ", round(lambda[iter, ], 4))
      cat(", total time", (tt1 - tt0)[3], "s\n")
    }
    if (stochastic) {
      return(structure(list(data = x, posteriors = sumpost/iter, 
                            lambda = lambda, bandwidth = bw, 
                            blockid = u, lambdahat = colMeans(lambda),
                            loglik = loglik), 
                       class="npEM"))
    }
    else {
      return(structure(list(data = x, posteriors = z.hat, 
                            lambda = lambda[1:iter,], bandwidth = bw, 
                            blockid = u, lambdahat = lambda[iter,],
                            loglik = loglik),
                       class="npEM"))
    }
  }

# Not sure whether the following function is really necessary:
npEMindrepbw <- function (x, mu0, blockid = 1:ncol(x), bw =
                            bw.nrd0(as.vector(as.matrix(x))), eps = 1e-08, 
                          maxiter = 500, stochastic = FALSE, verb = TRUE){
  npEM(x=x, mu0=mu0, blockid=blockid, bw=bw, samebw=FALSE, eps=eps,
       maxiter=maxiter, stochastic=stochastic, verb=verb)
}

logisregmixEM=function (y, x, N = NULL, lambda = NULL, beta = NULL, k = 2, 
    addintercept = TRUE, epsilon = 1e-08, maxit = 10000, verb = FALSE) 
{
    if (addintercept) {
        x = cbind(1, x)
    }
    else x = as.matrix(x)
    n <- length(y)
    p <- ncol(x)
    if (is.null(N)) {
        N = rep(1, n)
    }
    tmp <- logisregmix.init(y = y, x = x, N = N, lambda = lambda, 
        beta = beta, k = k)
    lambda <- tmp$lambda
    beta <- tmp$beta
    k <- tmp$k
    xbeta <- x %*% beta
    z <- matrix(0, n, k)
    diff <- 1
    iter <- 0
    comp <- t(t(dbinom(y, size = N, prob = inv.logit(xbeta))) * lambda)
    compsum <- apply(comp, 1, sum)
    obsloglik <- sum(log(compsum))
    ll <- obsloglik
	restarts <- 0
    while (diff > epsilon && iter < maxit) {
        j.star = apply(xbeta, 1, which.max)
        for (i in 1:n) {
            for (j in 1:k) {
                z[i, j] = lambda[j]/lambda[j.star[i]] * (inv.logit(xbeta[i, 
                  j])/inv.logit(xbeta[i, j.star[i]]))^y[i] * ((1 - 
                  inv.logit(xbeta[i, j]))/(1 - inv.logit(xbeta[i, j.star[i]])))^(N[i] - 
                  y[i])
            }
        }
        z = z/apply(z, 1, sum)
	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)

        if (sum(is.na(z)) > 0) {
            cat("Need new starting values due to underflow...", 
                "\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
            tmp <- logisregmix.init(y = y, x = x, N = N, k = k)
            lambda <- tmp$lambda
            beta <- tmp$beta
            k <- tmp$k
            diff <- 1
            iter <- 0
            xbeta <- x %*% beta
            comp <- t(t(dbinom(y, size = N, prob = inv.logit(xbeta))) * 
                lambda)
            compsum <- apply(comp, 1, sum)
            obsloglik <- sum(log(compsum))
            ll <- obsloglik
        }
        else {
            lambda <- apply(z, 2, mean)
            lm.out = lapply(1:k, function(j) try(glm.fit(x, cbind(y, 
                N - y), weights = z[, j], family = binomial()), 
                silent = TRUE))
            beta = sapply(lm.out, coef)
            xbeta <- x %*% beta
            comp <- t(t(dbinom(y, size = N, prob = inv.logit(xbeta))) * 
                lambda)
            compsum <- apply(comp, 1, sum)
            newobsloglik <- sum(log(compsum))
            if (abs(newobsloglik) == Inf || is.na(newobsloglik) || 
                newobsloglik < obsloglik){# || sum(z) != n) {
                cat("Need new starting values due to singularity...", 
                  "\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
                tmp <- logisregmix.init(y = y, x = x, N = N, 
                  k = k)
                lambda <- tmp$lambda
                beta <- tmp$beta
                k <- tmp$k
                diff <- 1
                iter <- 0
                xbeta <- x %*% beta
                comp <- t(t(dbinom(y, size = N, prob = inv.logit(xbeta))) * 
                  lambda)
                compsum <- apply(comp, 1, sum)
                obsloglik <- sum(log(compsum))
                ll <- obsloglik
            }
            else {
                diff <- newobsloglik - obsloglik
                obsloglik <- newobsloglik
                ll <- c(ll, obsloglik)
                iter <- iter + 1
                if (verb) {
                  cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                    obsloglik, "\n")
                }
            }
        }
    }
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
	beta <- matrix(beta,ncol=k)
    rownames(beta) <- c(paste("beta", ".", 0:(p - 1), sep = ""))
    colnames(beta) <- c(paste("comp", ".", 1:k, sep = ""))
    colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    cat("number of iterations=", iter, "\n")
    a = list(x = x, y = y, lambda = lambda, beta = beta, loglik = obsloglik, 
        posterior = z, all.loglik = ll, restarts=restarts, ft = "logisregmixEM")
    class(a) = "mixEM"
    a
}

regmix.lambda.init = function (y, x, lambda = NULL, beta = NULL, s = NULL, k = 2, 
    addintercept = TRUE, arbmean = TRUE, arbvar = TRUE) 
{
    x <- as.matrix(x)
    n <- length(y)
    p <- ncol(x)
    if (addintercept) {
        x = x[, -1]
    }
    else x = x
    w = cbind(y, x)
    w = w[order(w[, 1]), ]
    w.bin = list()
    for (j in 1:k) {
        w.bin[[j]] <- w[max(1, floor((j - 1) * n/k)):ceiling(j * 
            n/k), ]
    }
    if (addintercept) {
        lm.out <- lapply(1:k, function(i) lm(w.bin[[i]][, 1] ~ 
            w.bin[[i]][, 2:p]))
    }
    else lm.out <- lapply(1:k, function(i) lm(w.bin[[i]][, 1] ~ 
        w.bin[[i]][, 2:(p + 1)] - 1))
    if (is.null(s)) {
        s.hyp = lapply(lm.out, anova)
        s.hyp = as.vector(sqrt(sapply(1:k, function(i) s.hyp[[i]]$Mean[length(s.hyp[[i]]$Mean)])))
        if (arbvar) {
            s = 1/rexp(k, rate = s.hyp)
        }
        else {
            s.hyp = mean(s.hyp)
            s = 1/rexp(1, rate = s.hyp)
        }
    }
    if (is.null(s) == FALSE && arbvar == TRUE) {
        k = length(s)
    }
    if (is.null(beta)) {
        beta.hyp = matrix(sapply(lm.out, coef), ncol = k)
        beta = matrix(0, nrow = p, ncol = k)
        for (j in 1:k) {
            beta[, j] = rnorm(p, mean = as.vector(beta.hyp[, 
                j]), sd = s[arbvar * j + (1 - arbvar)])
        }
        if (arbmean == FALSE) {
            beta = apply(beta, 1, mean)
        }
    }
    if (is.null(beta) == FALSE && arbmean == TRUE) {
        k = ncol(beta)
    }
    if (is.null(lambda)) {
        lam = runif(k)
        lam = lam/sum(lam)
	for(i in 1:n) {lambda <- rbind(lambda,lam)}
    }
    else k = ncol(lambda)
    list(lambda = lambda, beta = beta, s = s, k = k)
}

###########################################################
# semi-parametric Reliability Mixture Models with Censored data
#	for 2 components                                     
# D. Chauveau
# ref: Bordes L. and Chauveau D. Computational Statistics (2016)
###########################################################

# Simulate from a lognormal scale mixture
# like model (3) in compstat paper
# lambda = vector of component probabilities
#	xi = scaling parameter
rlnormscalemix <- function(n, lambda=1, 
			meanlog=1, sdlog=1, scale=0.1) {
	m <- length(lambda) # nb of components
	z <- sample(m, n, replace = TRUE, prob = lambda) # component indicator
	x <- rlnorm(n, meanlog, sdlog)
	x[z==2] <- x[z==2]/scale
	x
}


##############################################
# Kaplan-Meier survival estimate 
# vectorized version ONLY on ** already ordered data **
# cpd = (n,2) matrix where
# cpd[,1] = t = ORDERED censored life data = min(x,c)
# cpd[,2] = d = censoring indicator, 1=observed 0=censored
# returned value = a step function object allowing to evaluate S()
# 	at any point, needed by SEM for computing S(xi*t_i)'s
KMod <- function(cpd, already.ordered=TRUE){
	# order stat of t and d ordered accordingly
	if (already.ordered) {
		n <- dim(cpd)[1]
		s <- cumprod(1 - cpd[,2]/(n - (1:n) +1))
		stepfun(cpd[,1], c(1,s))} # starts with 1 for t<=t(1)
	}


##############################################
## KM estimate integration
# s = a stepfun object as returned by KMod()
# returned value = int_0^max(t_i) S(u)du
KMintegrate <- function(s) {
	ks <- knots(s)
	n <- length(ks) # number of knots=data points
	ks2 <- c(0,ks[1:(n-1)]) # shifted right
	hs <- c(1,s(ks)[1:(n-1)]) # heights of survival
	sum((ks-ks2)*hs)
	}



################################################
###    HAZARD RATE SMOOTH KERNEL ESTIMATES   ###
################################################
# WKDE with Triangular Kernel and global bandwidth, vectorized
# returns vector of K_h(t_i-u_j), j=1,...,length(u)
# t = vector of data
# u = vector of points at which K(u) is computed
# w = weights defaults to 1/n, not forced to be normalized
# bw=bandwidth, can be a n-vector or a scalar
# nb: this is not specific for hazard rate, just a wkde 
# NB: triang_wkde is the default wkde called by HRkde() for hazard rate 
triang_wkde <- function(t, u=t, w=rep(1/length(t),length(t)), 
						bw=rep(bw.nrd0(t),length(t))) {
	n <- length(t); p <- length(u)
	xx <- outer(t, u, function(a,b) a-b) # K[i,j] = (t_i-u_j)
	xx <- abs(xx) # not necessarily for all kernels, separated from outer
	h <- matrix(bw,n,p) # bw repeated over p columns
						# works also if a scalar bw is passed!
	xx <- xx/h  # now xx is |t_i-u_j|/h_i
	K <- (xx <= 1)*(1-xx)/bw  # K((t-u)/h)/h
	mw <- matrix(w, nrow=1) # weights for matrix product
	as.vector(mw %*% K)
	}





## Hazard Rate kernel density estimate based on *ordered* censored data
# weighted kernel density estimate passed as function argument, 
# defaults to symmetric triangle kernel 
# cpd = (n,2) matrix where
# cpd[,1] = t = ORDERED censored life data = min(x,c)
# cpd[,2] = d = censoring indicator, 1=observed 0=censored
# u = vector of points at which alpha() is evaluated, 
#     defaults to t itself
# bw = bandwidth *vector* for the kernel density estimate
# kernelft = kernel definition
# return value = alpha(.) evaluated at u_j's
# NB: ordered data are necessary from weights definitions
#     (n-i+1) items at risk etc
HRkde <- function(cpd, u = cpd[,1], kernelft = triang_wkde, 
					bw = rep(bw.nrd0(as.vector(cpd[,1])), length(cpd[,1]))){
	# gaussian case (not recommended)
	# if (kernelft==gaussian) HRkdeGauss(cpd,u,bw) else {
	n <- length(cpd[,1])
	aw <- cpd[,2]/(n - (1:n) +1) # weights d(i)/(n-i+1) 
	kernelft(cpd[,1], u, w=aw, bw=bw)
# old, non vectorized version (deprecated)	
#	nu <- length(u)
#		hr <- rep(0,nu)
#		for (k in 1:nu) {
#			K <- kernelft(cpd[,1], u[k], bw)
#			hr[k] <- sum(K*aw)
#			}
#		}
#	hr
	}
	


#################################################################
#################################################################
## Stochastic EM algorithm for semiparametric Scaling
## Reliability Mixture Models (RMM) with Censoring, 2 components
# t = lifetime data, censored by random c if d is not NULL, in which case
# t = min(x,c) and d = I(x <= c)
# rate = scaling parameter
# centers = initial centers for initial call to kmeans
# averaged = TRUE if averaging performed at each iteration (cf Nielsen 2000)
# 	NB: averaging can be done in (at least) 2 ways; 
#	here the current theta for E & M steps is the average over the sequence,
#	but the theta^next not averaged is stored on the sequence 
# batchsize = number of last iterations to use for "averaging" unscaled samples 
#   for computing "average" final KM and HR estimates 
#	alpha() and S() 
#  kernelft = kernel used in HRkde for hazard rate nonparametric estimate
#
# NB: since the sequence of parameters is a stoch. process, the
# stopping criterion is usually not satisfied until maxit 

spRMM_SEM <- function (t, d = NULL, lambda = NULL, scaling = NULL, 
          centers = 2, kernelft = triang_wkde, 
          bw = rep(bw.nrd0(t),length(t)), averaged = TRUE,
          epsilon = 1e-08, maxit = 100, batchsize = 1, verb = FALSE) {
	k = 2 # fixed number of components for this model identifiability
	n <- length(t)
	if (is.null(d))  {
		d <- rep(1,n)  # but better use specific algo for noncensored data
		cat("warning: undefined censoring indicator d replaced by 1's")
		cat(" i.e. all data are assumed observed")
		cat(" better use instead a specific St-EM algorithm for uncensored data")
		}
									
	##### Initializations #####	
	# init function call to do later, in case lambda = scaling = NULL  
	if (is.null(lambda)) lambda <- rep(1/k,k) # not USED for init!?
	if (is.null(scaling)) scaling <- 1/2      # ToDo better init using centers fro kmeans!
	# sequences for storing along iterations 	
	lambda_seq <- matrix(0, nrow = maxit, ncol = k)
 	lambda_seq[1,] <- lambda
 	scaling_seq <- rep(0,maxit)
 	scaling_seq[1] <- scaling 
 	sumNaNs <- 0 # for total nb of NaN's in iterations while computing post
    # dll <- epsilon+1 # not applicable for SEM versions, always run maxit
    post <- z <- sumpost <- posthat <- matrix(0, nrow = n, ncol = k)
    
    qt=round(maxit/10); pc <- 0 # for printing of % of iteration done
	if (qt == 0) qt <- 1

    # initialization for batch storing
    if (batchsize > maxit) batchsize <- maxit
    batch_t <- NULL # for storing batchsize last unscaled samples
    batch_d	<- NULL # and associated event indicators

	# kmeans method for initial posterior and z matrix
	kmeans <- kmeans(t, centers=centers)
	for(j in 1:k) {
   		z[kmeans$cluster==j, j] <- 1
  		}
    # init values for Survival s() and hazard rate a()
	# strategy: use a M-step with z from kmeans result 
	zt <- z*t # recycling t m times => zt[i,] holds  
	zt[,2] <- scaling*zt[,2] # t if comp 1 and xi*t if comp 2
	newt <- rowSums(zt) # redressed "unscaled" sample
	newo <- order(newt)
	new.od <- cbind(newt,d)[newo,] # ordered data with associated d
	s <- KMod(new.od) # stepfun object, can be evaluated by s(t)
	# note: <=> to use KMsurvfit(newt,d) which calls survival package functions
	a1 <- HRkde(new.od, t, kernelft, bw=bw) # evaluated at the original t's
	a2  <- HRkde(new.od, scaling*t, kernelft, bw=bw) # and at scaling*t's

	####### SEM iterations #######
	iter <- 1
    # while (dll > epsilon && iter < maxit) { # EM-only version
    while (iter < maxit) {
	### E-step
		post[,1] <- ((lambda[1]*a1*s(t))^(d))*((lambda[1]*s(t))^(1-d))
		post[,2] <- (lambda[2]*scaling*a2*s(scaling*t))^d # observed
		post[,2] <- post[,2]*(lambda[2]*s(scaling*t))^(1-d) # censored
		# print(post)		
		rs <- rowSums(post) # post normalized per row
		post <- sweep(post, 1, rs, "/") # posteriors p_{ij}^t 's
		# check and solve NaN's: may cause theoretical pbs!?
		snans <- sum(is.na(post))
		if (snans > 0) {
			post[is.na(post[,1]),] <- 1/k # NaN's replaced by uniform weights
			sumNaNs <- sumNaNs + snans
			if (verb) cat(snans, "NaN's in post: ")
			}
		sumpost <- sumpost + post
				
	### S-step
	# ~ matrix of component indicators simu checked OK
		z <- t(apply(post, 1, function(prob) rmultinom(1, 1, prob))) 
		# cbind(post,z) # checking simulation
		zt <- z*t # each row of z is 0,1 hence zt holds {0,t}
		nsets <- colSums(z) # subsets per component sizes
	
	### M-step for scalar parameters		
		newlambda <- nsets/n # or colMeans(post) if EM strategy preferred!
		lambda_seq[iter+1, ] <- newlambda
		
	# update of the scaling (xi) parameter needs building of KM estimates
	# from each subsample, and integration of both
	# ToDo: use parallel on 2 cores to perform each subsample task!
		tsub1 <- t[z[,1]==1]; tsub2 <- t[z[,2]==1] # t subsamples
		dsub1 <- d[z[,1]==1]; dsub2 <- d[z[,2]==1] # d subsamples
		o1 <- order(tsub1); od1 <- cbind(tsub1,dsub1)[o1,]
		o2 <- order(tsub2); od2 <- cbind(tsub2,dsub2)[o2,]
		km1 <- KMod(od1); km2 <- KMod(od2)
		newscaling <- KMintegrate(km1)/KMintegrate(km2)
		scaling_seq[iter+1] <- newscaling # stored for final plotting
		if (averaged) {
			scaling <- mean(scaling_seq[1:(iter+1)])
			lambda <- colMeans(lambda_seq[1:(iter+1),])
			} else {	# averaged=FALSE case, just use last update
				scaling <- newscaling
				lambda <- newlambda
				} 		
		
	    ### M-step for nonparametric alpha() and s():
		# unscaling the sample and order it,
		# keeping the associated d censoring indicator
		zt[,2] <- scaling*zt[,2] # unscale t's from component 2
		newt <- rowSums(zt) # "unscaled" sample (sum 0 and t_i or scale*t_i )
		newo <- order(newt)
		new.od <- cbind(newt,d)[newo,] # with associated (unchanged) d
		s <- KMod(new.od) # stepfun object, can be evaluated by s(t)
	  	a1 <- HRkde(new.od, t, kernelft, bw=bw) # evaluated at the original t's
	  	a2  <- HRkde(new.od, scaling*t, kernelft, bw=bw) # and at scaling*t's
		
		### batch storage: collecting unscaled samples 
		if (iter >= (maxit - batchsize)) {
			batch_t <- c(batch_t, newt)
    		batch_d	<- c(batch_d, d)
			cat("-- adding unscaled sample at iteration",iter,"\n")
			}
		
		# change <- c(lambda_seq[iter,] - lambda_seq[iter-1,], 
        #	scaling_seq[iter]- scaling_seq[iter-1])
      	# dll <- max(abs(change))
      	if (verb)  {
      		cat("iteration", iter, ": sizes=", nsets," ")
      		cat("lambda_1=",lambda[1], "scaling=",scaling, "\n")
      		}
      			
      	# printing % done
		b=round(iter/qt); r=iter-b*qt
		if (r==0) {pc <- pc+10; cat(pc,"% done\n")}

      	iter <- iter + 1
      }  ###### end of SEM loops over iterations ######
    
	colnames(post) <- colnames(posthat) <- c(paste("comp", ".", 1:k, sep = ""))
	lambdahat <- colMeans(lambda_seq[1:iter,])
	scalinghat <- mean(scaling_seq[1:iter])
	posthat <- sumpost/(iter-1) # average posteriors
	hazard <- HRkde(new.od, kernelft=kernelft, bw=bw) # at final unscaled sample
	# Would it be better to return the final unscaled sample??
	
	cat(iter, "iterations: lambda=",lambdahat,", scaling=",scalinghat,"\n")
	if (sumNaNs > 0) cat("warning: ", sumNaNs,"NaN's occured in posterior computations")
	
	###### FINISHING ######
	# ToDo: average over batch set of last unscaled samples
	# compute average estimates via a S+M-step using posthat etc
	loglik <- sum(log(rs)) 	# "sort of" loglik with nonparam densities estimated
							# mimmics the parametric case, just like npEM does 
	z <- t(apply(posthat, 1, function(prob) rmultinom(1, 1, prob)))
	zt <- z*t
	zt[,2] <- scalinghat*zt[,2]
	avgt <- rowSums(zt) # unscaled sample
	avgo <- order(avgt)
	avg.od <- cbind(avgt,d)[avgo,] # with associated d
	shat <- KMod(avg.od) # stepfun object
	ahat <- HRkde(avg.od, kernelft=kernelft, bw=bw) # eval at the unscaled ordered avgt

    a=list(t=t, d=d, lambda=lambdahat, scaling=scalinghat, 
          posterior=post,     # final posterior probabilities
          all.lambda=lambda_seq[1:iter,],  # sequence of lambda's
          all.scaling=scaling_seq[1:iter], # sequence of scale
          loglik=loglik,	  # analog to parametric loglik, like npEM
          meanpost=posthat,   # posterior proba averaged over iterations
          survival=s,         # Kaplan-Meier last iter estimate (stepfun object)
          hazard=hazard,      # hazard rate final estimate evaluated at final.t 
          final.t=new.od[,1], # last unscaled sample
          s.hat=shat,         # Kaplan-Meier average estimate
          t.hat= avg.od[,1],  # ordered unscaled sample based on meanpost USEFUL??
          avg.od=avg.od,	  # t.hat with associated d (for computing ahat outside)
		  hazard.hat=ahat,    # hazard rate average estimate on t.hat
		  batch.t=batch_t,    # batch sample t (not ordered)
		  batch.d=batch_d,    # associated event indicators just rep(d,batchsize)
		  sumNaNs = sumNaNs,  # total nb of NaN's in iterations while computing post
          ft="spRMM_SEM")
	class(a) = "spRMM"
  return(a)
}




##############################################
## plot function for spRMM object : remove lognormal true pdf!
# sem = spRMM_SEM object
# other are true parameters, if available, for plotting references
# ToDo: plot true f and S only if true param availables
# 		pass the true pdf (like dlnorm) as well
plotspRMM <- function(sem, tmax = NULL){
	t <- sem$t   
	ym <- max(sem$all.scaling)
	par(mfrow=c(2,2))
	plot(sem$all.scaling, type="l", ylim=c(0, ym),
		xlab="iterations", main="scaling", ylab="")
	plot(sem$all.lambda[,1], ylim=c(0,1), type="l", xlab="iterations",
		main="weight of component 1",ylab="")
	# plots of Kaplan-Meier estimates
	# finding max time for plots
	if (is.null(tmax)){tmax <- max(sem$scaling*t) + 2}
	u <- seq(0, tmax, len=200)
	plot(sem$survival, xlim=c(0,tmax), ylim=c(0,1), pch=20, 
		verticals=F, do.points=F, xlab="time", main="Survival function estimate")
	# plot(sem$s.hat, pch=20, verticals=F, do.points=F, col=2, add = TRUE)
	# pdf estimates
	fhat <- sem$s.hat(sem$t.hat)*sem$hazard.hat
	ffinal <- sem$survival(sem$final.t)*sem$hazard
  plot(sem$final.t, ffinal, type="l", col=1, xlim=c(0,tmax), 
				xlab="time", ylab="density", main="Density estimate")
}



## S3 method of summary for class "spRMM"
summary.spRMM <- function(object, digits = 6, ...) {
  sem <- object
  if (sem$ft != "spRMM_SEM") stop("Unknown object of type ", sem$ft)
  cat("summary of", sem$ft, "object:\n")
  o <- matrix(NA, nrow = 2, ncol = 2)
  o[1,] <- sem$lambda
  o[2,2] <- sem$scaling
  colnames(o) <- paste("comp",1:2)
  rownames(o) <- c("lambda", "scaling")
  print(o, digits = digits, ...)
  cat("(pseudo) loglik at estimate: ", sem$loglik, "\n")
  pcc <- round(100*(1-mean(sem$d)), 2)
  cat(pcc, "% of the data right censored\n")
}

plotly_mixMCMC <- function(
  x , trace.plot = TRUE , summary.plot = FALSE , burnin = 2000, credit.region = 0.95, col.cr = NULL,
  cex.trace = 3 , width.trace = 3 , 
  cex.summary = 3 , width.summary = 1,
  title.trace = "", title.trace.x = 0.5 , title.trace.y = 0.95, title.trace.size = 15,
  xlab.trace = "Index" , xlab.trace.size = 15, xtick.trace.size = 15,
  ylab.trace = NULL , ylab.trace.size = 15, ytick.trace.size = 15,
  title.summary = "Credible Regions", title.summary.x = 0.5 , title.summary.y = 0.95, title.summary.size = 15,
  xlab.summary = "Predictor" , xlab.summary.size = 15, xtick.summary.size = 15,
  ylab.summary = "Response" , ylab.summary.size = 15, ytick.summary.size = 15
){
  mix.object <- x
  
  if (!inherits(mix.object, "mixMCMC")){
    stop("Use only with \"mixMCMC\" objects!")
  }
  if (trace.plot){
    k<-mix.object$components
    theta<-mix.object$theta
    p.k=ncol(theta)
    p=p.k/k
    name.theta<-colnames(theta)
    
    for (i in 1:dim(theta)[2]){
      if (is.null(ylab.trace)){
        ylab.trace <- name.theta[i]
      }
      plot.trace <- plot_ly()%>%
        add_trace(x=seq(from = 1 , to = dim(theta)[1] , by=1) , 
                  y=theta[,i] , type = 'scatter' , mode = 'lines+markers',
                  marker = list(size = cex.trace),
                  line = list(width = width.trace),
                  showlegend = FALSE) %>%
        plotly::layout(
          title = list(text = title.trace,
                       x = title.trace.x,
                       y = title.trace.y,
                       font = list(size=title.trace.size)),
          xaxis = list(title = list(text = xlab.trace,
                                    font = list(size = xlab.trace.size)),
                       tickfont = list(size = xtick.trace.size)),
          yaxis = list(title = list(text = ylab.trace,
                                    font = list(size = ylab.trace.size)),
                       tickfont = list(size = ytick.trace.size))
          )
      print(plot.trace)
    }
  }
  if(is.matrix(mix.object$x) == TRUE && is.null(mix.object$y) == FALSE && summary.plot == TRUE){
    y<-mix.object$y
    n<-length(y)
    x<-mix.object$x
    p<-ncol(x)
    k<-mix.object$components
    theta<-mix.object$theta
    if(p!=2 || sum(x[,1])!=n){                                                                  
      stop(paste("This only works for simple linear regression!","\n"))
    }
    plot.summary <- plot_ly()%>%
      add_trace(x=x[,2] , 
                y=y , type = 'scatter' , mode = 'markers',
                marker = list(size = cex.summary),
                showlegend = FALSE) %>%
      plotly::layout(
        title = list(text = paste(credit.region*100, "% " , title.summary ,sep = ""),
                     x = title.summary.x,
                     y = title.summary.y,
                     font = list(size=title.summary.size)),
        xaxis = list(title = list(text = xlab.summary,
                                  font = list(size = xlab.summary.size)),
                     tickfont = list(size = xtick.summary.size)
        ),
        yaxis = list(title = list(text = ylab.summary,
                                  font = list(size = ylab.summary.size)),
                     tickfont = list(size = ytick.summary.size)
        )
      )
    if(is.null(col.cr)){
      col.cr <- hue_pal()(k)
    }
    if (length(col.cr) != k){
      print(paste("Please specify",k,"colors in 'col.cr'."))
    }
    for (i in 1:k){
      beta.summary <- cbind(theta[-c(1:burnin),2*i-1],theta[-c(1:burnin),2*i])
      xbar = apply(beta.summary, 2, mean)
      n = nrow(beta.summary)
      cbeta = t(t(beta.summary) - xbar)
      S = t(cbeta) %*% cbeta/(n - 1)
      eS = eigen(S)
      B = eS$vec %*% diag(sqrt(eS$val))
      theta.summary = seq(0, 2 * pi, len = 250)
      alpha = 1-credit.region
      v = cbind(cos(theta.summary), sin(theta.summary)) * sqrt(qchisq(1 - alpha,2))
      h = t(B %*% t(v) + xbar)
      nh = nrow(h)
      m = which.max(h[, 2])
      h = rbind(h[m:nh, ], h[((1:nh) < m), ], h[m, ])
      bound = h
      x.summary <- x[,2]
      z <- length(x.summary) * 5
      u <- seq(min(x.summary), max(x.summary), length = z)
      lower <- c()
      upper <- c()
      v <- c()
      for (q in 1:z) {
        for (l in 1:nrow(beta.summary)) {
          v[l] <- as.matrix(beta.summary[, 1][l] + beta.summary[, 2][l] * u[q])
        }
        uv <- cbind(u[q], v)
        lower <- rbind(lower, uv[order(v), ][1, ])
        upper <- rbind(upper, uv[order(v), ][nrow(beta.summary), ])
      }
      plot.summary <- plot.summary%>%
        add_trace(x=lower[,1] , 
                  y=lower[,2] , type = 'scatter' , mode = 'lines',
                  line = list(width = width.summary , color = col.cr[i]),
                  name = paste("Lower Bound for Component",i), showlegend = FALSE)%>%
        add_trace(x=upper[,1] , 
                  y=upper[,2] , type = 'scatter' , mode = 'lines',
                  line = list(width = width.summary , color = col.cr[i]),
                  name = paste("Upper Bound for Component",i), showlegend = FALSE)
    }
    print(plot.summary)
  }
}

# Method for "density" generic:  Takes an npEM object and returns
# a corresponding KDE for the appropriate component and block, evaluated
# at the given set of points.  
# Does not use the FFT like the density.default function does; still
# quite fast, but not optimized for speed
density.npEM <- function (x, u = NULL, component = 1, block = 1, scale = FALSE, 
    ...) 
{
  if (is.null(blockid <- x$blockid)) {
    coords <- 1
  } 
  else {
    coords <- blockid == block
    if (!any(coords)) 
      stop("Illegal value of block argument")
  }
  m <- NCOL(x$post)
  xx <- as.vector(as.matrix(x$data)[, coords])
  if(is.matrix(x$bandwidth))
    bw <- x$bandwidth[block, component]
  else bw <- x$bandwidth
  if (is.null(u)) {
    u = seq(min(xx) - 4 * bw, max(xx) + 4 * bw, len = 250)
  }
  if (component > m || component < 1) 
    stop("Illegal value of component argument")
  if (!is.null(x$symmetric) && x$symmetric) {
    n <- length(xx)
    d <- wkde(x = rep(xx, m) - rep(x$muhat, each = n), u = u - 
              x$muhat[component], w = as.vector(x$post), bw = bw, 
              sym = TRUE)
  }
  else {
    n <- NROW(x$data)
    wts <- rep(x$post[, component], sum(coords))
    d <- wkde(xx, u = u, w = wts, bw = bw)
  }
  if (scale) 
    d <- d * x$lambdahat[component]
  structure(list(x = u, y = d, bw = bw, n = n, call = match.call(), 
                 data.name = deparse(substitute(x)), has.na = FALSE), 
             class = "density")
}

summary.npEM <- function(object, ...) {
  normpost <- sweep(object$post, 2, sums <- colSums(object$post), "/")
  n <- NROW(object$data)
  r <- NCOL(object$data)  
  m <- length(object$lambdahat)
  B <- 1
  if (r>1) B <- max(object$blockid)
  lambda <- sums/n
  means <- variances <- NULL
  for(i in 1:B) {    
    if (r>1) {
    	coords <- object$blockid == i
    	xx <- as.vector(object$data[,coords])
    	}
    	else {
    		coords <- 1
    		xx <- as.vector(object$data)
    		}
    sc <- sum(coords)
    M <- V <- NULL
    for (j in 1:m) {
      wts <- rep(normpost[,j]/sc, sc)
      M <- c(M, tmp <- sum(xx*wts))
      V <- c(V, sum((xx-tmp)^2 *wts))
#      cat(M," ")
    }
    means <- rbind(means, M)
    variances <- rbind(variances, V)
  }
  rownames(means) <- rownames(variances) <- paste("block", 1:B)
  colnames(means) <- colnames(variances) <- paste("component", 1:m)
  ans <- list(n=n, m=m, r=r, B=B, blockid=object$blockid,
              means = means, variances=variances)
  class(ans) <- "summary.npEM"
  ans
}

print.summary.npEM <- function(x, digits=3, ...) {	
	if (x$r>1) cat (paste(x$n,"observations,",x$r,"coordinates,",
  		x$m,"components, and",x$B,"blocks.\n\n"))
  	else cat(paste(x$n,"univariate observations, and",
  		x$m,"components.\n\n"))
   		
  cat ("Means (and std. deviations) for each component:\n")
  for(i in 1:x$B) {
  	coords <- 1
  	if (x$r>1) {
  		coords <- x$blockid == i
  		cat(paste("  Block #",i,":  Coordinate", sep=""))
  		cat(ifelse(sum(coords)>1, "s ", " "))
  		   cat(which(coords))
    	cat("\n    ")
  		}   
    cat(paste(signif(x$means[i,],digits), 
              " (", signif(sqrt(x$variances[i,]),digits), ")  ", sep=""))
    cat("\n")
  }
}

# fast function for two-component univariate normal mixture

normalmixEM2comp <- function(x, lambda, mu, sigsqrd, eps=1e-8, maxit=1000, verb=FALSE) {
  arbvar <- (length(sigsqrd)==2)
  mu1 <- mu[1]; mu2 <- mu[2]
  sigsqrd1 <- sigsqrd[1]; sigsqrd2 <- sigsqrd[arbvar+1]
  mx <- mean(x)
  const <- length(x) * 0.918938533204673 # i.e., times log(2*pi)/2
  dl <- 1 + eps
  iter<-0
  ll <- rep(0, maxit+1)
  a1<-(x-mu1)^2; b1<-(lambda/sqrt(sigsqrd1))*exp(-a1/2/sigsqrd1)
  a2<-(x-mu2)^2; b2<-((1-lambda)/sqrt(sigsqrd2))*exp(-a2/2/sigsqrd2)
  l <- sum(log(b1+b2))

  while (dl>eps && iter<maxit) {
    iter<-iter+1
    ll[iter] <- l
    postprobs <- b1/(b1+b2)
    lambda<-mean(postprobs)
    mu1<-mean(postprobs*x)/lambda
    mu2<-(mx-lambda*mu1)/(1-lambda)
    if (arbvar)  {   
      sigsqrd1<-mean(postprobs*a1)/lambda
      sigsqrd2<-mean((1-postprobs)*a2)/(1-lambda)
    } else {
      sigsqrd1 <- sigsqrd2 <- mean(postprobs*a1 + (1-postprobs)*a2) 
    }
    a1<-(x-mu1)^2; b1<-(lambda/sqrt(sigsqrd1))*exp(-a1/2/sigsqrd1)
    a2<-(x-mu2)^2; b2<-((1-lambda)/sqrt(sigsqrd2))*exp(-a2/2/sigsqrd2)

    oldl<-l    
    l <- sum(log(b1+b2))
    dl<-l-oldl
    if (verb) {
      cat("iteration =", iter, " log-lik diff =", dl, " log-lik =", 
          l-const, "\n")
    }
  }
  cat("number of iterations=", iter, "\n")
  iter <- iter+1
  ll[iter] <- l
  postprobs <- cbind(postprobs, 1-postprobs)
  colnames(postprobs) <- c(paste("comp", ".", 1:2, sep = ""))
  out <- list(x=x, lambda = c(lambda,1-lambda), mu = c(mu1, mu2), 
       sigma = sqrt(c(sigsqrd1, sigsqrd2)[1:(1+arbvar)]), 
       loglik = l - const, posterior = postprobs, 
       all.loglik=ll[1:iter] - const, 
       restarts=0, ft="normalmixEM")
  class(out) <- "mixEM"
  out
}

plot.mixMCMC <- function(x, trace.plots = TRUE, summary.plots = FALSE, burnin = 2000, ...){

mix.object <- x

if (!inherits(mix.object, "mixMCMC")) 
    stop("Use only with \"mixMCMC\" objects!")

if(trace.plots==TRUE){
k<-mix.object$components
theta<-mix.object$theta
p.k=ncol(theta)
p=p.k/k
name.theta<-colnames(theta)
par(mfrow=c(p,k))
for(i in 1:p){
    for(j in 1:k){
    plot(theta[,(i-1)*k+j],type="l",ylab=name.theta[(i-1)*k+j])
    }
}
}

#regmixMH
if(is.matrix(mix.object$x) == TRUE && is.null(mix.object$y) == FALSE && summary.plots == TRUE){
y<-mix.object$y
n<-length(y)
x<-mix.object$x
p<-ncol(x)
k<-mix.object$components
theta<-mix.object$theta
if(p!=2 || sum(x[,1])!=n){                                                                  
stop(paste("This only works for simple linear regression!","\n"))
}
par(mfrow=c(1,1))
plot(x[,2],y,main="Credible Regions",xlab="Predictor",ylab="Response")
#plot(theta[-c(1:burnin),seq(1,2*k-1,by=2)],theta[-c(1:burnin),seq(2,2*k,by=2)],col=0)
for(i in 1:k){
#points(theta[-c(1:burnin),2*i-1],theta[-c(1:burnin),2*i],col=(i+1))
regcr(beta=cbind(theta[-c(1:burnin),2*i-1],theta[-c(1:burnin),2*i]),col=(i+1), x=x[,2], plot=TRUE,...)
}

}

}

plotly_ellipse <- function(mu, sigma, alpha=.05, npoints=250,
                           draw=TRUE,
                           cex = 3,
                           col = "#1f77b4",
                           lwd = 3,
                           title = "",
                           title.x = 0.5,
                           title.y = 0.95,
                           title.size = 15,
                           xlab = "X",
                           xlab.size = 15,
                           xtick.size = 15,
                           ylab = "Y",
                           ylab.size = 15,
                           ytick.size = 15
) {
  es <- eigen(sigma)
  e1 <- es$vec%*%diag(sqrt(es$val))
  r1 <- sqrt(qchisq(1-alpha,2))
  theta <- seq(0,2*pi,len=npoints)
  v1 <- cbind(r1*cos(theta),r1*sin(theta))
  pts=t(mu-(e1%*%t(v1)))
  
  updatemenus <- list(
    list(
      x = 1.2,
      y = 1,
      visible = TRUE,
      font = list(size=12),
      active = 0,
      type= 'dropdown',
      buttons = list(
        list(
          label = "Show Markers",
          method = "update",
          args = list(list(visible = c(TRUE, FALSE , FALSE)))),
        list(
          label = "Show Lines",
          method = "update",
          args = list(list(visible = c(FALSE , TRUE , FALSE)))),
        list(
          label = "Show Lines+Markers",
          method = "update",
          args = list(list(visible = c(FALSE , FALSE , TRUE)))))
    )
  )
  
  plot <- plot_ly() %>%
    add_markers(x=pts[,1] , 
                y=pts[,2] ,
                type = 'scatter' , mode = 'markers' ,
                marker = list(color = col , size = cex) , 
                name = 'Data' , showlegend = FALSE) %>%
    add_trace(x=pts[,1] , 
              y=pts[,2] , type = 'scatter' , mode = 'lines',
              line = list(color = col , width = lwd),
              name = "Data" , showlegend = FALSE) %>%
    add_trace(x=pts[,1] , 
              y=pts[,2] , type = 'scatter' , mode = 'lines+markers',
              marker = list(color = col , size = cex),
              line = list(color = col , width = lwd),
              name = "Data" , showlegend = FALSE) %>%
    plotly::layout(
      updatemenus = updatemenus,
      title = list(text = title,
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)
      ),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  print(plot)
  
  invisible(pts)
}

# Alternative version of dmvnorm to eliminate dependence of mixtools
# on additional package 'mvtnorm'
# Written (hopefully) to be more efficient than mvtnorm version, which uses both
# a call to "eigen" and a call to "mahalanobis", by using only a single
# call to the more efficient "qr" (read "Note" under ?qr)

# Note:  These functions assume that each ROW of y is a separate position vector.
# i.e., y is assumed to be nxd, where d=dimension
dmvnorm <- function(y, mu=NULL, sigma=NULL) {
  exp(logdmvnorm(y, mu=mu, sigma=sigma))
}


logdmvnorm <- function(y, mu=NULL, sigma=NULL) {
  if (is.vector(y)) 
    y <- matrix(y, nrow=1)
  d <- ncol(y)
  if (!is.null(mu))
    y <- sweep(y, 2, mu, '-')
  if (is.null(sigma))
    sigma <- diag(d)
  k <- d * 1.8378770664093454836 # that constant is log(2*pi)
  a <- qr(sigma)
  logdet <- sum(log(abs(diag(a$qr))))
  if(nrow(y)==1) 
    mahaldist <- as.vector(y %*% qr.solve(a,t(y)))
  else
    mahaldist <- rowSums((y %*% qr.solve(a)) * y)
  -0.5*(mahaldist + logdet + k)
}

regmixEM = function (y, x, lambda = NULL, beta = NULL, sigma = NULL, k = 2, 
    addintercept = TRUE, arbmean = TRUE, arbvar = TRUE, epsilon = 1e-08, maxit = 10000, 
    verb = FALSE) 
{
    if(arbmean == FALSE && arbvar == FALSE){
    stop(paste("Must change constraints on beta and/or sigma!","\n"))
    }
    s = sigma
    if (addintercept) {
        x = cbind(1, x)
    }
    n <- length(y)
    p <- ncol(x)
    tmp <- regmix.init(y = y, x = x, lambda = lambda, beta = beta, 
        s = s, k = k, addintercept = addintercept, arbmean = arbmean, arbvar = arbvar)
    lambda <- tmp$lambda
    beta <- tmp$beta
    s <- tmp$s
    k <- tmp$k
    diff <- 1
    iter <- 0
    xbeta <- x %*% beta
    res <- (y - xbeta)^2
    if(arbmean == FALSE){
    res <- sapply(1:k,function(i) res)
    }
#    comp <- lapply(1:k, function(i) lambda[i] * dnorm(y, xbeta[,i * arbmean + (1 - arbmean)], 
#        s[i * arbvar + (1 - arbvar)]))
#    comp <- sapply(comp, cbind)
#    compsum <- apply(comp, 1, sum)
#    obsloglik <- sum(log(compsum))
    comp <- t((lambda/sqrt(2 * pi * s^2)) * t(exp(-t(t(res)/(2 * 
        s^2)))))
    obsloglik <- sum(log(apply(comp, 1, sum)))
    ll <- obsloglik
    z = matrix(nrow = n, ncol = k)
	restarts <- 0
    while (diff > epsilon && iter < maxit) {
        for (i in 1:n) {
            for (j in 1:k) {
                z.denom = c()
                for (h in 1:k) {
                  z.denom = c(z.denom, (lambda[h]/lambda[j]) * 
                    (s[j * arbvar + (1 - arbvar)]/s[h * arbvar + 
                      (1 - arbvar)]) * exp(-0.5 * ((1/s[h * arbvar + 
                    (1 - arbvar)]^2) * res[i, h] - (1/s[j * arbvar + 
                    (1 - arbvar)]^2) * res[i, j])))
                }
                z[i, j] = 1/sum(z.denom)
            }
        }
#	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)
	z = z/apply(z,1,sum)
        lambda.new <- apply(z, 2, mean)
        if (sum(lambda.new < 1e-08)>0 || is.na(sum(lambda.new))) {
            sing <- 1
        }
        else {
 
        if (arbmean == FALSE) {
            if (addintercept) {
			beta.new <- lm(y~x[,-1],weights=apply(t(t(z)/(s^2)),1,sum))$coef
            }
            else beta.new <- lm(y~x-1,weights=apply(t(t(z)/(s^2)),1,sum))$coef
#            beta.new <- sapply(lm.out, coef)
#        beta.new1 <- apply(t(apply(z,2,sum)*t(beta.new)),1,sum)/n
#	beta.new2 <- lm(y~x[,-1],weights=apply(t(t(z)/(s^2)),1,sum))$coef
#	beta.new<-as.vector(solve(t(x) %*% sweep(x, 1, t(t(z)/(s^2)), "*")) %*% apply(t(t(z)/(s^2))*y*x,2,sum) )
        } else {
           if (addintercept) {
                lm.out <- lapply(1:k, function(i) lm(y ~ x[, 
                  -1], weights = z[, i]))
            }
            else lm.out <- lapply(1:k, function(i) lm(y ~ x - 
                1, weights = z[, i]))
            beta.new <- sapply(lm.out, coef)
		}
        xbeta.new <- x %*% beta.new
        res <- (y - xbeta.new)^2
            if(arbmean == FALSE){
            res <- sapply(1:k,function(i) res)
            }
            if (arbvar) {
                s.new <- sqrt(sapply(1:k, function(i) sum(z[, 
                  i] * (res[, i]))/sum(z[, i])))
            }
            else s.new <- sqrt(sum(z * res)/n)
            lambda <- lambda.new
            beta <- beta.new
          xbeta <- x%*%beta
            s <- s.new
            sing <- sum(s < 1e-08)
    comp <- lapply(1:k, function(i) lambda[i] * dnorm(y, xbeta[,i * arbmean + (1 - arbmean)], 
        s[i * arbvar + (1 - arbvar)]))
    comp <- sapply(comp, cbind)
    compsum <- apply(comp, 1, sum)
    newobsloglik <- sum(log(compsum))
#            comp <- t((lambda/sqrt(2 * pi * s^2)) * t(exp(-t(t(res)/(2 * 
#                s^2)))))
#            newobsloglik <- sum(log(apply(comp, 1, sum)))
        }
        if (sing > 0 || is.na(newobsloglik) || newobsloglik < obsloglik || abs(newobsloglik) == 
            Inf){# || sum(z) != n) {
            cat("Need new starting values due to singularity...", 
                "\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
            tmp <- regmix.init(y = y, x = x, k = k, addintercept = addintercept, 
                arbmean = arbmean, arbvar = arbvar)
            lambda <- tmp$lambda
            beta <- tmp$beta
            s <- tmp$s
            k <- tmp$k
            diff <- 1
            iter <- 0
            xbeta <- x %*% beta
            res <- (y - xbeta)^2
    if(arbmean == FALSE){
    res <- sapply(1:k,function(i) res)
    }
#    comp <- lapply(1:k, function(i) lambda[i] * dnorm(y, xbeta[,i * arbmean + (1 - arbmean)], 
#        s[i * arbvar + (1 - arbvar)]))
#    comp <- sapply(comp, cbind)
#    compsum <- apply(comp, 1, sum)
#    obsloglik <- sum(log(compsum))
            comp <- t((lambda/sqrt(2 * pi * s^2)) * t(exp(-t(t(res)/(2 * 
                s^2)))))
            obsloglik <- sum(log(apply(comp, 1, sum)))
    ll <- obsloglik

        }
        else {
            diff <- newobsloglik - obsloglik
            obsloglik <- newobsloglik
          ll <- c(ll, obsloglik)
            iter <- iter + 1
            if (verb) {
                cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                  obsloglik, "\n")
            }
        }
    }
    scale.order = order(s)
    sigma.min = min(s)
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
    cat("number of iterations=", iter, "\n")
    if(arbmean == FALSE){
    z=z[,scale.order]
names(beta) <- c(paste("beta", ".", 0:(p-1), sep = ""))
colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=x, y=y, lambda = lambda[scale.order], beta = beta, sigma = sigma.min, scale = s[scale.order]/sigma.min, loglik = obsloglik, 
        posterior = z[,scale.order], all.loglik=ll, restarts = restarts, ft="regmixEM")
    class(a) = "mixEM"
    a
    } else {
rownames(beta) <- c(paste("beta", ".", 0:(p-1), sep = ""))
colnames(beta) <- c(paste("comp", ".", 1:k, sep = ""))
colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=x, y=y, lambda = lambda, beta = beta, sigma = s, loglik = obsloglik, 
        posterior = z, all.loglik=ll, restarts = restarts, ft="regmixEM")
    class(a) = "mixEM"
    a
    }
}

wquantile <- function(wt=rep(1,length(x)), x, probs, already.sorted=FALSE, already.normalized=FALSE) {
  if(!already.sorted) {
    wt <- wt[o<-order(x)]
    x <- x[o]
  }
  if(!already.normalized) {
    wt <- wt/sum(wt)
  }
  x[findInterval(probs,cumsum(wt))]
}

wIQR <- function(wt=rep(1,length(x)), x, already.sorted=FALSE, already.normalized=FALSE) {
  diff(wquantile(wt, x, c(.25,.75), already.sorted, already.normalized))
}

mvnormalmix.init = function (x, lambda = NULL, mu = NULL, sigma = NULL, k = 2, arbmean = TRUE, arbvar = TRUE) 
{
    n <- nrow(x)
    p <- ncol(x)
    y <- apply(x, 1, mean)
    x <- x[order(y), ]
    x.bin <- list()
    for (j in 1:k) {
        x.bin[[j]] <- x[max(1, floor((j - 1) * n/k)):ceiling(j * 
            n/k), ]
    }
    if (is.null(sigma)) {
        if (arbvar) {
            sigma.hyp = lapply(1:k, function(i) (apply(x.bin[[i]], 
                2, var))^-1)
            sigma = lapply(1:k, function(i) diag(1/rexp(p, rate = sigma.hyp[[i]])))
        }
        else {
            sigma.hyp = apply(sapply(1:k, function(i) (apply(x.bin[[i]], 
                2, var))^-1), 2, mean)
            sigma = diag(1/rexp(p, rate = sigma.hyp))
        }
    }
    if (is.null(sigma) == FALSE && arbvar == TRUE) {
        k = length(sigma)
    }
    if (is.null(mu)) {
        mu.hyp <- lapply(1:k, function(i) apply(x.bin[[i]], 2, 
            mean))
        if (arbvar) {
            mu <- lapply(1:k, function(i) as.vector(rmvnorm(1, 
                mu = as.vector(mu.hyp[[i]]), sigma = as.matrix(sigma[[i]]))))
        }
        else mu <- lapply(1:k, function(i) as.vector(rmvnorm(1, 
            mu = as.vector(mu.hyp[[i]]), sigma = as.matrix(sigma))))
	  if (arbmean==FALSE) {
		mu <- apply(sapply(mu,as.vector),1,mean)
#		mu <- lapply(1:k, function(i) mu)
    }
	}
    if (is.null(mu) == FALSE && arbmean == TRUE){
	k=length(mu)
	}
    if (is.null(lambda)) {
        lambda <- runif(k)
        lambda <- lambda/sum(lambda)
    }
    else k <- length(lambda)
    list(lambda = lambda, mu = mu, sigma = sigma, k = k)
}

# plots SEQuence from npEM object
# function for plotting the scalar parameters sequences along iterations
# if object x comes from npEM, just the x$lambda sequence
# if object x comes from spEM, both x$lambda and x$mu sequences
plotseq <- function(x, ...) UseMethod("plotseq") 



plotseq.npEM <- function(x, ...) {
#  ask <- par(ask=TRUE)
  r <- NCOL(x$data)
  n <- NROW(x$data)
  m <- length(x$lambdahat)
  iter <- NROW(x$lambda)
  xlabel <- paste("iterations")
  nbcol <- 1
  if (!is.null(x$symmetric) && x$symmetric)
		nbcol <- 2
	par(mfcol=c(m, nbcol))

# in all cases, plots the lambda's  
  for (j in 1:m) {
  	estim <- paste(round(x$lambdahat[j],3))
  	tt <- substitute(expression(paste("sequence of ",lambda[j],
  			", estimate ",widehat(lambda[j]),"=", estim, sep="")))
  	ylabel <- substitute(expression(paste(lambda[j],sep="")))
  	plot(x$lambda[,j], type="l", main=eval(tt), xlab=xlabel, 
  		ylab=eval(ylabel), ...)
  	lines(c(0,iter),rep(x$lambdahat[j],2),col=2,lty=2)
  	}

## for symmetric location spEM case plots mu
    if (!is.null(x$symmetric) && x$symmetric) {
  	for (j in 1:m) {
  		estim <- paste(round(x$muhat[j],3))
  	  	tt <- substitute(expression(paste("sequence of ",mu[j],
  	  		", estimate ",widehat(mu[j]),"=",estim,sep="")))
  		ylabel <- substitute(expression(paste(mu[j],sep="")))

  	plot(x$mu[,j], type="l", main=eval(tt),  ylab=eval(ylabel),
  			 xlab=xlabel, ...)
  	lines(c(0,iter),rep(x$muhat[j],2),col=2,lty=2)
   		}
#   	legend("topright", legend=round(x$muhat,3), fill=2:(m+1)) 
    } 
#      structure (list (call=match.call()))
}

matsqrt <- function(x) {
xe <- eigen(x)
xe.v <- xe$values
if(all(xe.v >= 0)) {
xe.v1 <- diag(sqrt(xe.v))
}
xvalue <- cbind(xe$vectors)
xvalue.1 <- solve(xvalue)
out <- xvalue %*% xe.v1 %*% xvalue.1
out
}

###################################################################
## "EC-MM" algorithm 
## to search for a local maximum of the likelihood surface for a 
## univariate finite mixture of normals with possible 
## linear constraints on the mean and variance parameters.
##
## (EC-MM is ECM in the sense of Meng and Rubin, 
## Biometrika 1993, where the M step is replaced by a 
## Conditional MM step)
##
## version allowing for linear constraints on the mean
## mu = M beta + C, where M is matrix(k,p) and C is vector()
## C-MM step required for the linear constraint on the Variances
## var.lincstr = matrix A (k,q) s.t.  iv = A g, where "i"nverse "v"ar
## iv = k-vector of 1/v_j's, and g = q-vector of unknown parameters
## no fast option, & ECM-MM algorithm forced (no ECM option available)
## init values for gparam are *required* here for the MM algorithm
## # default value for A could be diag(k) so that iv=g

normalmixMMlc <- function (x, 
			lambda = NULL, mu = NULL, sigma = NULL, k = 2,
			mean.constr = NULL,
			mean.lincstr = NULL, mean.constant = NULL,
			var.lincstr = NULL, gparam = NULL,
			epsilon = 1e-08, maxit = 1000, maxrestarts=20, 
			verb = FALSE) {
  ECM <- TRUE  # always required in this case
  A <- var.lincstr
  x <- as.vector(x);     n <- length(x)
  tmp <- normalmix.init(x = x, lambda = lambda, mu = mu, s = sigma, 
                        k = k) # no arbmean & arbvar parameters
  lambda <- tmp$lambda; mu <- tmp$mu; sigma <- tmp$s; k <- tmp$k
  # arbvar <- tmp$arbvar; arbmean <- tmp$arbmean
	arbmean <- arbvar <- TRUE # forced for parse.constraints()
	warn <- options(warn=-1) # Turn off warnings (for parsing only)
    z <- parse.constraints(mean.constr, k=k, allsame=!arbmean)    
    options(warn) # Reset warnings to original value
    meancat <- z$category; meanalpha <- z$alpha 
    
    if (!is.null(mean.lincstr)) {   # linear constraint on the means
		M <- mean.lincstr
		cat("linear constraint mu = M beta + C version\n")
		p <- dim(M)[2] # nb of columns = size of constr. mean parameter
		if (dim(M)[1] != k) 
			stop("mean.lincstr and mu dimensions must agree")
		if (is.null(mean.constant)) 
			C <- matrix(0,k,1) else C <- matrix(mean.constant,k,1)
   		}
    notdone <- TRUE
    while(notdone) {
      # Initialize everything
      notdone <- FALSE
      tmp <- normalmix.init(x=x, lambda=lambda, mu=mu, s=sigma, k=k)
      lambda <- tmp$lambda; mu <- tmp$mu; k <- tmp$k; sigma <- tmp$s

      q <- dim(A)[2] # nb of inverse variance parameters (gamma)
      if(is.null(gparam)) g <- rexp(q)
      else g <- gparam 
      iv <- A %*% g # inverse variances, as a one-column matrix
  	  v <- 1/iv
  	  # is it necessary to redefined sigma from g here ?
  	  sigma <- as.vector(sqrt(v))      
      diff <- epsilon+1
      iter <- 0
      postprobs <- matrix(nrow = n, ncol = k)
      restarts <- 0
      mu <- rep(mu, k)[1:k]  # is this useful? 
      sigma <- rep(sigma,k)[1:k] # sigma still needed for post computation   
      
      ## Initialization E-step here:
      z <- .C(C_normpost, as.integer(n), as.integer(k),
              as.double(x), as.double(mu), 
              as.double(sigma), as.double(lambda),
              res2 = double(n*k), double(3*k), post = double(n*k),
              loglik = double(1), PACKAGE = "mixtools")
      postprobs <- matrix(z$post, nrow=n)
      res <- matrix(z$res2, nrow=n) # n,m matrix of squared residuals (x_i-mu_j)^2
      ll <- obsloglik <- z$loglik
      
      ## EC-MM iterations
      while (diff > epsilon && iter < maxit) {      
        # ECM loop, 1st M-step: condition on sigma, update lambda and mu :
        lambda <- colMeans(postprobs) # update for lambda
        # update for mu, depending on constraint type
        mu[meancat==0] <- meanalpha[meancat==0]
        if (max(meancat)>0 && is.null(mean.lincstr)) { # simple constraint
          for(i in 1:max(meancat)) {
            w <- which(meancat==i)
            if (length(w)==1) {
              mu[w] <- sum(postprobs[,w]*x) / (n*lambda[w])
            } else {
              tmp <- t(postprobs[,w])*(meanalpha[w]/sigma[w]^2)
              mu[w] <- meanalpha[w] * sum(t(tmp)*x) / sum(tmp*meanalpha[w])
            	}
          	}
        	}
	
		if (!is.null(mean.lincstr)) {  # linear constraint mu = M beta + C
			iv2 <- as.vector(iv)
			# A1_j = sum_i p_ij x_i/v_j
			A1 <- apply(postprobs*matrix(x,n,k),2,sum)*iv2
			B <- diag(apply(postprobs,2,sum)*iv2)
			Stemp <- solve(t(M) %*% B %*% M)
			beta <- Stemp %*% t(M) %*% (A1 - B %*% C)
			mu <- as.vector(M %*% beta + C) # coerce to vector 
			}

          # ECM E-step number one:
          z <- .C(C_normpost, as.integer(n), as.integer(k),
                  as.double(x), as.double(mu), 
                  as.double(sigma), as.double(lambda),
                  res2 = double(n*k), double(3*k), post = double(n*k),
                  loglik = double(1), PACKAGE = "mixtools")
          postprobs <- matrix(z$post, nrow=n)
          res <- matrix(z$res2, nrow=n)

        #### ECM loop 2nd M-step: condition on mu, update lambda 
        #### and sigma via the MM step           
        lambda <- colMeans(postprobs)          
        # Update variances with the MM algo on g, conditional on mu
        # note: code in too much steps/details for debugging
		# computing q-vector of denominators
		r0 <- postprobs*res # (i,j)th is p_{ij}*(x_i - mu_j)^2
		r1 <- r0 %*% A
		# r1(n,q) matrix, (i,l) term =\sum_j p_{ij}A_{jl}(x_i - mu_j)^2
		den <- colSums(r1) # q-vector of denominators for updating g 
		# computing q-vector of numerators
		r3 <- matrix(v,nrow=k,ncol=q)*A # (k,q) matrix of v_j.A_{jl}
		r4 <- postprobs %*% r3 # (n,q) matrix of \sum_j p_{ij} A_{jl} v_j
		num <- colSums(r4) #
		# update of gamma parameters which gives iv, v and sigma 
		g.new <- g*(num/den) 	
		iv <- A %*% g.new
		v <- 1/iv 	# needed for next computation of r3
		sigma <- as.vector(sqrt(v)) # needed in next E-steps
		g <- g.new # iterates
		                
        ## test for variance degeneration
        if(any(sigma < 1e-08)) {
            notdone <- TRUE
            cat("One of the variances is going to zero; ",
                "trying new starting values.\n")
            restarts <- restarts + 1
            lambda <- mu <- sigma <- NULL  # WHAT TO DO WITH g in this case?
            if(restarts>maxrestarts) { stop("Too many tries!") }
            break
          }        
        
        # ECM E-step number two:
        z <- .C(C_normpost, as.integer(n), as.integer(k),
                as.double(x), as.double(mu), 
                as.double(sigma), as.double(lambda),
                res2 = double(n*k), double(3*k), post = double(n*k),
                loglik = double(1), PACKAGE = "mixtools")
        postprobs <- matrix(z$post, nrow=n)
        res <- matrix(z$res2, nrow=n)
        newobsloglik <- z$loglik
        diff <- newobsloglik - obsloglik # does not increase that one?
        obsloglik <- newobsloglik
        ll <- c(ll, obsloglik)
        iter <- iter + 1
        if (verb) {
          cat("iteration", iter, ": log-lik diff =", round(diff,4), 
          		" log-lik =", obsloglik, "\n")
        }
      }
    }

# summurizing and returning structure
    if (iter == maxit) {
      cat("WARNING! NOT CONVERGENT!", "\n")
    	}
    cat("number of iterations=", iter, "\n")
    if(arbmean == FALSE){
      scale.order = order(sigma)
      sigma.min = min(sigma)
      postprobs = postprobs[,scale.order]
      colnames(postprobs) <- c(paste("comp", ".", 1:k, sep = ""))
      a=list(x=x, lambda = lambda[scale.order], mu = mu, sigma = sigma.min, 
             scale = sigma[scale.order]/sigma.min, loglik = obsloglik, 
             posterior = postprobs, all.loglik=ll, restarts=restarts, 
             gamma = g,
             ft="normalmixMMlc")
    } else {
      colnames(postprobs) <- c(paste("comp", ".", 1:k, sep = ""))
      a=list(x=x, lambda = lambda, mu = mu, sigma = sigma, loglik = obsloglik, 
             posterior = postprobs, all.loglik=ll, restarts=restarts, 
             beta = beta, gamma = g,
             ft="normalmixMMlc")
    }
  
  class(a) = "mixEM"
  a
}

hmeEM <- function (y, x, lambda=NULL, beta = NULL, sigma = NULL, w = NULL, 
        k = 2, addintercept = TRUE, epsilon = 1e-08, maxit = 10000, verb=FALSE)
    {
    NR <- function (w, x, z)
    {
    A = (inv.logit(x %*% w)/(1 + exp(x %*% w)))^-1
    B = z[,1]-inv.logit(x %*% w)
    C = matrix(c(sum(B),sum(x[,2]*B)),2,1)
    D = matrix(nrow=2,ncol=nrow(x))
    D = solve(matrix(c(sum(A),sum(A * x[,2]),sum(A * x[,2]),sum(A * x[,2]^2)),nrow=2,ncol=2))
    E = w + apply(D %*% C, 1, sum)
    list(w.new = E)
    }
    if (k != 2) cat("This algorithm currently only works for a 2-component mixture-of-experts model!")
    s=sigma
    if (addintercept) {
        x = cbind(1, x)
    }
    n <- length(y)
    p <- ncol(x)
    if (is.null(w)) {
        w = rep(0,p*(k-1))
    }

    tmp <- regmix.init(y = y, x = x, beta = beta, 
        s = s, k = k, addintercept = addintercept)
    beta = tmp$beta
    s = tmp$s
    k = tmp$k    
    if(is.null(lambda)) lambda = inv.logit(x %*% w)
    diff <- 1
    iter <- 0
    nlogroot2pi = n * log(sqrt(2 * pi))
    xbeta <- x %*% beta
    tmp <- t(-1/2/s[2]^2 * t(y - xbeta[, -1])^2) + 1/2/s[1]^2 *
        (y - xbeta[, 1])^2
    tmp2 <- (1-lambda)/s[2] * s[1]/lambda
    tmp3 <- log(1 + (tmp2 * exp(tmp)))
    obsloglik <- sum(log(lambda)) - nlogroot2pi - n * log(s[1]) -
        1/2/s[1]^2 * sum((y - xbeta[, 1])^2) + sum(tmp3)
    ll <- obsloglik
    z = matrix(nrow = n, ncol = k)
	restarts <- 0
    while (diff > epsilon & iter < maxit) {

    lambda.mat <- cbind(lambda,1-lambda)

    res <- (y - xbeta)^2

        for (i in 1:n) {
            for (j in 1:k) {
                z.denom = c()
                for (h in 1:k) {
                  z.denom = c(z.denom, (lambda.mat[i,h]/lambda.mat[i,j]) * 
                    (s[j]/s[h]) * exp(-0.5 * ((1/s[h]^2) * res[i, h] - (1/s[j]^2) * res[i, j])))
                }
                z[i, j] = 1/sum(z.denom)
            }
        }
#        z[, k] = 1 - apply(as.matrix(z[, (1:(k - 1))]), 1, sum)
	z = z/apply(z,1,sum)

                if (addintercept) {
                  lm.out <- lapply(1:k, function(i) lm(y ~ x[, 
                    -1], weights = z[, i]))
                }
                else lm.out <- lapply(1:k, function(i) lm(y ~ 
                  x - 1, weights = z[, i]))
                beta.new <- sapply(lm.out, coef)


	w.diff=10
	while(sum(abs(w.diff))>(p*.001)){
	w.temp <- NR(w,x,z)$w.new
	w.diff <- w-w.temp
	w <- w.temp
	}
        w.new <- w
        lambda.new <- inv.logit(x %*% w.new)
	xbeta.new <- x %*% beta.new

        res <- (y - xbeta.new)^2

        s.new <- sqrt(sapply(1:k, function(i) sum(z[,i] * (res[, i]))/sum(z[, i])))

        sing <- sum(s.new < 1e-08)

        lambda <- lambda.new
        beta <- beta.new
        s <- s.new
        w <- w.new
        xbeta <- xbeta.new
        tmp <- t(-1/2/s[2]^2 * t(y - xbeta[, -1])^2) + 1/2/s[1]^2 *
            (y - xbeta[, 1])^2
        tmp2 <- (1-lambda)/s[2] * s[1]/lambda
        tmp3 <- log(1 + (tmp2 * exp(tmp)))
        newobsloglik <- sum(log(lambda)) - nlogroot2pi - n * log(s[1]) -
            1/2/s[1]^2 * sum((y - xbeta[, 1])^2) + sum(tmp3)


        if (sing > 0 || is.na(newobsloglik) || newobsloglik < 
            obsloglik || abs(newobsloglik) == Inf){# || sum(z) != n) {
            cat("Need new starting values due to singularity...", 
                "\n")
            restarts <- restarts + 1
            if (restarts > 15) 
                stop("Too many tries!")
	    tmp <- regmix.init(y = y, x = x, k = k, addintercept = addintercept)
	    beta = tmp$beta
	    s = tmp$s
	    k = tmp$k
	    w = rep(0,p*(k-1))
	    lambda = inv.logit(x %*% w)
	    diff <- 1
	    iter <- 0
	    xbeta <- x %*% beta
	    tmp <- t(-1/2/s[2]^2 * t(y - xbeta[, -1])^2) + 1/2/s[1]^2 *
	        (y - xbeta[, 1])^2
	    tmp2 <- (1-lambda)/s[2] * s[1]/lambda
	    tmp3 <- log(1 + (tmp2 * exp(tmp)))
	    obsloglik <- sum(log(lambda)) - nlogroot2pi - n * log(s[1]) -
	        1/2/s[1]^2 * sum((y - xbeta[, 1])^2) + sum(tmp3)
	    ll <- obsloglik
        }
	

	else{
        diff <- newobsloglik - obsloglik
        obsloglik <- newobsloglik
	ll <- c(ll, obsloglik)
        iter <- iter + 1
           if (verb) {
                cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                  obsloglik, "\n")}
	}

    }
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
    cat("number of iterations=", iter, "\n")
        rownames(beta) <- c(paste("beta", ".", 0:(p - 1), sep = ""))
        colnames(beta) <- c(paste("comp", ".", 1:k, sep = ""))
        colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
        a = list(x = x, y = y, w=w, lambda = cbind(lambda,1-lambda), beta = beta, 
            sigma = s, loglik = obsloglik, posterior = z, all.loglik = ll, 
            restarts = restarts, ft = "regmixEM")
        class(a) = "mixEM"
        a

}

poisregmixEM = function (y, x, lambda = NULL, beta = NULL, k = 2, addintercept = TRUE, 
    epsilon = 1e-08, maxit = 10000, verb=FALSE) 
{
    if (addintercept) {
        x = cbind(1, x)
    } else x = as.matrix(x)
    n <- length(y)
    p <- ncol(x)
    tmp <- poisregmix.init(y=y, x=x, lambda=lambda, beta=beta, k=k)
    lambda <- tmp$lambda
    beta <- tmp$beta
    k <- tmp$k

    xbeta <- x %*% beta
    z <- matrix(0, n, k)
    diff <- 1
    iter <- 0
    comp <- t(t(dpois(y, exp(xbeta))) * lambda)
    compsum <- apply(comp, 1, sum)
    obsloglik <- sum(log(compsum))
    ll <- obsloglik
	restarts <- 0
    while (diff > epsilon && iter < maxit) {
        j.star = apply(xbeta, 1, which.max)
        for (i in 1:n) {
            for (j in 1:k) {
                z[i, j] = lambda[j]/lambda[j.star[i]] * exp(y[i] * 
                  (xbeta[i, j] - xbeta[i, j.star[i]]) + exp(xbeta[i, 
                  j.star[i]]) - exp(xbeta[i, j]))
            }
        }
        z = z/apply(z, 1, sum)
	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)
    if(sum(is.na(z))>0){
        cat("Need new starting values due to underflow...","\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
        tmp <- poisregmix.init(y=y, x=x, k=k)
        lambda <- tmp$lambda
        beta <- tmp$beta
        k <- tmp$k
          diff <- 1
            iter <- 0
        xbeta <- x %*% beta
        comp <- t(t(dpois(y, exp(xbeta))) * lambda)
        compsum <- apply(comp, 1, sum)
        obsloglik <- sum(log(compsum))
    ll <- obsloglik
    } else{
            lambda <- apply(z, 2, mean)
        lm.out <- lapply(1:k, function(j) try(glm.fit(x, y, weights = z[, j], family = poisson())
,silent=TRUE))
        beta = sapply(lm.out,coef)
        xbeta <- x %*% beta
        comp <- t(t(dpois(y, exp(xbeta))) * lambda)
        compsum <- apply(comp, 1, sum)
        newobsloglik <- sum(log(compsum))
    if(abs(newobsloglik)==Inf || is.na(newobsloglik) || newobsloglik < obsloglik){# || sum(z)!=n){
        cat("Need new starting values due to singularity...","\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
        tmp <- poisregmix.init(y=y, x=x, k=k)
        lambda <- tmp$lambda
        beta <- tmp$beta
        k <- tmp$k
          diff <- 1
            iter <- 0
        xbeta <- x %*% beta
        comp <- t(t(dpois(y, exp(xbeta))) * lambda)
        compsum <- apply(comp, 1, sum)
        obsloglik <- sum(log(compsum))
        ll <- obsloglik
    } else{
      diff <- newobsloglik - obsloglik
        obsloglik <- newobsloglik
    ll <- c(ll, obsloglik)
        iter <- iter + 1
        if (verb) {
            cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                obsloglik, "\n") }

    }    
}
}
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
    cat("number of iterations=", iter, "\n")
	beta <- matrix(beta,ncol=k)
rownames(beta) <- c(paste("beta", ".", 0:(p-1), sep = ""))
colnames(beta) <- c(paste("comp", ".", 1:k, sep = ""))
colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=x, y=y, lambda = lambda, beta = beta, loglik = obsloglik, posterior = z, all.loglik=ll, restarts=restarts, ft="poisregmixEM")
    class(a) = "mixEM"
    a
}

summary.mixEM <- function(object, digits=6, ...){
  x <- object
  o <- switch(x$ft,
              "multmixEM" = rbind(x$lambda, t(x$theta)),
              "normalmixEM" = rbind(x$lambda, x$mu, x$sigma),
              "repnormmixEM" = rbind(x$lambda, x$mu, x$sigma),
              "regmixEM" = rbind(x$lambda, x$sigma, x$beta),
              "regmixEM.lambda" = rbind(x$lambda, x$sigma, x$beta),
              "regmixEM.mixed" = rbind(x$lambda, x$sigma, x$beta),
              "regmixEM.loc" = rbind(x$sigma, x$beta),
              "regmixEM.chgpt" = rbind(x$lambda, x$sigma),
              "logisregmixEM" = rbind(x$lambda, x$beta),
              "poisregmixEM" = rbind(x$lambda, x$beta),
              "expRMM_EM" = rbind(x$lambda, x$rate),       # Reliability Mixt Model censored exp mixture
              "weibullRMM_SEM" = rbind(x$lambda, x$shape, x$scale), # RMM censored weibull mixture
              "mvnormalmixEM" = rbind(x$lambda, matrix(unlist(x$mu), byrow=TRUE, nrow=length(x$lambda))),
              "normalmixMMlc" = rbind(x$lambda, x$mu, x$sigma),
              stop("Unknown mixEM object of type ", x$ft))
  colnames(o) <- paste("comp",1:ncol(o))
  rownames(o) <- switch(x$ft,
                        "multmixEM" = c("lambda", paste("theta", 1:ncol(x$theta), sep="")),
                        "normalmixEM" = c("lambda", "mu", "sigma"),
                        "repnormmixEM" = c("lambda", "mu", "sigma"),
                        "regmixEM" = c("lambda", "sigma", paste("beta", 1:nrow(x$beta), sep="")),
                        "regmixEM.lambda" = c("lambda", "sigma", paste("beta", 1:nrow(x$beta), sep="")),
                        "regmixEM.mixed" = c("lambda", "sigma", paste("beta", 1:nrow(x$beta), sep="")),
                        "regmixEM.loc" = c("sigma", paste("beta", 1:nrow(x$beta), sep="")),
                        "regmixEM.chgpt" = c("lambda", "sigma"),
                        "logisregmixEM" = c("lambda", paste("beta", 1:nrow(x$beta), sep="")),
                        "poisregmixEM" = c("lambda", paste("beta", 1:nrow(x$beta), sep="")),
                        "expRMM_EM" = c("lambda", "rate"),
                        "weibullRMM_SEM" = c("lambda", "shape", "scale"),
                        "mvnormalmixEM" = c("lambda", paste("mu", 1:length(x$mu), sep="")),
                        "normalmixMMlc" = c("lambda", "mu", "sigma"))
	cat("summary of", x$ft, "object:\n")
	print(o, digits=digits)
	cat("loglik at estimate: ", x$loglik, "\n")
	if (x$ft == "expRMM_EM") cat(100*(1-mean(x$d)), "% of the data right censored\n")
	if (x$ft == "weibullRMM_SEM") cat(100*(1-mean(x$d)), "% of the data right censored\n")
}

# code for Reliability Mixture Models (RMM) with Censored data
# D. Chauveau
# ref: Bordes L. and Chauveau D. Computational Statistics (2016)

# Simulate from an exponential mixture.  
# lambda = vector of component probabilities
#	rate = vector of component rates
rexpmix <- function(n,lambda=1,rate=1) {
	m <- length(lambda) # nb of components
	z <- sample(m, n, replace = TRUE, prob = lambda) # component indicator
	rexp(n, rate = rate[z])
}


# pdf of exponential mixture
dexpmixt <- function(t,lam,rate){
	m <- length(lam)
	f <- 0
	for (j in 1:m)
		f <- f + lam[j]*dexp(t,rate=rate[j])
	f
	}



##############################################
## EM algorithm for Reliability Mixture Models (RMM) with Censoring
## Parametric model, for 
## univariate finite mixture of exponentials with right censoring
# x = lifetime data, censored by random c if d is not NULL, in which case
# x= min(x,c) and d = I(x <= c)
# complete data may be (t,d,z) or (x,z), see option complete
expRMM_EM <- function (x, d=NULL, lambda = NULL, rate = NULL, k = 2, 
		  complete="tdz", epsilon = 1e-08, maxit = 1000, verb = FALSE) {
#  warn <- options(warn=-1) # Turn off warnings
#  x <- as.vector(x)
	n <- length(x)
	if (!is.null(lambda)) k=length(lambda)
	if (!is.null(rate)) k=length(rate) # either should be define !!!
	if (is.null(d))  d <- rep(1,n)  # simple EM for noncensored case
	xx <- matrix(x, nrow=n, ncol=k) # x repeated k times, used in E-steps
	dd <- matrix(d, nrow=n, ncol=k) # idem for d
# init function call to do later, in case lambda = rate = NULL  
	if (is.null(lambda)) lambda <- rep(1/k,k)
	if (is.null(rate)) rate <- rep(1,k)  # to do : init.rate(k)
	 # handle the 2 complete data models
	if (complete=="tdz") comptdz=TRUE else comptdz=FALSE
# sequences for storing along iterations 	
	lambda_seq <- rate_seq <- matrix(0, nrow = maxit, ncol = k)
 	lambda_seq[1,] <- lambda
 	rate_seq[1,] <- rate 
	loglik_seq <- NULL
	oldloglik <- -Inf
#    notdone <- TRUE
#    while(notdone) {
      # Initialize everything
      notdone <- FALSE
      dll <- epsilon+1
      iter <- 1
      post <- matrix(nrow = n, ncol = k)
      restarts <- 0     

      while (dll > epsilon && iter < maxit) { # EM iterations
		### E-step ###
		rr <- matrix(rate, n, k, byrow=T)
		ll <- matrix(lambda, n, k, byrow=T)
		# handling censored & non-censored cases
		post <- ((ll*dexp(xx,rr))^dd)*((ll*(1-pexp(xx,rr)))^(1-dd))
		rs <- rowSums(post)
		loglik <- sum(log(rs)) # loglik without the constant term
		# post normalized per row
		post <- sweep(post, 1, rs, "/") # posteriors p_{ij}^t 's

		### M-step ###
		lambda <- colMeans(post)
		lambda_seq[iter+1, ] <- lambda
		if (comptdz) rate <- colSums(post*dd)/colSums(post*xx) # complete=(t,d,z)
		# 			 rate <- colSums(post*d)/colSums(post*x) # gives same answer
		# 						cf automatic recycling
		if (!comptdz) {  # complete data = (x,z)
			mean_xi <- matrix(1/rate, n, k, byrow=T)
			rate <- colSums(post)/colSums(post*(xx + (1-dd)*mean_xi))
			# rate <- colSums(post)/colSums(post*(x + (1-d)*mean_xi)) # same answer
			}
		rate_seq[iter+1, ] <- rate

        dll <- loglik - oldloglik # = Inf for iter=0 1st time
        oldloglik <- loglik
        loglik_seq <- c(loglik_seq, loglik)
        if (verb) {
          cat("iteration", iter, " log-lik diff =", dll, " log-lik =", 
              loglik, "\n")
        }
        iter <- iter + 1
      }  # end of EM loops over iterations

      if (iter == maxit) cat("WARNING! NOT CONVERGENT!", "\n")   
    cat("number of iterations =", iter - 1, "\n")
	colnames(post) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=x, d=d, lambda = lambda, rate = rate, loglik = loglik, 
             posterior = post, all.loglik=loglik_seq, 
             all.lambda = lambda_seq[1:iter,],
             all.rate = rate_seq[1:iter,],
  #           restarts=restarts, 
              ft="expRMM_EM")
	class(a) = "mixEM"
  a
}





##################################################
# plot EM sequences from expRMM_EM:
# color by components, one plot per parameter type
# commented-out abline() usages are for true values for rate and lambda if available
plotexpRMM <- function(a, title=NULL, 
							rowstyle=TRUE, subtitle=NULL, ...)
	{
	n <- length(a$x); m <- dim(a$all.lambda)[2]
	pcc <- round(100*(1-mean(a$d)),2)
	sizes <- paste("n=",n,", ", pcc, "% censored", sep="")
	if (is.null(subtitle)) {
				subtitle <- paste("n=",n,", ", pcc, "% censored", sep="")}
	if (is.null(title)) {
		tt1 <- "Rate parameters" 
		tt2 <- "Weight parameters"
		} else tt1 <- tt2 <- title
lwdset <- 2
if (rowstyle) par(mfrow=c(1,2)) else par(mfrow=c(2,1))
plot(a$all.rate[,1], type="l", ylim=c(0,max(a$all.rate)), 
	xlab="iterations", ylab="estimates", main=tt1, ...)
title(sub=subtitle,cex.sub = 0.75)
lgd <- expression(xi[1]); lcol <- 1
for (j in 2:m) {
	lines(a$all.rate[,j], col=j, ...)
	# abline(rate[j],0,col=j,lty=3)
	lgd <- c(lgd,substitute(xi[j])); lcol <- c(lcol,j)
	}
legend("topright",lgd,col=lcol,lty=1,...)
plot(a$all.lambda[,1], type="l", ylim=c(0,1), 
	xlab="iterations", ylab="estimates", main=tt2, ...)
title(sub=subtitle,cex.sub = 0.75)
lgd <- expression(lambda[1]); lcol <- 1
if (m>2) {
	for (j in 2:m) {
		lines(a$all.lambda[,j], col=j, ...)
		lgd <- c(lgd,substitute(lambda[j]))
		lcol <- c(lcol,j)
		}
	}
legend("topright",lgd,col=lcol,lty=1,...)
	}

regmixEM.mixed = function (y, x, w = NULL, sigma = NULL, arb.sigma = TRUE, alpha = NULL, 
    lambda = NULL, mu = NULL, rho=NULL, R = NULL, arb.R = TRUE, k = 2, ar.1 = FALSE,
    addintercept.fixed = FALSE, addintercept.random = TRUE, 
    epsilon = 1e-08, maxit = 10000, verb = FALSE) 
{
Omega.def <- function(rho,n){
Omega.1stcol <- NULL
for(i in 1:n){
Omega.1stcol <- c(Omega.1stcol,rho^(i-1))
}
A=Omega.1stcol
for(i in 1:(n-1)){
A=cbind(A,c(rep(0,i),Omega.1stcol[1:(n-i)]))
}
B=A+t(A)
diag(B)=rep(1,n)
B=B/(1-rho^2)
B
}

rho.max<-function(rho,z,sigma,y,x,w,alpha,beta,V.beta){
n.i <- length(y)
Omega.ij <- Omega.def(rho=rho,n=n.i)
-.5*z*(log(det(Omega.ij))+(1/sigma)*(t(y- as.vector(w %*% alpha)-x%*%beta))%*%solve(Omega.ij)%*%(y- as.vector(w %*% alpha)-x%*%beta)+sum(diag(solve(Omega.ij) %*% x %*% V.beta %*% t(x) )))
}

    `%L*%` = function(x, y) lapply(1:length(x), function(z) {
        t(x[[z]]) %*% y[[z]]
    })
    N <- length(y)
    n <- sapply(y, length)
    if (addintercept.random) {
        x.1 = lapply(1:N, function(i) cbind(1, x[[i]]))
        x = x.1
    }
    if (is.null(w)) {
        w = as.list(rep(0, N))
	mixed=FALSE
    } else mixed=TRUE
    p <- ncol(x[[1]])
    if (mixed == TRUE) {
        if (addintercept.fixed) {
            w.1 = lapply(1:N, function(i) cbind(1, w[[i]]))
            w = w.1
        }
        tww = w %L*% w
        tww.inv = 0
        for (i in 1:N) {
            tww.inv = tww.inv + tww[[i]]
        }
        tww.inv = solve(tww.inv)
        twx = w %L*% x
        twy = w %L*% y
        q <- ncol(w[[1]])
    }
    tmp <- regmix.mixed.init(y = y, x = x, w = w, sigma = sigma, 
        arb.sigma = arb.sigma, alpha = alpha, lambda = lambda, 
        mu = mu, R = R, arb.R = arb.R, k = k, mixed = mixed, 
        addintercept.fixed = addintercept.fixed, addintercept.random = addintercept.random)
    alpha <- tmp$alpha
    lambda <- tmp$lambda
    mu <- tmp$mu
    R <- tmp$R
    sigma <- tmp$sigma
    s.2 <- sigma^2
if(ar.1==TRUE & is.null(rho)==TRUE){
rho=matrix(runif(N*k,-1,1),nrow=N,ncol=k)
}
    txx = x %L*% x
    diff <- 1
    iter <- 0
obsloglik=-Inf
    ll <- NULL

	restarts <- 0

if(ar.1) Omega.k <- lapply(1:N, function(i) lapply(1:k, function(j) Omega.def(rho=rho[i,j],n=n[i]))) else Omega.k <- lapply(1:N, function(i) lapply(1:k, function(j) diag(1, n[i])))

    while (diff > epsilon && iter < maxit) {

txOx<-lapply(1:N, function(i) lapply(1:k, function(j) t(x[[i]]) %*% solve(Omega.k[[i]][[j]]) %*% x[[i]]))

        if (arb.R) {
            V.beta = list()
            for (i in 1:N) {
                V.beta[[i]] = list()
                V.beta[[i]] = lapply(1:k, function(j) {
                  solve(txOx[[i]][[j]]/s.2[j * arb.sigma + (1 - arb.sigma)] + 
                    solve(R[[j]]))
                })
            }
            beta = list()
            for (i in 1:N) {
                beta[[i]] = matrix(nrow = p, ncol = k)
                for (j in 1:k) {
                  beta[[i]][, j] = V.beta[[i]][[j]] %*% (t(x[[i]])/s.2[j * 
                    arb.sigma + (1 - arb.sigma)]) %*% solve(Omega.k[[i]][[j]]) %*% (y[[i]] - 
                    as.vector(w[[i]] %*% alpha) - x[[i]] %*% 
                    mu[, j]) + mu[, j]
                }
            }
            z = matrix(nrow = N, ncol = k)
            for (i in 1:N) {
                for (j in 1:k) {
                  z.denom = c()
                  for (m in 1:k) {
                    z.denom = c(z.denom, lambda[m]/lambda[j] * 
                      (det(x[[i]] %*% R[[j]] %*% t(x[[i]]) + 
                        s.2[j * arb.sigma + (1 - arb.sigma)] * 
                          Omega.k[[i]][[j]])/det(x[[i]] %*% R[[m]] %*% 
                        t(x[[i]]) + s.2[m * arb.sigma + (1 - 
                        arb.sigma)] * Omega.k[[i]][[m]]))^(0.5) * 
                      exp(-0.5 * (t(y[[i]] - as.vector(w[[i]] %*% 
                        alpha) - x[[i]] %*% mu[, m]) %*% solve(x[[i]] %*% 
                        R[[m]] %*% t(x[[i]]) + s.2[m * arb.sigma + 
                        (1 - arb.sigma)] * Omega.k[[i]][[m]]) %*% 
                        (y[[i]] - as.vector(w[[i]] %*% alpha) - 
                          x[[i]] %*% mu[, m]) - t(y[[i]] - as.vector(w[[i]] %*% 
                        alpha) - x[[i]] %*% mu[, j]) %*% solve(x[[i]] %*% 
                        R[[j]] %*% t(x[[i]]) + s.2[j * arb.sigma + 
                        (1 - arb.sigma)] * Omega.k[[i]][[j]]) %*% 
                        (y[[i]] - as.vector(w[[i]] %*% alpha) - 
                          x[[i]] %*% mu[, j]))))
                  }
                  z[i, j] = 1/sum(z.denom)
                }
            }
#	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)
			z = z/apply(z,1,sum)

            sing <- sum(is.na(z))
            sum.z = apply(z, 2, sum)
            lambda.new <- sum.z/N
            if (sum(lambda.new < 1e-08)>0 || is.na(sum(lambda.new))) {
                sing <- 1
            }
            else {
                mu.new <- matrix(nrow = p, ncol = k)
                for (j in 1:k) {
                  mu.2 <- matrix(sapply(1:N, function(i) {
                    beta[[i]][, j]
                  }), ncol = N)
                  mu.new[, j] <- apply(t(apply(t(mu.2), 2, "*", 
                    matrix(z[, j], nrow = 1))), 1, sum)
                }
                mu.new = t(apply(t(mu.new), 2, "/", matrix(sum.z, 
                  nrow = 1)))
                if (mixed == TRUE) {
                  a.vec <- c()
		  A.mat <- 0
                  for (i in 1:N) {
                    for (j in 1:k) {
		      A.mat = A.mat + (z[i, j]/s.2[j * arb.sigma + 
                        (1 - arb.sigma)])*tww[[i]]
                      a.vec = cbind(a.vec, (z[i, j]/s.2[j * arb.sigma + 
                        (1 - arb.sigma)]) * (twy[[i]] - 
                        twx[[i]] %*% beta[[i]][, j]))
#                      a.vec = cbind(a.vec, z[i, j] * (twy[[i]] - 
#                        twx[[i]] %*% beta[[i]][, j]))
                    }
                  }
#                  alpha.new <- tww.inv %*% apply(a.vec, 1, sum)
                  alpha.new <- solve(A.mat) %*% apply(a.vec, 1, sum)
                  alpha = alpha.new
                }
                s.tr <- matrix(nrow = N, ncol = k)
                z.n <- matrix(nrow = N, ncol = k)
                for (i in 1:N) {
                  for (j in 1:k) {
		      s.tr[i, j] = sum(diag(z[i, j] * (solve(Omega.k[[i]][[j]]) %*% (y[[i]] -
                      as.vector(w[[i]] %*% alpha) - x[[i]] %*%
                      beta[[i]][, j]) %*% t(y[[i]] - as.vector(w[[i]] %*%
                      alpha) - x[[i]] %*% beta[[i]][, j]) + x[[i]] %*%
                      V.beta[[i]][[j]] %*% t(x[[i]]))))
                    z.n[i, j] = z[i, j] * n[i]
                  }
                }
                if (arb.sigma) {
                  s.2.new <- apply(s.tr, 2, sum)/apply(z.n, 2, 
                    sum)
                }
                else s.2.new <- sum(s.tr)/sum(n)
                R.new = list()
                for (j in 1:k) {
                  r.2 <- 0
                  for (i in 1:N) {
                    r <- z[i, j] * (V.beta[[i]][[j]] + t(t(beta[[i]][, 
                      j] - mu.new[, j])) %*% (beta[[i]][, j] - 
                      mu.new[, j]))
                    r.2 <- r.2 + r
                  }
                  R.new[[j]] = r.2/sum(z[, j])
                }
                lambda = lambda.new
                mu = mu.new
                s.2 = s.2.new
                R = R.new
                L = matrix(nrow = N, ncol = k)
                L = t(sapply(1:N, function(i) {
                  sapply(1:k, function(j) {
                    dmvnorm(as.vector(y[[i]]), as.vector(x[[i]] %*% 
                      mu[, j] + as.vector(w[[i]] %*% alpha)), 
                      x[[i]] %*% R[[j]] %*% t(x[[i]]) + s.2[j * 
                        arb.sigma + (1 - arb.sigma)] * Omega.k[[i]][[j]])
                  })
                }))
                L.n = t(apply(t(L), 2, "*", matrix(lambda, nrow = 1)))
                newobsloglik = sum(log(apply(L.n, 1, sum)))
            }
        }
        else {
            R.inv = solve(R)
            V.beta = list()
            for (i in 1:N) {
                V.beta[[i]] = list()
                V.beta[[i]] = lapply(1:k, function(j) {
                  solve(txOx[[i]][[j]]/s.2[j * arb.sigma + (1 - arb.sigma)] + 
                    R.inv)
                })
            }
            beta = list()
            for (i in 1:N) {
                beta[[i]] = matrix(nrow = p, ncol = k)
                for (j in 1:k) {
                  beta[[i]][, j] = V.beta[[i]][[j]] %*% (t(x[[i]])/s.2[j * 
                    arb.sigma + (1 - arb.sigma)]) %*% solve(Omega.k[[i]][[j]]) %*% (y[[i]] - 
                    as.vector(w[[i]] %*% alpha) - x[[i]] %*% 
                    mu[, j]) + mu[, j]
                }
            }
            z = matrix(nrow = N, ncol = k)
            for (i in 1:N) {
                for (j in 1:k) {
                  z.denom = c()
                  for (m in 1:k) {
                    z.denom = c(z.denom, lambda[m]/lambda[j] * 
                      (det(x[[i]] %*% R %*% t(x[[i]]) + s.2[j * 
                        arb.sigma + (1 - arb.sigma)] * Omega.k[[i]][[j]])/det(x[[i]] %*% R %*% t(x[[i]]) + 
                        s.2[m * arb.sigma + (1 - arb.sigma)] * 
                          Omega.k[[i]][[m]]))^(0.5) * exp(-0.5 * 
                      (t(y[[i]] - as.vector(w[[i]] %*% alpha) - 
                        x[[i]] %*% mu[, m]) %*% solve(x[[i]] %*% 
                        R %*% t(x[[i]]) + s.2[m * arb.sigma + 
                        (1 - arb.sigma)] * Omega.k[[i]][[m]]) %*% 
                        (y[[i]] - as.vector(w[[i]] %*% alpha) - 
                          x[[i]] %*% mu[, m]) - t(y[[i]] - as.vector(w[[i]] %*% 
                        alpha) - x[[i]] %*% mu[, j]) %*% solve(x[[i]] %*% 
                        R %*% t(x[[i]]) + s.2[j * arb.sigma + 
                        (1 - arb.sigma)] * Omega.k[[i]][[j]]) %*% 
                        (y[[i]] - as.vector(w[[i]] %*% alpha) - 
                          x[[i]] %*% mu[, j]))))
                  }
                  z[i, j] = 1/sum(z.denom)
                }
            }
#	  z[,k]=1-apply(as.matrix(z[,(1:(k-1))]),1,sum)
	z = z/apply(z,1,sum)
            sing <- sum(is.nan(z))
            sum.z = apply(z, 2, sum)
            lambda.new <- sum.z/N
            if (sum(lambda.new < 1e-08)>0 || is.na(sum(lambda.new))) {
                sing <- 1
            }
            else {
                mu.new <- matrix(nrow = p, ncol = k)
                for (j in 1:k) {
                  mu.2 <- matrix(sapply(1:N, function(i) {
                    beta[[i]][, j]
                  }), ncol = N)
                  mu.new[, j] <- apply(t(apply(t(mu.2), 2, "*", 
                    matrix(z[, j], nrow = 1))), 1, sum)
                }
                mu.new = t(apply(t(mu.new), 2, "/", matrix(sum.z, 
                  nrow = 1)))
                if (mixed == TRUE) {
                  a.vec <- c()
		  A.mat <- 0
                  for (i in 1:N) {
                    for (j in 1:k) {
		      A.mat = A.mat + (z[i, j]/s.2[j * arb.sigma + 
                        (1 - arb.sigma)])*tww[[i]]
                      a.vec = cbind(a.vec, (z[i, j]/s.2[j * arb.sigma + 
                        (1 - arb.sigma)]) * (twy[[i]] -
                        twx[[i]] %*% beta[[i]][, j])) 
#                      a.vec = cbind(a.vec, z[i, j] * (twy[[i]] - 
#                        twx[[i]] %*% beta[[i]][, j]))
                    }
                  }
#                  alpha.new <- tww.inv %*% apply(a.vec, 1, sum)
                  alpha.new <- solve(A.mat) %*% apply(a.vec, 1, sum)
                  alpha = alpha.new
                }
                s.tr <- matrix(nrow = N, ncol = k)
                z.n <- matrix(nrow = N, ncol = k)
                for (i in 1:N) {
                  for (j in 1:k) {
		      s.tr[i, j] = sum(diag(z[i, j] * (solve(Omega.k[[i]][[j]]) %*% (y[[i]] -
                      as.vector(w[[i]] %*% alpha) - x[[i]] %*%
                      beta[[i]][, j]) %*% t(y[[i]] - as.vector(w[[i]] %*%
                      alpha) - x[[i]] %*% beta[[i]][, j]) + x[[i]] %*%
                      V.beta[[i]][[j]] %*% t(x[[i]]))))
                    z.n[i, j] = z[i, j] * n[i]
                  }
                }
                if (arb.sigma) {
                  s.2.new <- apply(s.tr, 2, sum)/apply(z.n, 2, 
                    sum)
                }
                else s.2.new <- sum(s.tr)/sum(n)
                r.3 <- 0
                for (j in 1:k) {
                  r.2 <- 0
                  for (i in 1:N) {
                    r <- z[i, j] * (V.beta[[i]][[j]] + t(t(beta[[i]][, 
                      j] - mu.new[, j])) %*% (beta[[i]][, j] - 
                      mu.new[, j]))
                    r.2 <- r.2 + r
                  }
                  r.3 <- r.3 + r.2
                }
                R.new = r.3/N
                lambda = lambda.new
                mu = mu.new
                s.2 = s.2.new
                R = R.new
if(ar.1){
rho.new<-matrix(0,nrow=N,ncol=k)
for(i in 1:N){
    for(j in 1:k){
rho.new[i,j] <- optimize(rho.max,interval=c(-1,1),maximum=TRUE,tol=.Machine$double.eps,z=z[i,j],sigma=s.2[j * arb.sigma + (1 - arb.sigma)],y=y[[i]],x=x[[i]],w=w[[i]],alpha=alpha,beta=beta[[i]][,j],V.beta=V.beta[[i]][[j]])$max
}
}
rho = rho.new
Omega.k <- lapply(1:N, function(i) lapply(1:k, function(j) Omega.def(rho=rho[i,j],n=n[i] )))
}
                L = matrix(nrow = N, ncol = k)
                L = t(sapply(1:N, function(i) {
                  sapply(1:k, function(j) {
                    dmvnorm(as.vector(y[[i]]), as.vector(x[[i]] %*% 
                      mu[, j] + as.vector(w[[i]] %*% alpha)), 
                      x[[i]] %*% R %*% t(x[[i]]) + s.2[j * arb.sigma + 
                        (1 - arb.sigma)] * Omega.k[[i]][[j]])
                  })
                }))
                L.n = t(apply(t(L), 2, "*", matrix(lambda, nrow = 1)))
                newobsloglik = sum(log(apply(L.n, 1, sum)))
            }
        }

        if (sing > 0 || is.na(newobsloglik) || abs(newobsloglik) == Inf ||  
            newobsloglik < obsloglik) {# || sum(z) != N
            cat("Need new starting values due to singularity...", 
                "\n")
		restarts <- restarts + 1
		if(restarts>15) stop("Too many tries!")
            tmp <- regmix.mixed.init(y = y, x = x, w = w, arb.sigma = arb.sigma, 
                arb.R = arb.R, k = k, mixed = mixed, addintercept.fixed = addintercept.fixed, 
                addintercept.random = addintercept.random)
            alpha <- tmp$alpha
            lambda <- tmp$lambda
            mu <- tmp$mu
            R <- tmp$R
            sigma <- tmp$sigma
            s.2 <- sigma^2
            diff <- 1
            iter <- 0
	obsloglik=-Inf
        ll <- NULL

        }
        else {
            diff <- newobsloglik - obsloglik
            obsloglik <- newobsloglik
        ll <- c(ll, obsloglik)
            iter <- iter + 1
            if (verb) {
                cat("iteration=", iter, "diff=", diff, "log-likelihood", 
                  obsloglik, "\n")
            }
        }
    }
    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
    cat("number of iterations=", iter, "\n")
colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=x, y=y, w=w, lambda = lambda, mu = mu, R = R, sigma = sqrt(s.2), alpha = alpha, 
        loglik = obsloglik, posterior.z = z, posterior.beta = beta, all.loglik = ll, restarts=restarts, ft="regmixEM.mixed")
    class(a) = "mixEM"
    a
}

### multivariate npEM with multivariate conditionnally independent blocks
### functions for mvnpEM with mvwkde code all in C

######################################################################
## mvnpEM final (late 2015) version calling C codes for mvwkde's
## both for same and adaptive bandwidths
## adding a default bandwidth matrix passed as an argument
## bwdefault: a r-vector of fixed bandwidths per coordinates
##  default to Silverman's rule per coordinates, only when samebw=TRUE
## bw returns the standard deviation of the kernel density
## init= for initialization option (June 2015)
######################################################################
mvnpEM <- function (x, mu0, blockid = 1:ncol(x), samebw = TRUE,
                    bwdefault = apply(x,2,bw.nrd0), 
                    init=NULL,
                    eps=1e-8, maxiter = 500, verb = TRUE){
  x = as.matrix(x)
  n = nrow(x); r = ncol(x)
  bk = blockid #b_k=l indicate the kth coordinate belongs to lth block.
  B = max(bk) # total number of blocks 
  # ! Caution if blocks are numbered not consecutively, use unique?
  loglik = NULL
  tt0 <-  proc.time() # for total time
  # coordinate of X in lth block (block of dl-variate densities)
  # dl[l] = number of coordinates in l-th block
  # xx[[j]] = (n,dl[l]) -matrix of block l observations 
  dl=c(); for (l in 1:B){dl[l] = sum(bk==l)}
  xx = list()
  for (l in 1:B){
    xx[[l]] = as.matrix(x[,bk==l]) # coordinate of X in lth block
  }
  if (is.matrix(mu0))     m <- dim(mu0)[1]  # mu0 = initial means
  else     m <- mu0  # when mu0 = number of components
  ## NB initial means used only if kmeans used for init
  ## z.hat are posterior zij's, initialized using kmeans
  if (is.null(init)){
    z.hat <- matrix(0, nrow = n, ncol = m) 
    if (m == 1) z.hat <- matrix(1, nrow = n, ncol = m) # ?? m can never be 1
    else{
      kmeans <- kmeans(x, mu0)
      for(j in 1:m)   z.hat[kmeans$cluster==j, j] <- 1}
  }
  
  if (! is.null(init)) {z.hat <- init} # ToDo: check consistency dim (n,m)
  
  
  lambda = matrix(0, maxiter, m) # storing lambda's along iterations
  finished = FALSE 
  bw=matrix(0,m,r) # only used for adaptbw version
  
  if (samebw){ # bw computed once for all iterations and mvwkde calls
    # bw <- apply(x,2,bw.nrd0)
    # bw <- bwdefault^2*diag(r) # diagonal bw matrix appropriate for mvwkde
    bw <- bwdefault*diag(r) # diagonal bw matrix, not variances, appropriate for mvwkde
  }
  ## loop
  iter = 0 #start point
  while (!finished){
    iter = iter + 1 #step "iter + 1"
    #	t0 = proc.time()
    #################################################################
    ## M-step for the Euclidean parameter
    lambda[iter,] = colMeans(z.hat) #\lambda_j^(t+1) = 1/n* \sum_i^n (p_ij)^t
    #################################################################
    ## Weighted Kernel Density function step
    # wts[,j] = normalized weights for component j 
    cs <- colSums(z.hat) # or use n*lambda[iter,] to avoid re summing over n ? 
    wts <- sweep(z.hat, 2, cs, "/")
    wts [, cs==0] <- 1/NROW(wts)
    
    if (samebw){ # default samebw=TRUE	
      lambda.f <- matrix(NA,n,m) # lambda.f[i,j] = lambda_j f_j(x_i)
      fkernel <- matrix(1,n,m)
      # Faster new version: no loop in j, all is done in C block per block
      for (l in 1:B) {
        d = dl[l]
        tmp=as.matrix(bw[bk==l,bk==l]) # updated here, not sqrt anymore
        h=as.vector(diag(tmp));
        ans <- .C(C_mvwkde_samebw, n = as.integer(n),d = as.integer(d), 
                  m = as.integer(m), h = as.double(h), x=as.double(xx[[l]]), 
                  u=as.double(xx[[l]]), z=as.double(wts), f=double(n*m))
        fl = matrix(ans$f,n,m) # f_jl estimate
        fkernel <- fkernel*fl # product over blocks
      }
      lambda.f <- sweep(fkernel, 2, lambda[iter, ], "*")
    } # end of samebw case
    
    # Adaptive bandwidth case - NOT YET BEST C VERSION (for all j simultaneously ?)
    if (!samebw) {   
      for (k in 1:r) {
        # compute adaptive bw h_jk^t
        # use o <- order(x[,k]) to re-use it twice
        var <- colSums(wts*outer(x[,k], colSums(wts*x[,k]),'-')^2)
        iqr <- apply(as.matrix(wts[order(x[,k]),]),2,wIQR,
                     x[,k][order(x[,k])], 
                     already.sorted=TRUE, already.normalized=TRUE)
        bw[,k] <- 0.9*pmin(sqrt(var), iqr/1.34)*pmax(1,n*lambda[iter, ])^(-1/5)
      }
      lambda.f <- matrix(NA,n,m) # lambda.f[i,j] = lambda_j f_j(x_i)
      fkernel <- matrix(1,n,m)
      #for (j in 1:m){
      #lda.f = lambda[iter, j]
      for (l in 1:B){
        d = dl[l];
        H = as.matrix(bw[, bk==l]);
        ans <- .C(C_mvwkde_adaptbw, n = as.integer(n),d = as.integer(d), 
                  m = as.integer(m), H = as.double(H), x=as.double(xx[[l]]), 
                  u=as.double(xx[[l]]), z=as.double(wts), f=double(n*m))
        fl = matrix(ans$f,n,m)# f_jl estimate
        fkernel <- fkernel*fl # product over blocks
      }
      lambda.f <- sweep(fkernel, 2, lambda[iter, ], "*")
      #}
    }# end of adaptive case
    
    ################################################################
    ## E-step (for next iteration)
    z.hat = lambda.f/rowSums(lambda.f) #p_ij^t, Z_ij^t, update z.hat
    loglik <- c(loglik,sum(log(rowSums(lambda.f)))) # log-likelihood
    ## End
    finished = iter >= maxiter
    if (iter>1) {maxchange = max(abs(lambda[iter,] - lambda[iter-1,]))
    finished = finished | (maxchange < eps) }   
    if (verb) {
      # 		t1 <- proc.time()
      cat("iteration", iter, ": lambda ", round(lambda[iter, ], 4),"\n")
      #	      cat(" time", (t1 - t0)[3], "\n")   
    }
  }# End while
  ##### Output
  if (verb) {
    tt1 <- proc.time() # total time ending
    cat("# iter", iter)
    cat(", lambda ", round(lambda[iter, ], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")  }
  # final bandwidth output depending on samebw switch
  if (samebw) bw <- diag(bw)
  return(structure(list(data = x, posteriors = z.hat, lambda = lambda[1:iter,], 
                        blockid = bk, samebw=samebw, bandwidth = bw, lambdahat = lambda[iter,], 
                        loglik = loglik), 
                   class = "mvnpEM"))
} # End function.









#######################################################
# plot marginal (univariate) wkde's  from mvnpEM output
# a plot.mvnpEM method for mvnpEM class
# lambda, mu, v: true parameters, for gaussian true models only
# ... passed to hist first level plotting
plot.mvnpEM <- function(x, truenorm=FALSE, lambda=NULL, mu=NULL, v=NULL,
                        lgdcex =1,
                        ...) {
  mix.object <- x
  if (!inherits(mix.object, "mvnpEM"))
  a <- x                      
  x <- a$data;  r <- ncol(x); m <- ncol(a$posteriors)
  rr <- sqrt(r); frr <- floor(rr)
  if (frr == rr) par(mfrow=c(rr,rr)) else {
    if (frr*(frr+1) >= r) 
      par(mfrow=c(frr, frr+1)) else par(mfrow=c(frr+1, frr+1))}
  #	if ((r %% 2) == 0) par(mfrow=c(floor(r/2), floor(r/2))) else {
  #		par(mfrow=c(floor(r/2)+1, floor(r/2)+1))}
  for (k in 1:r) {	# for each coord (whatever its block)
    xx <- x[,k]
    uk <- seq(min(xx),max(xx),len=100)
    tt <- paste("block", a$blockid[k],", coord",k)
    hist(xx, col=8, freq=F, xlab="", main=tt, ...)
    for (j in 1:m) {
      wj <- a$post[,j]   # weights for component j
      if (a$samebw) bw <- a$bandwidth[k] else {
        bw <- a$bandwidth[j,k]}
      f <- wkde(xx, u=uk, w=wj, bw=bw, sym=F)
      lines(uk, a$lambdahat[j]*f, col=j)
      # add true Gaussian marginal if requested/known
      if (truenorm) {
        lines(uk,lambda[j]*dnorm(uk,mean=mu[j,k], sd=sqrt(v[j,k,k])), 
              lty=2,lwd=2, col=j)}
    }
    if (a$samebw) subt <- paste("same bw:",round(a$bandwidth[k],3)) else {
      subt <- "adapt bw: "
      for (j in 1:m) subt <- paste(subt, round(a$bandwidth[j,k],3))
    }
    title(sub=subt)
  }
  lgd <- NULL
  for (j in 1:m) lgd <- c(lgd, paste("comp",j))
  legend("topright", lgd, col=1:m, lty=1, cex=lgdcex)
}





###########################################################
print.mvnpEM <- function(x,...)
{
	n <- NROW(x$data)
	r <- NCOL(x$data)
	m <- length(x$lambdahat)
	cat(paste("Observations:", n, "\n"))
	cat(paste("Coordinates per observation:", r, "\n"))
	cat(paste("Mixture components:", m, "\n"))
	if (r > 1) {
	B <- max(x$blockid)
	cat(paste("Blocks (of conditionally iid coordinates):",B, "\n\n"))
	}
	dp = match(c("data", "posteriors", "lambda", "mu"), names(x), nomatch = 0)
	print.default(structure(x[-dp], class = class(x)), ...)
	invisible(x)
}


print.summary.mvnpEM <-function (x, digits = 3, ...) 
{
  if (x$r > 1) 
    cat(paste(x$n, "observations,", x$r, "coordinates,", 
              x$m, "components, and", x$B, "blocks.\n\n"))
  else cat(paste(x$n, "univariate observations, and", x$m, 
                 "components.\n\n"))
  cat("Means (and std. deviations) for each component:\n")
  for (l in 1:x$B) {
    coords <- 1
    if (x$r > 1) {
      coords <- x$blockid == l
      cat(paste("  Block #", l, ":  Coordinate", sep = ""))
      cat(ifelse(sum(coords) > 1, "s ", " "))
      cat(which(coords))
      cat("\n")
    }
    if (sum(coords)==1) { 
      for (j in 1:x$m){
        cat(paste("   Component", j,": "))
        cat(paste(signif(x$means[j,coords], digits), 
                  " (", signif(x$variances[[j]][coords,coords],digits), ")  ", sep = ""))
      }
      cat("\n")
    }
    else {
      for (j in 1:x$m){
        cat(paste("   Component", j,": "))
        cat(paste(signif(x$means[j,coords ], digits),  sep = ""))
        cat("\n")
        print(signif(x$variances[[j]][coords,coords], digits))
        cat("\n")
      }}
  }
}

summary.mvnpEM <- function(object,...)
{
  n <- NROW(object$data);n
  r <- NCOL(object$data);r
  m <- length(object$lambdahat);m
  B <- max(object$blockid);B
  mu <- matrix(0, m, r);mu
  v <- array(0,c(m,r,r));v[1,,];v[2,,]
  normpost <- sweep(object$post, 2, sums <- colSums(object$post),"/")
  for (l in 1:B){
    coords <- object$blockid == l;coords
    sc <- sum(coords);sc
    xx <- as.matrix(object$data[, coords]);xx
    var<- list();mean<-list()
    for (j in 1:m){
      wts <- normpost[, j]
      M<-mu[j,coords] <- colSums(wts*as.matrix(xx))
      if (sc==1) v[j,coords,coords]<-
        colSums(wts*outer(as.matrix(xx), colSums(wts*as.matrix(xx)),'-')^2)
      else{
        for (t1 in 1:sc){
          for (t2 in 1:sc){
            v[j,coords,coords][t1,t2] <- v[j,coords,coords][t2,t1] <-
              colSums(wts*outer(as.matrix(xx[,t1]), colSums(wts*as.matrix(xx[,t1])),'-')
                      *outer(as.matrix(xx[,t2]), colSums(wts*as.matrix(xx[,t2])),'-'))}
        }
      }
      var[[j]] = v[j,,]
    }
  }
  rownames(mu) <-  paste("component",1:m)
  colnames(mu) <-  paste("coordinate", 1:r)
  names(var) <- paste("component",1:m)
  ans <- list(n = n, m = m, r = r, B = B, blockid = object$blockid,
              means = mu, variances = var)
  class(ans)<-"summary.mvnpEM"	
  ans
}

plot.mixEM <-function (x, whichplots = 1, 
                       loglik = 1 %in% whichplots, 
                       density = 2 %in% whichplots,
                       xlab1="Iteration", ylab1="Log-Likelihood",
                       main1="Observed Data Log-Likelihood", col1=1, lwd1=2,
                       xlab2=NULL, ylab2=NULL, main2=NULL, col2=NULL, 
                       lwd2=2,
                       alpha = 0.05, marginal = FALSE, ...)  {
  def.par <- par(ask=(loglik + density > 1), "mar") # only ask and mar are changed
  mix.object <- x
  if (!inherits(mix.object, "mixEM"))
    stop("Use only with \"mixEM\" objects!")
  if (loglik) {
    plot(mix.object$all.loglik, xlab = xlab1, ylab = ylab1, main = main1, 
         type="l", lwd=lwd1, col=col1, ...)
  }
  if (density) {
    if (mix.object$ft == "logisregmixEM") {
      if (ncol(mix.object$x) != 2) {
        stop("The predictors must have 2 columns!")
      }
      if (sum((mix.object$y == 1) + (mix.object$y == 0)) != length(mix.object$y)) {
        stop("The response must be binary!")
      }
      k = ncol(mix.object$beta)
      x = mix.object$x[, 2]
      if(is.null(main2)) { main2 <- "Most Probable Component Membership" }
      if(is.null(xlab2)) { xlab2 <- "Predictor" }
      if(is.null(ylab2)) { ylab2 <- "Response" }
      if(is.null(col2)) { col2 <- 2:(k+1) }
      plot(x, mix.object$y, main=main2, xlab=xlab2, ylab=ylab2,
           col = col2[apply(mix.object$posterior, 1, which.max)], ...)
      a = cbind(x, mix.object$y)
      a = a[order(a[, 1]), ]
      for (i in 1:k) {
        lines(a[, 1], plogis(mix.object$beta[1, i] + mix.object$beta[2, 
                             i] * a[, 1]), col = col2[i])
      }
    }
    if (mix.object$ft == "normalmixEM") {
      k <- ncol(mix.object$posterior)
      x <- sort(mix.object$x)
      a <- hist(x, plot = FALSE)
      maxy <- max(max(a$density), .3989*mix.object$lambda/mix.object$sigma)
      if(is.null(main2)) { main2 <- "Density Curves" }
      if(is.null(xlab2)) { xlab2 <- "Data" }
      if(is.null(col2)) { col2 <- 2:(k+1) }
      hist(x, prob = TRUE, main = main2, xlab = xlab2, 
           ylim = c(0,maxy), ...)
      if (length(mix.object$mu) == 1) {
        arbvar <- TRUE
        mix.object$sigma <- mix.object$scale * mix.object$sigma
        arbmean <- FALSE
      }
      if (length(mix.object$mu) == k && length(mix.object$sigma) == 1) {
        arbmean <- TRUE
        arbvar <- FALSE
      }
      if (length(mix.object$sigma) == k && length(mix.object$mu) == k) {
        arbmean <- TRUE
        arbvar <- TRUE
      }
      for (i in 1:k) {
        lines(x, mix.object$lambda[i] * 
              dnorm(x, mean = mix.object$mu[i * arbmean + (1 - arbmean)], 
                    sd = mix.object$sigma[i * arbvar + (1 - arbvar)]), 
              col = col2[i], lwd = lwd2)
      }
    }
    if (mix.object$ft == "repnormmixEM") {
      x <- as.vector(as.matrix(x))
      k <- ncol(mix.object$posterior)
      x <- sort(mix.object$x)
      a <- hist(x, plot = FALSE)
      maxy <- max(max(a$density), .3989*mix.object$lambda/mix.object$sigma)
      if (is.null(main2)) { main2 <- "Density Curves" }
      if(is.null(xlab2)) { xlab2 <- "Data" }
      if(is.null(col2)) { col2 <- 2:(k+1) }
      hist(x, prob = TRUE, main = main2, xlab = xlab2, 
           ylim = c(0, maxy), ...)
      if (length(mix.object$mu) == 1) {
        arbvar <- TRUE
        mix.object$sigma = mix.object$scale * mix.object$sigma
        arbmean <- FALSE
      }
      if (length(mix.object$mu) == k && length(mix.object$sigma) == 1) {
        arbmean <- TRUE
        arbvar <- FALSE
      }
      if (length(mix.object$sigma) == k && length(mix.object$mu) == k) {
        arbmean <- TRUE
        arbvar <- TRUE
      }
      for (i in 1:k) {
        lines(x, mix.object$lambda[i] * 
              dnorm(x, mean = mix.object$mu[i * arbmean + (1 - arbmean)], 
                    sd = mix.object$sigma[i * arbvar + (1 - arbvar)]), 
              col = col2[i], lwd = lwd2)
      }
    }
    if (mix.object$ft == "regmixEM.mixed") {
      x.1 = mix.object$x
      n = sum(sapply(x.1, nrow))
      x.1.sum = sum(sapply(1:length(x.1), function(i) length(x.1[[i]][, 
                                                             1])))
      if (x.1.sum == n) {
        x = lapply(1:length(x.1), function(i) matrix(x.1[[i]][, 
                                                     -1], ncol = 1))
      }
      else {
        x = x.1
      }
      post.beta(x = x, y = mix.object$y, p.beta = mix.object$posterior.beta, 
                p.z = mix.object$posterior.z)
    }
    if (mix.object$ft == "mvnormalmixEM") {
      x = mix.object$x
      if (ncol(x) != 2) {
        stop("The data must have 2 columns!")
      }
      post = apply(mix.object$posterior, 1, which.max)
      k <- ncol(mix.object$posterior)
      if (is.list(mix.object$sigma)) {
        sigma = mix.object$sigma
      }
      else {
        sigma = lapply(1:k, function(i) mix.object$sigma)
      }
      if (is.list(mix.object$mu)) {
        mu = mix.object$mu
      }
      else {
        mu = lapply(1:k, function(i) mix.object$mu)
      }
      if(is.null(xlab2)) { xlab2 <- "X.1" }
      if(is.null(ylab2)) { ylab2 <- "X.2" }
      if(is.null(col2)) { col2 <- 2:(k+1) }
      if (marginal == FALSE) {
        if (is.null(main2)) { main2 <- "Density Curves" }
        plot(x, col = col2[post], xlab = xlab2, ylab = ylab2, main = main2, ...)
        lapply(1:k, function(i) points(mu[[i]][1], mu[[i]][2], pch = 19))
        for (i in 1:k) {
          for (j in 1:length(alpha)) {
            ellipse(mu = mu[[i]], sigma = sigma[[i]], alpha = alpha[j], 
                    col = col2[i])
          }
        }
      } else {
        if (is.null(main2)) { main2 <- "" } # FIXME:  What's the right main2 here?
        x <- mix.object$x[, 1]
        y <- mix.object$x[, 2]
        xhist <- hist(x, plot = FALSE)
        yhist <- hist(y, plot = FALSE)
        top <- max(c(xhist$counts, yhist$counts))
        xrange <- range(x)
        yrange <- range(y)
        nf <- layout(matrix(c(2, 0, 1, 3), 2, 2, byrow = TRUE), 
                     c(4, 1), c(1, 4), TRUE)
        layout.show(nf)
        par(mar = c(3, 3, 1, 1))
        plot(mix.object$x[, 1], mix.object$x[,2], col = col2[post], 
             xlab = xlab2, ylab = ylab2, main = main2, ...)
        lapply(1:k, function(i) points(mu[[i]][1], mu[[i]][2], pch = 19))
        for (i in 1:k) {
          for (j in 1:length(alpha)) {
            ellipse(mu = mu[[i]], sigma = sigma[[i]], alpha = alpha[j], 
                    col = (i + 1))
          }
        }
        par(mar = c(0, 3, 1, 1))
        barplot(xhist$counts, axes = FALSE, ylim = c(0, top), 
                space = 0, ...)
        par(mar = c(3, 0, 1, 1))
        barplot(yhist$counts, axes = FALSE, xlim = c(0, top), 
                space = 0, horiz = TRUE, ...)
      }
    }
    if (mix.object$ft == "regmixEM") {
      if (ncol(mix.object$x) != 2) {
        stop("The predictors must have 2 columns!")
      }
      post <- apply(mix.object$posterior, 1, which.max)
      k <- ncol(mix.object$posterior)
      x <- mix.object$x[, 2]
      y <- mix.object$y
      n <- length(y)
      if(is.null(main2)) { main2 <- "Most Probable Component Membership" }
      if(is.null(xlab2)) { xlab2 <- "Predictor" }
      if(is.null(ylab2)) { ylab2 <- "Response" }
      if(is.null(col2)) { col2 <- 2:(k+1) }
      plot(x, y, main = main2, xlab=xlab2, ylab=ylab2, type="n", ...)
      a = cbind(mix.object$x[, 2], mix.object$y, post)
      for (i in 1:k) {
        xy = subset(cbind(a, mix.object$posterior[, i]), a[, 3] == i)[, -3]
        xy = matrix(xy, ncol=3)
        points(xy[, 1], xy[, 2], col = col2[i])
        if (is.matrix(mix.object$beta) == FALSE) {
          abline(coef = mix.object$beta)
          beta = matrix(mix.object$beta, ncol = k, nrow = 2)
        }
        else {
          abline(coef = mix.object$beta[, i], col = col2[i])
          beta = mix.object$beta
        }
        out = lm(y ~ x, weights = mix.object$posterior[,i])
        fit = beta[1, i] + beta[2, i] * x
        out.aov = anova(out)
        MSE = out.aov$Mean[2]
        xy.f = cbind(x, y, fit)
        xy.sort = xy.f[order(xy.f[, 1]), ]
        x.new = seq(from=min(x),to=max(x),length.out=100)
        y.new = beta[1, i] + beta[2, i] * x.new
        s.h <- sqrt(MSE * (1/n + (x.new - mean(xy.sort[,1]))^2 / 
                           var(xy.sort[, 1])/(n - 1)))
        for (j in 1:length(alpha)) {
          W = sqrt(qf(1 - alpha[j], 2, n - 2))
          upper = y.new + W * s.h
          lower = y.new - W * s.h
          lines(x.new, upper, col = (i + 1))
          lines(x.new, lower, col = (i + 1))
        }
      }
    }
    if (mix.object$ft == "expRMM_EM") {plotexpRMM(mix.object, ...)} # all default
    if (mix.object$ft == "weibullRMM_SEM") {plotweibullRMM(mix.object, ...)} # all default
  }
  par(def.par) # reset ask and mar to original values
}

boot.se <- function (em.fit, B = 100,
    arbmean = TRUE, arbvar = TRUE, N=NULL, ...)
{
    mix.type <- em.fit$ft
    if (mix.type == "regmixEM") {


#Start Here...

                k=length(em.fit$lambda)
                y=em.fit$y
                n=length(y)
                if(sum(em.fit$x[,1]==1)==n){
                    x=em.fit$x[,-1]
                }
                else x=em.fit$x
                    if (arbmean == FALSE) {
                      scale = em.fit$scale
                      beta = matrix(rep(em.fit$beta, k), ncol = k)
                    }
                    else {
                      scale = 1
                      beta = em.fit$beta
                    }
                  
                  xbeta.new=em.fit$x%*%beta
                  lambda = em.fit$lambda
                  if(arbvar==FALSE){
                    sigma = rep(em.fit$sigma,k)
                  } else{
                    sigma = scale*em.fit$sigma
                    }

                  j = 0
                lambda.bs=NULL
                beta.bs=NULL
                sigma.bs=NULL
                scale.bs=NULL
                while (j < B) {
                  j = j + 1
                  w=rmultinom(n,size=1,prob=lambda)
                  y.sim=sapply(1:n,function(i) rnorm(1,mean=xbeta.new[i,(w[,i]==1)],sd=sigma[w[,i]==1]) )
                  em.bs = try(regmixEM(y = y.sim, x = x, k = k,
                    arbmean = arbmean, arbvar = arbvar, lambda=em.fit$lambda, beta=em.fit$beta,
                    sigma=(scale*em.fit$sigma), ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }

                  else {
                  if(arbmean==FALSE){
                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    beta.bs = cbind(beta.bs,as.vector(em.bs$beta))
                    sigma.bs = cbind(sigma.bs,em.bs$sigma)
                    scale.bs = cbind(scale.bs,em.bs$scale)
                    }
                  else {
                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    beta.bs = cbind(beta.bs,as.vector(em.bs$beta))
                    sigma.bs = cbind(sigma.bs,em.bs$sigma)
                  }
                  }

                }
    if(arbmean==FALSE){
    lambda.se=apply(lambda.bs,1,sd)
    beta.se=apply(beta.bs,1,sd)
    sigma.se=apply(sigma.bs,1,sd)
    scale.se=apply(scale.bs,1,sd)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, beta = beta.bs, beta.se = beta.se, 
    sigma=sigma.bs, sigma.se = sigma.se, scale=scale.bs, scale.se=scale.se)
    }
    else{
    lambda.se=apply(lambda.bs,1,sd)
    beta.se=matrix(apply(beta.bs,1,sd),ncol=k)
    sigma.se=apply(sigma.bs,1,sd)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, beta = beta.bs, beta.se = beta.se, 
    sigma=sigma.bs, sigma.se = sigma.se)
   }
}

#####
   if (mix.type == "regmixEM.mixed"){




#Start Here...

                k=length(em.fit$lambda)
                y=em.fit$y
		    x=em.fit$x
        if (length(y) != length(x))
            stop("Number of elements in lists for x and y must match!")
		    w=em.fit$w
		    p.z=em.fit$posterior.z
		    p.b=em.fit$posterior.beta
		    mu=em.fit$mu
		    R=em.fit$R
		    sigma=em.fit$sigma
		    lambda=em.fit$lambda
		    n=length(y)
		    p=nrow(mu)
		    n.i=sapply(y,length)
		    alpha=em.fit$alpha
                    
                    w.test <- NULL
			for(i in 1:length(w)){
				w.test <- c(w.test,as.vector(w[[i]]))
				}
		    if(sum(w.test)==0){
			 w1=NULL} else w1=w


		    if(length(R)==k) arb.R=TRUE else arb.R=FALSE
		    if(length(sigma)==k) arb.sigma=TRUE else arb.sigma=FALSE


                j = 0
                lambda.bs=NULL
                mu.bs=NULL
                sigma.bs=NULL
                R.bs=NULL
		    alpha.bs=NULL

                while (j < B) {
                  j = j + 1


		    k.bs=sapply(1:n, function(i) rmultinom(1,size=1,prob=lambda))
		    bs.i <- sample(1:n,n,replace=TRUE)
##
		    if(arb.R==FALSE){
			if(arb.sigma==FALSE){
		    		y.sim <- lapply(1:n, function(i) w[[i]] %*% 
                  alpha + as.vector(rmvnorm(1, mu = x[[i]] %*% mu[, 
                  (k.bs[, i] == 1)], sigma = (x[[i]]%*%R%*%t(x[[i]])  +diag(sigma,n.i[i])) )))
				} else y.sim <- lapply(1:n, function(i) w[[i]] %*% 
                  alpha + as.vector(rmvnorm(1, mu = x[[i]] %*% mu[, 
                  (k.bs[, i] == 1)], sigma = (x[[i]]%*%R%*%t(x[[i]])  +diag(sigma[(k.bs[,i] == 1)],n.i[i])) )))
		    } else{
				if(arb.sigma==FALSE){
					y.sim <- lapply(1:n, function(i) w[[i]] %*% 
                  alpha + as.vector(rmvnorm(1, mu = x[[i]] %*% mu[, 
                  (k.bs[, i] == 1)], sigma = (x[[i]]%*%R[(k.bs[, i] == 
                  1)][[1]]%*%t(x[[i]])  +diag(sigma,n.i[i])) )))   
					} else y.sim <- lapply(1:n, function(i) w[[i]] %*% 
                  alpha + as.vector(rmvnorm(1, mu = x[[i]] %*% mu[, 
                  (k.bs[, i] == 1)], sigma = (x[[i]]%*%R[(k.bs[, i] == 
                  1)][[1]]%*%t(x[[i]])  +diag(sigma[(k.bs[,i] == 1)],n.i[i])) )))   
			}

##

                  em.bs = try(regmixEM.mixed(y = y.sim, x = x, w=w1, sigma=sigma, mu=mu, alpha=alpha, R=R, 
					lambda=lambda, k = k, arb.R=arb.R, arb.sigma=arb.sigma, addintercept.fixed=FALSE,
					addintercept.random=FALSE, ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }

                  else {

			lambda.bs = cbind(lambda.bs,em.bs$lambda)
			alpha.bs = cbind(alpha.bs,em.bs$alpha)
			sigma.bs = cbind(sigma.bs,em.bs$sigma)
			mu.bs = cbind(mu.bs,as.vector(em.bs$mu))
			R.bs = cbind(R.bs,as.vector(sapply(em.bs$R,c)))
			}


lambda.se=apply(lambda.bs,1,sd)
alpha.se=apply(alpha.bs,1,sd)
sigma.se=apply(sigma.bs,1,sd)
mu.se=matrix(apply(mu.bs,1,sd),ncol=k)
R.se1=apply(R.bs,1,sd)
	if(arb.R==TRUE){
		R.se2=matrix(R.se1,ncol=k)
		R.se=lapply(1:k,function(i) matrix(R.se2[,i],ncol=p))
		} else R.se=matrix(R.se1,ncol=p)

    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, alpha=alpha.bs, alpha.se=alpha.se,
			mu = mu.bs, mu.se = mu.se, sigma=sigma.bs, sigma.se = sigma.se, R=R.bs, R.se=R.se)

}



}


#####
# Commented out by DRH on 8-29-2008 due to absence of gammamixEM function
#    if (mix.type == "gammamixEM") {
#        x <- em.fit$x
#        n <- length(x)
#        k <- length(em.fit$lambda)
#        alpha <- em.fit$gamma.pars[1,]
#        beta <- em.fit$gamma.pars[2,]
#        lambda <- em.fit$lambda
#        j = 0
#        lambda.bs = NULL
#	  alpha.bs = NULL
#        beta.bs = NULL
#        while (j < B) {
#            j = j + 1
#		comp = sample(1:k,size=n,replace=T,prob=lambda)
#		x.sim = sapply(1:n,function(i) rgamma(1,shape=alpha[comp[i]],scale=beta[comp[i]]))
#            em.bs = try(gammamixEM(x = x.sim, k = k, lambda = lambda, 
#                alpha = alpha, beta = beta, ...), silent = TRUE)
#            if (class(em.bs) == "try-error") {
#                j = j - 1
#            }
#            else {
#                lambda.bs = cbind(lambda.bs, em.bs$lambda)
#                alpha.bs = cbind(alpha.bs, as.vector(em.bs$gamma.pars[1,]))
#                beta.bs = cbind(beta.bs, as.vector(em.bs$gamma.pars[2,]))
#            }
#        }
#        lambda.se = apply(lambda.bs, 1, sd)
#	  alpha.se = apply(alpha.bs, 1, sd)
#	  beta.se = apply(beta.bs, 1, sd)
#        bs.list = list(lambda = lambda.bs, lambda.se = lambda.se, 
#            alpha = alpha.bs, alpha.se = alpha.se, beta = beta.bs, beta.se = beta.se)
#    }

#####
    if (mix.type == "repnormmixEM") {


#Start Here...

                k=length(em.fit$lambda)
                y=em.fit$y
                m=nrow(y)
        n=ncol(y)

                    if (arbmean == FALSE) {
                      scale = em.fit$scale
                      mu = rep(em.fit$mu, k)
                    }
                    else {
                      scale = 1
              mu = em.fit$mu
                    }

                  lambda = em.fit$lambda
                  if(arbvar==FALSE){
                    sigma = rep(em.fit$sigma,k)
                  } else{
                    sigma = scale*em.fit$sigma
                    }

                  j = 0
                lambda.bs=NULL
                mu.bs=NULL
                sigma.bs=NULL
                scale.bs=NULL
                while (j < B) {
                  j = j + 1
                  w=rmultinom(n,size=1,prob=lambda)
                  y.sim=sapply(1:n,function(i) rnorm(m,mean=mu[w[,i]==1],sd=sigma[w[,i]==1]) )
                  em.bs = try(repnormmixEM(x = y.sim,k = k,
                    arbmean = arbmean, arbvar = arbvar, lambda=em.fit$lambda, mu=em.fit$mu,
                    sigma=(scale*em.fit$sigma), ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }

                  else {
                  if(arbmean==FALSE){
                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    mu.bs = cbind(mu.bs,as.vector(em.bs$mu))
                    sigma.bs = cbind(sigma.bs,em.bs$sigma)
                    scale.bs = cbind(scale.bs,em.bs$scale)
                    }
                  else {
                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    mu.bs = cbind(mu.bs,as.vector(mu.bs$beta))
                    sigma.bs = cbind(sigma.bs,em.bs$sigma)
                  }
                  }

                }
    if(arbmean==FALSE){
    lambda.se=apply(lambda.bs,1,sd)
    mu.se=apply(mu.bs,1,sd)
    sigma.se=apply(sigma.bs,1,sd)
    scale.se=apply(scale.bs,1,sd)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, mu = mu.bs, mu.se = mu.se, 
    sigma=sigma.bs, sigma.se = sigma.se, scale=scale.bs, scale.se=scale.se)
    }
    else{
    lambda.se=apply(lambda.bs,1,sd)
    mu.se=apply(mu.bs,1,sd)
    sigma.se=apply(sigma.bs,1,sd)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, mu = mu.bs, mu.se = mu.se, 
    sigma=sigma.bs, sigma.se = sigma.se)
   }
}

####    
    if (mix.type == "mvnormalmixEM") {


#Start Here...

                k=length(em.fit$lambda)
                y=em.fit$x
                n=nrow(y)
        p=ncol(y)

                    if (arbmean == FALSE) {
                      mu = lapply(1:k,function(i) em.fit$mu)
                    }
                    else {
              mu = em.fit$mu
                    }

                  lambda = em.fit$lambda

                  if(arbvar==FALSE){
                    sigma = lapply(1:k, function(i) em.fit$sigma)
                  } else{
                    sigma = em.fit$sigma
                    }

                  j = 0
                lambda.bs=NULL
                mu.bs=NULL
                sigma.bs=NULL

                while (j < B) {
                  j = j + 1
                  w=rmultinom(n,size=1,prob=lambda)
                  y.sim=t(sapply(1:n,function(i) rmvnorm(1,mu=mu[w[,i]==1][[1]],sigma=sigma[w[,i]==1][[1]]) ))
                  em.bs = try(mvnormalmixEM(x = y.sim, k = k,
                    arbmean = arbmean, arbvar = arbvar, lambda=em.fit$lambda, mu=em.fit$mu,
                    sigma=em.fit$sigma, ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }

                  else {

                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    mu.bs1 = as.vector(sapply(em.bs$mu,c))
            mu.bs = cbind(mu.bs, mu.bs1)
                    if(arbvar==FALSE){
                    sigma.bs=cbind(sigma.bs,as.vector(em.bs$sigma))
                    } else{
                    sigma.bs1 = lapply(1:k, function(i) as.vector(em.bs$sigma[[i]]))
            sigma.bs2 = as.vector(sapply(sigma.bs1,c))
            sigma.bs = cbind(sigma.bs,sigma.bs2)
                  }
                  }

                }

    lambda.se=apply(lambda.bs,1,sd)
    mu.se1=apply(mu.bs,1,sd)
    if(arbmean==TRUE){
    mu.se = lapply(1:k,function(i) mu.se1[((i-1)*p+1):(i*p)])
    } else mu.se = mu.se1
    sigma.se1=apply(sigma.bs,1,sd)
    if(arbvar==TRUE){
    sigma.se=lapply(1:k, function(i) matrix(sigma.se1[((i-1)*(p^2)+1):(i*(p^2))], nrow=p,ncol=p))
    } else sigma.se=matrix(sigma.se1,nrow=p)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, mu = mu.bs, mu.se = mu.se, 
    sigma=sigma.bs, sigma.se = sigma.se)
 
}

####
    if (mix.type == "normalmixEM") {


#Start Here...

                k=length(em.fit$lambda)
                y=em.fit$x
                n=length(y)
                x=em.fit$x
                    if (arbmean == FALSE) {
                      scale = em.fit$scale
                      mu = rep(em.fit$mu, k)
                    }
                    else {
                      scale = 1
                      mu = em.fit$mu
                    }
                  
                  lambda = em.fit$lambda
                  if(arbvar==FALSE){
                    sigma = rep(em.fit$sigma,k)
                  } else{
                    sigma = scale*em.fit$sigma
                    }

                  j = 0
                lambda.bs=NULL
                mu.bs=NULL
                sigma.bs=NULL
                scale.bs=NULL
                while (j < B) {
                  j = j + 1
                  w=rmultinom(n,size=1,prob=lambda)
                  y.sim=sapply(1:n,function(i) rnorm(1,mean=mu[(w[,i]==1)],sd=sigma[w[,i]==1]) )
                  em.bs = try(normalmixEM(x = y.sim, k = k,
                    arbmean = arbmean, arbvar = arbvar, lambda=em.fit$lambda, mu=em.fit$mu,
                    sigma=(scale*em.fit$sigma), ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }

                  else {
                  if(arbmean==FALSE){
                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    mu.bs = cbind(mu.bs,as.vector(em.bs$mu))
                    sigma.bs = cbind(sigma.bs,em.bs$sigma)
                    scale.bs = cbind(scale.bs,em.bs$scale)
                    }
                  else {
                    lambda.bs = cbind(lambda.bs,em.bs$lambda)
                    mu.bs = cbind(mu.bs,as.vector(em.bs$mu))
                    sigma.bs = cbind(sigma.bs,em.bs$sigma)
                  }
                  }

                }
    if(arbmean==FALSE){
    lambda.se=apply(lambda.bs,1,sd)
    mu.se=apply(mu.bs,1,sd)
    sigma.se=apply(sigma.bs,1,sd)
    scale.se=apply(scale.bs,1,sd)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, mu = mu.bs, mu.se = mu.se, 
    sigma=sigma.bs, sigma.se = sigma.se, scale=scale.bs, scale.se=scale.se)
    }
    else{
    lambda.se=apply(lambda.bs,1,sd)
    mu.se=matrix(apply(mu.bs,1,sd),ncol=k)
    sigma.se=apply(sigma.bs,1,sd)
    bs.list=list(lambda = lambda.bs, lambda.se = lambda.se, mu = mu.bs, mu.se = mu.se, 
    sigma=sigma.bs, sigma.se = sigma.se)
   }
}

###
if (mix.type == "multmixEM") {


y<-em.fit$y
n<-nrow(y)
n.i<-apply(y,1,sum)
p<-ncol(y)
k<-length(em.fit$lambda)
theta<-em.fit$theta
lambda<-em.fit$lambda


                  j = 0
                lambda.bs=NULL
                theta.bs=NULL


                while (j < B) {
                  j = j + 1

                  w=rmultinom(n,size=1,prob=lambda)
                  y.sim=t(sapply(1:n,function(i) rmultinom(1,size=n.i[i],prob=theta[(w[,i]==1),]) ))

                  em.bs = try(multmixEM(y = y.sim, k = k, lambda=lambda, theta=theta, ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }
        
            else{
            lambda.bs=cbind(lambda.bs,em.bs$lambda)
            theta.bs=cbind(theta.bs,as.vector(em.bs$theta))
            }

        }


        lambda.se=apply(lambda.bs,1,sd)
        theta.se=matrix(apply(theta.bs,1,sd),nrow=k)
        bs.list=list(lambda=lambda.bs, lambda.se=lambda.se, theta=theta.bs, theta.se=theta.se)

}

    
####



if (mix.type == "logisregmixEM") {

        y=em.fit$y
	n=length(y)
        if (is.null(N)) N=rep(1,n)
                k=length(em.fit$lambda)

                
                if(sum(em.fit$x[,1]==1)==n){
                    x=em.fit$x[,-1]
                } else x=em.fit$x


        lambda<-em.fit$lambda
        beta<-em.fit$beta
        xbeta.new=em.fit$x%*%beta
        prob=inv.logit(xbeta.new)

                  j = 0
                lambda.bs=NULL
                beta.bs=NULL


                while (j < B) {
                  j = j + 1

                  w=rmultinom(n,size=1,prob=lambda)

                  y.sim = sapply(1:n, function(i) rbinom(1, size=N[i], prob=prob[,(w[,i]==1)]))


                  em.bs = try(logisregmixEM(y = y.sim, x=x, N=N, k = k, lambda=lambda, beta=beta,...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }
        
            else{
            lambda.bs=cbind(lambda.bs,em.bs$lambda)
            beta.bs=cbind(beta.bs,as.vector(em.bs$beta))
            }

        }


        lambda.se=apply(lambda.bs,1,sd)
        beta.se=matrix(apply(beta.bs,1,sd),nrow=k)
        bs.list=list(lambda=lambda.bs, lambda.se=lambda.se, beta=beta.bs, beta.se=beta.se)

    


}
####




if (mix.type == "poisregmixEM") {


                k=length(em.fit$lambda)
                y=em.fit$y
                n=length(y)
                if(sum(em.fit$x[,1]==1)==n){
                    x=em.fit$x[,-1]
                } else x=em.fit$x


        lambda<-em.fit$lambda
        beta<-em.fit$beta
        xbeta.new=em.fit$x%*%beta
        prob=exp(xbeta.new)

                j = 0
                lambda.bs=NULL
                beta.bs=NULL


                while (j < B) {
                  j = j + 1

                  w=rmultinom(n,size=1,prob=lambda)

                  y.sim = sapply(1:n, function(i) rpois(1, lambda=prob[,(w[,i]==1)]))


                  em.bs = try(poisregmixEM(y = y.sim, x=x, k = k, lambda=lambda, beta=beta, ...), silent = TRUE)
                  if (class(em.bs) == "try-error" || em.bs$restarts!=0) {
                    j = j - 1
                  }
        
            else{
            lambda.bs=cbind(lambda.bs,em.bs$lambda)
            beta.bs=cbind(beta.bs,as.vector(em.bs$beta))
            }

        }


        lambda.se=apply(lambda.bs,1,sd)
        beta.se=matrix(apply(beta.bs,1,sd),nrow=k)
        bs.list=list(lambda=lambda.bs, lambda.se=lambda.se, beta=beta.bs, beta.se=beta.se)

}


    bs.list
    }

boot.comp <- function (y, x = NULL, N = NULL, max.comp = 2, B = 100, sig = 0.05, 
    arbmean = TRUE, arbvar = TRUE, mix.type = c("logisregmix", 
        "multmix", "mvnormalmix", "normalmix", "poisregmix", 
        "regmix", "regmix.mixed", "repnormmix"), hist = TRUE, 
    ...) 
{
    mix.type <- match.arg(mix.type)
    k = max.comp
    p = 0
    sigtest = 1
    Q.star = list()
    i = 0
    if (mix.type == "regmix") {
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  H0.fit = lm(y ~ x)
                  beta = coef(H0.fit)
                  Q0[i] = as.numeric(logLik(H0.fit))
                  H1.fit = try(regmixEM(y = y, x = x, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) 
                      w = 1
                    else w = 2
                    beta = coef(H0.fit)
                    xbeta = cbind(1, x) %*% beta
                    xy.sd = sqrt(sum(H0.fit$res^2)/(length(y) - 
                      2))
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = rnorm(length(y), mean = xbeta, sd = xy.sd)
                  xy.simout = lm(y.sim ~ x)
                  em.out = try(regmixEM(y = y.sim, x = x, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - as.numeric(logLik(xy.simout)))
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(regmixEM(y = y, x = x, k = i, 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  H1.fit = try(regmixEM(y = y, x = x, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    Q0[i] = H0.fit$loglik
                    if (arbmean == FALSE) {
                      scale = H0.fit$scale
                      beta = matrix(rep(H0.fit$beta, i), ncol = i)
                    }
                    else {
                      scale = 1
                    }
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) 
                      w = 1
                    else w = 2
                  }
                  beta.new = H0.fit$beta
                  xbeta.new = cbind(1, x) %*% beta.new
                  j = 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(length(y), size = 1, prob = H0.fit$lambda)
                  if (arbmean == FALSE) {
                    y.sim = sapply(1:length(y), function(i) rnorm(1, 
                      mean = xbeta.new, sd = ((scale * H0.fit$sigma)[wt[, 
                        i] == 1])))
                  }
                  else {
                    if (arbvar == FALSE) {
                      y.sim = sapply(1:length(y), function(i) rnorm(1, 
                        mean = xbeta.new[i, (wt[, i] == 1)], 
                        sd = H0.fit$sigma))
                    }
                    else {
                      y.sim = sapply(1:length(y), function(i) rnorm(1, 
                        mean = xbeta.new[i, (wt[, i] == 1)], 
                        sd = H0.fit$sigma[wt[, i] == 1]))
                    }
                  }
                  em.out.0 = try(regmixEM(y = y.sim, x = x, k = i, 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  em.out.1 = try(regmixEM(y = y.sim, x = x, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (mix.type == "repnormmix") {
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  dens = dnorm(y, mean = mean(y), sd = sd(y))
                  Q0[i] = sum(log(dens[dens > 0]))
                  H1.fit = try(repnormmixEM(x = y, k = (i + 1), 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = rmvnormmix(nrow(y), mu = rep(mean(y), 
                    ncol(y)), sigma = rep(sd(y), ncol(y)))
                  dens.sim = dnorm(y.sim, mean = mean(y), sd = sd(y))
                  x.simout = sum(log(dens.sim[dens.sim > 0]))
                  em.out = try(repnormmixEM(x = y.sim, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - x.simout)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(repnormmixEM(x = y, k = i, arbmean = arbmean, 
                    arbvar = arbvar, ...), silent = TRUE)
                  H1.fit = try(repnormmixEM(x = y, k = (i + 1), 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    Q0[i] = H0.fit$loglik
                    if (arbmean == FALSE) 
                      scale = H0.fit$scale
                    else scale = 1
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                  }
                  j = 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(length(y), size = 1, prob = H0.fit$lambda)
                  if (arbmean == FALSE) {
                    y.sim = sapply(1:ncol(y), function(i) rnorm(nrow(y), 
                      mean = H0.fit$mu, sd = ((scale * H0.fit$sigma)[wt[, 
                        i] == 1])))
                  }
                  else {
                    if (arbvar == FALSE) {
                      y.sim = sapply(1:ncol(y), function(i) rnorm(nrow(y), 
                        mean = H0.fit$mu[wt[, i] == 1], sd = H0.fit$sigma))
                    }
                    else {
                      y.sim = sapply(1:ncol(y), function(i) rnorm(nrow(y), 
                        mean = H0.fit$mu[wt[, i] == 1], sd = H0.fit$sigma[wt[, 
                          i] == 1]))
                    }
                  }
                  em.out.0 = try(repnormmixEM(x = y.sim, k = i, 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  em.out.1 = try(repnormmixEM(x = y.sim, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (mix.type == "regmix.mixed") {
        if (is.list(y)) {
            if (length(y) != length(x)) 
                stop("Number of elements in lists for x and y must match!")
        }
        tt = sapply(1:length(x), function(i) x[[i]][, 1])
        beta = t(sapply(1:length(y), function(i) lm(y[[i]] ~ 
            x[[i]])$coef))
        y = beta
        mix.type = "mvnormalmix"
    }
    if (mix.type == "mvnormalmix") {
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  y.mean = apply(y, 2, mean)
                  y.cov = cov(y)
                  dens = dmvnorm(y, mu = y.mean, sigma = y.cov)
                  Q0[i] = sum(log(dens[dens > 0]))
                  H1.fit = try(mvnormalmixEM(x = y, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) 
                      w = 1
                    else w = 2
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = rmvnorm(nrow(y), mu = apply(y, 2, mean), 
                    sigma = y.cov)
                  dens.sim = dmvnorm(y.sim, mu = y.mean, sigma = y.cov)
                  y.simout = sum(log(dens.sim[dens.sim > 0]))
                  em.out = try(mvnormalmixEM(x = y.sim, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - y.simout)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(mvnormalmixEM(x = y, k = i, ...), 
                    silent = TRUE)
                  H1.fit = try(mvnormalmixEM(x = y, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    Q0[i] = H0.fit$loglik
                    if (arbmean == FALSE) {
                      H0.fit$mu = lapply(1:i, function(l) H0.fit$mu)
                    }
                    if (arbvar == FALSE) {
                      H0.fit$sigma = lapply(1:i, function(l) H0.fit$sigma)
                    }
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                  }
                  j <- 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(nrow(y), size = 1, prob = H0.fit$lambda)
                  if (arbmean == FALSE) {
                    y.sim = t(sapply(1:nrow(y), function(i) rmvnorm(1, 
                      mu = H0.fit$mu, sigma = H0.fit$sigma[wt[, 
                        i] == 1][[1]])))
                  }
                  else {
                    if (arbvar == FALSE) {
                      y.sim = t(sapply(1:nrow(y), function(i) rmvnorm(1, 
                        mu = H0.fit$mu[wt[, i] == 1][[1]], sigma = H0.fit$sigma)))
                    }
                    else {
                      y.sim = t(sapply(1:nrow(y), function(i) rmvnorm(1, 
                        mu = H0.fit$mu[wt[, i] == 1][[1]], sigma = H0.fit$sigma[wt[, 
                          i] == 1][[1]])))
                    }
                  }
                  em.out.0 = try(mvnormalmixEM(x = y.sim, k = i, 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  em.out.1 = try(mvnormalmixEM(x = y.sim, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (mix.type == "normalmix") {
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  dens = dnorm(y, mean = mean(y), sd = sd(y))
                  Q0[i] = sum(log(dens[dens > 0]))
                  H1.fit = try(normalmixEM(x = y, k = (i + 1), 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = rnorm(length(y), mean = mean(y), sd = sd(y))
                  dens.sim = dnorm(y.sim, mean = mean(y), sd = sd(y))
                  x.simout = sum(log(dens.sim[dens.sim > 0]))
                  em.out = try(normalmixEM(x = y.sim, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - x.simout)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(normalmixEM(x = y, k = i, arbmean = arbmean, 
                    arbvar = arbvar, ...), silent = TRUE)
                  H1.fit = try(normalmixEM(x = y, k = (i + 1), 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    Q0[i] = H0.fit$loglik
                    if (arbmean == FALSE) 
                      scale = H0.fit$scale
                    else scale = 1
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                  }
                  j = 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(length(y), size = 1, prob = H0.fit$lambda)
                  if (arbmean == FALSE) {
                    y.sim = sapply(1:length(y), function(i) rnorm(1, 
                      mean = H0.fit$mu, sd = ((scale * H0.fit$sigma)[wt[, 
                        i] == 1])))
                  }
                  else {
                    if (arbvar == FALSE) {
                      y.sim = sapply(1:length(y), function(i) rnorm(1, 
                        mean = H0.fit$mu[(wt[, i] == 1)], 
                        sd = H0.fit$sigma))
                    }
                    else {
                      y.sim = sapply(1:length(y), function(i) rnorm(1, 
                        mean = H0.fit$mu[(wt[, i] == 1)], 
                        sd = H0.fit$sigma[wt[, i] == 1]))
                    }
                  }
                  em.out.0 = try(normalmixEM(x = y.sim, k = i, 
                    arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  em.out.1 = try(normalmixEM(x = y.sim, k = (i + 
                    1), arbmean = arbmean, arbvar = arbvar, ...), 
                    silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (mix.type == "multmix") {
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  m = apply(y, 1, sum)
                  n.i = apply(y, 2, sum)
                  theta = n.i/sum(n.i)
                  Q0[i] = sum(log(exp(apply(y, 1, ldmult, theta = theta))))
                  H1.fit = try(multmixEM(y = y, k = (i + 1), 
                    ...), silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = matrix(0, ncol = ncol(y), nrow = nrow(y))
                  for (l in 1:length(m)) {
                    y.sim[l, ] <- rmultinom(1, size = m[l], prob = theta)
                  }
                  theta.sim = apply(y.sim, 2, sum)/sum(apply(y.sim, 
                    2, sum))
                  y.simout = sum(log(exp(apply(y.sim, 1, ldmult, 
                    theta = theta))))
                  em.out = try(multmixEM(y = y.sim, k = (i + 
                    1), ...), silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - y.simout)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(multmixEM(y = y, k = i, ...), 
                    silent = TRUE)
                  H1.fit = try(multmixEM(y = y, k = (i + 1), 
                    ...), silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    theta = H0.fit$theta
                    Q0[i] = H0.fit$loglik
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                  }
                  j = 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(nrow(y), size = 1, prob = H0.fit$lambda)
#                  y.sim = t(sapply(1:nrow(y), function(i) rmultinom(1, 
#                    size = n.i[i], prob = H0.fit$theta[(wt[, 
#                      i] == 1), ])))
                  new.y.sim = t(sapply(1:nrow(y), function(i) rmultinom(1, 
                    size = n.i, prob = H0.fit$theta[(wt[, 
                      i] == 1), ])))
#                  new.y.sim = 0
                  em.out.0 = try(multmixEM(y = new.y.sim, k = i, 
                    ...), silent = TRUE)
                  em.out.1 = try(multmixEM(y = new.y.sim, k = (i + 
                    1), ...), silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    Q.star[[i]][j]
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (mix.type == "logisregmix") {
        if (is.null(N)) 
            stop("Number of trials must be specified!")
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  H0.fit = glm(cbind(y, N - y) ~ x, family = binomial())
                  Q0[i] = logLik(H0.fit)
                  H1.fit = try(logisregmixEM(y = y, x = x, N = N, 
                    k = (i + 1), ...), silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    beta = coef(H0.fit)
                    xbeta = cbind(1, x) %*% beta
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = rbinom(length(y), size = N, prob = inv.logit(xbeta))
                  xy.simout = glm(cbind(y.sim, N - y.sim) ~ x, 
                    family = binomial())
                  em.out = try(logisregmixEM(y = y.sim, x = x, 
                    N = N, k = (i + 1), ...), silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - logLik(xy.simout))
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(logisregmixEM(y = y, x = x, N = N, 
                    k = i, ...), silent = TRUE)
                  H1.fit = try(logisregmixEM(y = y, x = x, N = N, 
                    k = (i + 1), ...), silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    Q0[i] = H0.fit$loglik
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    beta = H0.fit$beta
                    xbeta = cbind(1, x) %*% beta
                  }
                  j = 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(length(y), size = 1, prob = H0.fit$lambda)
                  y.sim = sapply(1:length(y), function(i) rbinom(1, 
                    size = N[i], prob = inv.logit(xbeta)[, (wt[, 
                      i] == 1)]))
                  em.out.0 = try(logisregmixEM(y = y.sim, x = x, 
                    N = N, k = i, ...), silent = TRUE)
                  em.out.1 = try(logisregmixEM(y = y.sim, x = x, 
                    N = N, k = (i + 1), ...), silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (mix.type == "poisregmix") {
        Q0 = 0
        Q1 = 0
        obs.Q = 0
        i = 1
        while (sigtest == 1 && i <= k) {
            Q.star[[i]] = 0
            if (i == 1) {
                w = 1
                while (w == 1) {
                  H0.fit = glm(y ~ x, family = poisson())
                  Q0[i] = logLik(H0.fit)
                  H1.fit = try(poisregmixEM(y = y, x = x, k = (i + 
                    1), ...), silent = TRUE)
                  if (class(H1.fit) == "try-error") {
                    w = 1
                  }
                  else {
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    beta = coef(H0.fit)
                    xbeta = cbind(1, x) %*% beta
                    j = 0
                  }
                }
                while (j < B) {
                  j = j + 1
                  y.sim = rpois(length(y), lambda = exp(xbeta))
                  xy.simout = glm(y.sim ~ x, family = poisson())
                  em.out = try(poisregmixEM(y = y.sim, x = x, 
                    k = (i + 1), ...), silent = TRUE)
                  if (class(em.out) == "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out$loglik - logLik(xy.simout))
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            else {
                w = 1
                while (w == 1) {
                  H0.fit = try(poisregmixEM(y = y, x = x, k = i, 
                    ...), silent = TRUE)
                  H1.fit = try(poisregmixEM(y = y, x = x, k = (i + 
                    1), ...), silent = TRUE)
                  if (class(H0.fit) == "try-error" || class(H1.fit) == 
                    "try-error") {
                    w = 1
                  }
                  else {
                    Q0[i] = H0.fit$loglik
                    Q1[i] = H1.fit$loglik
                    obs.Q[i] = 2 * (Q1[i] - Q0[i])
                    if (obs.Q[i] < 0) {
                      w = 1
                    }
                    else {
                      w = 2
                    }
                    beta = H0.fit$beta
                    xbeta = cbind(1, x) %*% beta
                  }
                  j = 0
                }
                while (j < B) {
                  j = j + 1
                  wt = rmultinom(length(y), size = 1, prob = H0.fit$lambda)
                  y.sim = sapply(1:length(y), function(i) rpois(1, 
                    lambda = exp(xbeta)[, (wt[, i] == 1)]))
                  em.out.0 = try(poisregmixEM(y = y.sim, x = x, 
                    k = i, ...), silent = TRUE)
                  em.out.1 = try(poisregmixEM(y = y.sim, x = x, 
                    k = (i + 1), ...), silent = TRUE)
                  if (class(em.out.0) == "try-error" || class(em.out.1) == 
                    "try-error") {
                    j = j - 1
                  }
                  else {
                    Q.star[[i]][j] = 2 * (em.out.1$loglik - em.out.0$loglik)
                    if (Q.star[[i]][j] < 0) {
                      j = j - 1
                    }
                  }
                }
            }
            p[i] = mean(Q.star[[i]] >= obs.Q[i])
            sigtest = (p[i] < sig)
            i = i + 1
        }
    }
    if (hist) {
        if (length(p) == 2) {
            par(mfrow = c(1, 2))
            for (i in 1:length(p)) {
                hist(Q.star[[i]], xlab = c("Bootstrap Likelihood", 
                  "Ratio Statistic"), main = paste(i, "versus", 
                  i + 1, "Components"))
                segments(obs.Q[i], 0, obs.Q[i], B, col = 2, lwd = 2)
            }
        }
        else {
            g = ceiling(sqrt(length(p)))
            par(mfrow = c(g, g))
            for (i in 1:length(p)) {
                hist(Q.star[[i]], xlab = c("Bootstrap Likelihood", 
                  "Ratio Statistic"), main = paste(i, "versus", 
                  i + 1, "Components"))
                segments(obs.Q[i], 0, obs.Q[i], B, col = 2, lwd = 2)
            }
        }
    }
    if (p[length(p)] < sig) {
        cat("Decision: Select", length(p) + 1, "Component(s)", 
            "\n")
    }
    else {
        cat("Decision: Select", length(p), "Component(s)", "\n")
    }
    list(p.values = p, log.lik = Q.star, obs.log.lik = obs.Q)
}

## EM-like algorithm for a location-scale mixture model with
## independent repeated measures -- some ID, some not --
## where each block has its own location and scale but otherwise
## all blocks (within a component or globally, depending) have the same
## shape.

## Correction:  For now, this algorithm only implements model (17) in 
## Benaglia et al -- in other words, each component and block has exactly
## the same shape and they differ only by location and scale.
spEM <- function(x, mu0, blockid = 1:ncol(x),
                 bw=bw.nrd0(as.vector(as.matrix(x))), constbw = TRUE,
                 h=bw, eps=1e-8,
                 maxiter=500, stochastic = FALSE, verb = TRUE){
  bw <- h # h is alternative bandwidth argument, for backward compatibility
  x <- as.matrix(x)
  n <- nrow(x)      # number of subjects
  r <- ncol(x)      # number of measurements in each subject
  u <- match(blockid, unique(blockid)) # convert blockid to integers 1, 2, ...
  if (is.matrix(mu0)) 
    m <- dim(mu0)[1]  # mu0=centers
  else 
    m <- mu0  # mu0=number of clusters
  z.hat <- matrix(0, nrow = n, ncol = m)
  tt0 <-  proc.time() # for total time

  ## Initial Values
  if(m == 1) z.hat <- matrix(1, nrow = n, ncol = m)
  else{
    kmeans <- kmeans(x, mu0)
    for(j in 1:m)
      z.hat[kmeans$cluster==j, j] <- 1
  }
  iter <- 0
  if (stochastic) {
    sumpost <- matrix(0, n, m)
  }
  finished <- FALSE
  lambda <- matrix(0, nrow = maxiter, ncol = m)
  mu <- sigma <- array(0, dim=c(maxiter, m, max(u)))
  stackedx <- x[rep(1:n,m),]
  loglik <- NULL

  while (!finished) {
    iter <- iter + 1
    bw.old <- bw
    t0 <- proc.time()
    
    ## Note:  Enter loop assuming E-step is done -- i.e., z.hat is in place    
    ## M-Step
    lambda[iter, ] <- colMeans(z.hat)
    
    if (stochastic) {
      z <- t(apply(z.hat, 1, function(prob) rmultinom(1, 1, prob)))
      cs <- colSums(z)
      z.tmp <- sweep(z, 2, cs, "/")
      z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case
    }
    else {
      cs <- colSums(z.hat)
      z.tmp <- sweep(z.hat, 2, cs, "/")
      z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case
    }
    h <- bw 
    
    ## More M-step (means and std devs are location / scale params)
    for (k in 1:max(u)) { # k is the block number
      r2 <- sum(u == k)
      x2 <- x[, u==k] # Subset of data belonging to kth block (n x r2 matrix)
      mu[iter, , k] <- as.vector(rowMeans(t(z.tmp) %*% x2))
      for (j in 1:m)
        sigma[iter, j, k] <- sqrt(sum(z.tmp[,j] * (x2-mu[iter, j, k])^2)/r2)
    }
    
    ## density estimation step
    if (!constbw) {
      wts <- rep(as.vector(z.tmp),r)
      scaledx <- as.vector((stackedx - mu[iter, rep(1:m, each=n), u])/
                           sigma[iter, rep(1:m, each=n), u])
      h <- bw <- 0.9 / (n*r)^(1/5) * min(1, wiqr<-wIQR(wt=wts, x=scaledx)/1.34)
    }
    ans <- .C(C_KDElocscale, n = as.integer(n), m = as.integer(m), 
              r = as.integer(r), blockid=as.integer(u), 
              mu = as.double(mu[iter, , ]), 
              sigma = as.double(sigma[iter, , ]),
              x = as.double(x),
              bw = as.double(h), z = as.double(z.tmp), f = double(n*m), 
              PACKAGE="mixtools")
    lambda.f <- sweep(matrix(ans$f, ncol=m), 2, lambda[iter, ], "*")

    ## E-step (for next iteration)
    z.hat <- lambda.f/rowSums(lambda.f)
    
    loglik <- c(loglik,sum(log(rowSums(lambda.f)))) # log-likelihood
    finished <- iter >= maxiter
    if (stochastic) {
      sumpost <- sumpost + z.hat
    } 
    else if (iter > 1) { # This convergence criterion may be too simplistic:
      maxchange <- max(abs(lambda[iter,] - lambda[iter-1,]))
      if (!constbw) 
        maxchange <- max(maxchange, max(abs(bw.old - bw)))
      finished <- finished | (maxchange < eps)
    }
    if (verb) {
      t1 <- proc.time()
      cat("iteration", iter, "  lambda ", round(lambda[iter, ], 4))
      cat(" time", (t1 - t0)[3], "\n")
    }
  }
  if (verb) {
    tt1 <- proc.time()
    cat("lambda ", round(lambda[iter, ], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")
  }
  if (stochastic) {
    return(structure(list(data = x, posteriors = sumpost/iter, 
                          lambda = lambda, bandwidth = bw, 
                          blockid = u, lambdahat = colMeans(lambda),
                          mu = mu, muhat = colMeans(mu), 
                          sigma = sigma, sigmahat = colMeans(sigma),
                          loglik = loglik), 
                        class="spEM"))
  }
  else {
    return(structure(list(data = x, posteriors = z.hat, 
                          lambda = lambda[1:iter,], bandwidth = bw, 
                          blockid = u, lambdahat = lambda[iter,],
                          mu = mu[1:iter, , ], muhat = mu[iter, , ],
                          sigma = sigma[1:iter, , ], sigmahat = sigma[iter, , ],
                          loglik = loglik),
                        class="spEM"))
  }
}

plotly_post.beta <- function(y, x, p.beta, p.z,
                             cex = 6,lwd=1,
                             title.size = 15,
                             xlab.size = 15 , xtick.size = 15,
                             ylab.size = 15 , ytick.size = 15,
                             col.data = "#1f77b4",
                             col.comp = NULL){
  N = length(y)
  k = ncol(p.z)
  g = apply(p.z, 1, function(i) (1:length(i))[i == max(i)])
  
  if(is.null(col.comp)){
    col.comp <- hue_pal()(k)
  }
  
  if (length(col.comp) != k){
    print(paste("Please specify" , k , "colors in 'col.comp'."))
  }
  
  abline.fun <- function(x , intercept , slope){
    y <- slope * x + intercept
    return(y)
  }
  
  
  min.int=min(sapply(1:N, function(i) min(p.beta[[i]][1,])))
  max.int=max(sapply(1:N, function(i) max(p.beta[[i]][1,])))
  min.slp=min(sapply(1:N, function(i) min(p.beta[[i]][2,])))
  max.slp=max(sapply(1:N, function(i) max(p.beta[[i]][2,])))
  
  # All posterior regression lines.
  fig1 <- plot_ly()%>%
    plotly::layout(
      xaxis = list(title = list(text = "x-values",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "y-values",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (i in 1:length(x)){
    fig1 <- add_trace(fig1,
      x=as.vector(x[[i]]) , 
      y=as.vector(y[[i]]), 
      type = 'scatter' , mode = 'markers',
      marker = list(size = cex , color = col.data),
      name = paste("Comp" , i) , showlegend = FALSE)
  }
  for (j in 1:k){
    for (l in 1:length(p.beta)){
      fig1 <- add_trace(fig1,
                        x=c(min(unlist(x)) , max(unlist(x))) , 
                        y=c(abline.fun(x=min(unlist(x)) , 
                                       intercept = p.beta[[l]][1,j] , 
                                       slope = p.beta[[l]][2,j]),
                            abline.fun(x=max(unlist(x)) , 
                                       intercept = p.beta[[l]][1,j] , 
                                       slope = p.beta[[l]][2,j])), 
                        type = 'scatter' , mode = 'lines',
                        line = list(width = lwd , color = col.comp[j]),
                        name = paste("Fit" , j) , showlegend = FALSE)
    }
  }
  
  #Posterior regression lines chosen according to the membership probabilities. 
  fig2 <- plot_ly()%>%
    plotly::layout(
      xaxis = list(title = list(text = "x-values",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "y-values",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (i in 1:length(x)){
    fig2 <- add_trace(fig2,
                      x=as.vector(x[[i]]) , 
                      y=as.vector(y[[i]]), 
                      type = 'scatter' , mode = 'markers',
                      marker = list(size = cex , color = col.data),
                      name = paste("Comp" , i) , showlegend = FALSE)
  }
  for (l in 1:length(p.beta)){
    fig2 <- add_trace(fig2,
                      x=c(min(unlist(x)) , max(unlist(x))) , 
                      y=c(abline.fun(x=min(unlist(x)) , 
                                     intercept = p.beta[[l]][1,g[l]] , 
                                     slope = p.beta[[l]][2,g[l]]),
                          abline.fun(x=max(unlist(x)) , 
                                     intercept = p.beta[[l]][1,g[l]] , 
                                     slope = p.beta[[l]][2,g[l]])), 
                      type = 'scatter' , mode = 'lines',
                      line = list(width = lwd , color = col.comp[g[l]]),
                      name = paste("Fit" , j) , showlegend = FALSE)
  }
  
  #All posterior beta values.
  fig3 <- plot_ly()%>%
    plotly::layout(
      xaxis = list(title = list(text = "Posterior Intercepts",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "Posterior Slopes",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (j in 1:k){
    for (l in 1:length(p.beta)){
      fig3 <- add_trace(fig3,
                        x=p.beta[[l]][1,j], 
                        y=p.beta[[l]][2,j], 
                        type = 'scatter' , mode = 'markers',
                        marker = list(size = cex , color = col.comp[j]),
                        name = paste("Comp" , j) , showlegend = FALSE)
    }
  }
  #Posterior beta values chosen according to the membership probabilities.
  fig4 <- plot_ly()%>%
    plotly::layout(
      xaxis = list(title = list(text = "Posterior Intercepts",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "Posterior Slopes",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (l in 1:length(p.beta)){
    fig4 <- add_trace(fig4,
                      x=p.beta[[l]][1,g[l]] , 
                      y=p.beta[[l]][2,g[l]], 
                      type = 'scatter' , mode = 'markers',
                      marker = list(size = cex , color = col.comp[g[l]]),
                      name = paste("Comp" , l) , showlegend = FALSE)
  }
  
  fig <- subplot(fig1, fig2, fig3 , fig4 , nrows = 2,
                 titleX = TRUE , titleY = TRUE,
                 margin = c(0.03,0.03,0.15,0.15)) %>%
    plotly::layout(annotations = list(
      list(
        x = 0.225, 
        y = 1.0, 
        font = list(size = title.size), 
        text = "Data and Posterior Regression Lines", 
        xref = "paper", 
        yref = "paper", 
        xanchor = "center", 
        yanchor = "bottom", 
        showarrow = FALSE
      ), 
      list(
        x = 0.775, 
        y = 1.0, 
        font = list(size = title.size), 
        text = "Data and Most Probable Posterior Regression Lines", 
        xref = "paper", 
        yref = "paper", 
        xanchor = "center", 
        yanchor = "bottom", 
        showarrow = FALSE
      ), 
      list(
        x = 0.225, 
        y = 0.375, 
        font = list(size = title.size), 
        text = "All Posterior Regression Coefficients", 
        xref = "paper", 
        yref = "paper", 
        xanchor = "center", 
        yanchor = "bottom", 
        showarrow = FALSE
      ), 
      list(
        x = 0.775, 
        y = 0.375, 
        font = list(size = title.size), 
        text = "Most Probable Posterior Regression Coefficients", 
        xref = "paper", 
        yref = "paper", 
        xanchor = "center", 
        yanchor = "bottom", 
        showarrow = FALSE
      )
    )
  )
  print(fig)
}

plotly_FDR <- function(post1, post2=NULL, lg1="FDR 1", lg2=NULL, 
                       compH0=1, alpha=0.1, complete.data =NULL, pctfdr=0.3,
                       col = NULL, width = 3 ,
                       title = NULL , title.size = 15 , title.x = 0.5 , title.y = 0.95,
                       xlab = "Index" , xlab.size = 15 , xtick.size = 15,
                       ylab = "Probability" , ylab.size = 15 , ytick.size = 15,
                       legend.text = "" , legend.text.size = 15 , legend.size = 15
                       ){
  hline <- function(y = 0, color = '#1f77b4') {
    list(
      type = "line",
      y0 = y,
      y1 = y,
      xref = "paper",
      x0 = 0,
      x1 = 1,
      line = list(color = '#1f77b4',
                  dash = "dash",
                  width = 1)
    )
  }
  
  if(is.null(col)){
    col <- hue_pal()(3)
  }
  if(length(col) != 3){
    print("Please specify 3 colors in 'col'.")
  }
  
  n <- dim(post1)[1]
  cs1 <- cumsum(post1[,compH0]) # local FDR(p_i)'s
  fdr1 <- cs1/(1:n) # FDR(p_i)'s
  if (is.null(title)) title <- paste("FDR estimate(s), n=",n)
  if (!is.null(post2)) {
    cs2 <- cumsum(post2[,compH0]) # local FDR(p_i)'s
    fdr2 <- cs2/(1:n)
    if (is.null(lg2)) {lg2 <- "FDR 2"}
  }
  i1 <- sum(fdr1<pctfdr) # plot up to pctfdr % FDR
  if (i1 == 0) {i1 <- n}   # for very bad fit, fdr[1] > pctfdr
  # cat("index",i1)
  plot <- plot_ly()%>%
    add_trace(x=seq(from = 1 , to = i1 , by = 1) , 
              y=fdr1[1:i1] , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col[1]),
              name = lg1, showlegend = TRUE)%>%
    plotly::layout(
      legend = list(title=list(text=legend.text,
                               font=list(size=legend.text.size)),
                    font = list(size=legend.size)),
      title = list(text = title,
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
                   ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
                   ),
      shapes = list(type = "line",
                    y0 = alpha,
                    y1 = alpha,
                    xref = "paper",
                    x0 = 0,
                    x1 = 1,
                    line = list(color = '#1f77b4',
                                dash = "dash",
                                width = 1)
      )
    )
    if (!is.null(post2)){
      plot <- plot%>%
        add_trace(x=seq(from = 1 , to = i1 , by = 1) , 
                  y=fdr2[1:i1] , type = 'scatter' , mode = 'lines',
                  line = list(width = width , color = col[2]),
                  name = lg2, showlegend = TRUE)
    }
  if (!is.null(complete.data)){
    V <- cumsum(complete.data[,1]==1) # cumulative nb of items under H0 
    trueFDR <- V/(1:n)
    plot <- plot%>%
      add_trace(x=seq(from = 1 , to = i1 , by = 1) , 
                y=trueFDR[1:i1] , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = col[3] , dash = "dash"),
                name = "True FDR", showlegend = TRUE)
  }
  print(plot)
}

spregmix = function (lmformula, bw = NULL, constbw = FALSE,
                     bwmult = 0.9, # gives Silverman's rule; or use 1.06 for Scott's
                     z.hat = NULL, symm = TRUE, betamethod="LS",
                     m = ifelse(is.null(z.hat), 2, ncol(z.hat)),
                     epsilon = 1e-04, maxit = 1000, verbose = FALSE, 
                     ...) 
{
# m, not k, is the number of components here
  y <- model.frame(lmformula, ...)[,1]
  x <- model.matrix(lmformula, ...)
  n <- length(y)
  p <- ncol(x)
  tt0 <-  proc.time() # for total time

  # Now initialize z.hat, which is the only thing needed to start the iterations
  # as long as we use a method such as least squares to find beta
  if(is.null(z.hat)) {
    z.hat <- matrix(runif(n*m), n, m)
    z.hat <- sweep(z.hat, 1, rowSums(z.hat), "/")
  }

  L1norm <- function(beta, y, x, p) sum(p*abs(y - x %*% beta))
  nploglik <- function(beta, y, x, p, bw, res, wts, symm) {
    sum(p*log(wkde(x=res, u=as.vector(y-x%*%beta),  w=wts, bw=bw, sym=symm))) 
  }
  lambda <- matrix(0, maxit, m)
  beta <- matrix(0, p, m)
  for (j in 1:m) { # Initialize beta to weighted LS solution
        wx <- sweep(x, 1, z.hat[,j], "*")
        beta[,j] <- solve(t(wx) %*% x, t(wx) %*% y) 
  }
  iter <- 0
  finished <- FALSE
#  j <- 0
#plot(tonedata)
  SPweight <- ifelse(betamethod=="transition", 0, 1)
  while (!finished) {
#    j <- j + 1; if (j>m) {
#browser();
#plot(x[,2],y);
#      j <- 1}
    iter <- iter + 1
    t0 <- proc.time()
    ## M-step
    lambda[iter, ] <- colMeans(z.hat)
    oldbeta <- beta
    for (j in 1:m) {
#abline(beta[1,j],beta[2,j], col=1+j, lwd=3, lty=2)
      if (betamethod=="LS") {
        wx <- sweep(x, 1, z.hat[,j], "*")
        beta[,j] <- solve(t(wx) %*% x, t(wx) %*% y) # Weighted least squares solution
      } else if (betamethod=="L1") { # Weighted L1 solution
        beta[,j] <- optim(par=beta[,j], fn=L1norm, y=y, x=x, p=z.hat[,j])$par
      } else if (betamethod=="transition") { # transition from LS to NP
        wx <- sweep(x, 1, z.hat[,j], "*")
        beta[,j] <- SPweight * 
                    optim(par=beta[,j], fn=nploglik, y=y, x=x, p=z.hat[,j], 
                          bw=bw, res=as.vector(y-x%*%beta), 
                          wts=as.vector(z.hat), symm=symm, 
                          control=list(fnscale=-1))$par + # NP loglik
                    (1-SPweight) *
                    solve(t(wx) %*% x, t(wx) %*% y) # Weighted least squares 
        SPweight <- min(1, SPweight + (1e-4)*2^(iter-1))
      } else { # Nonparametric loglikelihood 
        beta[,j] <- optim(par=beta[,j], fn=nploglik, y=y, x=x, p=z.hat[,j], 
                          bw=bw, res=as.vector(y-x%*%beta), 
                          wts=as.vector(z.hat), symm=symm, 
                          control=list(fnscale=-1))$par
#res=as.vector(y-x%*%beta)
#plot(res,wkde(x=res, u=res,  w=as.vector(z.hat), bw=bw, sym=symm))
#browser()

      }
      ## Here, we might try other methods of estimating beta.
    }

    ## update the bandwidth, if necssary
    xbetavec <- as.double(x %*% beta)
    zhatvec <- as.double(z.hat)
    if (is.null(bw) || !constbw) {
      res <- y-xbetavec
      np.sigma <- sqrt(sum(res^2 * zhatvec)/(n-1))
      bw <- bwmult / n^(1/5) * min(np.sigma, wiqr<-wIQR(wt=zhatvec, x=res)/1.34) 
#print(c(np.sigma, wiqr))
    }

    ## density estimation step  
    if (symm) {
      ans <- .C(C_KDEsymloc2, n=as.integer(n), m=as.integer(m),
                 mu = xbetavec, y=as.double(y), bw=as.double(bw),
                 z=as.double(z.hat), f=double(n*m))
    } else {
      ans <- .C(C_KDEloc2, n=as.integer(n), m=as.integer(m),
                 mu = xbetavec, y=as.double(y), bw=as.double(bw),
                 z=as.double(z.hat), f=double(n*m))
    }
    fkernel <- matrix(ans$f, ncol=m)
    lambda.f <- sweep(fkernel, 2, lambda[iter,], "*")

    ## E-step (for next iteration)
    z.hat <- lambda.f/rowSums(lambda.f)    
    
    ## Test convergence criteria
    finished <- (iter > 1 && SPweight == 1 &&
                 (iter >= maxit  ||
                  max(max(abs(beta-oldbeta)),
                      max(abs(lambda[iter,]-lambda[iter-1,]))) < epsilon
                  ))
    
    ## Print message for each iteration
    if (verbose) {
      t1 <- proc.time()
      cat("iteration ", iter, "  lambda ", round(lambda[iter,], 4), 
          "  bandwidth ", round(bw, 3))
      cat(" time", (t1 - t0)[3], "\n")
    }
  }

  ## Print final message after convergence
  if (verbose) {
    tt1 <- proc.time()
    cat("lambda ", round(lambda[iter,], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")
  }

  res <- ans$y - ans$mu
  xx <- res
  yy <- ans$f
  if (symm) {
    xx <- c(xx, -res) # exploiting symmetry
    yy <- c(yy, yy)
  }
  ox <- order(xx)
  np.sigma <- sqrt(sum(res^2 * ans$z)/(n-1))
  
  loglik <- rep(0, n)
  for(j in 1:m) {
    loglik <- loglik + z.hat[,j]*wkde(as.vector(y-x%*%beta[,j]),
                                       w=as.vector(z.hat[,j]), bw=bw, sym=symm)
  }
  loglik <- sum(log(loglik))
  
  a <- list(x=x, y=y, lambda = lambda[1:iter,], beta = beta,
         posterior = z.hat, np.stdev = np.sigma, bandwidth=bw,
         density.x = xx[ox], density.y = yy[ox], symmetric=symm,
         loglik = loglik,
         ft="regmixEM")
  class(a) = "npEM"
  a
} 

multmixEM <- function (y, lambda = NULL, theta = NULL, k = 2, maxit = 10000, 
    epsilon = 1e-08, verb = FALSE) {
  if (class(y)[1]=="list" && !is.null(y$y)) {
    y <- y$y
  }
  n <- nrow(y)
  p <- ncol(y)
  m <- colSums(y)
  r <- rowSums(y) # These need not all be the same

  tmp <- multmix.init(y=y, lambda=lambda, theta=theta, k=k)
  lambda <- tmp$lambda
  theta <- tmp$theta
  k <- tmp$k
  restarts<-0
  mustrestart <- FALSE

  llconstant <- sum(lgamma(r+1)) - sum(lgamma(y+1))
  while (restarts < 50) {
    ll <- NULL
    iter <- 0
    diff <- epsilon+1 # to ensure entering main EM loop
    # Normalize rows of theta matrix
    theta <- theta/rowSums(theta)
    theta <- pmax(theta, 1e-100) # ensure no zeros
    # preparatory E-step prior to entering main EM loop
    loglamcd <- log(lambda) + log(theta) %*% t(y) # kxn matrix of log(lambda * component densities)
    z <- .C(C_multinompost, as.integer(n), as.integer(k),
        as.double(loglamcd), post=double(n*k), loglik=as.double(llconstant),
        PACKAGE = "mixtools")
    post <- matrix(z$post, ncol=k)
    newll <- z$loglik
	tmp.post <- (post==0)
	if(any(apply(tmp.post,2,sum)==n)){
		diff <- epsilon
		mustrestart <- TRUE
	}
    while ((iter < maxit) && diff > epsilon) {  # main EM loop
      iter <- iter + 1
      oldll <- newll
      ll <- c(ll, oldll)
      # M-step:  First update theta values (proportional to sum of posteriors * data)
      theta <- t(post) %*% y
      theta <- theta/rowSums(theta)
      theta <- pmax(theta, 1e-100) # ensure no zeros
      # M-step:  Update the lambdas as usual for a finite mixture
      lambda <- colMeans(post)      
      # E-step:  prepare to find posteriors using C function
      loglamcd <- log(lambda) + log(theta) %*% t(y) # kxn matrix of log(lambda * component densities)
      # E-step:  Call C function to return nxk matrix of posteriors along with loglikelihood
      z <- .C(C_multinompost, as.integer(n), as.integer(k),
              as.double(loglamcd), post=double(n*k), loglik=as.double(llconstant),
              PACKAGE = "mixtools")
      post <- matrix(z$post, ncol=k)
      newll <- z$loglik
      diff <- newll - oldll      
      if (diff<0 || is.na(newll) || is.infinite(newll) || is.nan(newll)) {
        mustrestart <- TRUE
        break
      }
      if (verb) {
        cat("iteration=", iter, "diff=", diff, "log-likelihood", 
            ll[iter], "lambda", lambda, "\n") 
      }
    }
    if (mustrestart) {
      cat("Restarting due to numerical problem.\n")
      mustrestart <- FALSE
      restarts <- restarts + 1
      tmp <- multmix.init(y=y, k=k)
      lambda <- tmp$lambda
      theta <- tmp$theta
      k <- tmp$k
    } else {
      if (iter == maxit) {
        cat("Warning: Not convergent after", maxit, "iterations\n")
      }
      theta[,p] <- 1-apply(as.matrix(theta[,1:(p-1)]),1,sum)
      colnames(theta) <- c(paste("theta", ".", 1:p, sep = ""))
      rownames(theta) <- c(paste("comp", ".", 1:k, sep = ""))
      colnames(post) <- c(paste("comp", ".", 1:k, sep = ""))
      cat("number of iterations=", iter, "\n")
      out <-list(y=y, lambda = lambda, 
                  theta = theta, loglik = ll[length(ll)], posterior = post, 
                  all.loglik=ll, restarts=restarts, ft="multmixEM")
      class(out) <- "mixEM"
      return(out)
    }
  }
  stop("Too many restarts!")
}

plotly_spEMN01 <- function(x, bw=x$bandwidth, knownpdf=dnorm, add.plot=FALSE,
                           width = 3 , col.dens = NULL, col.hist =  '#1f77b4',
                           title = NULL , title.size = 15 , 
                           title.x = 0.5 , title.y = 0.95,
                           xlab = "t" , xlab.size = 15 , xtick.size = 15,
                           ylab = "Density" , ylab.size = 15 , ytick.size = 15,
                           legend.text = "Densities" , legend.text.size = 15 , legend.size = 15
){
  t <- seq(min(x$data), max(x$data), len=200)
  f1 <- x$lambdahat[1]*knownpdf(t)
  f2 <- x$lambdahat[2]*wkde(x$data-x$muhat, u=t-x$muhat, w=x$post[,2], bw=bw, sym=TRUE)
  f <- f1+f2
  
  if(is.null(col.dens)){
    col.dens <- hue_pal()(3)
  }
  if (length(col.dens) != 3){
    print("Please sepcify 3 colors in 'col.dens'.")
  }
  if (is.null(title)){
    title <- ""
  }
  plot <- plot_ly()%>%
    add_trace(x=t , 
              y=f , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col.dens[1]),
              name = "f", showlegend = TRUE)%>%
    add_trace(x=t , 
              y=f1 , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col.dens[2]),
              name = "f1", showlegend = TRUE)%>%
    add_trace(x=t , 
              y=f2 , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col.dens[3]),
              name = "f2", showlegend = TRUE)%>%
    plotly::layout(
      legend = list(title=list(text=legend.text,
                               font=list(size=legend.text.size)),
                    font = list(size=legend.size)),
      title = list(text = title,
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  if (add.plot){
    plot <- plot%>%
      add_trace(x=x$data ,
                type = 'histogram', histnorm = "probability density",
                name = 'Data' , showlegend = FALSE,
                marker = list(color = col.hist,
                              line = list(color = col.hist))
      )%>%
      plotly::layout(bargap = 0.01)
  }
  print(plot)
}

plot.spEM <- plot.npEM <- function(x, blocks = NULL, hist=TRUE, addlegend=TRUE,
                                   scale = TRUE, title=NULL, breaks="Sturges", 
                                   ylim=NULL, dens.col, newplot=TRUE, 
                                   pos.legend="topright", cex.legend=1, ...) {
  r <- NCOL(x$data)
  m <- NCOL(x$posteriors)
  blockid <- x$blockid
  if (is.null(blocks)) {
    if(!is.null(blockid)) {
      blocks <- 1:max(blockid)
    } else {
      blocks <- blockid <- 1:r
    }
  }
  ask <- par(ask=(length(unique(blocks))>1))
  ylim.orig <- ylim
  out <- list(x=list(), y=list())
  if (!newplot) {
    hist <- FALSE
  }
  for(i in 1:length(blocks)) {
    coords <- blockid == blocks[i]
    ylim <- ylim.orig
    if (is.null(title)) {
      if (r>1) {
        tt <- paste(which(coords), collapse=",")
        tt <- paste("Coordinate", ifelse(sum(coords)>1, "s ", " "), tt, sep="")
      } else {
        tt <- "Density Curves"
      }
    } else {
      tt <- rep(title,length(blocks))[i]
    }
    dx <- dy <- NULL
    for (j in 1:m) {
      d <- density(x, component=j, block=blocks[i], scale=scale)
      dx <- cbind(dx, d$x)
      dy <- cbind(dy, d$y)
    }
    xx <- as.vector(as.matrix(x$data)[,coords])
    if (is.null(ylim)) {
      ylim=range(dy)
      if (hist) {
        ylim[2] <- max(ylim[2], hist(xx, breaks=breaks, plot=FALSE)$density)
      }
    }
    if (!hist && newplot) {
      pf <- plot # Use plot or hist as plotting fn the 1st time only, then lines
    } else {
      pf <- lines 
    }
    if (hist) {
      hist(xx, breaks=breaks, prob=TRUE, ylim=ylim, main="", ...)
    }
    if (missing(dens.col)) 
      dens.col <- 2:(m+1)
    dens.col <- rep(dens.col, length.out=m)
    for (j in 1:m) {
      pf(dx[,j],dy[,j], type="l", lwd=2, col=dens.col[j], ylim=ylim, ...)
      pf <- lines
    }
    if (addlegend) {
      legend(pos.legend, legend=round(x$lambdahat,3), fill=dens.col, cex=cex.legend)
      out$x[[i]]<-dx
      out$y[[i]]<-dy
    }
    if (newplot) {
      title(main=tt, ...)
    }
  }
  par(ask=ask)
  invisible(out)
}

    

try.flare <- function (y, x, lambda = NULL, beta = NULL, sigma = NULL, alpha = NULL, nu=1,
		epsilon = 1e-04, maxit = 10000, verb = FALSE, restart=50) 
{

loglik <- function(res, sigma, lambda, alpha) {
  tmp <- lambda*dnorm(res,sd=sqrt(sigma)) + (1-lambda)*dexp(res,rate=alpha)
  sum(log(tmp))
}



Q=function(res, sigma, lambda, alpha, z) {  
  Q <- sum(z*log(lambda)) + sum((1-z)*log(1-lambda)) -
       log(2*pi*sigma)*sum(z)/2 - sum(z*res^2)/2/sigma +
       log(alpha)*sum(1-z) - alpha*sum((1-z)*res)
  Q
}


Z <- function(res, sigma, lambda, alpha) {
  z=rep(1, length(res))
  z[res>0] = lambda / (lambda+(1-lambda)* sqrt(2*pi*sigma) * alpha *
                       as.vector(exp(res[res>0]^2/2/sigma - alpha*res[res>0])))
  z
}


    x <- cbind(1, x)
    n <- length(y)
    p <- ncol(x)

    est <- flaremix.init(y=y, x=x, lambda=lambda, beta=beta, sigma=sigma, alpha=alpha)
    lambda <- est$lambda
    beta <- est$beta
    sigma <- est$sigma
    alpha <- est$alpha


    diff <- 1
    iter <- 0
    counts <- 0
	ll.counts<-0
    xbeta <- x %*% beta
    res <- y - xbeta


	dn <- dnorm(res,sd=sqrt(sigma))
	de <- dexp(res,rate=alpha)

	obsloglik <- loglik(res, sigma, lambda, alpha)
	ll<-obsloglik

  Q1 <- -Inf
  all.Q <- NULL

	z=Z(res,sigma,lambda,alpha)

    while (sum(abs(diff) > epsilon)>0 && iter < maxit) {


      iter=iter+1


temp=(solve(-1/sigma*t(x) %*% sweep(x, 1, z, "*") + nu*t(x) %*% sweep(x, 1, (1-z)/(y-x%*%beta), "*"))%*%(1/sigma*(apply(sweep(x,1,z*(y-x%*%beta),"*"),2,sum)) +alpha*apply(sweep(x,1,1-z,"*"),2,sum)))

m=1
while(m<restart){
j=1
while(j<restart){

beta.new <- try(beta - (1/2^j)*temp,silent=TRUE)

if(any(class(beta.new)=="try-error") || sum(is.na(beta.new))>0) beta.new=beta else beta.new=beta.new

	  xbeta.new <- x %*% beta.new
	  res.new <- y-xbeta.new


Q.beta <- Q(res.new,sigma,lambda,alpha,z)


if(Q.beta < Q1) j=j+1 else j=101

}

if(j==restart) stop(paste("Too many attempts at step-halving!","\n"))


	z.new=Z(res.new,sigma,lambda,alpha)


      lambda.new <- mean(z.new)


    sigma.new <- sum(z.new*(res.new^2))/sum(z.new)

  alpha.new <- sum(1-z.new[res.new>0])/sum((1-z.new[res.new>0])*res.new[res.new>0])




diff <- c(lambda.new,beta.new,sigma.new,alpha.new)-c(lambda,beta,sigma,alpha)

  	  z.new2=Z(res,sigma,lambda,alpha)
#		z.new2=z.new2/apply(z.new2,1,sum)
	  Q.new <- Q(res.new,sigma.new,lambda.new,alpha.new,z.new2)
q.diff=Q.new-Q1

if(q.diff<0) m=m+1 else m=101

}

if(m==restart) stop(paste("Too many attempts at step-halving!","\n"))

	  lambda <- lambda.new
	  beta <- beta.new
	  xbeta <- xbeta.new
	  res <- res.new
	  sigma <- sigma.new
	  alpha <- alpha.new
	  z<-z.new2

		newobsloglik <- loglik(res.new, sigma.new, lambda.new, alpha.new)
	ll<-c(ll, newobsloglik)


	  counts <- counts + (Q.new<Q1)
	  all.Q <- c(all.Q,Q.new)
		ll.counts <- ll.counts + (obsloglik>newobsloglik)




	  Q1 <- Q.new
	


            obsloglik <- newobsloglik



	if(verb==TRUE) cat("iteration=", iter, "diff=", diff, "log-likelihood", obsloglik, "\n")
	
	}

    if (iter == maxit) {
        cat("WARNING! NOT CONVERGENT!", "\n")
    }
#	par(mfrow=c(2,1))
#    plot(all.Q,type="l")
#    plot(ll,type="l")
    cat("number of iterations=", iter, "\n")
    a=list(x=x,y=y,posterior=cbind(z,1-z),lambda=c(lambda,1-lambda),beta=beta,sigma=sigma,alpha=alpha,loglik=obsloglik,all.loglik=ll,ft="flaremixEM")
	class(a)="mixEM"
	a

}

flaremixEM<-function(y, x, lambda = NULL, beta = NULL, sigma = NULL, alpha = NULL, nu=NULL,
		epsilon = 1e-04, maxit = 10000, verb = FALSE, restart=50){
j=1
while(j<=length(nu)){
temp=try(try.flare(y=y, x=x, lambda = lambda, beta = beta, sigma = sigma, alpha = alpha, nu=nu[j],
		epsilon = epsilon, maxit = maxit, verb = verb, restart=restart),silent=TRUE)

if(any(class(temp)=="try-error")) j=j+1 else j=2^100
}



if(j==(length(nu)+1)) stop(paste("Too many attempts. Select a different barrier constant.","\n"))


temp

}

plotly_mixEM <-function (x, 
                         loglik = TRUE, 
                         density = FALSE,
                         xlab1="Iteration", xlab1.size=15 , xtick1.size=15,
                         ylab1="Log-Likelihood", ylab1.size=15 , ytick1.size=15,
                         title1="Observed Data Log-Likelihood", title1.size=15,
                         title1.x = 0.5,title1.y=0.95,
                         col1="#1f77b4", lwd1=3, cex1=6,
                         xlab2=NULL, xlab2.size=15 , xtick2.size=15,
                         ylab2=NULL, ylab2.size=15 , ytick2.size=15,
                         title2=NULL, title2.size=15,
                         title2.x = 0.5,title2.y=0.95, col.hist = "#1f77b4",
                         col2=NULL, lwd2=3, cex2=6,
                         alpha = 0.05, marginal = FALSE){
  def.par <- par(ask=(loglik + density > 1), "mar") # only ask and mar are changed
  mix.object <- x
  if (!inherits(mix.object, "mixEM"))
    stop("Use only with \"mixEM\" objects!")
  ### iteration plot ###
  if (loglik) {
    plot.loglik <- plot_ly()%>%
      add_trace(x= seq(from=1 , to=length(mix.object$all.loglik) , by=1), 
                y= mix.object$all.loglik , type = 'scatter' , mode = 'lines+markers',
                marker = list(color = col1 , size = cex1),
                line = list(color = col1 , width = lwd1),
                name = "Log-Likelihood" , showlegend = FALSE)%>%
      plotly::layout(
        title = list(text = title1,
                     x = title1.x,
                     y = title1.y,
                     font = list(size=title1.size)
        ),
        xaxis = list(title = list(text = xlab1,
                                  font = list(size = xlab1.size)),
                     tickfont = list(size = xtick1.size)
        ),
        yaxis = list(title = list(text = ylab1,
                                  font = list(size = ylab1.size)),
                     tickfont = list(size = ytick1.size)
        )
      )
    print(plot.loglik)
  }
  ### density plot ###
  if (density){
    if (mix.object$ft == "logisregmixEM") {
      if (ncol(mix.object$x) != 2) {
        stop("The predictors must have 2 columns!")
      }
      if (sum((mix.object$y == 1) + (mix.object$y == 0)) != length(mix.object$y)) {
        stop("The response must be binary!")
      }
      k = ncol(mix.object$beta)
      x = mix.object$x[, 2]
      if(is.null(title2)) { title2 <- "Most Probable Component Membership" }
      if(is.null(xlab2)) { xlab2 <- "Predictor" }
      if(is.null(ylab2)) { ylab2 <- "Response" }
      if (is.null(col2)){
        col2 <- hue_pal()(k)
      }
      if (length(col2) != k){
        print(paste("Please specify" , k , "colors in 'col2'."))
      }
      plot.density <- plot_ly()%>%
        add_trace(x=x , 
                  y=mix.object$y, 
                  type = 'scatter' , mode = 'markers',
                  marker = list(color = col2[apply(mix.object$posterior,1, which.max)] , size = cex2),
                  name = "Data" , showlegend = FALSE)%>%
        plotly::layout(
          title = list(text = title2,
                       x = title2.x,
                       y = title2.y,
                       font = list(size=title2.size)
          ),
          xaxis = list(title = list(text = xlab2,
                                    font = list(size = xlab2.size)),
                       tickfont = list(size = xtick2.size)
          ),
          yaxis = list(title = list(text = ylab2,
                                    font = list(size = ylab2.size)),
                       tickfont = list(size = ytick2.size)
          )
        )
      
      a = cbind(x, mix.object$y)
      a = a[order(a[, 1]), ]
      
      for (i in 1:k) {
        plot.density <- add_trace(plot.density,
                                  x=a[,1] , 
                                  y=plogis(mix.object$beta[1, i]+mix.object$beta[2,i] * a[,1]), 
                                  type = 'scatter' , mode = 'lines',
                                  line = list(width = lwd2 , color = col2[i]),
                                  name = paste("Component" , i) , showlegend = FALSE)
      }
    }
    if (mix.object$ft == "normalmixEM") {
      k <- ncol(mix.object$posterior)
      x <- sort(mix.object$x)
      a <- hist(x, plot = FALSE)
      maxy <- max(max(a$density), 0.3989*mix.object$lambda/mix.object$sigma)
      if(is.null(title2)) { title2 <- "Density Curves" }
      if(is.null(xlab2)) { xlab2 <- "Data" }
      if (is.null(col2)){
        col2 <- hue_pal()(k)
      }
      if (length(col2) != k){
        print(paste("Please specify" , k , "colors in 'col2'."))
      }
      
      plot.density <- plot_ly()%>%
        add_trace(x=x ,
                  type = 'histogram', histnorm = "probability density",
                  name = 'Data' , showlegend = FALSE,
                  marker = list(color = col.hist,
                                line = list(color = col.hist))
        )%>%
        plotly::layout(
          title = list(text = title2,
                       x = title2.x,
                       y = title2.y,
                       font = list(size=title2.size)
          ),
          xaxis = list(title = list(text = xlab2,
                                    font = list(size = xlab2.size)),
                       tickfont = list(size = xtick2.size)
          ),
          yaxis = list(title = list(text = ylab2,
                                    font = list(size = ylab2.size)),
                       tickfont = list(size = ytick2.size),
                       range = c(0 , maxy)
          ),
          bargap = 0.01
        )
      if (length(mix.object$mu) == 1) {
        arbvar <- TRUE
        mix.object$sigma <- mix.object$scale * mix.object$sigma
        arbmean <- FALSE
      }
      if (length(mix.object$mu) == k && length(mix.object$sigma) == 1) {
        arbmean <- TRUE
        arbvar <- FALSE
      }
      if (length(mix.object$sigma) == k && length(mix.object$mu) == k) {
        arbmean <- TRUE
        arbvar <- TRUE
      }
      for (i in 1:k) {
        plot.density <- add_trace(plot.density,
                                  x=x , 
                                  y=mix.object$lambda[i] *
                                    dnorm(x, mean = mix.object$mu[i * arbmean + (1 - arbmean)],
                                          sd = mix.object$sigma[i * arbvar + (1 - arbvar)]), 
                                  type = 'scatter' , mode = 'lines',
                                  line = list(width = lwd2 , color = col2[i]),
                                  name = paste("Component" , i) , showlegend = FALSE)
      }
    }
    if (mix.object$ft == "repnormmixEM") {
      x <- as.vector(as.matrix(mix.object$x))
      k <- ncol(mix.object$posterior)
      x.sort <- sort(x)
      a <- hist(x.sort, plot = FALSE)
      maxy <- max(max(a$density), .3989*mix.object$lambda/mix.object$sigma)
      if (is.null(title2)) { title2 <- "Density Curves" }
      if(is.null(xlab2)) { xlab2 <- "Data" }
      if (is.null(col2)){
        col2 <- hue_pal()(k)
      }
      if (length(col2) != k){
        print(paste("Please specify" , k , "colors in 'col2'."))
      }
      
      plot.density <- plot_ly()%>%
        add_trace(x=x ,
                  type = 'histogram', histnorm = "probability density",
                  name = 'Data' , showlegend = FALSE,
                  marker = list(color = col.hist,
                                line = list(color = col.hist))
        )%>%
        plotly::layout(
          title = list(text = title2,
                       x = title2.x,
                       y = title2.y,
                       font = list(size=title2.size)
          ),
          xaxis = list(title = list(text = xlab2,
                                    font = list(size = xlab2.size)),
                       tickfont = list(size = xtick2.size)
          ),
          yaxis = list(title = list(text = ylab2,
                                    font = list(size = ylab2.size)),
                       tickfont = list(size = ytick2.size),
                       range = c(0 , maxy)
          ),
          bargap = 0.01
        )
      if (length(mix.object$mu) == 1) {
        arbvar <- TRUE
        mix.object$sigma = mix.object$scale * mix.object$sigma
        arbmean <- FALSE
      }
      if (length(mix.object$mu) == k && length(mix.object$sigma) == 1) {
        arbmean <- TRUE
        arbvar <- FALSE
      }
      if (length(mix.object$sigma) == k && length(mix.object$mu) == k) {
        arbmean <- TRUE
        arbvar <- TRUE
      }
      for (i in 1:k) {
        plot.density <- add_trace(plot.density,
                                  x=x.sort , 
                                  y=mix.object$lambda[i] * 
                                    dnorm(x.sort, mean = mix.object$mu[i * arbmean + (1 - arbmean)], 
                                          sd = mix.object$sigma[i * arbvar + (1 - arbvar)]), 
                                  type = 'scatter' , mode = 'lines',
                                  line = list(width = lwd2 , color = col2[i]),
                                  name = paste("Component" , i) , showlegend = FALSE)
      }
    }
    if (mix.object$ft == "regmixEM.mixed") {
      if (is.null(col2)){
        col2 <- hue_pal()(ncol(x$posterior.z))
      }
      if (length(col2) != ncol(x$posterior.z)){
        print(paste("Please specify", ncol(x$posterior.z) ,"color in 'col2'."))
      }
      x.1 = mix.object$x
      n = sum(sapply(x.1, nrow))
      x.1.sum = sum(sapply(1:length(x.1), function(i) length(x.1[[i]][,1])))
      if (x.1.sum == n) {
        x = lapply(1:length(x.1), function(i) matrix(x.1[[i]][,-1], ncol = 1))
      }else {
        x = x.1
      }
      plot.density <- plotly_post.beta(x = x, y = mix.object$y, p.beta = mix.object$posterior.beta, 
                                       p.z = mix.object$posterior.z ,
                                       cex = cex2,lwd=lwd2,
                                       title.size = title2.size,
                                       xlab.size = xlab2.size , xtick.size = xtick2.size,
                                       ylab.size = ylab2.size , ytick.size = ytick2.size,
                                       col.comp = col2) 
    }
    if (mix.object$ft == "mvnormalmixEM") {
      x = mix.object$x
      if (ncol(x) != 2) {
        stop("The data must have 2 columns!")
      }
      post = apply(mix.object$posterior, 1, which.max)
      k <- ncol(mix.object$posterior)
      if (is.null(col2)){
        col2 <- hue_pal()(k)
      }
      if (length(col2) != k){
        print(paste("Please specify" ,k," colors in 'col2'."))
      }
      if (is.list(mix.object$sigma)) {
        sigma = mix.object$sigma
      } else {
        sigma = lapply(1:k, function(i) mix.object$sigma)
      }
      if (is.list(mix.object$mu)) {
        mu = mix.object$mu
      } else {
        mu = lapply(1:k, function(i) mix.object$mu)
      }
      if(is.null(xlab2)) { xlab2 <- "X.1" }
      if(is.null(ylab2)) { ylab2 <- "X.2" }
      if (!marginal) {
        if (is.null(title2)) { title2 <- "Density Curves" }
      }
      if (marginal) {
        title2 <- ""
      }
      
      plot.main <- plot_ly()%>%
        add_trace(x=mix.object$x[,1] , 
                  y=mix.object$x[,2], 
                  type = 'scatter' , mode = 'markers',
                  marker = list(color = col2[post] , size = cex2),
                  name = "Data" , showlegend = FALSE)%>%
        plotly::layout(
          title = list(text = title2,
                       x = title2.x,
                       y = title2.y,
                       font = list(size=title2.size)
          ),
          xaxis = list(title = list(text = xlab2,
                                    font = list(size = xlab2.size)),
                       tickfont = list(size = xtick2.size)
          ),
          yaxis = list(title = list(text = ylab2,
                                    font = list(size = ylab2.size)),
                       tickfont = list(size = ytick2.size)
          )
        )
      for (i in 1:k){
        plot.main <- add_markers(plot.main,
                                 x = mu[[i]][1],
                                 y = mu[[i]][2],
                                 marker = list(color = "black" , size = cex2+3),
                                 name = paste("Center" , i) , showlegend = FALSE)
      }
      es.multi <- lapply(sigma, eigen)
      e1.multi <- lapply(es.multi, function(x){x$vectors%*%diag(sqrt(x$values))})
      r1.multi <- sapply(alpha, function(x){sqrt(qchisq(1-x,2))})
      theta <- seq(0,2*pi,len=300)
      v1.multi <- lapply(r1.multi , function(x){cbind(x*cos(theta),x*sin(theta))})
      pts.multi <- rep(list(NA),length(sigma))
      for (i in 1:length(sigma)){
        pts.multi[[i]] <- rep(list(NA) , length(alpha))
        for (j in 1:length(alpha)){
          pts.multi[[i]][[j]] <- t(mu[[i]]-e1.multi[[i]]%*%t(v1.multi[[j]]))
        }
      }
      for (i in 1:k) {
        for (j in 1:length(alpha)) {
          plot.main <- add_trace(
            plot.main,
            x=pts.multi[[i]][[j]][,1] , 
            y=pts.multi[[i]][[j]][,2] , type = 'scatter' , mode = 'lines',
            line = list(color = col2[i] , width = lwd2),
            name = paste((1-alpha[j])*100,'% Ellipse'),showlegend = FALSE)
        }
      }  
      if (!marginal){
        plot.density <- plot.main
      }
      if (marginal){
        x.marginal <- plot_ly()%>%
          add_trace(x=mix.object$x[, 1],
                    type = 'histogram',
                    name = "Dist X",
                    showlegend = FALSE,
                    marker = list(color = col.hist,
                                  line = list(color = col.hist))
          )%>%
          plotly::layout(
            bargap = 0.01
          )
        
        y.marginal <- plot_ly()%>%
          add_trace(y=mix.object$x[, 2],
                    type = 'histogram',
                    name = "Dist X",
                    showlegend = FALSE,
                    marker = list(color = col.hist,
                                  line = list(color = col.hist))
          )%>%
          plotly::layout(
            bargap = 0.01
          )
        
        plot.density <- subplot(
          x.marginal,
          plotly_empty(type = 'scatter' , mode = "markers"),
          plot.main,
          y.marginal,
          nrows = 2, heights = c(.2, .8), widths = c(.8,.2), margin = 0,
          shareX = TRUE, shareY = TRUE) %>%
          plotly::layout(showlegend = FALSE)
      }
    }
    print(plot.density)
    
#    if (mix.object$ft == "expRMM_EM") {plotexpRMM(mix.object, ...)} # all default
#    if (mix.object$ft == "weibullRMM_SEM") {plotweibullRMM(mix.object, ...)} # all default
    if (mix.object$ft == "expRMM_EM") {plotly_expRMM(mix.object)} # all default
    if (mix.object$ft == "weibullRMM_SEM") {plotly_weibullRMM(mix.object)} # all default
  }
  par(def.par) # reset ask and mar to original values
}

repnormmix.init = function (x, lambda = NULL, mu = NULL, s = NULL, k = 2, arbmean = TRUE, arbvar = TRUE) 
{
    n <- ncol(x)
    m <- nrow(x)
    y <- apply(x, 2, mean)
    x <- x[, order(y)]
    x.bin = list()
    for (j in 1:k) {
        x.bin[[j]] <- x[, max(1, floor((j - 1) * n/k)):ceiling(j * 
            n/k)]
    }
    if (is.null(s)) {
        s.hyp = sapply(lapply(x.bin, as.vector), sd)
        if (arbvar) {
            s = 1/rexp(k, rate = s.hyp)
        }
        else {
            s.hyp = mean(s.hyp)
            s = 1/rexp(1, rate = s.hyp)
        }
    }
    if (is.null(s) == FALSE && arbvar == TRUE) {
        k = length(s)
    }
    if (is.null(mu)) {
        mu.hyp <- sapply(lapply(x.bin, as.vector), mean)
        mu = rnorm(k, mean = mu.hyp, sd = s)
		if(arbmean==FALSE){
		mu = mean(mu)
		}
    }
	if (is.null(mu)==FALSE && arbmean==TRUE){
    		k = length(mu)
	}
    if (is.null(lambda)) {
        lambda = runif(k)
        lambda = lambda/sum(lambda)
    }
    else k = length(lambda)
    list(lambda = lambda, mu = mu, s = s, k = k)
}

post.beta <- function(y, x, p.beta, p.z)
{
N = length(y)
k = ncol(p.z)
g = apply(p.z, 1, function(i) (1:length(i))[i == max(i)])


min.int=min(sapply(1:N, function(i) min(p.beta[[i]][1,])))
max.int=max(sapply(1:N, function(i) max(p.beta[[i]][1,])))
min.slp=min(sapply(1:N, function(i) min(p.beta[[i]][2,])))
max.slp=max(sapply(1:N, function(i) max(p.beta[[i]][2,])))


par(mfrow=c(2,2))



#All posterior regression lines.
plot(c(min(sapply(x,min)),max(sapply(x,max))),c(min(sapply(y,min)),max(sapply(y,max))),type="n",xlab="x-values",ylab="y-values",main="Data and Posterior Regression Lines")
sapply(1:length(x),function(i) points(x[[i]], y[[i]]))
for(j in 1:k){
sapply(1:length(p.beta), function(i) abline(coef=p.beta[[i]][,j], col=(j+1)))
}


#Posterior regression lines chosen according to the membership probabilities.
plot(c(min(sapply(x,min)),max(sapply(x,max))),c(min(sapply(y,min)),max(sapply(y,max))),type="n",xlab="x-values",ylab="y-values",main="Data and Most Probable Posterior Regression Lines")
sapply(1:length(x),function(i) points(x[[i]], y[[i]]))
sapply(1:length(p.beta), function(i) abline(coef=p.beta[[i]][,g[i]], col=(g[i]+1)))



#All posterior beta values.
plot(c(min.int,max.int),c(min.slp,max.slp),type="n",xlab="Posterior Intercepts",ylab="Posterior Slopes",main="All Posterior Regression Coefficients")
for(j in 1:k){
sapply(1:length(p.beta), function(i) points(t(p.beta[[i]][,j]), col=(j+1)))
}


#Posterior beta values chosen according to the membership probabilities.
min.max=c(min(sapply(p.beta,min)),max(sapply(p.beta,max)))
plot(c(min.int,max.int),c(min.slp,max.slp),type="n",xlab="Posterior Intercepts",ylab="Posterior Slopes",main="Most Probable Posterior Regression Coefficients")
sapply(1:length(p.beta), function(i) points(t(p.beta[[i]][,g[i]]), col=(g[i]+1)))


}

mixturegram <- function(data, pmbs, method=c("pca","kpca","lda"), all.n=FALSE, id.con=NULL, score=1, iter.max=50, nstart=25, ...){
  col.blind=rep(c("#000000", "#E69F00", "#56B4E9", "#009E73", 
                  "#F0E442", "#0072B2", "#D55E00", "#CC79A7",
                  "#7FFF00","#7D26CD"),100)
  method <- match.arg(method)
  k=length(pmbs)+1
  if (is.null(id.con)) id.con=lapply(pmbs,function(x) data%*%x/apply(x,2,sum))
  ###regular pca
  if(method=="pca"){
    x.star=c(list(data), lapply(1:(k-1), function(i) cbind(data,pmbs[[i]][,order(id.con[[i]])])))
    Data=lapply(x.star,scale)
    if(score>(ncol(Data[[1]])+1)) warning(paste("The largest value that can be specified for 'score' is ",ncol(Data[[1]])+1,", which is what will be used for the mixturegram.",sep=""))
    score<-min(score,ncol(x.star[[2]]))
    if(score==1){
      PCA=lapply(Data,function(x) x%*%princomp(x)$loadings[,score])
    } else{
      PCA <- vector("list",k)
      PCA[[1]]=Data[[1]]%*%princomp(Data[[1]])$loadings[,1]
      PCA[2:k]=lapply(1:(k-1),function(i) Data[[(i+1)]]%*%princomp(Data[[(i+1)]])$loadings[,score])
    }
    K=lapply(2:k,function(i) kmeans(PCA[[i]],i,iter.max=iter.max,nstart=nstart))
    lim=ceiling(max(abs(unlist(PCA))))
    plot(1:k,rep(1e1000,k),ylim=c(-lim,lim),col='white',xlab='k',ylab='PC Scores',xaxt="n", ...)
    rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], col = colors()[356])
    axis(1,at=1:k,labels=1:k)  
    abline(v=1:k,col="gray80")
    mix.method <- PCA
  }else{
    ###kernel pca
    if(method=="kpca"){
      x.star=c(list(data), lapply(1:(k-1), function(i) cbind(data,pmbs[[i]][,order(id.con[[i]])])))
      Data=lapply(x.star,scale)
      if(score>(ncol(Data[[1]])+1)) warning(paste("The largest value that can be specified for 'score' is ",ncol(Data[[1]])+1,", which is what will be used for the mixturegram.",sep=""))
      score<-min(score,ncol(x.star[[2]]))
      if(score==1){
        kPCA=lapply(Data,function(x) cbind(pcv(kpca(x))[,1]))
      } else{
        kPCA <- vector("list",k)
        kPCA[[1]]=cbind(pcv(kpca(Data[[1]]))[,1])
        kPCA[2:k]=lapply(1:(k-1),function(i) cbind(pcv(kpca(Data[[i+1]]))[,score]))
      }
      K=lapply(2:k,function(i) kmeans(kPCA[[i]],i,iter.max=iter.max,nstart=nstart))
      lim=max(abs(unlist(kPCA)))+0.1
      plot(1:k,rep(1e1000,k),ylim=c(-lim,lim),col='white',xlab='k',ylab='Kernel PC Scores',xaxt="n", ...)
      rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], col = colors()[356])
      axis(1,at=1:k,labels=1:k)  
      abline(v=1:k,col="gray80")
      mix.method <- kPCA
    }else{
      if(method=="lda"){
        class=lapply(pmbs, function(post) apply(post,1,which.max))
        ldcdata = c(list(as.matrix(data) %*% ldc(data, rep(1,nrow(as.matrix(data))),score=score)),
                    lapply(class, function(class) as.matrix(data) %*% ldc(data, class,score=score)))
        K=lapply(2:k,function(i) kmeans(ldcdata[[i]],i,iter.max=iter.max,nstart=nstart))
        lim=ceiling(max(abs(unlist(ldcdata))))
        plot(1:k,rep(1e1000,k),ylim=c(-lim,lim),col='white',xlab='k',ylab='LDC Scores',xaxt="n", ...)
        rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], col = colors()[356])
        axis(1,at=1:k,labels=1:k)  
        abline(v=1:k,col="gray80")
        mix.method <- ldcdata
      }
    }
  }
  Kcol=function(x){
    temp=unlist(sapply(1:length(x$size),function(i) rep(rank(x$center)[i],x$size[i])))
    index=unlist(sapply(1:length(x$size),function(i) which(x$cluster==i)))
    K.col=replace(x$cluster,index,temp)
  }
  all.K.col=lapply(1:(k-1), function(i) Kcol(K[[i]]))
  K.col=all.K.col[[k-1]]
  n=length(K.col)
  K.centers=c(0, lapply(1:(k-1), function(i) sort(c(K[[i]]$centers))))
  if(all.n){
    sapply(1:(k-1),function(i) segments(i,mix.method[[i]],(i+1),mix.method[[i+1]],col=col.blind[K.col]))
    points(1,mean(mix.method[[1]]),pch=21,cex=1.2,bg=colors()[553],col=1)
    sapply(2:k,function(i) points(rep(i,i),sort(c(K[[i-1]]$centers)),pch=21,cex=1.2,bg=colors()[553],col=1))
  } else{
    ride.medians=sapply(1:k, function(i) unlist(by(c(mix.method[[i]]),col.blind[K.col],median)))
    prop.mad=sapply(1:k, function(i) unlist(by(c(mix.method[[i]]),col.blind[K.col],mad)))
    L1=ride.medians-prop.mad
    U1=ride.medians+prop.mad
    L2=ride.medians-2*prop.mad
    U2=ride.medians+2*prop.mad
    L3=ride.medians-3*prop.mad
    U3=ride.medians+3*prop.mad
    
    srt.colors=rownames(ride.medians)
    srt.colors1=adjustcolor(srt.colors, alpha.f = 0.7)
    srt.colors2=adjustcolor(srt.colors, alpha.f = 0.4)
    srt.colors3=adjustcolor(srt.colors, alpha.f = 0.2)
    
    
    invisible(sapply(1:k, function(i) polygon(c(1:k,k:1),c(L3[i,],rev(U3[i,])),col=srt.colors3[i], border=FALSE) ))
    invisible(sapply(1:k, function(i) polygon(c(1:k,k:1),c(L2[i,],rev(U2[i,])),col=srt.colors2[i], border=FALSE) ))
    invisible(sapply(1:k, function(i) polygon(c(1:k,k:1),c(L1[i,],rev(U1[i,])),col=srt.colors1[i], border=FALSE) ))
    invisible(sapply(1:(k-1),function(i) segments(rep(i,k),ride.medians[,i],rep((i+1),k),ride.medians[,(i+1)],col=srt.colors) ))
    points(1,mean(mix.method[[1]]),pch=21,cex=1.2,bg=colors()[553],col=1)
    invisible(sapply(2:k,function(i) points(rep(i,i),sort(c(K[[i-1]]$centers)),pch=21,cex=1.2,bg=colors()[553],col=1)))
  }
  props=c(1,sapply(1:length(K), function(i) K[[i]][[5]]/sum(unlist(K[[i]][5:6]))))
  print(list(stopping=props))
}

test.equality <- function(y, x = NULL, arbmean=TRUE, arbvar=FALSE, mu=NULL, sigma=NULL, beta=NULL, lambda=NULL,...){

if(arbmean==arbvar) stop("Change either 'arbmean' or 'arbvar'!")

if(arbmean==FALSE){
	w=1
	while(w==1){
	if(is.null(x)){
	H0=normalmixEM(x=y,arbmean=FALSE,arbvar=TRUE, mu=mu, sigma=sigma, lambda=lambda,...)
	k=length(H0$lambda)
#	H1=normalmixEM(x=y,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,mu=rep(H0$mu,k)*(1:k),sigma=(H0$scale*H0$sigma),...)
	H1=normalmixEM(x=y,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,mu=NULL,sigma=(H0$scale*H0$sigma),...)
	D=2*(H1$loglik-H0$loglik)
	df=k-1
	alpha=1-pchisq(D,df=df)
	} else{
	H0=regmixEM(y=y,x=x,arbmean=FALSE,arbvar=TRUE,beta=beta, sigma=sigma, lambda=lambda,...)
	k=length(H0$lambda)
#	H1=regmixEM(y=y,x=x,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,beta=matrix(rep(H0$beta,k),k)*(1:k),sigma=(H0$scale*H0$sigma),...)
	H1=regmixEM(y=y,x=x,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,beta=NULL,sigma=(H0$scale*H0$sigma),...)
	p=nrow(H1$beta)
	D=2*(H1$loglik-H0$loglik)
	df=p*(k-1)
	alpha=1-pchisq(D,df=df)
	}
	if(D<0){
	w=1 
	mu=NULL
	sigma=NULL
	lambda=NULL
	} else w=2 
	}
}

if(arbvar==FALSE){
	w=1
	while(w==1){
	if(is.null(x)){
	H0=normalmixEM(x=y,arbmean=TRUE,arbvar=FALSE,mu=mu, sigma=sigma, lambda=lambda,...)
	k=length(H0$lambda)
#	H1=normalmixEM(x=y,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,mu=H0$mu,sigma=rep(H0$sigma,k),...)
	H1=normalmixEM(x=y,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,mu=H0$mu,sigma=NULL,...)
	D=2*(H1$loglik-H0$loglik)
	df=k-1
	alpha=1-pchisq(D,df=df)
	} else{
	H0=regmixEM(y=y,x=x,arbmean=TRUE,arbvar=FALSE,beta=beta, sigma=sigma, lambda=lambda,...)
	k=length(H0$lambda)
#	H1=regmixEM(y=y,x=x,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,beta=H0$beta,sigma=rep(H0$sigma,k),...)
	H1=regmixEM(y=y,x=x,arbmean=TRUE,arbvar=TRUE,lambda=H0$lambda,beta=H0$beta,sigma=NULL,...)
	D=2*(H1$loglik-H0$loglik)
	df=k-1
	alpha=1-pchisq(D,df=df)
	}
	if(D<0){
	w=1 
	mu=NULL
	sigma=NULL
	lambda=NULL
	} else w=2 
	}
}
a=list(chi.sq=D, df=df, p.value=alpha)
a
}

gammamixEM <- function(x, lambda = NULL, alpha = NULL, beta = NULL, k = 2, mom.start = TRUE, fix.alpha = FALSE,  
                            epsilon = 1e-08, maxit = 1000, maxrestarts = 20, verb = FALSE){
  if(!is.null(alpha) & is.numeric(fix.alpha))
    stop(paste("Both alpha and fix.alpha cannot be numeric!", 
               "\n"))
  if(!is.null(lambda)|!is.null(beta)) k <- max(length(lambda),length(beta))
  x <- as.vector(x)
  y <- x^(1/3)
  n <- length(x)
  if(is.logical(fix.alpha)&(fix.alpha==FALSE)){
    cond <- 1 
  } else if(is.logical(fix.alpha)&(fix.alpha==TRUE)){
    cond <- 2
  } else cond <- 3 
  fn.alpha <- function(alpha,beta,z,x) log(beta)+sum(z*log(x))/sum(z)-digamma(alpha)
  fn.alpha.2 <- function(alpha,beta,z,x) (log(beta)+sum(z*log(x))/sum(z)-digamma(alpha))^2
  fn.beta <- function(z,x,alpha) sum(z)/sum(z*x)*alpha
  
  fn.alpha2 <- function(alpha,beta,z,x) sum(z%*%cbind(log(beta)))/length(x)+sum(t(z)%*%cbind(log(x)))/length(x)-digamma(alpha)
  fn.alpha2.2 <- function(alpha,beta,z,x) (sum(z%*%cbind(log(beta)))/length(x)+sum(t(z)%*%cbind(log(x)))/length(x)-digamma(alpha))^2
  
  if(mom.start){
    out <- try(normalmixEM(y,k=k,maxit=5000,epsilon=1e-5,maxrestarts=10),silent=TRUE)
    if(class(out)=="try-error"){
      tmp <- gammamix.init(x = x, lambda = lambda, alpha = alpha, 
                           beta = beta, k = k)
    } else{
      z <- out$posterior
      wt.mean <- sapply(1:k,function(i) weighted.mean(x,w=z[,i]))
      wt.var <- sapply(1:k,function(i) sum(z[,i] * (x - wt.mean[i])^2)/((n-1)*sum(z[,i])/n))
      shape.mom <- wt.mean^2/wt.var
      scale.mom <- wt.var/wt.mean
      shape.mle <- try(sapply(1:k, function(i) uniroot(fn.alpha,interval=c(0.000001,1000000),beta=1/scale.mom[i],z=z[,i],x=x)$root),silent=TRUE)    
      if(class(shape.mle)=="try-error") shape.mle <- sapply(1:k, function(i) nlminb(shape.mom[i],fn.alpha.2,lower=0,beta=1/scale.mom[i],z=z[,i],x=x)$par)    
      scale.mle <- sapply(1:k, function(i) 1/fn.beta(z=z[,i],x=x,alpha=shape.mle[i]))
      lambda.mle <- apply(z,2,mean)
      tmp <- list(lambda=lambda.mle,alpha=scale.mle,beta=shape.mle)
    }
  } else tmp <- gammamix.init(x = x, lambda = lambda, alpha = alpha, 
                              beta = beta, k = k)
  lambda.mle <- tmp$lambda
  scale.mle <- tmp$beta
  if(cond==2){
    shape.mle <- rep(mean(tmp$alpha),k)
  } else if(cond==1){
    shape.mle <- tmp$alpha
  } else shape.mle <- rep(fix.alpha,k)
  
  dens <- function(x, lambda, alpha, beta) {
    k <- length(lambda)
    temp = sapply(1:k, function(j) dgamma(x, shape = alpha[j], scale = beta[j]))
    temp = t(lambda * t(temp))
    temp
  }
  iter <- 0
  mr <- 0
  diff <- epsilon + 1
  dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
  old.obs.ll <- sum(log(apply(dens1, 1, sum)))
  ll <- old.obs.ll
  if(cond==1){
    
    while (diff > epsilon && iter < maxit) {
      old.shape.mle <- shape.mle
      old.scale.mle <- scale.mle
      old.lambda.mle <- lambda.mle
      z <- dens1/apply(dens1, 1, sum)
      #M-step
      shape.mle <- try(sapply(1:k, function(i) uniroot(fn.alpha,interval=c(0.000001,10000),beta=1/old.scale.mle[i],z=z[,i],x=x)$root),silent=TRUE)
      if(class(shape.mle)=="try-error") shape.mle <- sapply(1:k, function(i) nlminb(old.shape.mle[i],fn.alpha.2,lower=0,beta=1/scale.mle[i],z=z[,i],x=x)$par)
      scale.mle <- sapply(1:k, function(i) 1/fn.beta(z=z[,i],x=x,alpha=shape.mle[i]))
      lambda.mle <- apply(z,2,mean)
      dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
      new.obs.ll <- sum(log(apply(dens1, 1, sum)))
      diff <- new.obs.ll - old.obs.ll
      old.obs.ll <- new.obs.ll
      ll <- c(ll, old.obs.ll)
      iter = iter + 1
      if (verb) {
        cat("iteration =", iter, " log-lik diff =", diff, 
            " log-lik =", new.obs.ll, "\n")
      }
      if(is.na(diff)){
        cat("Note: Choosing new starting values.", "\n")
        if (mr == maxrestarts) 
          stop(paste("Try different number of components?", 
                     "\n"))
        mr <- mr + 1
        tmp <- gammamix.init(x = x, lambda = lambda, alpha = alpha, 
                             beta = beta, k = k)
        lambda.mle <- tmp$lambda
        if(cond==2){
          shape.mle <- rep(mean(tmp$alpha),k)
        } else if(cond==1){
          shape.mle <- tmp$alpha
        } else shape.mle <- rep(fix.alpha,k)
        scale.mle <- tmp$beta
        iter <- 0
        diff <- epsilon + 1
        dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
        old.obs.ll <- sum(log(apply(dens1, 1, sum)))
        ll <- old.obs.ll
      }
    }
  } else if(cond==2){
    
    while (diff > epsilon && iter < maxit) {
      old.shape.mle <- shape.mle
      old.scale.mle <- scale.mle
      old.lambda.mle <- lambda.mle
      z <- dens1/apply(dens1, 1, sum)
      #M-step
      shape.mle <- try(rep(uniroot(fn.alpha2,interval=c(0.000001,10000),beta=1/old.scale.mle,z=z,x=x)$root,k),silent=TRUE)
      if(class(shape.mle)=="try-error") shape.mle <- rep(nlminb(old.shape.mle[1],fn.alpha2,lower=0,beta=1/old.scale.mle,z=z,x=x)$par,k)
      scale.mle <- sapply(1:k, function(i) 1/fn.beta(z=z[,i],x=x,alpha=shape.mle[1]))
      lambda.mle <- apply(z,2,mean)
      dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
      new.obs.ll <- sum(log(apply(dens1, 1, sum)))
      diff <- new.obs.ll - old.obs.ll
      old.obs.ll <- new.obs.ll
      ll <- c(ll, old.obs.ll)
      iter = iter + 1
      if (verb) {
        cat("iteration =", iter, " log-lik diff =", diff, 
            " log-lik =", new.obs.ll, "\n")
      }
      if(is.na(diff)){
        cat("Note: Choosing new starting values.", "\n")
        if (mr == maxrestarts) 
          stop(paste("Try different number of components?", 
                     "\n"))
        mr <- mr + 1
        tmp <- gammamix.init(x = x, lambda = lambda, alpha = alpha, 
                             beta = beta, k = k)
        lambda.mle <- tmp$lambda
        if(cond==2){
          shape.mle <- rep(mean(tmp$alpha),k)
        } else if(cond==1){
          shape.mle <- tmp$alpha
        } else shape.mle <- rep(fix.alpha,k)
        scale.mle <- tmp$beta
        iter <- 0
        diff <- epsilon + 1
        dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
        old.obs.ll <- sum(log(apply(dens1, 1, sum)))
        ll <- old.obs.ll
      }
    }     
    
  } else{
    
    while (diff > epsilon && iter < maxit) {
      old.scale.mle <- scale.mle
      old.lambda.mle <- lambda.mle
      z <- dens1/apply(dens1, 1, sum)
      #M-step
      scale.mle <- sapply(1:k, function(i) 1/fn.beta(z=z[,i],x=x,alpha=shape.mle[1]))
      lambda.mle <- apply(z,2,mean)
      dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
      new.obs.ll <- sum(log(apply(dens1, 1, sum)))
      diff <- new.obs.ll - old.obs.ll
      old.obs.ll <- new.obs.ll
      ll <- c(ll, old.obs.ll)
      iter = iter + 1
      if (verb) {
        cat("iteration =", iter, " log-lik diff =", diff, 
            " log-lik =", new.obs.ll, "\n")
      }
      if(is.na(diff)){
        cat("Note: Choosing new starting values.", "\n")
        if (mr == maxrestarts) 
          stop(paste("Try different number of components or different value for the fixed shape parameter?", 
                     "\n"))
        mr <- mr + 1
        tmp <- gammamix.init(x = x, lambda = lambda, alpha = alpha, 
                             beta = beta, k = k)
        lambda.mle <- tmp$lambda
        if(cond==2){
          shape.mle <- rep(mean(tmp$alpha),k)
        } else if(cond==1){
          shape.mle <- tmp$alpha
        } else shape.mle <- rep(fix.alpha,k)
        scale.mle <- tmp$beta
        iter <- 0
        diff <- epsilon + 1
        dens1 <- dens(x=x, lambda=lambda.mle, alpha=shape.mle, beta=scale.mle)
        old.obs.ll <- sum(log(apply(dens1, 1, sum)))
        ll <- old.obs.ll
      }
    }     
  }
  if (iter == maxit) {
    cat("WARNING! NOT CONVERGENT!", "\n")
  }
  cat("number of iterations=", iter, "\n")
  theta = rbind(shape.mle, scale.mle)
  rownames(theta) = c("alpha", "beta")
  colnames(theta) = c(paste("comp", ".", 1:k, sep = ""))
  a = list(x = x, lambda = lambda.mle, gamma.pars = theta, loglik = new.obs.ll, 
           posterior = z, all.loglik = ll, ft = "gammamixEM")
  class(a) = "mixEM"
  a
}  

ddirichlet <- function (x, alpha) 
{
    if (length(x) != length(alpha))
	stop("Mismatch between dimensions of x and alpha in ddirichlet().\n")
        logD <- sum(lgamma(alpha)) - lgamma(sum(alpha))
        s <- sum((alpha - 1) * log(x))
        pd <- exp(sum(s) - logD)
	  pd[any(x < 0 | x > 1)] <- 0
	  if(sum(x) != 1) pd <- 0
	  return(pd)
}

plotly_ise.npEM <- function(npEMout, component=1, block=1, truepdf=dnorm, lower=-Inf,
                            upper=Inf, plots = TRUE ,
                            col = NULL , width = 3,
                            title = NULL , title.size = 15 , title.x = 0.5 , title.y = 0.95,
                            xlab = "t" , xlab.size = 15 , xtick.size = 15,
                            ylab = "" , ylab.size = 15 , ytick.size = 15,
                            legend.text = "" , legend.text.size = 15 , legend.size = 15, ...){
  coords <- npEMout$blockid == block
  bs <- sum(coords) # block size
  xx <- as.vector(npEMout$data[,coords]) # flatten data 
  wts <- rep(npEMout$post[,component],bs) # duplicate weights
  if (is.matrix(npEMout$bandwidth)){
    bw <- npEMout$bandwidth[block,component]
  } else {bw <- npEMout$bandwidth}
  integrand = function(u,...) {
    (wkde(xx,u,wts,bw) - truepdf(u,...))^2
  }
  numint <- integrate(integrand,lower,upper, ...)
  
  if (is.null(col)){
    col <- hue_pal()(2)
  }
  if (length(col) != 2){
    print("Please specify 2 colors in 'col'.")
  }
  
  if (plots){
    # plot of estimated and truepdf
    ise <- paste(round(numint$value,4))
    temp=paste(component, block, sep="")
    if (is.null(title)){
      title = substitute(expression(paste("Integrated Squared Error for ",
                                          f[temp]," = ",ise,sep="")))
    }
    
    if (!is.finite(lower)) {
      lower <- min(xx)
    }
    if (!is.finite(upper)) {
      upper <- max(xx)
    }    
    u <- seq(lower,upper, 0.01)
    fhat <- wkde(xx,u,wts,bw)
    ymax <- max(max(truepdf(u, ...)),max(fhat))
    
    plot <- plot_ly()%>%
      add_trace(x=u , 
                y=fhat , type = 'scatter' , mode = 'lines',
                line = list(width = (width/2) , color = col[2]),
                name = "Fitted", showlegend = TRUE)%>%
      add_trace(x=u , 
                y=truepdf(u, ...) , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = col[1]),
                name = "True", showlegend = TRUE)%>%
      plotly::layout(
        legend = list(title=list(text=legend.text,
                                 font=list(size=legend.text.size)),
                      font = list(size=legend.size)),
        title = list(text = eval(title),
                     x = title.x,
                     y = title.y,
                     font = list(size=title.size)),
        xaxis = list(title = list(text = xlab,
                                  font = list(size = xlab.size)),
                     tickfont = list(size = xtick.size)
        ),
        yaxis = list(title = list(text = ylab,
                                  font = list(size = ylab.size)),
                     tickfont = list(size = ytick.size)
        )
      )
  }
  print(plot)
  numint
}

ldc <- function(data,class,score){
  data = as.matrix(data)
  K = length(unique(class)) # The number of classes (expect the classes to be labeled 1, 2, 3, ..., K-1, K 
  N = dim(data)[1] # The number of samples
  p = dim(data)[2] # The number of features
  
  # Compute the class dependent probabilities and class dependent centroids: 
  Pi = c(table(class))/N
  M = aggregate(data,list(class),mean)[,-1]
  # Compute W:
  W = cov(data) 
  # Compute M* = M W^{-1/2} using the eigen-decomposition of W :
  e = eigen(W)
  V = e$vectors
  W_Minus_One_Half = V %*% diag( 1/sqrt(e$values),nrow=p ) %*% t(V)
  MStar = as.matrix(M) %*% W_Minus_One_Half
  # Compute B* the covariance matrix of M* and its eigen-decomposition:
  if(p>1 & length(Pi)>1){
    BStar = cov(MStar)
    e = eigen(BStar)
    VStar = e$vectors
    # 1st linear discriminant coordinate
    ldc1 = W_Minus_One_Half %*% VStar[,score]
  } else if(p==1){
    ldc1 = c(W_Minus_One_Half)
  } else ldc1 = W_Minus_One_Half[,score]
  return(ldc1)
}

ellipse <- function(mu, sigma, alpha=.05, npoints=250,
                    newplot=FALSE, draw=TRUE, ...) {
  es <- eigen(sigma)
  e1 <- es$vec%*%diag(sqrt(es$val))
  r1 <- sqrt(qchisq(1-alpha,2))
  theta <- seq(0,2*pi,len=npoints)
  v1 <- cbind(r1*cos(theta),r1*sin(theta))
  pts=t(mu-(e1%*%t(v1)))
  if (newplot && draw) {
    plot(pts, ...)
  } else if (!newplot && draw) {
    lines(pts, ...)
  }
  invisible(pts)
}

  

# code for Reliability Mixture Models (RMM) with Censored data
# D. Chauveau
# ref: Bordes L. and Chauveau D. Computational Statistics (2016)

# Simulate from a Weibull mixture
# lambda = vector of component probabilities
#	shape, scale = vector of component rates
rweibullmix <- function(n,lambda=1, shape=1, scale=1) {
	m <- length(lambda) # nb of components
	z <- sample(m, n, replace=TRUE, prob=lambda) # component indicator
	rweibull(n, shape = shape[z], scale=scale[z])
	}


########################################################################
## Stochastic EM algorithm for Reliability Mixture Models (RMM) 
## with Censoring; Parametric model, for 
## univariate finite mixture of Weibull with right censoring
# x = lifetime data, censored by random c if d is not NULL, in which case
# x= min(x,c) and d = I(x <= c)
# uses parametric MLE for censored weibull data from the survival package
# caution: when fitted by a survreg object, the weibull parametrization is
#			shape=1/fit$scale and scale=exp(fit$coeff)
# averaged = TRUE if averaging of the parameters is computed at each iteration
#			 (cf Nielsen 2000)
# 	NB: averaging can be done in several ways; 
#	DEPRECATED: the current theta for E & M steps is the average over the sequence,
#	but the theta^next not averaged is stored on the sequence 
# 	CURRENT VERSION: the average is stored and used for next step
#
# About calls of survreg() from the survival package:
# 	maxit.survreg is
#	passed to survreg.control() for weibull MLE using survreg()

weibullRMM_SEM <- function (x, d=NULL, lambda = NULL, 
			shape = NULL, scale = NULL,  # dweibull() parameters
			k = 2, # default nb of components
          	maxit = 200, # maxrestarts = 20, 
          	maxit.survreg = 200, epsilon = 1e-03,
          	averaged = TRUE, verb = FALSE) {
#  warn <- options(warn=-1) # Turn off warnings
#  x <- as.vector(x)
#  require(survival)
	n <- length(x)
	if (!is.null(lambda)) k=length(lambda)
	if (!is.null(scale)) k=length(scale) # at least one
	if (!is.null(shape)) k=length(shape) # should be define !!!

	if (is.null(d))  d <- rep(1,n)  # noncensored case forced
	xx <- matrix(x, nrow=n, ncol=k) # x repeated k times, used in E-steps
	dd <- matrix(d, nrow=n, ncol=k) # idem for d
#
# note: d can be used instead of dd since automatic recycling gives
# dd*xx = d*xx, but we keep dd to avoid eventual futur change
	
# init function call to do later, in case lambda = rate = NULL  
	if (is.null(lambda)) lambda <- rep(1/k,k)
	if (is.null(scale)) scale <- rep(1,k)  # to do : init.functions(k)
	if (is.null(shape)) shape <- rep(1,k)  # 
# sequences for storing along iterations 	
	lambda_seq <- scale_seq <- shape_seq <- matrix(0, nrow = maxit, ncol = k)
 	lambda_seq[1,] <- lambda
 	scale_seq[1,] <- scale;  shape_seq[1,] <- shape
	loglik_seq <- NULL
	oldloglik <- -Inf
#    notdone <- TRUE # for handling restarts etc, ToDo later!
#    while(notdone) {
      # Initialize everything
      notdone <- FALSE
      # dll <- epsilon+1
      iter <- 1
      post <- z <- sumpost <- matrix(0, nrow = n, ncol = k)
      new_scale <- new_shape <- rep(0,k)
	  
	  ##### SEM iterations #####
      while (iter < maxit) { # SEM version
      	
		### E-step ###
		scn <- matrix(scale, n, k, byrow=T) # for vectorized post comput.
		shn <- matrix(shape, n, k, byrow=T)
		ll <- matrix(lambda, n, k, byrow=T)
		# handling censored & non-censored cases
		post <- ((ll*dweibull(xx,shn,scn))^dd)*((ll*(1-pweibull(xx,shn,scn)))^(1-dd))
		rs <- rowSums(post)
		loglik <- sum(log(rs)) # loglik without the constant term related to h pdf
		# post normalized per row
		post <- sweep(post, 1, rs, "/") # posteriors p_{ij}^t 's
		# check and solve NaN's: may cause theoretical pbs!?
		snans <- sum(is.na(post))
		if (snans > 0) {
			post[is.na(post[,1]),] <- 1/k
			cat("warning:",snans, "NaN's in post\n")
			}

		### S-step ###	
		# ~ matrix of component indicators simu checked ?
		z <- t(apply(post, 1, function(prob) rmultinom(1, 1, prob))) 
		nsets <- colSums(z) # subsets per component sizes
		
		# cat("it",iter,": sets=",nsets,"\n")
		
		### M-step ###
		# new_parameter = SEM(lambda,shape,scale) 
		new_lambda <- nsets/n # or colMeans(post) if EM version preferred
		for (j in 1:k) { # for each component; vectorize later?
			tj <- x[z[,j]==1] # subsample from component j
			dj <- d[z[,j]==1] # associated event indicator
			# passing maxit and epsilon parameters to survreg
			# and current shape & scale as init parameters
			fit=survreg(Surv(tj,dj)~1, dist = 'weibull',
						control = survreg.control(maxiter = maxit.survreg, 
											rel.tolerance=epsilon),
						init.beta=log(scale), init.scale=1/shape)
			new_scale[j] <- exp(fit$coeff)
			new_shape[j] <- 1/fit$scale
			}
		
		# Next parameter value, depending on average strategy
		if (averaged) {
			scale <- (new_scale + iter*scale_seq[iter,])/(iter+1)
			shape <- (new_shape + iter*shape_seq[iter,])/(iter+1)
			lambda <- (new_lambda + iter*lambda_seq[iter,])/(iter+1)
			
			} else {	# averaged=FALSE case, just use last update 
				scale <- new_scale
				shape <- new_shape
				lambda <- new_lambda
				} 		
		# new strategy = storing sequence of averages
		lambda_seq[iter+1, ] <- lambda
		scale_seq[iter+1, ] <- scale
		shape_seq[iter+1, ] <- shape

        dll <- loglik - oldloglik # = Inf for iter=0 1st time
        oldloglik <- loglik
        loglik_seq <- c(loglik_seq, loglik)
        if (verb) {
          cat("iteration", iter, " log-lik diff =", dll, " log-lik =", 
              loglik, "\n")
#          print(rbind(lambda, rate))		 
        }
        iter <- iter + 1
      }  # end of SEM loops over iterations
        
#    } # while notdone, if restarts control implemented
        
    # final estimates depending on average strategy
    if (averaged) {
    	final.lambda <- lambda
    	final.scale <- scale
    	final.shape <- shape
    	} else {
    		final.scale <- colMeans(scale_seq[1:iter,])
			final.shape <- colMeans(shape_seq[1:iter,])
			final.lambda <- colMeans(lambda_seq[1:iter,])
    		}
    
    cat("number of iterations =", iter, "\n")
	colnames(post) <- c(paste("comp", ".", 1:k, sep = ""))
    a=list(x=x, d=d, 
    		lambda = final.lambda, 
    		scale = final.scale, shape = final.shape,
    		loglik = loglik, 
            posterior = post, all.loglik=loglik_seq, 
            all.lambda = lambda_seq[1:iter,],
            all.scale = scale_seq[1:iter,],
            all.shape = shape_seq[1:iter,],
            ft="weibullRMM_SEM")
	class(a) = "mixEM"
  a
}



##################################################
# plot SEM sequences from weibullRMM_SEM:
# color by components, one plot per parameter type
plotweibullRMM <- function(a, title=NULL, rowstyle=TRUE, subtitle=NULL,...)
	{
	n <- length(a$x); m <- dim(a$all.lambda)[2]
	pcc <- round(100*(1-mean(a$d)),2)
	if (is.null(subtitle)) {
				subtitle <- paste("n=",n,", ", pcc, "% censored", sep="")}
	if (is.null(title)) {
		tt1 <- "Shape parameters"; tt2 <- "Scale parameters" 
		tt3 <- "Weight parameters"
		} else tt1 <- tt2 <- tt3 <- title
lwdset <- 2
if (rowstyle) par(mfrow=c(3,1)) else par(mfrow=c(1,3))

plot(a$all.shape[,1], type="l", ylim=c(0,max(a$all.shape)), 
	xlab="iterations", ylab="estimates", main=tt1, ...)
# if (truevalues) abline(shape[1],0,lty=3)
title(sub=subtitle, cex.sub = 0.75)
lgd <- expression(sh[1]); lcol <- 1
for (j in 2:m) {
	lines(a$all.shape[,j], col=j, ...)
	# if (truevalues) abline(shape[j],0,col=j,lty=3)
	lgd <- c(lgd,substitute(sh[j])); lcol <- c(lcol,j)
	}
legend("topright", lgd, col=lcol, lty=1,...)

plot(a$all.scale[,1], type="l", ylim=c(0,max(a$all.scale)), 
	xlab="iterations", ylab="estimates", main=tt2, ...)
# if (truevalues) abline(scale[1],0,lty=3)
title(sub=subtitle, cex.sub = 0.75)
lgd <- expression(sc[1]); lcol <- 1
for (j in 2:m) {
	lines(a$all.scale[,j], col=j, ...)
	# if (truevalues) abline(scale[j],0,col=j,lty=3)
	lgd <- c(lgd,substitute(sc[j])); lcol <- c(lcol,j)
	}
legend("topright", lgd, col=lcol, lty=1, ...)

plot(a$all.lambda[,1], type="l", ylim=c(0,1), 
	xlab="iterations", ylab="estimates", main=tt3, ...)
# if (truevalues) abline(lambda[1],0,lty=3)
title(sub=subtitle,cex.sub = 0.75)
lgd <- expression(lambda[1]); lcol <- 1
for (j in 2:m) {
		lines(a$all.lambda[,j], col=j, ...)
		# if (truevalues) abline(lambda[j],0,col=j,lty=3)
		lgd <- c(lgd,substitute(lambda[j]))
		lcol <- c(lcol,j)
		}
legend("topright", lgd, col=lcol, lty=1, ...)
	}

# Produce credible region for regression lines based on
# sample from posterior distribution of beta parameters.
# It is assumed that beta, upon entry, is an nx2 matrix,
# where the first column gives the intercepts and the
# second column gives the slopes.
# alpha is the proportion of beta to remove from the posterior.
# Thus, 1-alpha is the level of the credible region.
#
# If nonparametric=TRUE, then the region is based on the convex
# hull of the remaining beta after trimming, which is accomplished
# using a data depth technique.
#
# If nonparametric=FALSE, then the region is based on the
# asymptotic normal approximation.

regcr=function (beta, x, em.beta=NULL, em.sigma=NULL, alpha = 0.05, nonparametric = FALSE, plot = FALSE, 
    xyaxes = TRUE, ...) 
{
    if (nonparametric) {
	  beta.old=beta
	  if(is.null(em.sigma) && is.null(em.beta)){
	  beta=t(matsqrt(solve(cov(beta)))%*%(t(beta)-apply(beta,2,mean)))
	  } else beta=1/sqrt(length(x))*t(em.sigma^(-1)*matsqrt(t(cbind(1,x))%*%cbind(1,x))%*%(t(beta.old)-em.beta))
        d = depth(beta, beta)
        beta = beta.old[order(d), ]
        d = d[order(d)]
        n = length(d)
        trimbeta = beta[-(1:round(n * alpha)), ]
        h = unique(trimbeta[chull(trimbeta), ])
        nh = nrow(h)
        m = which.max(h[, 2])
        h = rbind(h[m:nh, ], h[((1:nh) < m), ], h[m, ])
        bound = NULL
        for (i in 1:nh) {
            bound = rbind(bound, h[i, ])
            bound = rbind(bound, cbind(seq(h[i, 1], h[i + 1, 
                1], len = 50), seq(h[i, 2], h[i + 1, 2], len = 50)))
        }
	beta=trimbeta
    }
    else {
        xbar = apply(beta, 2, mean)
        n = nrow(beta)
        cbeta = t(t(beta) - xbar)
        S = t(cbeta) %*% cbeta/(n - 1)
        eS = eigen(S)
        B = eS$vec %*% diag(sqrt(eS$val))
        theta = seq(0, 2 * pi, len = 250)
        v = cbind(cos(theta), sin(theta)) * sqrt(qchisq(1 - alpha, 
            2))
        h = t(B %*% t(v) + xbar)
        nh = nrow(h)
        m = which.max(h[, 2])
        h = rbind(h[m:nh, ], h[((1:nh) < m), ], h[m, ])
        bound = h
    }
    z <- length(x) * 5
    u <- seq(min(x), max(x), length = z)
    lower <- c()
    upper <- c()
    v <- c()
    for (j in 1:z) {
        for (i in 1:nrow(beta)) {
            v[i] <- as.matrix(beta[, 1][i] + beta[, 2][i] * u[j])
        }
        uv <- cbind(u[j], v)
        lower <- rbind(lower, uv[order(v), ][1, ])
        upper <- rbind(upper, uv[order(v), ][nrow(beta), ])
    }
    if (plot) {
        if (xyaxes) {
            lines(upper, ...)
            lines(lower, ...)
        }
        else {
            lines(bound, ...)
        }
    }
    invisible(list(boundary = bound, upper = upper, lower = lower))
}

# Note:  normmixrm.sim is here for backwards compatibility
rmvnormmix <- normmixrm.sim <- function(n,lambda=1,mu=0,sigma=1) {
  m <- length(lambda) # nb of components
  mu <- matrix(mu, nrow=m)
  sigma <- matrix(sigma, nrow=m)
  if ((r <- NCOL(mu)) != NCOL(sigma)) {
    stop("mu and sigma must have the same number of columns", call.=FALSE)
  }
  z <- sample(m,n,replace=TRUE,prob=lambda) # component 
  matrix(rnorm(n*r,mean=as.vector(mu[z,]),sd=as.vector(sigma[z,])),n,r)  
}

#################################################################
## FUNCTIONS FOR FALSE DISCOVERY RATE (FDR) ESTIMATION         ##
## USING SEMI-PARAMETRIC EM-LIKE ALGORITHMS                    ##
## D. CHAUVEAU                                                 ##
## mixtools 1.0 addition                                       ##
#################################################################


#################################################################
## EM-like algorithm for a nonparametric univariate mixture model
## - Component 1 known = N(0,1) 
##   = the pdf of a probit transform of pvalue under H0
## - component 2 symmetric shifted from a location parameter mu
## NB: stochastic=TRUE not implemented, parameter removed here

spEMsymlocN01 <- function(x, mu0=2, bw = bw.nrd0(x), h=bw, eps = 1e-8,
						maxiter=100, verbose = FALSE, plotf=FALSE){
  bw <- h # h is alternative bandwidth argument, for backward compatibility
  n <- length(x)
#  if (length(mu0)>1) m <- length(mu0) else m <- mu0 
  m <- 2 # fixed number of components in this model
  z.hat <- matrix(0, nrow=n, ncol=m)
  fkernel <- matrix(0, nrow=n, ncol=m)
  tt0 <- proc.time()
  o <- order(x) # for plotting fhat's if requested
  kmeans <- kmeans(x, mu0)  # is this a good init for probit data?
  for(j in 1:m) {
    z.hat[kmeans$cluster==j, j] <- 1
  }
  iter <- 0
  finished <- FALSE
  lambda <- matrix(0,maxiter,m)
  mu <- rep(0,maxiter) # only component 2 mean used in this case
  while (!finished) {
  #while (max(abs(change)) > eps & iter < maxiter) {
    iter <- iter + 1
    t0 <- proc.time()

    ## M-Step
    lambda[iter,] <- colMeans(z.hat) 
    # mu[iter,] <- apply(sweep(z.hat, 1, x, "*"), 2, mean)/lambda[iter,]
    mu[iter] <- sum(x*z.hat[,2])/(n*lambda[iter,2]) #

    ## second component density estimation step evaluated at x_i-mu's  
      ans <- .C(C_KDEsymloc1comp, n=as.integer(n),
                mu=as.double(mu[iter]), lbd2=as.double(lambda[iter,2]),
                x=as.double(x), bw=as.double(bw),
                z=as.double(z.hat), f = double(n)) 
                #add PACKAGE="mixtools" option when integrated

    # successive plots of fhat's (for debugging mostly)
    if (plotf) {
   		if (iter==1) plotfunc <- plot else plotfunc <- lines 
    	plotfunc(x[o],ans$f[o],type="l", col=iter) 
    	}
    
    # version lambda_j f_j(xi) specific for component one = N(0,1)
    # NB: this is the only place where the known component pdf is used
    lambda.f <- cbind(lambda[iter,1]*dnorm(x), lambda[iter,2]*ans$f)

    ## E-step (for next iteration)
    z.hat <- lambda.f/rowSums(lambda.f)    
    finished <- iter >= maxiter
    if (iter>1) { # This convergence criterion is too simplistic:
      change <- c(lambda[iter,] - lambda[iter-1,], mu[iter]-mu[iter-1])
      finished <- finished | (max(abs(change)) < eps)
    }
    if (verbose) {
      t1 <- proc.time()
      cat("iteration ", iter, "  lambda ", round(lambda[iter,], 4), 
          "  mu ", round(mu[iter], 4))
      cat(" time", (t1 - t0)[3], "\n")
    }
  }
  if (verbose) {
    tt1 <- proc.time()
    cat("lambda ", round(lambda[iter,], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")
  }
    return(structure(list(data=x, posteriors=z.hat, lambda=lambda[1:iter,],
                          bandwidth=bw, lambdahat=lambda[iter,], 
                          mu = mu[1:iter], muhat = mu[iter], symmetric=TRUE),
                          class="spEMN01"))
}





	
#######################################################
# plot mixture pdf for the semiparametric mixture model
# with component 1 pdf passed in the knownpdf parameter
# uses a weighted kernel density estimate of the nonparametric component 2 
# a = object of class "spEMN01" as returned by spEMsymlocNorm
plot.spEMN01 <- function(x, bw=x$bandwidth, knownpdf=dnorm, add.plot=FALSE, ...) {	
	t <- seq(min(x$data), max(x$data), len=200)
	f1 <- x$lambdahat[1]*knownpdf(t)
	f2 <- x$lambdahat[2]*wkde(x$data-x$muhat, u=t-x$muhat, w=x$post[,2], bw=bw, sym=TRUE)
	f <- f1+f2
	if (!add.plot) plot(t,f1+f2, type="l", ...) else lines(t,f1+f2, ...)
	lines(t,f1, col=2); lines(t,f2, col=3)
	}



####################################################
# plot and compare FDR for 1 or 2 EM-like strategies
# for mixtools 1.0
# post NEEDS to be sorted by p	
plotFDR <- function(post1, post2=NULL, lg1="FDR 1", lg2=NULL, title=NULL, 
					compH0=1, alpha=0.1, complete.data =NULL, pctfdr=0.3) {
	n <- dim(post1)[1]
	cs1 <- cumsum(post1[,compH0]) # local FDR(p_i)'s
	fdr1 <- cs1/(1:n) # FDR(p_i)'s
	if (is.null(title)) title <- paste("FDR estimate(s), n=",n)
	if (!is.null(post2)) {
			cs2 <- cumsum(post2[,compH0]) # local FDR(p_i)'s
			fdr2 <- cs2/(1:n)
			if (is.null(lg2)) lg2 <- "FDR 2"
			}
	i1 <- sum(fdr1<pctfdr) # plot up to pctfdr % FDR
	if (i1 == 0) i1 <- n   # for very bad fit, fdr[1] > pctfdr
	# cat("index",i1)
	plot(fdr1[1:i1], type="l", main=title, col=1, ylim=c(0,fdr1[i1]),
			xlab="index", ylab="probability")
	if (!is.null(post2)) lines(fdr2[1:i1], col=2)
	abline(alpha, 0, lty=3)
	if (!is.null(complete.data)) { # true complete data available
		V <- cumsum(complete.data[,1]==1) # cumulative nb of items under H0 
		trueFDR <- V/(1:n)
		lines(trueFDR[1:i1], lty=2, col=3)	
		if (!is.null(post2)) 
			legend("topleft", c(lg1,lg2,"True FDR"),col=1:3, lty=c(1,1,2))
		if (is.null(post2)) 
			legend("topleft", c(lg1,"True FDR"),col=c(1,3), lty=c(1,2))
				} else {
			if (!is.null(post2)) 
				legend("topleft", c(lg1,lg2), col=1:2, lty=c(1,1))
			if (is.null(post2)) 
				legend("topleft", lg1, col=1, lty=1)
			}
	}

ise.npEM <- function(npEMout, component=1, block=1, truepdf, lower=-Inf,
                     upper=Inf, plots = TRUE, ...){
	# returns the Integrated Squared Error
	# between f_{comp,block} as estimated by npEM,
	# and truepdf
  #true2 <- function(u) truepdf(u, ...)
	coords <- npEMout$blockid == block
	bs <- sum(coords) # block size
	xx <- as.vector(npEMout$data[,coords]) # flatten data 
	wts <- rep(npEMout$post[,component],bs) # duplicate weights
	if (is.matrix(npEMout$bandwidth)){
		bw <- npEMout$bandwidth[block,component]
		}
		else bw <- npEMout$bandwidth
	integrand = function(u,...) {
    (wkde(xx,u,wts,bw) - truepdf(u,...))^2
  }
	numint <- integrate(integrand,lower,upper, ...)
	if (plots) {
    # plot of estimated and truepdf
    ise <- paste(round(numint$value,4))
    temp=paste(component, block, sep="")
    title = substitute(expression(paste("Integrated Squared Error for ",
                                        f[temp]," = ",ise,sep="")))
    if (!is.finite(lower)) {
      lower <- min(xx)
    }
    if (!is.finite(upper)) {
      upper <- max(xx)
    }    
    u <- seq(lower,upper, 0.01)
    fhat <- wkde(xx,u,wts,bw)
    ymax <- max(max(truepdf(u, ...)),max(fhat))
    plot(u,fhat,type="l",ylim=c(0,ymax),
         main=eval(title),ylab="")
    legend("topleft",legend=c("true","fitted"),col=c(2,1),lty=c(1,1),lwd=c(2,1))
    lines(u,truepdf(u, ...),lwd=2,col=2)
	}
	numint
}

plotly_mixturegram <- function(
  data, pmbs, method=c("pca","kpca","lda"), 
  all.n=FALSE, id.con=NULL, score=1, iter.max=50, nstart=25,
  xlab = "K" , xlab.size = 15 , xtick.size = 15,
  ylab = NULL , ylab.size = 15 , ytick.size = 15,
  cex = 12 , col.dot = "red" , width = 1 ,
  title = "Mixturegram" , title.size = 15 , title.x = 0.5 , title.y = 0.95
){
  vline <- function(x = 0, color = '#1f77b4') {
    list(
      type = "line",
      y0 = 0,
      y1 = 1,
      yref = "paper",
      x0 = x,
      x1 = x,
      line = list(color = '#1f77b4',
                  dash = "dash",
                  width = 1)
    )
  }
  
  col.blind=rep(c("#000000", "#E69F00", "#56B4E9", "#009E73", 
                  "#F0E442", "#0072B2", "#D55E00", "#CC79A7",
                  "#7FFF00","#7D26CD"),100)
  
  method <- match.arg(method)
  
  if(is.null(ylab)){
    if (method == "pca"){ylab <- "PC Scores"}
    else if (method == "kpca"){ylab <- "Kernel PC Scores"}
    else if (method == "lda"){ylab <- "LDC Scores"}
  }
  
  k=length(pmbs)+1
  if (is.null(id.con)) {id.con=lapply(pmbs,function(x) data%*%x/apply(x,2,sum))}
  ### Find ylim: lim ###
  ### Regular PCA ###
  if(method=="pca"){
    x.star=c(list(data), lapply(1:(k-1), function(i) cbind(data,pmbs[[i]][,order(id.con[[i]])])))
    Data=lapply(x.star,scale)
    if(score>(ncol(Data[[1]])+1)) {
      warning(paste("The largest value that can be specified for 'score' is ",
                    ncol(Data[[1]])+1,
                    ", which is what will be used for the mixturegram.",sep=""))
    }
    score<-min(score,ncol(x.star[[2]]))
    if(score==1){
      PCA=lapply(Data,function(x) x%*%princomp(x)$loadings[,score])
    } else{
      PCA <- vector("list",k)
      PCA[[1]]=Data[[1]]%*%princomp(Data[[1]])$loadings[,1]
      PCA[2:k]=lapply(1:(k-1),function(i) Data[[(i+1)]]%*%princomp(Data[[(i+1)]])$loadings[,score])
    }
    K=lapply(2:k,function(i) kmeans(PCA[[i]],i,iter.max=iter.max,nstart=nstart))
    lim=ceiling(max(abs(unlist(PCA))))
    mix.method <- PCA
  }else{
    ### Kernel PCA ###
    if(method=="kpca"){
      x.star=c(list(data), lapply(1:(k-1), function(i) cbind(data,pmbs[[i]][,order(id.con[[i]])])))
      Data=lapply(x.star,scale)
      if(score>(ncol(Data[[1]])+1)) warning(paste("The largest value that can be specified for 'score' is ",ncol(Data[[1]])+1,", which is what will be used for the mixturegram.",sep=""))
      score<-min(score,ncol(x.star[[2]]))
      if(score==1){
        kPCA=lapply(Data,function(x) cbind(pcv(kpca(x))[,1]))
      } else{
        kPCA <- vector("list",k)
        kPCA[[1]]=cbind(pcv(kpca(Data[[1]]))[,1])
        kPCA[2:k]=lapply(1:(k-1),function(i) cbind(pcv(kpca(Data[[i+1]]))[,score]))
      }
      K=lapply(2:k,function(i) kmeans(kPCA[[i]],i,iter.max=iter.max,nstart=nstart))
      lim=max(abs(unlist(kPCA)))+0.1
      mix.method <- kPCA
    } else if(method=="lda"){
      class=lapply(pmbs, function(post) apply(post,1,which.max))
      ldcdata = c(list(as.matrix(data) %*% ldc(data, rep(1,nrow(as.matrix(data))),score=score)),
                  lapply(class, function(class) as.matrix(data) %*% ldc(data, class,score=score)))
      K=lapply(2:k,function(i) kmeans(ldcdata[[i]],i,iter.max=iter.max,nstart=nstart))
      lim=ceiling(max(abs(unlist(ldcdata))))
      mix.method <- ldcdata
    }
  }
  plot <- plot_ly()%>%
    plotly::layout(
      title = list(text = title,
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size),
                   range = c(0.25,k+0.25),
                   dtick = 1, 
                   tick0 = 1, 
                   tickmode = "linear"
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size),
                   range = c(-lim, lim)
      ),
      shapes = lapply(1:k , function(i){vline(i)})
    )
  
  Kcol=function(x){
    temp=unlist(sapply(1:length(x$size),function(i) rep(rank(x$center)[i],x$size[i])))
    index=unlist(sapply(1:length(x$size),function(i) which(x$cluster==i)))
    K.col=replace(x$cluster,index,temp)
  }
  all.K.col=lapply(1:(k-1), function(i) Kcol(K[[i]]))
  K.col=all.K.col[[k-1]]
  n=length(K.col)
  K.centers=c(0, lapply(1:(k-1), function(i) sort(c(K[[i]]$centers))))
  
  if (all.n){
    for (i in 1:(k-1)){
      for (j in 1:length(mix.method[[i]])){
        plot <- plot%>%
          add_trace(x=c(i , i+1) , 
                    y=c(mix.method[[i]][j] , mix.method[[i+1]][j]) , type = 'scatter' , mode = 'lines',
                    line = list(width = width , color = col.blind[K.col][j]), 
                    name = paste("Method" , i),showlegend = FALSE) 
      }
    }
    for (q in 2:k){
      plot <- plot%>%
        add_trace(x=rep(q,q), 
                  y=sort(c(K[[q-1]]$centers)) , type = 'scatter' , mode = 'markers',
                  marker = list(size = cex , color = col.dot),
                  name = paste("Cluster" , q), showlegend = FALSE)
    }
    plot <- plot%>%
      add_trace(x=1, 
                y=mean(mix.method[[1]]) , type = 'scatter' , mode = 'markers',
                marker = list(size = cex , color = col.dot),
                name = "Cluster 1",showlegend = FALSE)
  } else{
    ride.medians=sapply(1:k, function(i) unlist(by(c(mix.method[[i]]),col.blind[K.col],median)))
    prop.mad=sapply(1:k, function(i) unlist(by(c(mix.method[[i]]),col.blind[K.col],mad)))
    L1=ride.medians-prop.mad
    U1=ride.medians+prop.mad
    L2=ride.medians-2*prop.mad
    U2=ride.medians+2*prop.mad
    L3=ride.medians-3*prop.mad
    U3=ride.medians+3*prop.mad
    
    srt.colors=rownames(ride.medians)
    srt.colors1=adjustcolor(srt.colors, alpha.f = 0.7)
    srt.colors2=adjustcolor(srt.colors, alpha.f = 0.4)
    srt.colors3=adjustcolor(srt.colors, alpha.f = 0.2)
    
    for (l in 1:k){
      plot <- plot%>%
        add_trace(
          x = c(1:k,k:1),
          y = c(L3[l,],rev(U3[l,])),
          type = 'scatter', mode = "lines",
          fill = 'tozeroy', ## fill type
          fillcolor = srt.colors3[l],
          hoveron = 'points+fills',
          line = list(
            color = srt.colors3[l]
          ),
          text = "Points + Fills",
          hoverinfo = 'text',
          showlegend = FALSE
        )%>%
        add_trace(
          x = c(1:k,k:1),
          y = c(L2[l,],rev(U2[l,])),
          type = 'scatter', mode = "lines",
          fill = 'tozeroy', ## fill type
          fillcolor = srt.colors2[l],
          hoveron = 'points+fills',
          line = list(
            color = srt.colors2[l]
          ),
          text = "Points + Fills",
          hoverinfo = 'text',
          showlegend = FALSE
        )%>%
        add_trace(
          x = c(1:k,k:1),
          y = c(L1[l,],rev(U1[l,])),
          type = 'scatter', mode = "lines",
          fill = 'tozeroy', ## fill type
          fillcolor = srt.colors1[l],
          hoveron = 'points+fills',
          line = list(
            color = srt.colors1[l]
          ),
          text = "Points + Fills",
          hoverinfo = 'text',
          showlegend = FALSE
        )
    }
    # for (h in 1:(k-1)){
    #   plot <- plot%>%
    #     add_trace(x=c(rep(h,k),rep((h+1),k)) , 
    #               y=c(ride.medians[,h] , ride.medians[,(h+1)]) , type = 'scatter' , mode = 'lines',
    #               line = list(width = width , color = srt.colors), 
    #               showlegend = FALSE) 
    # }
    for (j in 2:k){
      plot <- plot%>%
        add_trace(x=rep(j,j), 
                  y=sort(c(K[[j-1]]$centers)) , type = 'scatter' , mode = 'markers',
                  marker = list(size = cex , color = col.dot),
                  name = paste("Cluster" , j),showlegend = FALSE)
    }
    plot <- plot%>%
      add_trace(x=1, 
                y=mean(mix.method[[1]]) , type = 'scatter' , mode = 'markers',
                marker = list(size = cex , color = col.dot),
                name = "Cluster 1",showlegend = FALSE)
  }
  print(plot)
  props=c(1,sapply(1:length(K), function(i) K[[i]][[5]]/sum(unlist(K[[i]][5:6]))))
  print(list(stopping=props))
}

aug.x <- function(X,cp.locs,cp,delta = NULL){
X=as.matrix(X)
if(is.null(delta)) delta=cp.locs
if(length(cp.locs)!=length(delta) | length(cp.locs)!=sum(cp)){
	stop(paste("You must specify a correct changepoint structure!", 
		"\n"))
}
if(sum(cp)==0){
	X
} else{
new.x=NULL
sum.cp=cumsum(cp)
for(i in 1:length(cp)){
	X.i=matrix(X[,i],ncol=max(1,cp[i]),nrow=length(X[,1]))
	if(cp[i]!=0){
		CPs=cp.locs[(max(1,sum.cp[i-1]+1)):sum.cp[i]]
		SPs=delta[(max(1,sum.cp[i-1]+1)):sum.cp[i]]
		new.x=cbind(new.x,t(t(X.i)-SPs)*t(t(X.i)>CPs))
	}
}
cbind(X,new.x)
}
}

## Use an ECM algorithm (in the sense of Meng and Rubin, Biometrika 1993)
## to search for a local maximum of the likelihood surface for a 
## univariate finite mixture of normals with possible equality
## constraints on the stdev parameters.

## It is assumed here that there are three components and the three normal means
## are equal to alpha, alpha-delta, and alpha+delta for unknown parameters
## alpha and delta.
## In other words, this function implements the specific model described in 
## Thomas et al (2009), Extensions of Reliability Theory.
## It is a modified version of normalmixEM.

tauequivnormalmixEM <-
function (x, lambda = NULL, mu = NULL, sigma = NULL, k = 3, 
          mean.constr = NULL, sd.constr = NULL, gparam = NULL,
          epsilon = 1e-08, maxit = 10000, maxrestarts=20, 
          verb = FALSE, fast=FALSE, ECM = TRUE,
          arbmean = TRUE, arbvar = TRUE) {
  
  M <- A <- NULL
  if (is.null(mean.constr)) {
  	  # In this case, we will be fitting a 3-component mixture model with means
  	  # constrained to be alpha, alpha-delta, and alpha+delta for 
  	  # parameters alpha and delta.
  	  k <- 3
  	  if (length(mu) != 3) mu <- NULL
  	  if (length(sigma) != 3) sigma <- NULL
  	  M <- matrix(c(1, 1, 1, 0, -1, 1), 3, 2)
  	  
  	  # We will also constain the reciprocals of the variances to be 
  	  # gamma_1+gamma_2, gamma_1, and gamma_1 for positive
  	  # parameters gamma_1 and gamma_2.
  	  A <- matrix(c(1, 1, 1, 1, 0, 0), 3, 2)
  }
  normalmixMMlc(x, 
    lambda = lambda,
    mu = mu,
    sigma = sigma,
    k = k,
    mean.constr = mean.constr,
    mean.lincstr = M,
    var.lincstr = A,
    gparam = gparam,
    epsilon = epsilon,
    maxit = maxit, 
    maxrestarts = maxrestarts,
    verb = verb)
}

regmixMH=function (y, x, lambda = NULL, beta = NULL, s = NULL, k = 2, 
    addintercept = TRUE, mu = NULL, sig = NULL, lam.hyp = NULL, sampsize = 1000, 
    omega = 0.01, thin = 1) 
{
    if (addintercept) {
        x = cbind(1, x)
    }
    XTX <- solve(t(x)%*%x) 
    n <- length(y)
    p <- ncol(x)
    if (is.null(s)) {
        s = sqrt(1/rexp(k))
    }
    else k = length(s)
    if (is.null(beta)) {
        beta = matrix(rnorm(p * k), p, k)
    }
    else k = ncol(beta)
    if (is.null(lambda)) {
        lambda = runif(k)
        lambda = lambda/sum(lambda)
    }
    else k = length(lambda)
    if (is.null(mu)) {
        mu = 0 * beta
    }
    if (is.null(sig)) {
        sig = rep(5 * sqrt(var(y)),k)
    }
    sig.beta = t(matrix(rep(sig,p),ncol=p)) * sqrt(matrix(rep(diag(XTX),k),ncol=k)) 
    if(is.null(lam.hyp)) lam.hyp = rep(1,k)
    L.theta <- matrix(nrow = n, ncol = k)
    pi.beta <- matrix(nrow = p, ncol = k)
    pi.sigma <- c()
    pi.lambda <- c()
    new.L.theta <- matrix(nrow = length(y), ncol = k)
    new.pi.beta <- matrix(nrow = p, ncol = k)
    new.pi.sigma <- c()
    new.pi.lambda <- c()
    accepts = 0
    theta <- matrix(c(beta, s, lambda), nrow = 1)
    thetalist <- matrix(theta, sampsize, ncol(theta), byrow=TRUE)
    for (i in 2:sampsize) {
        log.pi.beta <- dnorm(beta, mu, sig.beta, log=TRUE)
        log.pi.sigma <- dexp(s, 1/sig, log=TRUE)
        log.pi.lambda <- sum((lam.hyp-1)*log(lambda)) - lgamma(lam.hyp) +
                         lgamma(sum(lam.hyp)) # Dirichlet log-density
#        log.pi.lambda <- log(ddirichlet(lambda, lam.hyp)) 
        L.theta <- dnorm(y - x %*% beta, 0, matrix(s, n, k, byrow = TRUE)
                         ) %*% matrix(lambda, k, 1)
        log.Lik.theta <- sum(log(L.theta))
        log.prior <- sum(log.pi.beta) + sum(log.pi.sigma) + sum(log.pi.lambda)
        log.f.theta <- log.Lik.theta + log.prior
        new.beta <- beta + omega * matrix(rcauchy(k * p), p, k)
        new.sigma <- log(s) + omega * rcauchy(k) 
        new.sigma <- exp(new.sigma) 
	  new.lambda <- lambda.pert(lambda, omega*rcauchy(k)) 
        log.new.pi.beta <- dnorm(new.beta, mu, sig.beta, log=TRUE) 
        log.new.pi.sigma <- dexp(new.sigma, 1/sig, log=TRUE) 
        log.new.pi.lambda <- sum((lam.hyp-1)*log(new.lambda)) - lgamma(lam.hyp) +
                              lgamma(sum(lam.hyp)) # Dirichlet log-density
#        log.new.pi.lambda <- log(ddirichlet(new.lambda, lam.hyp)) 
        new.L.theta <- dnorm(y - x %*% new.beta, 0, matrix(new.sigma, 
            n, k, byrow = TRUE)) %*% matrix(new.lambda, k, 1)
        log.new.Lik.theta <- sum(log(new.L.theta))
        log.new.prior <- sum(log.new.pi.beta) + sum(log.new.pi.sigma) + 
            sum(log.new.pi.lambda)
        log.new.f.theta <- log.new.Lik.theta + log.new.prior
        new.theta <- c(new.beta, new.sigma, new.lambda)
        a <- log.new.f.theta - log.f.theta
        r <- log(runif(1))
        if (a > r & !is.na(a)) {
            theta <- new.theta
            beta <- new.beta
            s <- new.sigma
            lambda <- new.lambda
            accepts = accepts + 1
        }
        if (i%%thin == 0) 
            thetalist[i,] = theta
    }
    cat(paste("Acceptance rate: ", 100 * accepts/sampsize, "%\n", 
        sep = ""))
    colnames(thetalist) <- c(paste("beta", rep(0:(p - 1), k), 
        ".", rep(1:k, rep(p, k)), sep = ""), paste("s.", 1:k, 
        sep = ""), paste("lambda.", 1:k, sep = ""))
    invisible(thetalist)
	a=list(x=x, y=y, theta=thetalist, components=k)
	class(a)="mixMCMC"
	a
}

repnormmixmodel.sel <- function (x, k = 2, ...) 
{
    aic <- NULL
    bic <- NULL
    caic <- NULL
    icl <- NULL
        AIC <- function(emout) {
            emout$loglik - (length(emout$mu) + length(emout$stdev) + 
                length(emout$lambda) - 1)
        }
        BIC <- function(emout) {
            emout$loglik - log(nrow(x)) * (length(emout$mu) + 
                length(emout$stdev) + length(emout$lambda) - 
                1)/2
        }
        CAIC <- function(emout) {
            emout$loglik - (log(nrow(x)) + 1) * (length(emout$mu) + 
                length(emout$stdev) + length(emout$lambda) - 
                1)/2
        }
        ICL <- function(emout) {
            BIC(emout) - sum(emout$lambda * log(emout$lambda))
        }
        for (i in 1:k) {
            if (i == 1) {
                avx <- as.vector(x)
                mu <- mean(avx)
                s <- sd(avx)
                loglik <- sum(dnorm(avx, mean=mu, sd=s, log=TRUE))
                emout <- list(mu=mu, stdev=s, lambda=1, loglik=loglik)
            }
            else emout <- repnormmixEM(x, k = i, ...)
            aic[i] <- AIC(emout)
            bic[i] <- BIC(emout)
            caic[i] <- CAIC(emout)
            icl[i] <- ICL(emout)
        }
    out = rbind(aic, bic, caic, icl)
    Winner = apply(out, 1, function(x) (1:length(x))[x == max(x)])
    rownames(out) = c("AIC", "BIC", "CAIC", "ICL")
    colnames(out) = 1:k
    cbind(out, Winner)
}

plotly_weibullRMM <- function(a, title=NULL, rowstyle=TRUE, subtitle=NULL,
                             width = 3 , col = NULL , 
                             title.size = 15 , title.x = 0.5 , title.y = 0.95,
                             xlab = "Iterations" , xlab.size = 15 , xtick.size = 15,
                             ylab = "Estimates" , ylab.size = 15 , ytick.size = 15,
                             legend.size = 15){
  n <- length(a$x)
  m <- dim(a$all.lambda)[2]
  if (is.null(col)){
    col <- hue_pal()(m)
  }
  if (length(col) != m){
    print("Please specify",m,"colors in 'col'.")
  }
  pcc <- round(100*(1-mean(a$d)),2)
  if (is.null(subtitle)) {
    subtitle <- paste("n=",n,", ", pcc, "% censored", sep="")}
  if (is.null(title)) {
    tt1 <- "Shape parameters"; tt2 <- "Scale parameters" 
    tt3 <- "Weight parameters"
  } else tt1 <- tt2 <- tt3 <- title
  
  plot1 <- plot_ly()%>%
    add_trace(x = seq(from = 1 , to = length(a$all.shape[,1]) , by = 1),
              y = a$all.shape[,1] , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col[1]),
              name = "Shape 1",showlegend = TRUE)%>%
    plotly::layout(
      legend = list(font = list(size=legend.size)),
      title = list(text = paste(tt1 , "\n(", subtitle,")"),
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (j in 2:m){
    plot1 <- plot1%>%
      add_trace(x = seq(from = 1 , to = length(a$all.shape[,j]) , by = 1),
                y = a$all.shape[,j] , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = col[j]),
                name = paste("Shape",j),showlegend = TRUE)
  }
  
  plot2 <- plot_ly()%>%
    add_trace(x = seq(from = 1 , to = length(a$all.scale[,1]) , by = 1),
              y = a$all.scale[,1] , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col[1]),
              name = "Scale 1",showlegend = TRUE)%>%
    plotly::layout(
      legend = list(font = list(size=legend.size)),
      title = list(text = paste(tt2 , "\n(", subtitle,")"),
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (j in 2:m){
    plot2 <- plot2%>%
      add_trace(x = seq(from = 1 , to = length(a$all.scale[,j]) , by = 1),
                y = a$all.scale[,j] , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = col[j]),
                name = paste("Scale",j),showlegend = TRUE)
  } 
  
  plot3 <- plot_ly()%>%
    add_trace(x = seq(from = 1 , to = length(a$all.lambda[,1]) , by = 1),
              y = a$all.lambda[,1] , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col[1]),
              name = paste('&#955;',"<sub>",1,"</sub>" , sep=""),showlegend = TRUE)%>%
    plotly::layout(
      legend = list(font = list(size=legend.size)),
      title = list(text = paste(tt3 , "\n(", subtitle,")"),
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = xlab,
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = ylab,
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (j in 2:m){
    plot3 <- plot3%>%
      add_trace(x = seq(from = 1 , to = length(a$all.lambda[,j]) , by = 1),
                y = a$all.lambda[,j] , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = col[j]),
                name = paste('&#955;',"<sub>",j,"</sub>" , sep=""),showlegend = TRUE)
  } 
  
  print(plot1)
  print(plot2)
  print(plot3)
}

flaremix.init <- function(y, x, lambda=NULL, beta=NULL, sigma=NULL, alpha=NULL){

n<-length(y)

if(is.null(lambda)){
	lambda=runif(2)
	lambda=lambda/sum(lambda)
}

lm.out=lm(y~x[,2])

if(is.null(beta)){
	beta=lm.out$coef
	beta[1]=beta[1]+mean(sort(lm.out$residuals)[(n-10):n])
	beta[2]=rnorm(1,mean=beta[2],sd=abs(beta[2]/10))
}

if(is.null(sigma)){
	sigma=rexp(1,rate=sqrt(1/anova(lm.out)$Mean[2]))
}

if(is.null(alpha)){
	a=1/sum(lm.out$res[lm.out$res>0])
	alpha=abs(rnorm(1,a))
}

list(lambda=lambda[1], beta=matrix(beta,ncol=1), sigma=sigma, alpha=alpha)

}

segregmixEM=function (y, x, lambda = NULL, beta = NULL, sigma = NULL, k = 2, seg.Z, psi, psi.locs = NULL, delta = NULL,
	epsilon = 1e-08, maxit = 10000, verb = FALSE, max.restarts=15) 
{
	if (sum(x[,1]==1)==nrow(x)) x=x[,-1]
	x=data.frame(x) 
	col.names.x <- colnames(x) 
	xnam <- colnames(x) 
	fmla <- as.formula(paste("y ~ ", paste(xnam, collapse= "+"))) 
if(!is.null(psi.locs)){
psi.counts=apply(psi,1,sum)
for(i in 1:k){
	if(psi.counts[i]>0){
		TEMP <- (is.list(psi.locs[[i]]))&(length(psi.locs[[i]])==sum(psi[i,]>0)) 
	} else{
		TEMP <- is.null(psi.locs[[i]])
		}
	if(TEMP==FALSE) stop(paste("You must specify a correct changepoint structure!", 
		"\n"))
}
}

	if (!is.null(delta)) {
		cat("Estimation performed assuming the changepoints are known.", "\n")
		if (is.null(psi.locs)) {
			stop(paste("You must specify the changepoints for this setting!", 
				"\n"))
		}
	} 
	if ((length(seg.Z) != k) | class(seg.Z) != "list") {
		stop(paste("You must specify a list of length k for the segmented relationships!", 
			"\n"))
	}
	if (!identical(all.equal(dim(psi),c(k,ncol(x))),TRUE)) {
		stop(paste("You must specify a matrix with the correct dimension for psi!", 
			"\n"))
	}
	if (((length(psi.locs) != k) | class(psi.locs) != "list") & !is.null(psi.locs)) {
		stop(paste("You must specify a list of length k for the number of changepoints per predictor in each component!", 
			"\n"))
	}
	tot.cp <- apply(psi,1,sum)
	tmp.ind=1
	tmp <- try(suppressWarnings(segregmix.init(y=y, x=x, lambda = lambda, beta = beta, s = sigma, k = k, seg.Z=seg.Z, 
		psi=psi, psi.locs = psi.locs)),silent=TRUE)
	if(class(tmp)=="try-error"){
		cat("Generating new initial values.", "\n")
		while(tmp.ind<=10){
			tmp <- try(suppressWarnings(segregmix.init(y=y, x=x, lambda = NULL, beta = NULL, s = NULL, k = k, seg.Z=seg.Z, 
				psi=psi, psi.locs = NULL)),silent=TRUE)
			tmp.ind <- tmp.ind+1
			if(tmp.ind==11)  stop(paste("Had to reinitialize algorithm too many times.  Reconsider specified model.", "\n"))
			if(class(tmp)!="try-error") tmp.ind=20
		}
	}
	x.old=x
	x = cbind(1, x)
	data.x=cbind(y,x) 
	lambda <- tmp$lambda
	beta <- tmp$beta
	s <- tmp$s
	k <- tmp$k
	psi.locs <- tmp$psi.locs
	sing <- 0
	perms=perm(k,k)
	perm.ind=nrow(perms)
	if (is.null(delta)) delta <- lapply(1:k,function(i) NULL)
	n <- length(y)
	diff <- 1
	iter <- 0
	X.aug <- lapply(1:k, function(i) cbind(1,aug.x(x[,-1],unlist(psi.locs[[i]]),psi[i,],delta=delta[[i]])))
	X.aug.old <- X.aug
	psi.locs.old <- psi.locs
	xbeta <- lapply(1:k, function(i) X.aug[[i]] %*% matrix(beta[[i]],ncol=1))
	res <- sapply(1:k, function(i) as.numeric((y - xbeta[[i]])^2))
	comp <- t((lambda/sqrt(2 * pi * s^2)) * t(exp(-t(t(res)/(2 * s^2)))))
	obsloglik <- sum(log(apply(comp, 1, sum)))
	ll <- obsloglik
	z = matrix(nrow = n, ncol = k)
	restarts <- 0
	while (diff > epsilon && iter < maxit) {
	null.beta=0
	lambda.old <- lambda
	beta.old <- beta
	s.old <- s
        for (i in 1:n) {
		for (j in 1:k) {
			z.denom = c()
			for (h in 1:k) {
				z.denom = c(z.denom, (lambda[h]/lambda[j]) * 
					(s[j]/s[h]) * exp(-0.5 * ((1/s[h]^2) * res[i, h] - (1/s[j]^2) * res[i, j])))
			}
			z[i, j] = 1/sum(z.denom)
		}
	}
	z = z/apply(z, 1, sum)
	z[,k] = 1-apply(as.matrix(z[,-k]),1,sum)
	z<-round(z,15)
	z.old <- z
	lambda.new <- apply(z, 2, mean)
	if (sum(lambda.new < 1e-08) > 0 || is.na(sum(lambda.new))) {
		sing <- 1
	} else {
		lm.out <- vector("list",k)
		psi.temp=psi.locs
		psi.ind=lapply(1:k,function(i) which(psi[i,]!=0))
		for(i in 1:k){
			if(is.null(seg.Z[[i]]) | (sum(1-sapply(delta,is.null))>0)){
				temp.seg <- lm(fmla,data=data.x,weights=z[,i])
			} else temp.seg <- try(suppressWarnings(segmented(lm(fmla,data=data.x,weights=z[,i]),seg.Z=seg.Z[[i]],psi=psi.temp[[i]])),silent=TRUE)
			if(class(temp.seg)[1]=="try-error"){
				seq = 1
				temp.names = names(psi.locs.old[[i]])
				while(seq < 20){
					psi.temp2 <- vector("list",length(psi.temp[[i]]))
					for(ii in 1:length(psi.temp[[i]])){
						x.range <- range(data.x[,which(names(data.x)==temp.names[ii])])
						psi.temp2[[ii]] <- psi.temp[[i]][[ii]]+sample(c(-1,1),length(psi.temp[[i]][[ii]]),replace=TRUE)*runif(length(psi.temp[[i]][[ii]]),0,diff(x.range)/10)
						if((any(psi.temp2[[ii]]<=x.range[1]))|(any(psi.temp2[[ii]]>=x.range[2]))) psi.temp2[[ii]]=psi.temp[[i]][[ii]]
						psi.temp2[[ii]]=sort(psi.temp2[[ii]])
					}
					names(psi.temp2)=temp.names
					temp.seg <- try(suppressWarnings(segmented(lm(fmla,data=data.x,weights=z[,i]),seg.Z=seg.Z[[i]],psi=psi.temp2[[i]],control=seg.control(it.max=1))),silent=TRUE)
					if(class(temp.seg)[1]=="try-error"){
						seq = seq+1
					} else seq=40
				} 
				if(seq!=40){
					temp.seg <- try(suppressWarnings(segmented(lm(fmla,data=data.x,weights=z[,i]),seg.Z=seg.Z[[i]],psi=psi.temp[[i]],control=seg.control(it.max=1))),silent=TRUE)
				}
			}
			lm.out[[i]]=temp.seg			
		}
		lambda <- lambda.new
		if(sum(sapply(lm.out,class)=="try-error")>0){
			newobsloglik=-Inf
		} else{
			if(sum(1-sapply(delta,is.null))>0){
			psi.new <- psi.locs.old
			} else {
				psi.new <- psi.locs
				for(i in 1:k){
					if(class(lm.out[[i]])[1]=="segmented"){
						temp.names=names(psi.locs[[i]])
						temp.cumsum=cumsum(sapply(psi.locs[[i]],length))
						TC.ind = length(temp.cumsum)
						seg.temp = lm.out[[i]]$psi[,2]
						psi.new[[i]] = lapply(1:length(psi.locs[[i]]), function(j) as.numeric(lm.out[[i]]$psi[,2]))
						psi.new[[i]] = vector("list",TC.ind)
						psi.new[[i]][[1]]=sort(seg.temp[1:temp.cumsum[1]])
						if(TC.ind>1) for(j in 2:TC.ind) psi.new[[i]][[j]] = sort(seg.temp[(temp.cumsum[j-1]+1):temp.cumsum[j]])
						names(psi.new[[i]])=temp.names
					}
				}
			}
			X.aug.new <- lapply(1:k, function(i) cbind(1,aug.x(x[,-1],unlist(psi.new[[i]]),psi[i,],delta[[i]])))
			lm.out2=lapply(1:perm.ind, function(j) lapply(1:k, function(i) lm(y~X.aug.new[[i]][,-1],weights=z[,perms[j,i]])))
			beta.new <- lapply(1:perm.ind, function(j) lapply(lm.out2[[j]],coef))
			null.perms <- sapply(1:perm.ind,function(i) all(!is.na(lapply(beta.new,unlist)[[i]])))
			null.beta=0
			if(sum(null.perms)>0){
				xbeta.new <- lapply(1:perm.ind, function(j) lapply(1:k, function(i) X.aug.new[[i]] %*% matrix(beta.new[[j]][[i]],ncol=1)))
				res <- lapply(1:perm.ind, function(j) sapply(1:k, function(i) (y - xbeta.new[[j]][[i]])^2))
				s.new <- lapply(1:perm.ind, function(j) sqrt(sapply(1:k, function(i) sum(z[,perms[j,i]] * (res[[j]][, i]))/sum(z[,perms[j,i]]))))
				comp <- lapply(1:perm.ind, function(j) lapply(1:k, function(i) lambda.new[i] * dnorm(y, xbeta.new[[j]][[i]], s.new[[j]][i])))
				comp <- lapply(1:perm.ind, function(j) sapply(comp[[j]], cbind))
				compsum <- lapply(1:perm.ind, function(j) apply(comp[[j]], 1, sum))
				newobsloglik <- sapply(1:perm.ind, function(j) sum(log(compsum[[j]])))
				newobsloglik[c(1-null.perms)]=-Inf
				IND <- which.max(newobsloglik)
				z = z[,perms[IND,]]
				lambda.new <- apply(z, 2, mean)
				lambda <- lambda.new
				beta <- beta.new[[IND]]
				xbeta <- xbeta.new[[IND]]
				res <- res[[IND]]
				X.aug <- X.aug.old
				s <- s.new[[IND]]
				psi.locs <- psi.new
				sing <- sum(s < 1e-08)
				newobsloglik <- newobsloglik[IND]
			} else{
				newobsloglik=Inf
				null.beta=1
			} 
		}
		if((newobsloglik<obsloglik & !is.na(newobsloglik))|null.beta==1){
		lm.out1=lapply(1:perm.ind, function(j) lapply(1:k, function(i) lm(y~X.aug.old[[i]][,-1],weights=z[,perms[j,i]])))
		beta.new <- lapply(1:perm.ind, function(j) lapply(lm.out1[[j]],coef))
		null.perms <- sapply(1:perm.ind,function(i) all(!is.na(lapply(beta.new,unlist)[[i]])))
		if(sum(null.perms)>0){
			xbeta.new <- lapply(1:perm.ind, function(j) lapply(1:k, function(i) X.aug.old[[i]] %*% matrix(beta.new[[j]][[i]],ncol=1)))
			res <- lapply(1:perm.ind, function(j) sapply(1:k, function(i) (y - xbeta.new[[j]][[i]])^2))
			s.new <- lapply(1:perm.ind, function(j) sqrt(sapply(1:k, function(i) sum(z[,perms[j,i]] * (res[[j]][, i]))/sum(z[,perms[j,i]]))))
			comp <- lapply(1:perm.ind, function(j) lapply(1:k, function(i) lambda.new[i] * dnorm(y, xbeta.new[[j]][[i]], s.new[[j]][i])))
			comp <- lapply(1:perm.ind, function(j) sapply(comp[[j]], cbind))
			compsum <- lapply(1:perm.ind, function(j) apply(comp[[j]], 1, sum))
			newobsloglik <- sapply(1:perm.ind, function(j) sum(log(compsum[[j]])))
			newobsloglik[c(1-null.perms)]=-Inf
			IND <- which.max(newobsloglik)
			z = z[,perms[IND,]]
			lambda.new <- apply(z, 2, mean)
			lambda <- lambda.new
			beta <- beta.new[[IND]]
			xbeta <- xbeta.new[[IND]]
			res <- res[[IND]]
			X.aug <- X.aug.old
			s <- s.new[[IND]]
			psi.locs <- psi.locs.old
			sing <- sum(s < 1e-08)
			newobsloglik <- newobsloglik[IND]
		} else{
			newobsloglik=Inf
			sing=1
		}
	}
	if(newobsloglik<obsloglik & !is.na(newobsloglik) & abs(newobsloglik) != Inf){
		z <- z.old
		lambda <- apply(z, 2, mean)
		X.aug.1 <- lapply(1:k, function(i) cbind(1,aug.x(x[,-1],unlist(psi.locs.old[[i]]),psi[i,],delta[[i]])))
		lm.out.old=lapply(1:k, function(i) lm(y~X.aug.1[[i]][,-1],weights=z[,i]))
		beta <- lapply(lm.out.old,coef)
		xbeta <- lapply(1:k, function(i) X.aug.1[[i]] %*% matrix(beta[[i]],ncol=1))
		res <- sapply(1:k, function(i) (y - xbeta[[i]])^2)
		s <- sqrt(sapply(1:k, function(i) sum(z[,i] * (res[, i]))/sum(z[,i])))
		psi.locs <- psi.locs.old
		comp <- lapply(1:k, function(i) lambda[i] * dnorm(y, xbeta[[i]], s[i]))
		comp <- sapply(comp, cbind)
		compsum <- apply(comp, 1, sum)
		newobsloglik <- sum(log(compsum))
		if(is.na(newobsloglik)){
			sing <- 1
		} else{
			if(newobsloglik<obsloglik) sing <- 1
		}
	}
	}
	if (sing > 0 || is.na(newobsloglik) || abs(newobsloglik) == Inf) {
		cat("Need new starting values due to singularity...", "\n")
		restarts <- restarts + 1
		if (restarts > max.restarts) stop("Too many tries!")
		tmp.ind=1
		while(tmp.ind==1){
			if(sum(1-sapply(delta,is.null))>0) psi.temp=psi.locs
			tmp <- try(suppressWarnings(segregmix.init(y=y, x=x.old, lambda = NULL, beta = NULL, s = NULL, k = k, seg.Z=seg.Z, 
				psi=psi, psi.locs = NULL)),silent=TRUE)
		if(class(tmp)!="try-error") tmp.ind=2
	}
	lambda <- tmp$lambda
	beta <- tmp$beta
	s <- tmp$s
	k <- tmp$k
	psi.locs <- tmp$psi.locs
	n <- length(y)
	diff <- 1
	iter <- 0
	X.aug <- lapply(1:k, function(i) cbind(1,aug.x(x[,-1],unlist(psi.locs[[i]]),psi[i,],delta[[i]])))
	xbeta <- lapply(1:k, function(i) X.aug[[i]] %*% matrix(beta[[i]],ncol=1))
	res <- sapply(1:k, function(i) as.numeric((y - xbeta[[i]])^2))
	comp <- t((lambda/sqrt(2 * pi * s^2)) * t(exp(-t(t(res)/(2 * s^2)))))
	obsloglik <- sum(log(apply(comp, 1, sum)))
	ll <- obsloglik
	} else {
		diff <- newobsloglik - obsloglik
		obsloglik <- newobsloglik
		ll <- c(ll, obsloglik)
		X.aug.old <- X.aug
		psi.locs.old <- psi.locs
		iter <- iter + 1
		if (verb) {
			cat("iteration =", iter, "diff =", diff, "log-likelihood =", obsloglik, "\n")
		}
	}
	}
	if (iter == maxit) {
		warning("Maximum number of iterations reached.", call. = FALSE)
	}
	if (iter == 1) {
		cat("Converged in 1 iteration. Consider rerunning with different starting values or smaller stopping criterion.", "\n")
	}
	cat("number of iterations=", iter, "\n")
	names(delta) <- c(paste("comp", ".", 1:k, sep = ""))
	names(seg.Z) <- c(paste("comp", ".", 1:k, sep = ""))
	names(psi.locs) <- c(paste("comp", ".", 1:k, sep = ""))
	names(beta) <- c(paste("comp", ".", 1:k, sep = ""))
	for(i in 1:k){
		names(beta[[i]])[1]="(Intercept)"
		names(beta[[i]])[2:ncol(x)]=colnames(x)[-1]
		if(!is.null(psi.locs[[i]])){
			for(j in 1:ncol(psi)){
				if(psi[i,j]>0 & j==1){
					names(beta[[i]])[(ncol(x)+1):(ncol(x)+cumsum(psi[i,])[j])]=c(paste(colnames(x)[j+1], ".", 1:psi[i,j], sep = ""))
				} else if(psi[i,j]>0) names(beta[[i]])[(ncol(x)+cumsum(psi[i,])[j-1]+1):(ncol(x)+cumsum(psi[i,])[j])]=c(paste(colnames(x)[j+1], ".", 1:psi[i,j], sep = ""))
			}
		}
	}
	colnames(z) <- c(paste("comp", ".", 1:k, sep = ""))
		a = list(x = x, y = y, lambda = lambda, beta = beta, 
		sigma = s, seg.Z = seg.Z, psi.locs = psi.locs, delta = delta, loglik = obsloglik, posterior = z, all.loglik = ll, 
		restarts = restarts, ft = "segregmixEM")
	class(a) = "mixEM"
	a
}

# original version, with weighted Silverman bandwidth unique option
# has been improved (mid 2014) by a k-fold CV option
# the original version is kept as "npMSL_old" in this file
################################################################ 
################################################################ 
## nonparametric algorithm for Smoothed Likelihood Maximization 
## implementing block structure
## and Silverman adaptive bandwidth
## 2014 additions: loglik stores all the sequence,
## post argument for passing an init matrix of posterior
## bwiter argument for the duration of the adaptive bw stage
## can be set to 0 for keeping an initial bw matrix when samebw=FALSE
################################################################ 
################################################################ 
npMSL_old <- function(x, mu0, blockid = 1:ncol(x),
                  bw=bw.nrd0(as.vector(as.matrix(x))), samebw = TRUE,
                  h=bw, eps=1e-8,
                  maxiter=500, bwiter = maxiter,
                  ngrid=200, post=NULL, verb = TRUE){
  bw <- h # h is alternative bandwidth argument, for backward compatibility
  x <- as.matrix(x)
  n <- nrow(x)      # number of subjects
  r <- ncol(x)      # number of measurements in each subject
  u <- match(blockid, unique(blockid))
  B <- max(u) 		# nb of blocks
  BlS <- rep(0,B)	# block sizes = C_ell in JCGS paper
  for (ell in 1:B) {
      BlS[ell] <- sum(u == ell)}      
      
  if (is.matrix(mu0)) # mu0=centers
    m <- dim(mu0)[1]  else m <- mu0  # mu0=number of clusters
    
  if(!samebw && !is.matrix(bw)) { # create initial bandwidth matrix
    cat("create initial bandwidth matrix\n")
    bw <- matrix(bw, nrow=max(u), ncol=m)  
  }
  z.hat <- matrix(0, nrow = n, ncol = m)
  tt0 <-  proc.time() # for total time
  
  ## Initial Values
  if(m == 1) z.hat <- matrix(1, nrow = n, ncol = m) 
  else if(is.null(post)) {
    kmeans <- kmeans(x, mu0)
    for(j in 1:m)
      z.hat[kmeans$cluster==j, j] <- 1
  } 
  else { 
    z.hat <- post ## Currently no error-checking is done here
  }
  
  iter <- 0
  finished <- FALSE
  lambda <- matrix(0, nrow = maxiter, ncol = m)
  loglik <- NULL; loglikseq <- rep(NA,maxiter)
  total_udfl <- 0; total_nan <- 0 # eventual NaN and underflow in C code

  tmp <- 1:n   # is this needed?
  xtra <- (max(x)-min(x))/10
  grid <- seq(min(x)-xtra, max(x)+xtra, length=ngrid)
  # f stored on a ngrid by m by B array 
  # f_{g,j,ell} = f_{j ell}(u_g)
  # f <- array(1/m/diff(grid[1:2]), c(ngrid, m, B)) 
  # this f was not normalized for being uniform over grid 
  Delta <- diff(grid[1:2])  
  f <- array(1/((ngrid-1)*Delta), c(ngrid, m, B)) 
  oldloglik <- -Inf

  orderx <- xx <- list() # preparation for adaptive bandwidth
  for(k in 1:B) {
    xx[[k]] <- as.vector(x[, u==k])
    if (!samebw) {
      orderx[[k]] = order(xx[[k]]) # only needed for IQR calculation for bw
    }
  }
  ## CftEstep <- ifelse(samebw, "npMSL_Estep", "npMSL_Estep_bw") #Called directly below
  # CftEstep <- "npMSL_Estep_bw" # temporary, for testing only the M-step  
  ## CftMstep <- ifelse(samebw, "npMSL_Mstep", "npMSL_Mstep_bw") #Called directly below
  
  while (!finished) { # Algorithm main iteration loop
    iter <- iter + 1
    bw.old <- bw	# is this needed?
    t0 <- proc.time()
    nb_udfl=0;  # nb underflows, K()*log(0) ~0 cancelled in nems_Estep.c
    nb_nan=0;  # nb nonzero K()*log(0) cancelled in nems_Estep.c

    ## Note:  Enter loop assuming E-step is done -- i.e., z.hat in place    
    ## M-Step
    lambda[iter, ] <- colMeans(z.hat)
    
    ## density estimation in M-step: WKDE-step  
      cs <- colSums(z.hat)
      z.tmp <- sweep(z.hat, 2, cs, "/")
      z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case

    ## adaptive bandwidth update IF in adaptive bw stage
    if (!samebw && iter <= bwiter) {
      for (ell in 1:B) {
        r2 <- BlS[ell]	# block size = nb of coordinates
        wts <- apply(z.tmp, 2, function(z) rep(z/r2, r2))
        variances <- colSums(wts*outer(xx[[ell]], colSums(wts*xx[[ell]]),'-')^2)
        iqr <- apply(as.matrix(wts[orderx[[ell]],]), 2, wIQR,
                     xx[[ell]][orderx[[ell]]],
                     already.sorted=TRUE, already.normalized=TRUE)
        h <- bw[ell, ] <- 0.9 * pmin(sqrt(variances), iqr/1.34) * 
                     pmax(1,r2*n*lambda[iter, ])^(-1/5) 
                     # Note:  Doesn't allow "sample size" < 1.
  #      browser()    
        } 
    } # end of bw adaptive stage

    if(samebw){
      z <- .C(C_npMSL_Mstep, as.integer(ngrid), as.integer(n),
           as.integer(m), as.integer(r), 
           as.integer(B), as.integer(BlS), as.integer(u),
           as.double(bw), as.double(x), as.double(grid), 
           new.f=as.double(f), 
           as.double(lambda[iter,]), 
           as.double(z.hat), PACKAGE = "mixtools")
    } else{
      z <- .C(C_npMSL_Mstep_bw, as.integer(ngrid), as.integer(n),
           as.integer(m), as.integer(r), 
           as.integer(B), as.integer(BlS), as.integer(u),
           as.double(bw), as.double(x), as.double(grid), 
           new.f=as.double(f), 
           as.double(lambda[iter,]), 
           as.double(z.hat), PACKAGE = "mixtools")
    }
#    z=.C(CftMstep, as.integer(ngrid), as.integer(n),
#       as.integer(m), as.integer(r), 
#       as.integer(B), as.integer(BlS), as.integer(u),
#       as.double(bw), as.double(x), as.double(grid), 
#       new.f=as.double(f), 
#       as.double(lambda[iter,]), 
#       as.double(z.hat))
       
    f <- array(z$new.f, c(ngrid, m, B)) # check sum(f == 0)
# print(apply(f,2:3,sum) * Delta)
# print(max(abs(f-f2)))
#    browser()

    ## E-step (for next iteration)
    if(samebw){
      z <- .C(C_npMSL_Estep, as.integer(ngrid), as.integer(n),
           as.integer(m), as.integer(r), 
           as.integer(B), as.integer(u),
           as.double(bw),
           as.double(x), as.double(grid), f=as.double(f),
           as.double(lambda[iter,]), post=as.double(z.hat),
           loglik = double(1),
           nb_udfl = as.integer(nb_udfl), nb_nan = as.integer(nb_nan), PACKAGE = "mixtools")
    } else{
      z <- .C(C_npMSL_Estep_bw, as.integer(ngrid), as.integer(n),
              as.integer(m), as.integer(r), 
              as.integer(B), as.integer(u),
              as.double(bw),
              as.double(x), as.double(grid), f=as.double(f),
              as.double(lambda[iter,]), post=as.double(z.hat),
              loglik = double(1),
              nb_udfl = as.integer(nb_udfl), nb_nan = as.integer(nb_nan), PACKAGE = "mixtools")
    }
#    z=.C(CftEstep, as.integer(ngrid), as.integer(n),
#       as.integer(m), as.integer(r), 
#       as.integer(B), as.integer(u),
#       as.double(bw),
#       as.double(x), as.double(grid), f=as.double(f),
#       as.double(lambda[iter,]), post=as.double(z.hat),
#       loglik = double(1),
#       nb_udfl = as.integer(nb_udfl), nb_nan = as.integer(nb_nan))

    nb_udfl = z$nb_udfl; nb_nan = z$nb_nan; 
    total_udfl <- total_udfl + nb_udfl
    total_nan <- total_nan + nb_nan
    
    z.hat <- matrix(z$post, n, m)
    if (sum(is.nan(z.hat)) > 0) cat("Error!! NaN in z.hat") # obsolete ?
    loglik <- loglikseq[iter] <- z$loglik
    loglikchange <- loglik - oldloglik
    oldloglik <- loglik
    finished <- iter >= maxiter
    if (iter>1 && max(abs(lambda[iter, ] - lambda[iter-1, ])) < eps)
      finished <- TRUE
    if (verb) {
      t1 <- proc.time()
      cat("iteration", iter, ": lambda ", round(lambda[iter, ], 4))
      cat(" obj function", round(loglik, 4))
      cat(" (change", round(loglikchange,4), ")")
      cat(" time", (t1 - t0)[3])
      if ((nb_udfl > 0) || (nb_nan >0)) cat("\n  ")
      if (nb_udfl > 0) cat("average underflows=", round(nb_udfl/(n*m*r),3)," ")
      if (nb_nan >0) cat("average NaNs=", round(nb_nan/(n*m*r),3))
# Note: these average mean nb of nan over ngrid convolution
      cat("\n")
    }
  }
  # f <- array(z$f, c(ngrid, m, r)) # obsolete in block version

 if (!samebw) {
    rownames(bw) <- paste("block", 1:max(u))
    colnames(bw) <- paste("component", 1:m)
  	}

  if (verb) {
    tt1 <- proc.time()
    cat("lambda ", round(lambda[iter, ], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")
  }
  return(structure(list(data = x, posteriors = z.hat, 
                        lambda = lambda[1:iter,], bandwidth = bw, 
                        blockid = u, lambdahat = lambda[iter,], f=f,
                        grid = grid, loglik = loglikseq[1:iter],
			meanUdfl = total_udfl/(n*m*r*iter),# average underflow
			meanNaN = total_nan/(n*m*r*iter)), # average NaN's
                    class="npEM")) # define a "npMSL" class ?
}




## updated late 2014 version
##################################################################### 
##################################################################### 
## nonparametric algorithm for Smoothed Likelihood Maximization 
## implementing block structure
## 2014 latests additions: 
## option for Silverman &  weighted k-fold CV adaptive bandwidth:
##  parameter bwmethod = "S" for Silverman's rule (the default), 
##                       "CV" for Cross-Validation
## loglik now stores all the sequence
## post argument for passing an init matrix of posterior
## bwiter argument for the duration of the adaptive bw stage
## can be set to 0 for keeping an initial bw matrix when samebw=FALSE
###################################################################### 
###################################################################### 

## preliminary function for computing the weighted version of
## k-fold Cross-Validation

# Splitting 1:n in nbsets subsets and return indices
# n = total sample size
# setsize = size of sets
# nseq = explicit 1st and last indices
splitsample <- function(n, nbsets=2) {
  k <- floor(n/nbsets)
  klast <- n - (nbsets-1)*k
  setsize <- c(rep(k, nbsets-1), klast) # n per set
  if (sum(setsize) != n) {cat("warning")}
  ni <- c(1,cumsum(setsize)+1) # indices for splitting   
  nseq <- matrix(NA, nrow=nbsets, ncol=2)
  for (j in 1:nbsets) {
    nseq[j,1] <- ni[j]     # 1st index for jth set
    nseq[j,2] <- ni[j+1]-1 # last index for jth set
  }
  a = list(setsize=setsize, nseq=nseq)
  return(a)
}

## computes CV(h) in k-fold CV case
# h  = bandwidth (first argument for optimizing)
# x = sample of data, vector
# nbsets = number of folds
# w = weights
# lower, upper = numerical integration limits, data-driven defaults
kfoldCV <- function(h, x, nbsets=2, w = rep(1, length(x)), lower=mean(x)-5*sd(x), upper=mean(x)+5*sd(x)) {
  n <- length(x)
  fold <- splitsample(n, nbsets)
  sumf <- 0
  for (k in 1:nbsets) {
    Sk <- fold$nseq[k,1]:fold$nseq[k,2] # indices of set k 
    learnset <- x[-Sk] # obs not in set k, from which f_h is "learned"
    evalset <- x[Sk]
    fk <- wkde(x = learnset, u = evalset, w = w[-Sk], bw = h)
    sumf <- sumf + sum(fk)
  }
  integrand <- function(u,...) {wkde(x, u, bw=h)^2} # computing ||f_h||^2 
  fh2 <- integrate(integrand, lower=lower, upper=upper)$value
  return(fh2 - 2*sumf/n)
}


## Weighted bw selection by k-fold CV ####
## min search done by call to optimize
# x = vector of data
# w = weights, defaults to 1, unnormalized
wbw.kCV <- function(x, nbfold=5, w = rep(1, length(x)), hmin=0.1*hmax, hmax=NULL) {
  n <- length(x)
  if (is.null(hmax)) hmax <- 1.144 * sqrt(var(x))*n^(-1/5) # default hmax as in bw.ucv
  # computing lower and upper integration limits for ||fh||^2
  wm <- weighted.mean(x, w)
  lowerf <- wm - 5*sd(x); upperf <- wm + 5*sd(x) # maybe use a weighted.sd version as well?
  fucv <- function(h) kfoldCV(h, x, nbsets=nbfold, w=w, lower=lowerf, upper=upperf)
  hopt <- optimize(fucv, lower = hmin, upper = hmax)$minimum
  return(hopt)
}


##################################################################### 
##################################################################### 
## nonparametric algorithm for Smoothed Likelihood Maximization 
## implementing block structure
## 2014 latests additions: 
## option for Silverman &  weighted k-fold CV adaptive bandwidth:
##  parameter bwmethod = "S" for Silverman's rule (the default), 
##                       "CV" for Cross-Validation
## loglik now stores all the sequence
## post argument for passing an init matrix of posterior
## bwiter argument for the duration of the adaptive bw stage
## nbfold parameter passed to wbw.kCV, for leave-[n/nbfold]-out CV
## can be set to 0 for keeping an initial bw matrix when samebw=FALSE
###################################################################### 
###################################################################### 
# ToDo: add a nbfold parameter passed to wbw.kCV
npMSL <- function(x, mu0, blockid = 1:ncol(x),
                      bw=bw.nrd0(as.vector(as.matrix(x))), samebw = TRUE,
                      bwmethod = "S", h=bw, eps=1e-8,
                      maxiter=500, bwiter = maxiter, nbfold = NULL,
                      ngrid=200, post=NULL, verb = TRUE){
  bw <- h # h is alternative bandwidth argument, for backward compatibility
  x <- as.matrix(x)
  n <- nrow(x)      # number of subjects
  r <- ncol(x)      # number of measurements in each subject
  u <- match(blockid, unique(blockid))
  B <- max(u)   	# nb of blocks
  BlS <- rep(0,B)	# block sizes = C_ell in JCGS paper
  for (ell in 1:B) {
    BlS[ell] <- sum(u == ell)}      
  
  if (is.matrix(mu0)) # mu0=centers
    m <- dim(mu0)[1]  else m <- mu0  # mu0 = number of clusters
  
  if(!samebw && !is.matrix(bw)) { # create initial bandwidth matrix h_lj
    bw <- matrix(bw, nrow=max(u), ncol=m)}
  z.hat <- matrix(0, nrow = n, ncol = m)
  tt0 <-  proc.time() # for total time  
  
  ## Initial Values
  if(m == 1) z.hat <- matrix(1, nrow = n, ncol = m) 
  else if(is.null(post)) {
    kmeans <- kmeans(x, mu0)
    for(j in 1:m)
      z.hat[kmeans$cluster==j, j] <- 1
  } 
  else { 
    z.hat <- post ## Currently no error-checking is done here
  }
  
  if (is.null(nbfold) && bwmethod == "CV") {nbfold <- 5} # default value for nbfold-CV
  
  iter <- 0
  finished <- FALSE
  lambda <- matrix(0, nrow = maxiter, ncol = m)
  loglik <- NULL
  loglikseq <- rep(NA,maxiter)
  total_udfl <- 0; total_nan <- 0 # eventual NaN and underflow in C code
  
  tmp <- 1:n
  xtra <- (max(x)-min(x))/10
  grid <- seq(min(x)-xtra, max(x)+xtra, length=ngrid)
  # f stored on a ngrid by m by B array 
  # f_{g,j,ell} = f_{j ell}(u_g)
  # f <- array(1/m/diff(grid[1:2]), c(ngrid, m, B)) 
  # this f was not normalized for being uniform over grid 
  Delta <- diff(grid[1:2])  
  f <- array(1/((ngrid-1)*Delta), c(ngrid, m, B)) 
  oldloglik <- -Inf
  
  orderx <- xx <- list() # preparation for Silverman adaptive bandwidth
  for(k in 1:B) {
    xx[[k]] <- as.vector(x[, u==k]) # data pooled from kth block
    if (!samebw && bwmethod == "S") {
      orderx[[k]] = order(xx[[k]]) # only needed for IQR calculation for bw
    }
  }
  ## CftEstep <- ifelse(samebw, "npMSL_Estep", "npMSL_Estep_bw") #Called directly below.
  # CftEstep <- "npMSL_Estep_bw" # temporary, for testing only the M-step  
  ## CftMstep <- ifelse(samebw, "npMSL_Mstep", "npMSL_Mstep_bw") #Called directly below.
  
  while (!finished) {
    iter <- iter + 1
    bw.old <- bw	# is this needed?
    t0 <- proc.time()
    nb_udfl=0;  # nb of underflows log(0) cancelled in nems_Estep.c
    nb_nan=0;  # nb of nonzero K()*log(0) cancelled in nems_Estep.c
    
    ## Note:  Enter loop assuming E-step is done i.e., z.hat in place    
    ## M-Step
    lambda[iter, ] <- colMeans(z.hat)
    
    ## density estimation in M-step: WKDE-step  
    cs <- colSums(z.hat)
    z.tmp <- sweep(z.hat, 2, cs, "/")
    z.tmp[, cs==0] <- 1/NROW(z.tmp) # Just in case
    
    
    
    ## adaptive bandwidth update - depends on bwmethod and adptbw stage duration in this version
    if (!samebw && iter <= bwiter) {
      # adaptive Silverman's rule
      if (bwmethod == "S") {
        for (ell in 1:B) { # for each block
          r2 <- BlS[ell]   # block size = nb of coordinates
          wts <- apply(z.tmp, 2, function(z) rep(z/r2, r2))
          variances <- colSums(wts*outer(xx[[ell]], colSums(wts*xx[[ell]]),'-')^2)
          iqr <- apply(as.matrix(wts[orderx[[ell]],]), 2, wIQR, 
                       xx[[ell]][orderx[[ell]]],
                       already.sorted=TRUE, already.normalized=TRUE)
          h <- bw[ell, ] <- 0.9 * pmin(sqrt(variances), iqr/1.34) * 
            pmax(1,r2*n*lambda[iter, ])^(-1/5) 
          # Note:  Doesn't allow "sample size" < 1.  
        }
      }
      
      # adaptive nbfold CV computation, nbfold=5 by default
      if (bwmethod == "CV") { 
        # k-fold CV      
        for (ell in 1:B) { # for each block
          r2 <- BlS[ell]   # block size = nb of coordinates
          wts <- apply(z.tmp, 2, function(z) rep(z, r2)) # replicate weights
          for (j in 1:m) bw[ell,j] <- wbw.kCV(xx[[ell]], nbfold = nbfold, w = wts[,j])
        }
      } # end of CV version  
    } # end of bw adaptive stage
    
    
    if(samebw){
      z <- .C(C_npMSL_Mstep, as.integer(ngrid), as.integer(n),
              as.integer(m), as.integer(r), 
              as.integer(B), as.integer(BlS), as.integer(u),
              as.double(bw), as.double(x), as.double(grid), 
              new.f=as.double(f), 
              as.double(lambda[iter,]), 
              as.double(z.hat), PACKAGE = "mixtools")
    } else{
      z <- .C(C_npMSL_Mstep_bw, as.integer(ngrid), as.integer(n),
              as.integer(m), as.integer(r), 
              as.integer(B), as.integer(BlS), as.integer(u),
              as.double(bw), as.double(x), as.double(grid), 
              new.f=as.double(f), 
              as.double(lambda[iter,]), 
              as.double(z.hat), PACKAGE = "mixtools")
    }    
    
#    z=.C(CftMstep, as.integer(ngrid), as.integer(n),
#         as.integer(m), as.integer(r), 
#         as.integer(B), as.integer(BlS), as.integer(u),
#         as.double(bw), as.double(x), as.double(grid), 
#         new.f=as.double(f), 
#         as.double(lambda[iter,]), 
#         as.double(z.hat))
    
    f <- array(z$new.f, c(ngrid, m, B)) # check sum(f == 0)
    # print(apply(f,2:3,sum) * Delta)
    # print(max(abs(f-f2)))
    #    browser()
    
    ## E-step (for next iteration)
    if(samebw){
      z <- .C(C_npMSL_Estep, as.integer(ngrid), as.integer(n),
              as.integer(m), as.integer(r), 
              as.integer(B), as.integer(u),
              as.double(bw),
              as.double(x), as.double(grid), f=as.double(f),
              as.double(lambda[iter,]), post=as.double(z.hat),
              loglik = double(1),
              nb_udfl = as.integer(nb_udfl), nb_nan = as.integer(nb_nan), PACKAGE = "mixtools")
    } else{
      z <- .C(C_npMSL_Estep_bw, as.integer(ngrid), as.integer(n),
              as.integer(m), as.integer(r), 
              as.integer(B), as.integer(u),
              as.double(bw),
              as.double(x), as.double(grid), f=as.double(f),
              as.double(lambda[iter,]), post=as.double(z.hat),
              loglik = double(1),
              nb_udfl = as.integer(nb_udfl), nb_nan = as.integer(nb_nan), PACKAGE = "mixtools")
    }
#    z=.C(CftEstep, as.integer(ngrid), as.integer(n),
#         as.integer(m), as.integer(r), 
#         as.integer(B), as.integer(u),
#         as.double(bw),
#         as.double(x), as.double(grid), f=as.double(f),
#         as.double(lambda[iter,]), post=as.double(z.hat),
#         loglik = double(1),
#         nb_udfl = as.integer(nb_udfl), nb_nan = as.integer(nb_nan))
    
    nb_udfl = z$nb_udfl; nb_nan = z$nb_nan; 
    total_udfl <- total_udfl + nb_udfl
    total_nan <- total_nan + nb_nan
    
    z.hat <- matrix(z$post, n, m)
    if (sum(is.nan(z.hat)) > 0) cat("Error!! NaN in z.hat") # obsolete ?
    loglik <- loglikseq[iter] <- z$loglik
    loglikchange <- loglik - oldloglik
    oldloglik <- loglik
    finished <- iter >= maxiter
    if (iter>1 && max(abs(lambda[iter, ] - lambda[iter-1, ])) < eps)
      finished <- TRUE
    if (verb) {
      t1 <- proc.time()
      cat("iteration", iter, ": lambda ", round(lambda[iter, ], 4))
      cat(" obj change", round(loglikchange,4))
      cat(" time", (t1 - t0)[3])
      if ((nb_udfl > 0) || (nb_nan >0)) cat("\n  ")
      if (nb_udfl > 0) cat("average underflows=", round(nb_udfl/(n*m*r),3)," ")
      if (nb_nan >0) cat("average NaNs=", round(nb_nan/(n*m*r),3))
      # Note: average nb of nan over ngrid convolution
      cat("\n")
    }
  }
  # f <- array(z$f, c(ngrid, m, r)) # obsolete in block version
  
  if (!samebw) {
    rownames(bw) <- paste("block", 1:max(u))
    colnames(bw) <- paste("component", 1:m)
  }
  
  if (verb) {
    tt1 <- proc.time()
    cat("lambda ", round(lambda[iter, ], 4))
    cat(", total time", (tt1 - tt0)[3], "s\n")
  }
  return(structure(list(data = x, posteriors = z.hat, 
                        lambda = lambda[1:iter,], bandwidth = bw, 
                        blockid = u, lambdahat = lambda[iter,], f=f,
                        grid = grid, loglik = loglikseq[1:iter],
                        meanUdfl = total_udfl/(n*m*r*iter),# average underflow
                        meanNaN = total_nan/(n*m*r*iter)), # average NaN's
                   class="npEM")) # should we define a "npMSL" class ?
}

plotly_seq.npEM <- function(x, col = '#1f77b4' , width = 6,
                            xlab = "Iteration" , xlab.size = 15 , xtick.size = 15,
                            ylab.size = 15 , ytick.size = 15,
                            title.size = 15 , title.x = 0.5 , title.y = 0.95){
  r <- NCOL(x$data)
  n <- NROW(x$data)
  m <- length(x$lambdahat)
  iter <- NROW(x$lambda)
  nbcol <- 1
  if (!is.null(x$symmetric) && x$symmetric){nbcol <- 2}
  # in all cases, plots the lambda's  
  for (j in 1:m){
    estim <- paste(round(x$lambdahat[j],3))
    # tt <- substitute(expression(paste("sequence of ",lambda[j],
    #                                   ", estimate ",widehat(lambda[j]),"=", estim, sep="")))
    tt <- paste("Sequence of " ,'&#955;',"<sub>",j,"</sub>",
                " (Estimated ","&#955;","<sub>",j,"</sub>","=",estim, ")",sep="")
    # ylabel <- substitute(expression(paste(lambda[j],sep="")))
    ylabel <- paste('&#955;',"<sub>",j,"</sub>",sep="")
    plot1 <- plot_ly()%>%
      add_trace(x = seq(from = 1 , to = length(x$lambda[,j]) , by = 1),
                y = x$lambda[,j] , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = col),
                showlegend = FALSE)%>%
      add_trace(x = c(0 , iter),
                y = rep(x$lambdahat[j],2) , type = 'scatter' , mode = 'lines',
                line = list(width = width , color = 'red' , dash = "dash"),
                showlegend = FALSE)%>%
      plotly::layout(
        title = list(text = tt,
                     x = title.x,
                     y = title.y,
                     font = list(size=title.size)),
        xaxis = list(title = list(text = xlab,
                                  font = list(size = xlab.size)),
                     tickfont = list(size = xtick.size)
        ),
        yaxis = list(title = list(text = eval(ylabel),
                                  font = list(size = ylab.size)),
                     tickfont = list(size = ytick.size)
        )
      )
    print(plot1)
  }
  ## for symmetric location spEM case plots mu
  if (!is.null(x$symmetric) && x$symmetric){
    for (j in 1:m){
      estim <- paste(round(x$muhat[j],3))
      # tt <- substitute(expression(paste("sequence of ",mu[j],
      #                                   ", estimate ",widehat(mu[j]),"=",estim,sep="")))
      tt <- paste("Sequence of " ,'&#956;',"<sub>",j,"</sub>",
                  " (Estimated ","&#956;","<sub>",j,"</sub>","=",estim, ")",sep="")
      # ylabel <- substitute(expression(paste(mu[j],sep="")))
      ylabel <- paste('&#956;',"<sub>",j,"</sub>",sep="")
      plot2 <- plot_ly()%>%
        add_trace(x = seq(from = 1 , to = length(x$mu[,j]) , by = 1),
                  y = x$mu[,j] , type = 'scatter' , mode = 'lines',
                  line = list(width = width , color = col),
                  showlegend = FALSE)%>%
        add_trace(x = c(0 , iter),
                  y = rep(x$muhat[j],2) , type = 'scatter' , mode = 'lines',
                  line = list(width = width , color = 'red' , dash = "dash"),
                  showlegend = FALSE)%>%
        plotly::layout(
          title = list(text = tt,
                       x = title.x,
                       y = title.y,
                       font = list(size=title.size)),
          xaxis = list(title = list(text = xlab,
                                    font = list(size = xlab.size)),
                       tickfont = list(size = xtick.size)
          ),
          yaxis = list(title = list(text = eval(ylabel),
                                    font = list(size = ylab.size)),
                       tickfont = list(size = ytick.size)
          )
        )
      print(plot2)
    }
  }
}

# Alternative version of rmvnorm to eliminate dependence of mixtools
# on additional package 'mvtnorm'

# Uses eigen decomposition assuming symmetric==TRUE.  Don't know how efficient
# this might be relative to other approaches, but some suggest this is among
# the most efficient methods to find a matrix square root.

rmvnorm <- function(n, mu=NULL, sigma=NULL) {
  if (is.null(mu)) {
    if (is.null(sigma)) {
      return(rnorm(n))
    } else {
      mu = rep(0, nrow(sigma))
    }
  } else if (is.null(sigma)) {
    sigma=diag(length(mu))
  }
  lmu <- length(mu)
  if (lmu != nrow(sigma) || lmu != ncol(sigma)) 
    stop("length of mu must equal nrow and ncol of sigma")
  e <- eigen(sigma, symmetric=TRUE)
  if (any(e$val<0)) 
    stop("Numerically negative definite covariance matrix")
  t(mu + e$vec %*% (t(e$vec) * sqrt(e$val)) %*% matrix(rnorm(n*lmu), lmu, n))  
}

perm <- function (n, r, v = 1:n) 
{ 
	if (r == 1) 
	matrix(v, n, 1) 
	else if (n == 1) 
	matrix(v, 1, r) 
	else { 
		X <- NULL 
		for (i in 1:n) X <- rbind(X, cbind(v[i], perm(n - 
			1, r - 1, v[-i]))) 
		X 
	} 
} 

regmixEM.loc=function (y, x, lambda = NULL, beta = NULL, sigma = NULL, k = 2, 
                        addintercept = TRUE, kern.l = c("Gaussian", "Beta", "Triangle", 
                                                        "Cosinus", "Optcosinus"), epsilon = 1e-08, maxit = 10000, 
                        kernl.g = 0, kernl.h = 1, verb = FALSE) 
{
  diff <- 1
  iter <- 0
  x.1 <- cbind(1, x)
  n <- length(y)
  kern.l <- match.arg(kern.l)
  out.EM <- regmixEM.lambda(y, x, lambda = lambda, beta = beta, 
                            sigma = sigma, k = k, epsilon = epsilon, maxit = maxit)
  ll = out.EM$loglik
  restarts <- 0
  while (diff > epsilon && iter < maxit) {
    old.l.x = out.EM$lambda
    old.loglik = out.EM$loglik
    l.x = matrix(nrow = n, ncol = k)
    for (i in 1:k) {
      #      l.x[, i] <- c((cbind(rep(1,n),0)*t(apply(matrix(x, ncol = 1), 1, lambda, 
      #                                               z = out.EM$post[, i], xi = x, kernel = kern.l, 
      #                                               g = kernl.g, h = kernl.h)))[,1])
      l.x[,i] <- t(apply(as.matrix(x), 1, lambda, 
                         z = out.EM$post[, i], xi = as.matrix(x), kernel = kern.l, 
                         g = kernl.g, h = kernl.h))[,1]
      l.x[,i] <- pmin(pmax(l.x[,i],0),1)
      #      l.x[,i]=apply(cbind(l.x[,i]),1,max,0)
      #      l.x[,i]=apply(cbind(l.x[,i]),1,min,1)
    }
    l.x <- l.x/matrix(rep(apply(l.x,1,sum),k),ncol = k)
    #    l.x <- cbind(l.x, 1 - apply(l.x, 1, sum))
    out.EM.loc <- regmixEM.lambda(y, x, beta = out.EM$beta, 
                                  sigma = out.EM$sigma, lambda = l.x, k = k)
    loglik.loc <- out.EM.loc$loglik
    out.EM.old <- regmixEM.lambda(y, x, beta = out.EM$beta, 
                                  sigma = out.EM$sigma, lambda = old.l.x, k = k)
    loglik.old <- out.EM.old$loglik
    if (loglik.loc > old.loglik) {
      out.EM <- out.EM.loc
    }
    else out.EM <- out.EM.old
    loglik.chosen <- out.EM$loglik
    ll <- c(ll, loglik.chosen)
    diff <- loglik.chosen - old.loglik
    if (diff < 0) {
      cat("Generating new starting values...", "\n")
      out.EM <- regmixEM.lambda(y, x, lambda = lambda, 
                                beta = beta, sigma = sigma, k = k, epsilon = epsilon, 
                                maxit = maxit)
      restarts <- restarts + 1
      if (restarts > 15) 
        stop("Too many tries!")
      iter <- 0
      diff <- 1
    }
    else {
      iter <- iter + 1
      if (verb) {
        cat("iteration=", iter, "diff=", diff, "log-likelihood", 
            loglik.chosen, "\n")
      }
    }
  }
  if (iter == maxit) {
    cat("WARNING! NOT CONVERGENT!", "\n")
  }
  cat("number of overall iterations=", iter, "\n")
  a = list(x = x, y = y, lambda.x = out.EM$lambda, beta = out.EM$beta, 
           sigma = out.EM$sigma, loglik = loglik.chosen, posterior = out.EM$post, 
           all.loglik = ll, restarts = restarts, ft = "regmixEM.loc")
  class(a) = "mixEM"
  a
}

######################################################################
# the following depth function can compute the depth for             #
# multi points at one time                                           #
######################################################################

#dyn.load("~fxuan/depth/spherical/sphericaldepth.so")
depth <- function(pts,x,Cx=var(x)){
  x=x%*%t(chol(solve(Cx)))
  pts=pts%*%t(chol(solve(Cx)))
#  y <- .Fortran("mudepth",
#                as.integer(nrow(x)),
#                as.integer(nrow(pts)),
#                as.integer(ncol(x)),
#                as.single(pts),
#                as.single(x),
#                as.integer(1:nrow(pts)),
#                as.single(1:nrow(pts)),
#                PACKAGE="mixtools")
# Now rewritten in C:
  y <- .C(C_mudepth,
                as.integer(nrow(x)),
                as.integer(nrow(pts)),
                as.integer(ncol(x)),
                as.double(pts),
                as.double(x),
                integer(nrow(pts)),
                double(nrow(pts)),
                PACKAGE="mixtools")  
  count <- y[[6]]
  n <- nrow(x)
#  depth <- sqrt(8)*(count-n*(n-1)/4)/sqrt((n*(n-1)))#this is to standardize the depth
#  depth <- (factorial(n)/(2*factorial(n-2)))^(-1)*count
  depth <- choose(n,2)^(-1)*count
  depth
}

plotly_spRMM <- function(sem, tmax = NULL,
                        width = 3 , col = '#1f77b4', cex = 3,
                        title.size = 15 , title.x = 0.5 , title.y = 0.95,
                        xlab.size = 15 , xtick.size=15 ,
                        ylab.size = 15 , ytick.size=15){
  t <- sem$t   
  ym <- max(sem$all.scaling)
  
  plot1 <- plot_ly()%>%
    add_trace(x = seq(from = 1 , to = length(sem$all.scaling) , by = 1),
              y = sem$all.scaling , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col),
              showlegend = FALSE)%>%
    plotly::layout(
      title = list(text = "Scaling",
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = "Iteration",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size),
                   range = c(0,ym)
      )
    )
  
  plot2 <- plot_ly()%>%
    add_trace(x = seq(from = 1 , to = length(sem$all.lambda[,1]) , by = 1),
              y = sem$all.lambda[,1] , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col),
              showlegend = FALSE)%>%
    plotly::layout(
      title = list(text = "Weight of Component 1",
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = "Iteration",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size),
                   range = c(0,1)
      )
    )
  
  if (is.null(tmax)){tmax <- max(sem$scaling*t) + 2}
  u <- seq(0, tmax, len=200)
  fhat <- sem$s.hat(sem$t.hat)*sem$hazard.hat
  ffinal <- sem$survival(sem$final.t)*sem$hazard
  
  plot3 <- plot_ly()%>%
    add_trace(x = seq(from = 1 , to = length(sem$survival) , by = 1),
              y = sem$survival , type = 'scatter' , mode = 'markers',
              marker = list(size = cex , color = col),
              showlegend = FALSE)%>%
    plotly::layout(
      title = list(text = "Survival Function Estimate",
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = "Time",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size),
                   range = c(0,tmax)
      ),
      yaxis = list(title = list(text = "",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size),
                   range = c(0,1)
      )
    )
  
  plot4 <- plot_ly()%>%
    add_trace(x = sem$final.t,
              y = ffinal , type = 'scatter' , mode = 'lines',
              line = list(width = width , color = col),
              showlegend = FALSE)%>%
    plotly::layout(
      title = list(text = "Density Estimate",
                   x = title.x,
                   y = title.y,
                   font = list(size=title.size)),
      xaxis = list(title = list(text = "Time",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size),
                   range = c(0,tmax)
      ),
      yaxis = list(title = list(text = "Density",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  
  print(plot1)
  print(plot2)
  print(plot3)
  print(plot4)
}

plotly_expRMM <- function(a , title = NULL , 
                          rowstyle = TRUE , subtitle=NULL,
                          width = 2 , cex = 2 , col.comp = NULL,
                          legend.text = NULL,
                          legend.text.size = 15,
                          legend.size = 15,
                          title.x = 0.5,
                          title.y = 0.95,
                          title.size = 15,
                          xlab.size = 15,
                          xtick.size = 15,
                          ylab.size = 15,
                          ytick.size = 15
){
  n <- length(a$x) 
  m <- dim(a$all.lambda)[2]
  if (is.null(col.comp)){
    col.comp <- hue_pal()(m)
  }
  if (length(col.comp) != m){
    print(paste("Please specify",m,"colors in 'col.comp'."))
  }
  pcc <- round(100*(1-mean(a$d)),2)
  sizes <- paste("n=",n,", ", pcc, "% censored", sep="")
  if (is.null(subtitle)) {
    subtitle <- paste("n=",n,", ", pcc, "% censored", sep="")}
  if (is.null(title)) {
    tt1 <- "Rate parameters" 
    tt2 <- "Weight parameters"
  } else {tt1 <- tt2 <- title}
  # lgd <- expression(xi[1]) ## Cannot be converted to plotly.
  
  plot1 <- plot_ly() %>%
    plotly::layout(
      legend = list(title=list(text=legend.text),
                    font = list(size=legend.size)),
      xaxis = list(title = list(text = "Iterations",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "Estimates",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (j in 1:m){
    plot1 <- plot1 %>%
      add_trace(x=seq(from = 1 , to = length(a$all.rate[,j]) , by = 1) , 
                y=a$all.rate[,j] , type = 'scatter' , mode = 'lines+markers',
                marker = list(size = cex , color = col.comp[j]),
                line = list(width = width , color = col.comp[j]),
                name = paste("Comp" , j),showlegend = TRUE)
  }
  
  plot2 <- plot_ly() %>%
    plotly::layout(
      legend = list(title=legend.text,
                    font = list(size=legend.size)),
      xaxis = list(title = list(text = "Iterations",
                                font = list(size = xlab.size)),
                   tickfont = list(size = xtick.size)
      ),
      yaxis = list(title = list(text = "Estimates",
                                font = list(size = ylab.size)),
                   tickfont = list(size = ytick.size)
      )
    )
  for (j in 1:m){
    plot2 <- plot2 %>%
      add_trace(x=seq(from = 1 , to = length(a$all.lambda[,j]) , by = 1) , 
                y=a$all.lambda[,j] , type = 'scatter' , mode = 'lines+markers',
                marker = list(size = cex , color = col.comp[j]),
                line = list(width = width , color = col.comp[j]),
                name = paste("Comp" , j),showlegend = FALSE)
  }
  
  if (rowstyle) {
    nrow <- 1
    x.1 <- 0.25
    x.2 <- 0.75
    y.1 <- y.2 <- 0.95
    share.X <- FALSE
  } else if (!rowstyle){
    nrow <- 2
    x.1 <- x.2 <- 0.5
    y.1 <- 0.95
    y.2 <- 0.45
    share.X <- TRUE
  }
  plot.all <- subplot(
    plot1 , plot2 , nrows = nrow , shareX = share.X , shareY = FALSE,
    titleX = TRUE , titleY = TRUE
  )%>%
    plotly::layout(annotations = list(
      list(
        x = x.1, 
        y = y.1, 
        font = list(size = title.size), 
        text = paste(tt1 , "\n(", subtitle,")"), 
        xref = "paper", 
        yref = "paper", 
        xanchor = "center", 
        yanchor = "bottom", 
        showarrow = FALSE
      ), 
      list(
        x = x.2, 
        y = y.2, 
        font = list(size = title.size), 
        text = paste(tt2 , "\n(", subtitle,")"), 
        xref = "paper", 
        yref = "paper", 
        xanchor = "center", 
        yanchor = "bottom", 
        showarrow = FALSE
      )
    )
  )
  print(plot.all)
}

\documentclass[nojss]{jss}
\usepackage{amsmath}
\usepackage{amssymb}
%% need no \usepackage{Sweave.sty}

%\VignetteIndexEntry{mixtools for mixture models} 

%% macros (Didier)
\newcommand{\CF}{{\mathcal F}}
\newcommand{\CN}{{\mathcal N}}

\def\Bg{\mathbf{g}}
\def\Bh{\mathbf{h}}
\def\Bk{\mathbf{k}}
\def\Bx{\mathbf{x}}
\def\By{\mathbf{y}}
\def\Bc{\mathbf{c}}
\def\BC{\mathbf{C}}
\def\Bz{\mathbf{z}}

\newcommand{\argmax}{\mathop{\mbox{argmax}}}
\def\bP{\mathbb{P}} % Probability
\def\I{\mathbb I}   % indicator function
\def\bE{\mathbb{E}} % expectation
\def\bR{\mathbb{R}} % real line
\newcommand{\f}{{\vec\theta}}
\newcommand{\lb}{{\lambda}}

\def\post{{p}}
\def\defn{{\stackrel{\rm def}{=}}}
\def\vec#1{\mathchoice{\mbox{\boldmath$\displaystyle\bf#1$}}
{\mbox{\boldmath$\textstyle\bf#1$}}
{\mbox{\boldmath$\scriptstyle\bf#1$}}
{\mbox{\boldmath$\scriptscriptstyle\bf#1$}}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% declarations for jss.cls %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% almost as usual
\author{Tatiana Benaglia \\ Pennsylvania State University \And
  Didier Chauveau \\ Universit\'e d'Orl\'eans \AND David R.~Hunter \\
  Pennsylvania State University \And Derek S. Young \\ Pennsylvania
  State University} \title{\pkg{mixtools}: An \proglang{R} Package for
  Analyzing Finite Mixture Models}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Tatiana Benaglia, Didier Chauveau, David R.~Hunter, Derek
  Young} %% comma-separated
\Plaintitle{mixtools: An R Package for Analyzing Mixture
  Models} %% without formatting
\Shorttitle{mixtools for Mixture Models} %% a short title (if necessary)

%% an abstract and keywords
\Abstract{ The \pkg{mixtools} package for \proglang{R} provides a set of
  functions for analyzing a variety of finite mixture models.  These
  functions include both traditional methods, such as EM algorithms for
  univariate and multivariate normal mixtures, and newer methods that
  reflect some recent research in finite mixture models.  In the latter
  category, \pkg{mixtools} provides algorithms for estimating parameters
  in a wide range of different mixture-of-regression contexts, in
  multinomial mixtures such as those arising from discretizing
  continuous multivariate data, in nonparametric situations where the
  multivariate component densities are completely unspecified, and in
  semiparametric situations such as a univariate location mixture of
  symmetric but otherwise unspecified densities.  Many of the algorithms
  of the \pkg{mixtools} package are EM algorithms or are based on
  EM-like ideas, so this article includes an overview of EM algorithms
  for finite mixture models.  } \Keywords{cutpoint, EM algorithm,
  mixture of regressions, model-based clustering, nonparametric mixture,
  semiparametric mixture, unsupervised clustering}
%, keywords, comma-separated, not capitalized, \proglang{Java}}
\Plainkeywords{keywords, comma-separated, not capitalized,
  Java} %% without formatting
%% at least one keyword must be supplied

%% publication information
%% NOTE: Typically, this can be left commented and will be filled out by the technical editor
%% \Volume{13}
%% \Issue{9}
%% \Month{September}
%% \Year{2004}
%% \Submitdate{2004-09-29}
%% \Acceptdate{2004-09-29}

%% The address of (at least) one author should be given
%% in the following format:
\Address{
Didier Chauveau\\
Laboratoire MAPMO - UMR 7349 - F\'ed\'eration Denis Poisson\\
Universit\'e d'Orl\'eans\\
BP 6759, 45067 Orl\'eans cedex 2, FRANCE.\\
E-mail: \email{didier.chauveau@univ-orleans.fr} \\
URL: \url{http://www.univ-orleans.fr/mapmo/membres/chauveau/}\\
\\
David R.~Hunter\\
Department of Statistics\\
326 Thomas Building\\
Pennsylvania State University\\
University Park, PA 16802\\
Telephone:  +1/814-863-0979\\
Fax: +1/814-863-7114\\
E-mail: \email{dhunter@stat.psu.edu} \\
URL: \url{http://www.stat.psu.edu/~dhunter/}\\
\\
Tatiana Benaglia\\
Department of Statistics, Penn State (see above)\\
E-mail:  \email{tab321@stat.psu.edu} \\
\\
Derek Young\\
Department of Statistics, Penn State (see above)\\
E-mail:  \email{dsy109@psu.edu}
}
%% It is also possible to add a telephone and fax number
%% before the e-mail in the following format:
%% Telephone: +43/1/31336-5053
%% Fax: +43/1/31336-734

%% for those who use Sweave please include the following line (with % symbols):
%% need no \usepackage{Sweave.sty}

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


\begin{document}
\SweaveOpts{concordance=FALSE}

%% include your article here, just as usual
%% Note that you should use the \pkg{}, \proglang{} and \code{} commands.

\section[Introduction to finite mixtures and mixtools]{Introduction to
  finite mixtures and \pkg{mixtools}}
%% Note: If there is markup in \(sub)section, then it has to be escape
%% as above.
\label{s:intro}

Authors' note: The original version of this vignette was produced using
an article that appears in the {\it Journal of Statistical Software}
(URL:  \url{http://www.jstatsoft.org/}); see
\citet{Benaglia+Chauveau+Hunter+Young:2009}.

Populations of individuals may often be divided into subgroups.  Yet
even when we observe characteristics of these individuals that provide
information about their subgroup memberships, we may not actually
observe these memberships {\em per se}. The basic goal of the tools in
the \pkg{mixtools} package (version 0.4.3, as of this writing) for
\proglang{R} \citep{r2009} is to examine a sample of measurements to
discern and describe subgroups of individuals, even when there is no
observable variable that readily indexes into which subgroup an
individual properly belongs.  This task is sometimes referred to as
``unsupervised clustering'' in the literature, and in fact mixture
models may be generally thought of as comprising the subset of
clustering methods known as ``model-based clustering''.  The
\pkg{mixtools} package is available from the Comprehensive \proglang{R}
Archive Network at \url{http://CRAN.R-project.org/package=mixtools}.

Finite mixture models may also be used in situations beyond those for
which clustering of individuals is of interest.  For one thing, finite
mixture models give descriptions of entire subgroups, rather than
assignments of individuals to those subgroups (though the latter may be
accomplished using mixture models).  Indeed, even the subgroups may not
necessarily be of interest; sometimes finite mixture models merely
provide a means for adequately describing a particular distribution,
such as the distribution of residuals in a linear regression model where
outliers are present.

Whatever the goal of the modeler when employing mixture models, much of
the theory of these models involves the assumption that the subgroups
are distributed according to a particular parametric form --- and quite
often this form is univariate or multivariate normal.  While
\pkg{mixtools} does provide tools for traditional fitting of finite
mixtures of univariate and multivariate normal distributions, it goes
well beyond this well-studied realm.  Arising from recent research whose
goal is to relax or modify the assumption of multivariate normality,
\pkg{mixtools} provides computational techniques for finite mixture
model analysis in which components are regressions, multinomial vectors
arising from discretization of multivariate data, or even distributions
that are almost completely unspecified.  This is the main feature that
distinguishes \pkg{mixtools} from other mixture-related \proglang{R}
packages, also available from the Comprehensive \proglang{R} Archive
Network at \url{http://CRAN.R-project.org/}, such as \pkg{mclust}
\citep{Fraley+Raftery:2009} and \pkg{flexmix} \citep{jss:Leisch:2004,
  Grun+Leisch:2008}.  We briefly mention these two packages in
Sections~\ref{section:EMexample} and \ref{section:pdmp}, respectively.

To make the mixture model framework more concrete, suppose the possibly
vector-valued random variables $\vec X_1, \ldots, \vec X_n$ are a simple
random sample from a finite mixture of $m>1$ arbitrary distributions,
which we will call {\em components} throughout this article.  The
density of each $\vec X_i$ may be written
\begin{equation}
\label{mvmixture}
g_{\f}(\vec x_i) = \sum_{j=1}^m\lambda_j\phi_j(\vec x_i), \quad
\vec x_i\in\bR^r,
\end{equation}
where $\f=(\vec\lambda, \vec \phi) = (\lambda_1, \ldots, \lambda_m,
\phi_1, \ldots, \phi_m)$ denotes the parameter and the $\lambda_m$ are
positive and sum to unity.  We assume that the $\phi_j$ are drawn from
some family $\cal F$ of multivariate density functions absolutely
continuous with respect to, say, Lebesgue measure.  The representation
\eqref{mvmixture} is not identifiable if no restrictions are placed on
$\cal F$, where by ``identifiable'' we mean that $g_{\f}$ has a {\em
  unique} representation of the form \eqref{mvmixture} and we do not
consider that ``label-switching'' --- i.e., reordering the $m$ pairs
$(\lambda_1, \phi_1), \ldots, (\lambda_m, \phi_m)$ --- produces a
distinct representation.

In the next sections we will sometimes have to distinguish between {\em
  parametric} and more general {\em nonparametric} situations.  This
distinction is related to the structure of the family $\CF$ of
distributions to which the component densities $\phi_j$ in model
\eqref{mvmixture} belong.  We say that the mixture is {\em parametric}
if $\CF$ is a parametric family, $\CF = \{\phi(\cdot|\vec\xi),
\vec\xi\in\bR^d\}$, indexed by a ($d$-dimensional) Euclidean parameter
$\vec\xi$.  A parametric family often used is the univariate Gaussian
family $\CF = \{\phi(\cdot|\mu,\sigma^2)=\mbox{density of
}\CN(\mu,\sigma^2), (\mu,\sigma^2)\in\bR\times\bR^+_*\}$, in which case
the model parameter reduces to $\f = (\vec \lambda,
(\mu_1,\sigma^2_1),\ldots,(\mu_m,\sigma^2_m))$.  For the multivariate
case, a possible parametric model is the {\em conditionally i.i.d.\
  normal model}, for which $\CF=\{\phi(\vec x_i) = \prod_{k=1}^r
f(x_{ik}), \mbox{$f(t)$ density of $\CN(\mu,\sigma^2)$}\}$ (this model
is included in \pkg{mixtools}; see Section~\ref{ss:nbcomp}).  An example
of a (multivariate) nonparametric situation is $\CF=\{\phi(\vec x_i) =
\prod_{k=1}^r f(x_{ik}), \mbox{$f(t)$ a univariate density on $\bR$}\}$, in
which case $\vec\f$ consists in a Euclidean part ($\vec\lb$) and a
nonparametric part $(f_1,\ldots,f_m)$.

As a simple example of a dataset to which mixture models may be applied,
consider the sample depicted in Figure \ref{geyser}. In the Old Faithful
dataset, measurements give time in minutes between eruptions of the Old
Faithful geyser in Yellowstone National Park, USA. These data are
included as part of the \pkg{datasets} package in \proglang{R}
\citep{r2009}; type \code{help("faithful")} in \proglang{R} for more
details.

<<faithful, echo=TRUE, results=hide>>=
library(mixtools)
data(faithful)
attach(faithful)
@

\setkeys{Gin}{width=0.6\textwidth}
\begin{figure}[h]
  \centering
<<geyser, fig=TRUE, eps=FALSE>>=
hist(waiting, main="Time between Old Faithful eruptions",
     xlab="Minutes",  ylab="", cex.main=1.5, cex.lab=1.5, cex.axis=1.4)
@
\caption{The Old Faithful dataset is clearly suggestive of a
  two-component mixture of symmetric components.}
\label{geyser}
\end{figure}

For the Old Faithful eruption data, a two-component mixture
model is clearly a reasonable model based on the bimodality evident in
the histogram. This example is analyzed by \citet{hunter2007ims}, who
compare a standard normal-mixture method for fitting it with a novel
semiparametric approach.  Both approaches are included in
\pkg{mixtools}; see Sections \ref{section:EMexample} and
\ref{section:SPexample} of this article.

In Section~\ref{section:EM} of the current article we review the
well-known class of EM algorithms for finite mixture models, a common
thread that runs throughout much of the rest of the article.  The
remaining sections discuss various categories of functions found in the
\pkg{mixtools} package, from cutpoint methods that relax distributional
assumptions for multivariate data by discretizing the data
(Section~\ref{section:cut}), to semi- and non-parametric methods that
eliminate distributional assumptions almost entirely depending on what
the identifiability of the model allows (Section~\ref{section:np}), to
methods that handle various mixtures of regressions
(Section~\ref{section:reg}).  Finally, Section \ref{section:misc}
describes several miscellaneous features of the \pkg{mixtools} package.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{EM algorithms for finite mixtures}
\label{section:EM}


\subsection{Missing data setup}

Much of the general methodology used in \pkg{mixtools} involves the
representation of the mixture problem as a particular case of maximum
likelihood estimation (MLE) when the observations can be viewed as
incomplete data.  This setup implies consideration of two sample spaces,
the sample space of the (incomplete) observations, and a sample space of
some ``complete'' observations, the characterization of which being that
the estimation can be performed explicitly at this level.  For instance,
in parametric situations, the MLE based on the complete data may exist
in closed form.  Among the numerous reference papers and monographs on
this subject are, e.g., the original EM algorithm paper by
\citet{dempster1977mli} and the finite mixture model book by
\citet{mclachlan2000fmm} and references therein.  We now give a brief
description of this setup as it applies to finite mixture models in
general.

The (observed) data consist of $n$ i.i.d. observations $\vec x = (\vec
x_1,\ldots,\vec x_n)$ from a density $g_\f$ given by
\eqref{mvmixture}. It is common to denote the density of the sample by
$\Bg_\f$, the $n$-fold product of $g_\f$, so that we write simply
$\Bx\sim \Bg_\f$. In the missing data setup, $\Bg_\f$ is called the
incomplete-data density, and the associated log-likelihood is
$L_{\Bx}(\f) = \sum_{i=1}^n \log g_\f(\vec x_i)$. The (parametric) ML
estimation problem consists in finding $\hat\f_{\Bx} =
\argmax_{\f\in\Phi} L_{\Bx}(\f)$, or at least finding a local maximum
--- there are certain well-known cases in which a finite mixture model
likelihood is unbounded \citep{mclachlan2000fmm}, but we ignore these
technical details for now. Calculating $\hat\f_{\Bx}$ even for a
parametric finite mixture model is known to be a difficult problem, and
considering $\Bx$ as incomplete data resulting from non-observed
complete data helps.

The associated complete data is denoted by $\Bc = (\vec c_1,\ldots, \vec
c_n)$, with density $\Bh_\f(\Bc)=\prod_{i=1}^n h_\f(\vec c_i)$ (there
exists a many-to-one mapping from $\Bc$ to $\Bx$, representing the loss
of information).  In the model for complete data associated with
model~\eqref{mvmixture}, each random vector $\vec C_i = (\vec X_i,\vec
Z_i)$, where $\vec Z_i = (Z_{ij},j=1,\ldots m)$, and $Z_{ij}\in\{0,1\}$
is a Bernoulli random variable indicating that individual $i$ comes from
component $j$. Since each individual comes from exactly one component,
this implies $\sum_{j=1}^m Z_{ij}=1$, and
$$
\Prob(Z_{ij} = 1) = \lambda_{j},\quad (\vec X_i|Z_{ij}=1) \sim \phi_j,
\quad j=1,\ldots,m.
$$
The complete-data density for one observation is thus
$$
h_\f(\vec c_i) = h_\f(\vec x_i,\vec z_i) = \sum_{j=1}^m \I_{z_{ij}}\lb_j
\phi_j (\vec x_i),
$$
In the parametric situation, i.e.\ when $\CF$ is a parametric family, it
is easy to check that the complete-data MLE $\hat\f_{\Bc}$ based on
maximizing $\log \Bh_\f(\Bc)$ is easy to find, provided that this is the
case for the family $\CF$.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{EM algorithms}
\label{sec:EM}

An EM algorithm iteratively maximizes, instead of the observed
log-likelihood $L_{\Bx}(\f)$, the operator
$$
Q(\f | \f^{(t)}) = \E \left[\log \Bh_\f(\BC)|\Bx,\f^{(t)} \right],
$$
where $\f^{(t)}$ is the current value at iteration~$t$, and the
expectation is with respect to the distribution $\Bk_\f(\Bc|\Bx)$ of
$\Bc$ given $\Bx$, for the value $\f^{(t)}$ of the parameter. The
iteration $\f^{(t)} \to \f^{(t+1)}$ is defined in the above general
setup by
\begin{enumerate}
\item E-step: compute $Q(\f | \f^{(t)})$

\item M-step: set $\f^{(t+1)} = \argmax_{\f\in\Phi}Q(\f | \f^{(t)})$
\end{enumerate}

For finite mixture models, the E-step does not depend on the structure
of $\CF$, since the missing data part is only related to the $\Bz$'s:
$$
\Bk_\f(\Bc|\Bx) = \prod_{i=1}^n k_\f(\vec z_i|\vec x_i).
$$
The $\Bz$ are discrete, and their distribution is given via Bayes'
theorem. The M-step itself can be split in two parts, the maximization
related to $\vec\lb$, which does not depend on $\CF$, and the
maximization related to $\vec \phi$, which has to be handled
specifically (say, parametrically, semi- or non-parametrically) for each
model.  Hence the EM algorithms for the models handled by the
\pkg{mixtools} package share the following common features:
\begin{enumerate}
\item{\bf E-step:\ } Calculate the ``posterior'' probabilities
  (conditional on the data and $\vec\theta^{(t)}$) of component
  inclusion,
\begin{equation}\label{posteriors}
  \post_{ij}^{(t)} \, \defn \,
  \Prob_{\vec\theta^{(t)}}(Z_{ij}=1| \vec x_i)
  = \frac{\lambda_j^{(t)} \phi_{j}^{(t)}(\vec x_{i})}
  {\sum_{j'=1}^m\lambda_{j'}^{(t)} \phi_{j'}^{(t)}(\vec x_{i})}
\end{equation}
for all $i=1,\ldots, n$ and $j=1, \ldots, m$.  Numerically, it can be
dangerous to implement equation (\ref{posteriors}) exactly as written
due to the possibility of the indeterminant form $0/0$ in cases where
$\vec x_i$ is so far from any of the components that all
$\phi_{j'}^{(t)}(\vec x_i)$ values result in a numerical underflow to
zero.  Thus, many of the routines in \pkg{mixtools} actually use the
equivalent expression
\begin{equation}\label{altposteriors}
  \post_{ij}^{(t)}
  = \left[ 1 +
    \sum_{j'\ne j} \frac{ \lambda_{j'}^{(t)} \phi_{j'}^{(t)}(\vec x_{i})}
    {\lambda_j^{(t)} \phi_{j}^{(t)}(\vec x_{i})} \right]^{-1}
\end{equation}
or some variant thereof.
\item{\bf M-step for $\vec\lb$:\ } Set
\begin{equation}\label{lambda}
  \lambda_j^{(t+1)} = \frac1n\sum_{i=1}^n \post_{ij}^{(t)} ,
  \quad\mbox{for $j=1, \ldots, m$.}
\end{equation}
\end{enumerate}


\subsection{An EM algorithm example}
\label{section:EMexample}

As an example, we consider the univariate normal mixture analysis of the
Old Faithful waiting data depicted in Figure \ref{geyser}. This fully
parametric situation corresponds to a mixture from the univariate
Gaussian family described in Section~\ref{s:intro}, where the $j$th
component density $\phi_j(x)$ in \eqref{mvmixture} is normal with mean
$\mu_j$ and variance $\sigma_j^2$.  This is a special case of the
general mixture-of-normal model that is well-studied in the literature
and for which other software, such as the \pkg{mclust}
\citep{Fraley+Raftery:2009} package for \proglang{R}, may also be used
for parameter estimation.  The M-step for the parameters
$(\mu_j,\sigma^2_j)$, $j=1,\ldots,m$ of this EM algorithm for such
mixtures of univariate normals is straightforward, and can be found,
e.g., in \citet{mclachlan2000fmm}.  The function \code{normalmixEM}
implements the algorithm in \pkg{mixtools}.  Code for the Old Faithful
example, using most of the default values (e.g., stopping criterion,
maximum number of iterations), is simply

<<normmixEM>>=
wait1 <- normalmixEM(waiting, lambda = .5, mu = c(55, 80), sigma = 5)
@

The code above will fit a 2-component mixture (because \code{mu} is a
vector of length two) in which the standard deviations are assumed equal
(because \code{sigma} is a scalar instead of a vector).  See
\code{help("normalmixEM")} for details about specifying starting values
for this EM algorithm.
<<geyserEM, eval=FALSE>>=
plot(wait1, density=TRUE, cex.axis=1.4, cex.lab=1.4, cex.main=1.8,
     main2="Time between Old Faithful eruptions", xlab2="Minutes")
@

\setkeys{Gin}{width=0.49\textwidth}
\begin{figure}[!h]
  \centering
<<geyserEM, echo=FALSE, results=tex>>=
for(i in 1:2){
  file=paste("geyserEM", i, ".pdf", sep="")
  pdf(file=file, paper="special", width=6, height=6)
  plot(wait1, whichplots=i, cex.axis = 1.4, cex.lab = 1.4, cex.main =
       1.8,   main2 = "Time between Old Faithful eruptions", xlab2 =
       "Minutes")
  dev.off()
  cat("\\includegraphics{", file, "}\n", sep="")
}
@
\caption{The Old Faithful waiting data fitted with a parametric EM
  algorithm in \pkg{mixtools}. Left: the sequence of log-likelihood
  values; Right: the fitted Gaussian components.}
\label{geyserEM}
\end{figure}

The \code{normalmixEM} function returns an object of class
\code{"mixEM"}, and the \code{plot} method for these objects delivers
the two plots given in Figure \ref{geyserEM}: the sequence $t\mapsto
L_{\Bx}(\f^{(t)})$ of observed log-likelihood values and the histogram
of the data with the $m$ ($m=2$ here) fitted Gaussian component
densities of $\CN(\hat\mu_j,\hat\sigma^2_j)$, $j=1,\ldots,m$, each
scaled by the corresponding $\hat\lambda_j$, superimposed.  The
estimator $\hat{\vec\theta}$ can be displayed by typing, e.g.,
<<geyserestimates>>=
wait1[c("lambda", "mu", "sigma")]
@

Alternatively, the same output may be obtained using the \code{summary}
method:
<<geysersummary>>=
summary(wait1)
@

\section{Cutpoint methods}
\label{section:cut}

Traditionally, most literature on finite mixture models has assumed that
the density functions $\phi_j(\vec x)$ of equation (\ref{mvmixture})
come from a known parametric family.  However, some authors have
recently considered the problem in which $\phi_j(\vec x)$ is unspecified
except for some conditions necessary to ensure the identifiability of
the parameters in the model.  One such set of conditions is as follows:

\citet{hettmansperger2000ani}; \citet{cruzmedina2004smm}; and
\citet{elmore2004ecc} treat the case in which $\phi_j(\vec x)$ equals
the product $f_j(x_i)\cdots f_j(x_r)$ for some univariate density
function $f_j$.  Thus, conditional on knowing that $\vec X$ comes from
the $j$th mixture component, the coordinates of $\vec X$ are independent
and identically distributed.  For this reason, this case is called the
conditionally i.i.d.\ model.

The authors named above have developed an estimation method for the
conditionally i.i.d.\ model.  This method, the {\em cutpoint approach},
discretizes the continuous measurements by replacing each
$r$-dimensional observation, say $\vec X_i= (x_{i1}, \ldots, x_{ir})$,
by the $p$-dimensional multinomial vector $(n_1, \ldots, n_p)$, where
$p\ge2$ is chosen by the experimenter along with a set of cutpoints
$-\infty = c_0 < c_1 < \cdots < c_p=\infty$, so that for $a=1, \ldots,
p$,
\[
n_a = \sum_{k=1}^r I\{c_{a-1} < x_{ik} \le c_a\}.
\]
Note that the multinomial distribution is guaranteed by the conditional
i.i.d.\ assumption, and the multinomial probability of the $a$th
category is equal to $\theta_a \equiv P_{}(c_{a-1}<X_{ik}\le c_a)$.

The cutpoint approach is completely general in the sense that it can be
applied to any number of components $m$ and any number of repeated
measures $r$, just as long as $r \ge 2m-1$, a condition that guarantees
identifiability \citep{elmore2003}.  However, some information is lost
in the discretization step, and for this reason it becomes difficult to
obtain density estimates of the component densities.  Furthermore, even
if the assumption of conditional independence is warranted, the extra
assumption of identically distributed coordinates may not be; and the
cutpoint method collapses when the coordinates are not identically
distributed.

As an illustration of the cutpoint approach applied to a dataset, we
show here how to use \pkg{mixtools} to reconstruct---almost---an example
from \citet{elmore2004ecc}.  The dataset is \code{Waterdata}, a
description of which is available by typing \code{help("Waterdata")}.
This dataset contains 8 observations on each of 405 subjects, where the
observations are angle degree measurements ranging from $-90$ to $90$
that describe the subjects' answers to a series of 8 questions related
to a conceptual task about how the surface of a liquid would be oriented
if the vessel containing it were tipped to a particular angle.  The
correct answer is 0 degree in all cases, yet the subjects showed a
remarkable variety of patterns of answers.  \citet{elmore2004ecc}
assumed the conditionally i.i.d.\ model (see \citet{benaglia2009} for an
in-depth discussion of this assumption and this dataset) with both $m=3$
and $m=4$ mixture components.  \citet{elmore2004ecc} summarized their
results by providing plots of estimated empirical distribution functions
for the component distributions, where these functions are given by
\begin{equation}\label{ecdf}
  \tilde F_j(x) = \frac{1}{mn\lambda_j}\sum_{i=1}^n \sum_{\ell=1}^r
  p_{ij}I\{x_{i\ell}\le x\}.
\end{equation}
In equation (\ref{ecdf}), the values of $\lambda_j$ and $p_{ij}$ are the
final maximum likelihood estimates of the mixing proportions and
posterior component membership probabilities that result from fitting a
mixture of $m$ multinomials (note in particular that the estimates of
the multinomial parameters $\theta_a$ for each component are not used in
this equation).

We cannot obtain the exact results of \citet{elmore2004ecc} because
those authors do not state specifically which cutpoints $c_a$ they use;
they merely state that they use thirteen cutpoints.  It appears from
their Figures 1 and 2 that these cutpoints occur approximately at
intervals of 10.5 degrees, starting at $-63$ and going through $63$;
these are the cutpoints that we adopt here.  The function
\code{makemultdata} will create a multinomial dataset from the original
data, as follows:
<<cutpoint>>=
data("Waterdata")
cutpts <- 10.5*(-6:6)
watermult <- makemultdata(Waterdata, cuts = cutpts)
@

Once the multinomial data have been created, we may apply the
\code{multmixEM} function to estimate the multinomial parameters via an
EM algorithm.
<<multmixEM>>=
set.seed(15)
theta4 <- matrix(runif(56), ncol = 14)
theta3 <- theta4[1:3,]
mult3 <- multmixEM(watermult, lambda = rep(1, 3)/3, theta = theta3)
mult4 <- multmixEM (watermult, lambda = rep (1, 4) / 4, theta = theta4)
@

Finally, \code{compCDF} calculates and plots the estimated distribution
functions of equation (\ref{ecdf}).  Figure \ref{WDcutpoint} gives plots
for both a 3-component and a 4-component solution; these plots are very
similar to the corresponding plots in Figures 1 and 2 of
\citet{elmore2004ecc}.
<<echo=TRUE, eval=FALSE>>=
cdf3 <- compCDF(Waterdata, mult3$posterior, lwd=2, lab=c(7, 5, 7),
                xlab="Angle in degrees", ylab="Component CDFs",
                main="Three-Component Solution")
cdf4 <- compCDF(Waterdata, mult4$posterior, lwd=2, lab=c(7, 5, 7),
                xlab="Angle in degrees", ylab="Component CDFs",
                main="Four-Component Solution")
@

<<cutpointplots, results=hide, echo=FALSE>>=
pdf(file="WDcutpoint3comp.pdf", paper="special", width=8, height=8)
cdf3 <- compCDF(Waterdata, mult3$posterior, lwd=3,
                xlab="Angle in degrees", lab=c(7, 5, 7), ylab="Component CDFs",
                main="Three-Component Solution", cex.axis=1.4, cex.lab=1.5,
                cex.main=1.5)
ltext <- paste(round(mult3$lam*100, 1), "%", sep="")
legend("bottomright", legend=ltext, pch=15:17, cex=1.5, pt.cex=1.35)
y <- compCDF(Waterdata, mult3$posterior, x=cutpts, makeplot=F)
for(i in 1:3) points(cutpts, y[i,], pch=14+i, cex=1.35)
dev.off()
pdf(file="WDcutpoint4comp.pdf", paper="special", width=8, height=8)
cdf4 <- compCDF(Waterdata, mult4$posterior, lwd=3,
                xlab="Angle in degrees", lab=c(7, 5, 7),
                ylab="Component CDFs", main="Four-Component Solution",
                cex.axis=1.4, cex.lab=1.5, cex.main=1.5)
ltext <- paste(round(mult4$lam*100,1), "%", sep="")
legend("bottomright", legend=ltext, pch=15:18, cex=1.5, pt.cex=1.35)
y <- compCDF(Waterdata, mult4$posterior, x=cutpts, makeplot=F)
for(i in 1:4) points(cutpts, y[i,], pch=14+i, cex=1.35)
dev.off()
@

\begin{figure}[!h]
  \centering
  \includegraphics[width=0.49\textwidth]{WDcutpoint3comp}
  \includegraphics[width=0.49\textwidth]{WDcutpoint4comp}
  \caption{Empirical cumulative distribution function (CDF) estimates
    for the three- and four-component multinomial cutpoint models for
    the water-level data; compare Figures 1 and 2 of
    \citet{elmore2004ecc}.  The 13 cutpoints used are indicated by the
    points in the plots, and the estimated mixing proportions for the
    various components are given by the legend.  }
  \label{WDcutpoint}
\end{figure}

As with the output of \code{normalmixEM} in Section~\ref{section:EM},
it is possible to summarize the output of the \code{multmixEM} function using
the \code{summary} method for \code{mixEM} objects:
<<summarymult4>>=
summary(mult4)
@


\section{Nonparametric and semiparametric methods}
\label{section:np}

In this section, we consider nonparametric multivariate finite mixture
models.  The first algorithm presented here was introduced by
\citet{benaglia2009} as a generalization of the stochastic
semiparametric EM algorithm of \citet{bordes2007sas}. Both algorithms
are implemented in \pkg{mixtools}.

\subsection{EM-like algorithms for mixtures of unspecified densities}
\label{section:EMlike}

Consider the mixture model described by equation \eqref{mvmixture}. If
we assume that the coordinates of the $\vec X_i$ vector are {\em
  conditionally independent}, i.e. they are independent conditional on
the subpopulation or component ($\phi_1$ through $\phi_m$) from which
$\vec X_i$ is drawn, the density in \eqref{mvmixture} can be rewritten
as:
\begin{equation}
  \label{mvmixture2}
  g_{\vec\theta}(\vec x_i) =
  \sum_{j=1}^m\lambda_j\prod_{k=1}^rf_{jk}(x_{ik}),
\end{equation}
where the function $f(\cdot)$, with or without subscripts, will always
denote a univariate density function. Here we do not assume that
$f_{jk}(\cdot)$ comes from a family of densities that may be indexed by
a finite-dimensional parameter vector, and we estimate these densities
using nonparametric density techniques. That is why we say that this
algorithm is a fully nonparametric approach.

The density in equation \eqref{mvmixture2} allows for a different
distribution for each component and each coordinate of $\vec
X_i$. Notice that if the density $f_{jk}(\cdot)$ does not depend on $k$,
we have the case in which the $\vec X_i$ are not only conditionally
independent but identically distributed as well. These are the two
extreme cases. In order to encompass both the conditionally i.i.d. case
and the more general case \eqref{mvmixture2} simultaneously in one
model, we allow that the coordinates of $\vec X_i$ are conditionally
independent and there exist {\em blocks} of coordinates that are also
identically distributed. If we let $b_k$ denote the block to which the
$k$th coordinate belongs, where $1\le b_k\le B$ and $B$ is the total
number of such blocks, then equation \eqref{mvmixture2} is replaced by
\begin{equation}\label{rmgeneral}
  g_{\vec\theta} (\vec x_i) = \sum_{j=1}^m \lambda_j \prod_{k=1}^r
  f_{j{b_k}} (x_{ik}).
\end{equation}

The indices $i$, $j$, $k$, and $\ell$ will always denote a generic
individual, component (subpopulation), coordinate (repeated
measurement), and block, respectively. Therefore, we will always have
$1\le i\le n$, $1\le j\le m$, $1\le k\le r$, and $1\le\ell\le B$.

The EM algorithm to estimate model \eqref{rmgeneral} has the E-step and
M-step described in Section~\ref{sec:EM}.  In equation
(\ref{posteriors}), we have $\phi_j^{(t)}(\vec x_i) = \prod_{k=1}^r
f_{jb_k}^{(t)}(x_{ik})$, where $f_{j\ell}^{(t)}(\cdot)$ is obtained by a
weighted nonparametric (kernel) density estimate, given by:
\begin{enumerate}
  \addtocounter{enumi}{2}
\item{\bf Nonparametric (Kernel) density estimation step:\ } For any
  real $u$, define for each component $j\in\{1, \ldots, m\}$ and each
  block $\ell\in\{1, \ldots, B\}$
  \begin{equation}
    \label{densest}
    f_{j\ell}^{t+1}(u) = \frac
    {1}{nh_{j\ell} C_\ell\lambda_{j}^{t+1}} \sum_{k=1}^r \sum_{i=1}^n
    \post_{ij}^{(t)} I\{b_k=\ell\}
    K\left(\frac{u-x_{ik}}{h_{j\ell}}\right),
  \end{equation}
  where $K(\cdot)$ is a kernel density function, $h_{j\ell}$ is the
  bandwidth for the $j$th component and $\ell$th block density estimate,
  and $C_\ell$ is the number of coordinates in the $\ell$th block.
\end{enumerate}

The function \code{npEM} implements this algorithm in
\pkg{mixtools}. This function has an argument \code{samebw} which, when
set to \code{TRUE} (the default), takes $h_{j\ell} = h$, for all $1 \le
j \le m$ and $1\le\ell\le B$, that is, the same bandwidth for all
components and blocks, while \code{samebw = FALSE} allows a different
bandwidth for each component and each block, as detailed in
\citet{bch:festchrift2009}.  This function will, if called using
\code{stochastic = TRUE}, replace the deterministic density estimation
step (\ref{densest}) by a {\em stochastic} density estimation step of
the type proposed by \citet{bordes2007sas}: First, generate $\vec
Z^{(t)}_{i} = (Z^{(t)}_{i1}, \ldots, Z^{(t)}_{im})$ as a multivariate
random vector with a single trial and success probability vector $\vec
p_i^{(t)} = (p_{i1}^{(t)}, \ldots, p_{1m}^{(t)})$, then in the M-step
for $\lambda_{j}^{t+1}$ in equation~(\ref{lambda}), replace
$p^{(t)}_{ij}$ by $Z^{(t)}_{ij}$ and let
\[
f_{j\ell}^{t+1}(u) = \frac {1}{nh_{j\ell} C_\ell\lambda_{j}^{t+1}}
\sum_{k=1}^r \sum_{i=1}^n Z_{ij}^{(t)} I\{b_k=\ell\}
K\left(\frac{u-x_{ik}}{h_{j\ell}}\right).
\]
In other words, the stochastic versions of these algorithms re-assign
each observation randomly at each iteration, according to the
$p_{ij}^{(t)}$ values at that iteration, to one of the $m$ components,
then the density estimate for each component is based only on those
observations that have been assigned to it.  Because the stochastic
algorithms do not converge the way a deterministic algorithm often does,
the output of \code{npEM} is slightly different when \code{stochastic =
  TRUE} than when \code{stochastic = FALSE}, the default.  See the
corresponding help file for details.

\citet{benaglia2009} also discuss specific cases of model
(\ref{rmgeneral}) in which some of the $f_{jb_k}(\cdot)$ densities are
assumed to be the same except for a location and scale change.  They
refer to such cases as semiparametric since estimating each
$f_{jb_k}(\cdot)$ involves estimating an unknown density as well as
multiple location and scale parameters.  For instance, equation (17) of
\citet{benaglia2009} sets
\begin{equation}
  \label{spEM}
  f_{j\ell}(x) = \frac{1}{\sigma_{j\ell}}f
  \left( \frac{x-\mu_{j\ell}}{\sigma_{j\ell}} \right),
\end{equation}
where $\ell=b_k$ for a generic $k$.

The \pkg{mixtools} package implements an algorithm for fitting model
(\ref{spEM}) in a function called \code{spEM}.  Details on the use of
this function may be obtained by typing \code{help("spEM")}.
Implementation of this algorithm and of that of the \code{npEM} function
requires updating the values of $f_{jb_k}(x_{ik})$ for all $i$, $j$, and
$k$ for use in the E-step (\ref{posteriors}).  To do this, the
\code{spEM} algorithm keeps track of an $n\times m$ matrix, called
$\Phi$ here, where
\[
\Phi_{ij} \equiv \phi_j(\vec x_i) = \prod_{k=1}^r f_{jb_k}(x_{ik}).
\]
The density estimation step of equation (\ref{densest}) updates the
$\Phi$ matrix for the $(t+1)$th iteration based on the most recent
values of all of the parameters.  For instance, in the case of model
(\ref{spEM}), we obtain
\begin{eqnarray*}
  \Phi_{ij}^{t+1}
  &=& \prod_{\ell=1}^B\prod_{k:b_k=\ell} \frac{1}{\sigma_{j\ell}^{t+1}}
  f^{t+1}
  \left( \frac{x-\mu_{j\ell}^{t+1}}{\sigma_{j\ell}^{t+1}} \right) \\
  &=& \prod_{\ell=1}^B
  \prod_{k:b_k=\ell} \frac{1}{\sigma_{j\ell}^{t+1}} \sum_{i'=1}^n
  \frac{p_{ij}^{t+1}}{hrn\lambda_j^{t+1}}
  \sum_{k'=1}^r
  K\left[
    \frac{\left(\frac{x_{ik}-\mu_{j\ell}^{t+1}}{\sigma_{j\ell}^{t+1}} \right) - (x_{i'k'} -
      \mu_{j\ell}^{t+1})}
    {h\sigma_{j\ell}^{t+1}}
  \right].
\end{eqnarray*}

\subsection{A univariate symmetric, location-shifted semiparametric
  example}
\label{section:SPexample}

Both \citet{hunter2007ims} and \citet{bordes2006set} study a particular
case of model (\ref{mvmixture}) in which $x$ is univariate and
\begin{equation}
  \label{spmodel}
  g_{\vec \theta}(x) = \sum_{j=1}^m\lambda_j \phi(x-\mu_j),
\end{equation}
where $\phi(\cdot)$ is a density that is assumed to be completely
unspecified except that it is symmetric about zero.  Because each
component distribution has both a nonparametric part $\phi(\cdot)$ and a
parametric part $\mu_j$, we refer to this model as semiparametric.

Under the additional assumption that $\phi(\cdot)$ is absolutely
continuous with respect to Lebesgue measure, \citet{bordes2007sas}
propose a stochastic algorithm for estimating the model parameters,
namely, $(\vec\lambda, \vec\mu, \phi)$.  This algorithm is implemented
by the \pkg{mixtools} function \code{spEMsymloc}.  This function also
implements a nonstochastic version of the algorithm, which is the
default and which is a special case of the general algorithm described
in Section~\ref{section:EMlike}.

<<spsymmplots, results=hide, echo=FALSE>>=
pdf(file="spsymmfig1.pdf", paper="special", width=8, height=8)
par(mar=0.1+c(5,4.2,4,1.8))
plot(wait1, which = 2, cex.axis = 1.4, cex.lab = 1.5, cex.main = 1.5,
     main2 = "Time between Old Faithful eruptions", xlab2 = "Minutes")
wait2 <- spEMsymloc(waiting, mu0 = c(55, 80))
plot(wait2, lty = 2, newplot = FALSE, addlegend = FALSE)
dev.off()
pdf(file="spsymmfig2.pdf", paper="special", width=8, height=8)
par(mar=0.1+c(5,4.2,4,1.8))
wait2a <- spEMsymloc(waiting, mu0 = c(55, 80), bw = 1)
wait2b <- spEMsymloc(waiting, mu0 = c(55, 80), bw = 6)
plot(wait2a, lty = 1, addlegend = FALSE, cex.axis = 1.4, cex.lab = 1.5,
     cex.main = 1.5, title = "Time between Old Faithful eruptions",
     xlab = "Minutes")
plot(wait2b, lty = 2, newplot = FALSE, addlegend = FALSE)
dev.off()
@

\begin{figure}[h]
  \centering
  \includegraphics[height=3in,width=3in]{spsymmfig1}
  \includegraphics[height=3in,width=3in]{spsymmfig2}
  \caption{The Old Faithful dataset, fit using different algorithms in
    \pkg{mixtools}. Left: the fitted Gaussian components (solid) and a
    semiparametric fit assuming model (\ref{spmodel}) with the default
    bandwidth of $4.0$ (dashed); Right: the same model (\ref{spmodel})
    using bandwidths of $1.0$ (solid) and $6.0$ (dashed).}
  \label{spsymmfig}
\end{figure}

As noted in Figure \ref{geyser}, model (\ref{spmodel}) appears to be an
appropriate model for the Old Faithful waiting times dataset.  Here, we
provide code that applies the \code{spEMsymloc} function to these data.
First, we display the normal mixture solution of Figure \ref{geyserEM}
with a semiparametric solution superimposed, in Figure
\ref{spsymmfig}(a):
<<plotspsymm, eval=FALSE>>=
plot(wait1, which = 2, cex.axis = 1.4, cex.lab = 1.4, cex.main = 1.8,
     main2 = "Time between Old Faithful eruptions", xlab2 = "Minutes")
wait2 <- spEMsymloc(waiting, mu0 = c(55, 80))
plot(wait2, lty = 2, newplot = FALSE, addlegend = FALSE)
@

Because the semiparametric version relies on a kernel density estimation
step (\ref{densest}), it is necessary to select a bandwidth for this
step.  By default, \code{spEMsymloc} uses a fairly simplistic approach:
It applies ``Silverman's rule of thumb'' \citep{silverman1986des} to the
entire dataset using the \code{bw.nrd0} function in \proglang{R}.  For
the Old Faithful waiting time dataset, this bandwidth is about~$4$:
<<bandwidth>>=
bw.nrd0(waiting)
@

But the choice of bandwidth can make a big difference, as seen in Figure
\ref{spsymmfig}(b).
<<plotbweffect, eval=FALSE>>=
wait2a <- spEMsymloc(waiting, mu0 = c(55, 80), bw = 1)
wait2b <- spEMsymloc(waiting, mu0 = c(55, 80), bw = 6)
plot(wait2a, lty = 1, addlegend = FALSE, cex.axis = 1.4,
     cex.lab = 1.4, cex.main = 1.8, xlab = "Minutes",
     title = "Time between Old Faithful eruptions")
plot(wait2b, lty = 2, newplot = FALSE, addlegend = FALSE)
@

We find that with a bandwidth near $2$, the semiparametric solution
looks quite close to the normal mixture solution of Figure
\ref{geyserEM}.  Reducing the bandwidth further results in the
``bumpiness'' exhibited by the solid line in Figure \ref{spsymmfig}(b).
On the other hand, with a bandwidth of 8, the semiparametric solution
completely breaks down in the sense that algorithm tries to make each
component look similar to the whole mixture distribution.  We encourage
the reader to experiment by changing the bandwidth in the above code.

\subsection{A trivariate Gaussian example}
\label{ss:trigauss}

As a first simple, nonparametric example, we simulate a Gaussian
trivariate mixture with independent repeated measures and a shift of
location between the two components in each coordinate, i.e., $m=2$,
$r=3$, and $b_k=k$, $k=1,2,3$.  The individual densities $f_{jk}$ are
the densities of $\CN(\mu_{jk},1)$, with component means $\vec\mu_1 =
(0,0,0)$ and $\vec\mu_2=(3,4,5)$. This example was introduced by
\citet{hall2005nim} then later reused by \citet{benaglia2009} for
comparison purposes.  Note that the parameters in this model are
identifiable, since \citet{hall2003nec} showed that for two components
($m=2$), identifiability holds in model~\eqref{mvmixture} is under mild
assumptions as long as $r\ge3$, even in the most general case in which
$b_k=k$ for all $k$.

A function \code{ise.npEM} has been included in \pkg{mixtools} for
numerically computing the integrated squared error (ISE) relative to a
user-specified true density for a selected estimated density $\hat
f_{jk}$ from \code{npEM} output.  Each density $\hat f_{jk}$ is computed
using equation~(\ref{densest}) together with the posterior probabilities
after convergence of the algorithm, i.e., the final values of the
$\post_{ij}^t$ (when \code{stochastic = FALSE}).  We illustrate the
usage of \code{ise.npEM} in this example by running a Monte Carlo
simulation for $S$ replications, then computing the square root of the
mean integrated squared error (MISE) for each density, where
\[ {\rm MISE} = \frac{1}{S}\sum_{s=1}^S \int \left(\hat
  f_{jk}^{(s)}(u)-f_{jk}(u)\right)^2\,du,\quad j=1,2 \mbox{ and }
k=1,2,3.
\]

For this example, we first set up the model true parameters with $S=100$
replications of $n=300$ observations each:
<<gaussexample>>=
m <- 2; r <- 3; n <- 300; S <- 100
lambda <- c(0.4, 0.6)
mu <- matrix(c(0, 0, 0, 3, 4, 5), m, r, byrow = TRUE)
sigma <- matrix(rep(1, 6), m, r, byrow = TRUE)
@

Next, we set up ``arbitrary'' initial centers, a matrix for storing sums
of integrated squared errors, and an integer storing the number of
suspected instances of label switching that may occur during the
replications:
<<gaussinitial>>=
centers <- matrix(c(0, 0, 0, 4, 4, 4), 2, 3, byrow = TRUE)
ISE <- matrix(0, m, r, dimnames = list(Components = 1:m, Blocks = 1:r))
nblabsw <- 0
@
Finally, we run the Monte Carlo simulation, using the \code{samebw =
  FALSE} option since it is more appropriate for this location-shift
model:
<<sqMISE>>=
set.seed(1000)
for (mc in 1:S) {
  x <- rmvnormmix(n, lambda, mu, sigma)
  a <- npEM(x, centers, verb = FALSE, samebw = FALSE)
  if (a$lambda[1] > a$lambda[2]) nblabsw <- nblabsw + 1
  for (j in 1:m) {
     for (k in 1:r) {
     ISE[j, k] <- ISE[j, k] + ise.npEM(a, j, k, dnorm,
         lower = mu[j, k] - 5, upper = mu[j, k] + 5, plots = FALSE,
         mean = mu[j, k], sd = sigma[j, k])$value #$
    }
  }
}
MISE <- ISE/S
print(sqMISE <- sqrt(MISE))
@

We can examine the \code{npEM} output from the last replication above
using
<<summarygauss>>=
summary(a)
@

We can also get plots of the estimated component densities for each
block (recall that in this example, block $\ell$ consists only of
coordinate $\ell$) using the \code{plot} function. The resulting plots
are given in Figure~\ref{fig:gausstrivariate}.
<<plotgauss3rm, eval=FALSE>>=
plot(a)
@

<<gauss3rm, echo=FALSE, results=hide>>=
pdf("gauss3rm.pdf", paper="special", width=10, height=5)
par(mfrow=c(1,3), ask=F)
plot(a)
dev.off()
@

\begin{figure}[h]
  \centering
  \includegraphics[width=.99\textwidth]{gauss3rm}
  \caption{Output of the \code{npEM} algorithm for the trivariate
    Gaussian model with independent repeated measures.}
  \label{fig:gausstrivariate}
\end{figure}


\subsection{A more general multivariate nonparametric example}
\label{sec:generalmv}

In this section, we fit a more difficult example, with non-multimodal
mixture densities (in block \#2), heavy-tailed distributions, and
different scales among the coordinates. The model is multivariate with
$r=5$ repeated measures and $m=2$ components (hence identifiability
holds; cf.\ \citet{hall2003nec} as cited in
Section~\ref{ss:trigauss}). The $5$ repeated measures are grouped into
$B=2$ blocks, with $b_1=b_2=b_3=1$ and $b_4=b_5=2$.  Block $1$
corresponds to a mixture of two noncentral Student $t$ distributions,
$t'(2,0)$ and $t'(10,8)$, where the first parameter is the number of
degrees of freedom, and the second is the non-centrality. Block~2
corresponds to a mixture of Beta distributions, ${\cal B}(1,1)$ (which
is actually the uniform distribution over $[0,1]$) and ${\cal
  B}(1,5)$. The first component weight is $\lambda_1 = 0.4$. The true
mixtures are depicted in Figure~\ref{fig:true5rm}.

<<true5rm, echo=FALSE, results=hide>>=
pdf("truepdf5rm_block1.pdf")
par(mar=0.1+c(5,4.2,4,1.5))
x <- seq(-10, 25, len=250)
plot(x, .4* dt(x, 2, 0) + .6 * dt(x, 10, 8), type="l", lwd=3, col=2,
     cex.axis=1.4, cex.lab=1.5, cex.main=1.5, main="Block 1", xlab="",
     ylab="Density")
lines (x, .4*dt(x, 2, 0), lwd=4, lty=2)
lines (x, .6*dt(x, 10, 8), lwd=4, lty=2)
dev.off()
pdf("truepdf5rm_block2.pdf")
par(mar=0.1+c(5,4.2,4,1.5))
x <- seq(0, 1, len=250)
plot(x, .4 + .6 * dbeta(x, 1, 5), type="l", lwd=3, col=2, cex.axis=1.4,
   cex.lab=1.5, cex.main=1.5, main="Block 2", xlab="", ylab="Density",
   ylim= c(0, 3.4))
lines (x, rep(.4, 250), lwd=4, lty=2)
lines (x, .6*dbeta(x, 1, 5), lwd=4, lty=2)
dev.off()
@

\begin{figure}[h]
  \centering
  \includegraphics[height=2.5in,width=2.5in]{truepdf5rm_block1}
  \includegraphics[height=2.5in,width=2.5in]{truepdf5rm_block2}
  \caption{True densities for the mixture of
    Section~\ref{sec:generalmv}, with individual component densities
    (scaled by $\lambda_j$) in dotted lines and mixture densities in
    solid lines.  The noncentral $t$ mixture of coordinates 1 through 3
    is on the left, the beta mixture of coordinates 4 and 5 on the
    right.}
  \label{fig:true5rm}
\end{figure}


To fit this model in \pkg{mixtools}, we first set up the model
parameters:
<<parameters5rm>>=
m <- 2; r <- 5
lambda <- c(0.4, 0.6)
df <- c(2, 10); ncp <- c(0, 8)
sh1 <- c(1, 1) ; sh2 <- c(1, 5)
@

Then we generate a pseudo-random sample of size $n=300$ from this model:
<<generate5rm>>=
n <- 300; z <- sample(m, n, rep = TRUE, prob = lambda)
r1 <- 3; z2 <- rep(z, r1)
x1 <- matrix(rt(n * r1, df[z2], ncp[z2]), n, r1)
r2 <- 2; z2 <- rep(z, r2)
x2 <- matrix(rbeta(n * r2, sh1[z2], sh2[z2]), n, r2)
x <- cbind(x1, x2)
@

For this example in which the coordinate densities are on different
scales, it is obvious that the bandwidth in \code{npEM} should depend on
the blocks and components.  We set up the block structure and some
initial centers, then run the algorithm with the option \code{samebw =
  FALSE}:
<<npEM5rm>>=
id <- c(rep(1, r1), rep(2, r2))
centers <- matrix(c(0, 0, 0, 1/2, 1/2, 4, 4, 4, 1/2, 1/2), m, r,
                  byrow = TRUE)
b <- npEM(x, centers, id, eps = 1e-8, verb = FALSE, samebw = FALSE)
@
Figure~\ref{fig:npEM5rm} shows the resulting density estimates, which
may be obtained using the plotting function included in \pkg{mixtools}:

<<plot5rm, eval=FALSE>>=
plot(b, breaks = 15)
@

% plot(b, breaks = 15, cex.main = 1.5, cex.lab = 1.5, cex.axis = 1.4,
%     cex.legend = 1.5)

<<plot5rmcommands, echo=FALSE, results=hide>>=
pdf("npEM5rm.pdf", width=8, height=5)
par(mfrow=c(1,2))
plot(b, breaks = 15)

dev.off()
@

\begin{figure}[h]
  \centering
  \includegraphics[width=.95\textwidth]{npEM5rm}
  \caption{Result of plotting \code{npEM} output for the example of
    Section~\ref{sec:generalmv}.  Since $n=300$, the histogram on the
    left includes 900 observations and the one on the right includes
    600.}
  \label{fig:npEM5rm}
\end{figure}

Finally, we can compute the ISE of the estimated density relative to the
truth for each block and component. The corresponding output is depicted
in Figure \ref{fig:ISEnpEM5rm}.
<<ISEnpEM5rm, eval=FALSE>>=
par(mfrow=c(2,2))
for (j in 1:2){
  ise.npEM(b, j, 1, truepdf = dt, lower = ncp[j] - 10,
           upper = ncp[j] + 10, df = df[j], ncp = ncp[j])
  ise.npEM(b, j, 2, truepdf = dbeta, lower = -0.5,
           upper = 1.5,  shape1 = sh1[j], shape2 = sh2[j])
}
@ 

<<plotISEnpEM5rm, echo=FALSE, results=hide>>=
options(warn=-1)
pdf("ISEnpEM5rm.pdf", width=8, height=8)
par(mfrow = c(2, 2))
for (j in 1:2){
  ise.npEM(b, j, 1, truepdf = dt, lower = ncp[j] - 10,
           upper = ncp[j] + 10, df = df[j], ncp = ncp[j])
  ise.npEM(b, j, 2, truepdf = dbeta, lower = -0.5,
           upper = 1.5,  shape1 = sh1[j], shape2 = sh2[j])
}
dev.off()
@

\begin{figure}[h]
  \centering
  \includegraphics[height=5in,width=6in]{ISEnpEM5rm}
  \caption{\code{ise.npEM} output for the 5-repeated measures example;
    the true densities are $f_{11}\equiv t'(2,0)$, $f_{21}\equiv
    t'(10,8)$, $f_{12}\equiv {\cal U}_{(0,1)}$, $f_{22}\equiv {\cal
      B}(1,5)$.}
  \label{fig:ISEnpEM5rm}
\end{figure}

\section{Mixtures of regressions}
\label{section:reg}

\subsection{Mixtures of linear regressions}

Consider a mixture setting where we now assume $\textbf{X}_{i}$ is a
vector of covariates observed with a response $Y_{i}$.  The goal of
mixtures of regressions is to describe the conditional distribution of
$Y_{i}|\textbf{X}_{i}$. Mixtures of regressions have been extensively
studied in the econometrics literature and were first introduced by
\citet{quandt1972sr} as the \textit{switching regimes} (or
\textit{switching regressions}) problem. A switching regimes system is
often compared to \textit{structural change} in a system
\citep{quandtram1978sr}. A structural change assumes the system depends
deterministically on some observable variables, but switching regimes
implies one is unaware of what causes the switch between regimes. In the
case where it is assumed there are two heterogeneous classes,
\citet{quandt1972sr} characterized the switching regimes problem ``by
assuming that nature chooses between regimes with probabilities
$\lambda$ and $1-\lambda$''.

Suppose we have $n$ independent univariate observations,
$y_{1},\ldots,y_{n}$, each with a corresponding vector of predictors,
$\textbf{x}_{1},\ldots,\textbf{x}_{n}$, with
$\textbf{x}_{i}=(x_{i,1},\ldots,x_{i,p})^\top$ for $i=1,\ldots,n$. We
often set $x_{i,1}=1$ to allow for an intercept term. Let
$\textbf{y}=(y_{1},\ldots,y_{n})^\top$ and let $\underline{\textbf{X}}$
be the $n\times p$ matrix consisting of the predictor vectors.

Suppose further that each observation $(y_{i}, \vec x_i)$ belongs to one
of $m$ classes. Conditional on membership in the $j$th component, the
relationship between $y_{i}$ and $\textbf{x}_{i}$ is the normal
regression model
\begin{equation}\label{regmodel}
  y_{i}=\textbf{x}_{i}^\top\vec{\beta}_{j}+\epsilon_{i},
\end{equation}
where $\epsilon_{i}\thicksim \CN(0,\sigma^{2}_{j})$ and
$\vec{\beta}_{j}$ and $\sigma_{j}^{2}$ are the $p$-dimensional vector of
regression coefficients and the error variance for component $j$,
respectively.

Accounting for the mixture structure, the conditional density of
$y_{i}|\vec x_i$ is
\begin{equation}\label{mor}
  g_{\vec\theta}(y_{i}|\textbf{x}_{i})=\sum_{j=1}^{m}\lambda_{j}
  \phi(y_{i} | \textbf{x}_{i}^\top\vec{\beta}_{j},\sigma^{2}_{j}),
\end{equation}
where $\phi(\cdot|\textbf{x}^\top\vec{\beta}_{j},\sigma^{2}_{j})$ is the
normal density with mean $\textbf{x}^\top\vec\beta$ and variance
$\sigma^2$.  Notice that the model parameter for this setting is
$\vec\theta=(\vec\lambda,(\vec{\beta}_{1},\sigma^2_{1}),\ldots,(\vec{\beta}_{m},\sigma^2_{m}))$.
The mixture of regressions model (\ref{mor}) differs from the well-known
mixture of multivariate normals model
$(Y_{i},\textbf{X}_{i}^\top)^\top\thicksim
\sum_{j=1}^{m}\lambda_{j}\CN_{p+1}(\vec{\mu}_{j},\Sigma_{j})$ because
model (\ref{mor}) makes no assertion about the marginal distribution of
$\textbf{X}_{i}$, whereas the mixture of multivariate normals specifies
that $\textbf{X}_{i}$ itself has a mixture of multivariate normals
distribution.

<<gnpdata, echo=FALSE, results=hide>>=
data("CO2data")
attach(CO2data)
pdf("gnpdata.pdf")
par(mar=0.1+c(5,4.2,4,1.5))
plot(GNP, CO2, xlab="Gross National Product",
     ylab=expression(paste(CO[2]," per Capita")),
     cex.lab=1.5, cex.main=1.5, cex.axis=1.4,
     main="1996 GNP and Emissions Data")
text(GNP, CO2, country, adj=c(.5,-.5))
dev.off()
@

\begin{figure}[h]
  \centering
  \includegraphics[height=3in,width=3in]{gnpdata.pdf}
  \caption{1996 data on gross national product (GNP) per capita and
    estimated carbon dioxide ($\textrm{CO}_{2}$) emissions per capita.
    Note that ``CH'' stands for Switzerland, not China.}
  \label{gnpdata}
\end{figure}

As a simple example of a dataset to which a mixture of regressions
models may be applied, consider the sample depicted in Figure
\ref{gnpdata}. In this dataset, the measurements of carbon dioxide
($\textrm{CO}_{2}$) emissions are plotted versus the gross national
product (GNP) for $n=28$ countries. These data are included
\pkg{mixtools}; type \code{help("CO2data")} in \proglang{R} for more
details.  \citet{hurn} analyzed these data using a mixture of
regressions from the Bayesian perspective, pointing out that ``there do
seem to be several groups for which a linear model would be a reasonable
approximation.''  They further point out that identification of such
groups could clarify potential development paths of lower GNP countries.


\subsection{EM algorithms for mixtures of regressions}

A standard EM algorithm, as described in Section~\ref{section:EM}, may
be used to find a local maximum of the likelihood surface.  
\citet{deveaux1989} describes EM algorithms for mixtures of
regressions in more detail, including proposing a method for
choosing a starting point in the parameter space.
The E-step
is the same as for any finite mixture model EM algorithm; i.e., the
$p_{ij}^{(t)}$ values are updated according to equation
(\ref{posteriors})---or, in reality, equation
(\ref{altposteriors})---where each $\phi_j^{(t)}(\vec x_i)$ is replaced
in the regression context by $\phi(y_i | \vec x_i^\top\vec\beta_j,
\sigma_j^2)$:
\begin{equation}\label{regposteriors}
  \post_{ij}^{(t)} = \left[ 1 + \sum_{j'\ne j} \frac{ \lambda_{j'}^{(t)} \phi(y_i | \vec x_i^\top\vec\beta_{j'}, \sigma_{j'}^2)}{\lambda_j^{(t)} \phi(y_i | \vec x_i^\top\vec\beta_j, \sigma_j^2)} \right]^{-1}
\end{equation}
The update to the $\lambda$ parameters in the M-step, equation
(\ref{lambda}), is also the same.  Letting
$\textbf{W}_{j}^{(t)}=\textrm{diag}(\post_{1j}^{(t)},\ldots,\post_{nj}^{(t)})$,
the additional M-step updates to the $\vec\beta$ and $\sigma$ parameters
are given by
\begin{eqnarray}\label{betaEM}
  \vec{\beta}_{j}^{(t+1)} &=& (\underline{\textbf{X}}^\top\textbf{W}_{j}^{(t)}\underline{\textbf{X}})^{-1}\underline{\textbf{X}}^\top
  \textbf{W}_{j}^{(t)}\textbf{y}
  \quad \mbox{and} \\
\label{sigma}
\sigma_{j}^{2(t+1)} &=& \frac{\biggl\|\textbf{W}_{j}^{1/2(t)}(\textbf{y}-\underline{\textbf{X}}^\top\vec{\beta}_{j}^{(t+1)})\biggr\|^{2}}{\mbox{tr}(\textbf{W}_{j}^{(t)})},
\end{eqnarray}
where $\|\textbf{A}\|^{2}=\textbf{A}^\top\textbf{A}$ and
$\mbox{tr}(\textbf{A})$ means the trace of the matrix $\textbf{A}$.
Notice that equation (\ref{betaEM}) is a weighted least squares (WLS)
estimate of $\vec{\beta}_{j}$ and equation (\ref{sigma}) resembles the
variance estimate used in WLS.

Allowing each component to have its own error variance $\sigma_j^2$
results in the likelihood surface having no maximizer, since the
likelihood may be driven to infinity if one component gives a regression
surface passing through one or more points exactly and the variance for
that component is allowed to go to zero.  A similar phenomenon is
well-known in the finite mixture-of-normals model where the component
variances are allowed to be distinct \citep{mclachlan2000fmm}.  However,
in practice we observe this behavior infrequently, and the
\pkg{mixtools} functions automatically force their EM algorithms to
restart at randomly chosen parameter values when it occurs.  A local
maximum of the likelihood function, a consistent version of which is
guaranteed to exist by the asymptotic theory as long as the model is
correct and all $\lambda_j$ are positive, usually results without any
restarts.

The function \code{regmixEM} implements the EM algorithm for mixtures of
regressions in \pkg{mixtools}.  This function has arguments that control
options such as adding an intercept term, \code{addintercept = TRUE};
forcing all $\vec\beta_j$ estimates to be the same, \code{arbmean =
  FALSE} (for instance, to model outlying observations as having a
separate error variance from the non-outliers); and forcing all
$\sigma_j^2$ estimates to be the same, \code{arbvar = FALSE}.  For
additional details, type \code{help("regmixEM")}.

As an example, we fit a 2-component model to the GNP data shown in
Figure \ref{gnpdata}.  \citet{hurn} and \citet{youngphd} selected 2
components for this dataset using model selection criteria, Bayesian
approaches to selecting the number of components, and a bootstrapping
approach.  The function \code{regmixEM} will be used for fitting a
2-component mixture of regressions by an EM algorithm:
<<results=hide>>=
data("CO2data")
attach(CO2data)
@ 
<<CO2reg>>=
CO2reg <- regmixEM(CO2, GNP, lambda = c(1, 3) / 4,
                   beta = matrix(c(8, -1, 1, 1), 2, 2), sigma = c(2, 1))
@

We can then pull out the final observed log-likelihood as well as
estimates for the 2-component fit, which include $\hat{\lambda}$,
$\hat{\vec{\beta}}_{1}$, $\hat{\vec{\beta}}_{2}$, $\hat{\sigma}_{1}$,
and $\hat{\sigma}_{2}$:
<<summaryCO2reg>>=
summary(CO2reg)
@

The reader is encouraged to alter the starting values or let the
internal algorithm generate random starting values.  However, this fit
seems appropriate and the solution is displayed in Figure \ref{co2EM}
along with 99\% Working-Hotelling Confidence Bands, which are
constructed automatically by the \code{plot} method in this case by
assigning each point to its most probable component and then fitting two
separate linear regressions:

<<plotCO2reg, eval=FALSE>>=
plot(CO2reg, density = TRUE, alpha = 0.01, cex.main = 1.5, cex.lab = 1.5,
     cex.axis = 1.4)
@

\setkeys{Gin}{width=0.49\textwidth}
\begin{figure}[!h]
  \centering
<<trueplotCO2reg, echo=FALSE, results=tex>>=
for(i in 1:2){
    file=paste("CO2reg", i, ".pdf", sep="")
    pdf(file=file, paper="special", width=6, height=6)
    plot(CO2reg, whichplots=i, alpha = 0.01, cex.main = 1.5, cex.lab = 1.5,
         cex.axis = 1.4)
    dev.off()
    cat("\\includegraphics{", file, "}\n", sep="")
}
@
\caption{The GNP data fitted with a 2-component parametric EM
  algorithm in \pkg{mixtools}. Left: the sequence of log-likelihood
  values, $L_{\Bx}(\f^{(t)})$; Right: the fitted regression lines with
  99\% Working-Hotelling Confidence Bands.}
\label{co2EM}
\end{figure}


\subsection{Predictor-dependent mixing proportions}
\label{section:pdmp}

Suppose that in model (\ref{mor}), we replace $\lambda_j$ by
$\lambda_{j}(\textbf{x}_{i})$ and assume that the mixing proportions
vary as a function of the predictors $\textbf{x}_{i}$.  Allowing this
type of flexibility in the model might be useful for a number of
reasons.  For instance, sometimes it is the proportions $\lambda_j$ that
are of primary scientific interest, and in a regression setting it may
be helpful to know whether these proportions appear to vary with the
predictors.  As another example, consider a \code{regmixEM} model using
\code{arbmean = FALSE} in which the mixture structure only concerns the
error variance: In this case, $\lambda_j(\vec x)$ would give some sense
of the proportion of outliers in various regions of the predictor space.

One may assume that $\lambda_{j}(\textbf{x})$ has a particular
parametric form, such as a logistic function, which introduces new
parameters requiring estimation. This is the idea of the
\textit{hierarchical mixtures of experts} (HME) procedure
\citep{jacobsall}, which is commonly used in neural networks and which
is implemented, for example, in the \pkg{flexmix} package for
\proglang{R} \citep{jss:Leisch:2004, Grun+Leisch:2008}.  However, a
parametric form of $\lambda_{j}(\textbf{x})$ may be too restrictive; in
particular, the logistic function is monotone, which may not
realistically capture the pattern of change of $\lambda_j$ as a function
of $\vec x$.  As an alternative, \citet{young2009mor} propose a
nonparametric estimate of $\lambda_{j}(\textbf{x}_{i})$ that uses ideas
from kernel density estimation.

The intuition behind the approach of \citet{young2009mor} is as follows:
The M-step estimate (\ref{lambda}) of $\lambda_j$ at each iteration of a
finite mixture model EM algorithm is simply an average of the
``posterior'' probabilities $p_{ij}=\E(Z_{ij}|\mbox{data})$.  As a
substitute, the nonparametric approach uses local linear regression to
approximate the $\lambda_j(\textbf{x})$ function.  Considering the case
of univariate $x$ for simplicity, we set $\lambda_j(x) =
\hat\alpha_{0j}(x)$, where
\begin{equation}\label{lambdax}
(\hat\alpha_{0j}(x), \hat\alpha_{1j}(x))=
\arg\min_{(\alpha_0, \alpha_1)} \sum_{i=1}^n
K_h(x-x_i) \left[ p_{ij} - \alpha_0 - \alpha_1(x-x_i) \right]^2
\end{equation}
and $K_h(\cdot)$ is a kernel density function with scale parameter
(i.e., bandwidth) $h$.  It is straightforward to generalize equation
(\ref{lambdax}) to the case of vector-valued $\vec x$ by using a
multivariate kernel function.

\citet{young2009mor} give an iterative algorithm for estimating mixture
of regression parameters that replaces the standard $\lambda_j$ updates
(\ref{lambda}) by the kernel-weighted version (\ref{lambdax}).  The
algorithm is otherwise similar to a standard EM; thus, like the
algorithm in Section~\ref{section:EMlike} of this article, the resulting
algorithm is an EM-like algorithm.  Because only the $\lambda_j$
parameters depend on $\vec x$ (and are thus ``locally estimated''),
whereas the other parameters (the $\vec\beta_j$ and $\sigma_j$) can be
considered to be globally estimated, \citet{young2009mor} call this
algorithm an iterative global/local estimation (IGLE) algorithm.
Naturally, it replaces the usual E-step (\ref{regposteriors}) by a
modified version in which each $\lambda_j$ is replaced by
$\lambda_j(x_i)$.

The function \code{regmixEM.loc} implements the IGLE algorithm in
\pkg{mixtools}. Like the \code{regmixEM} function, \code{regmixEM.loc}
has the flexibility to include an intercept term by using
\code{addintercept = TRUE}.  Moreover, this function has the argument
\code{kern.l} to specify the kernel used in the local estimation of the
$\lambda_{j}(\textbf{x}_{i})$.  Kernels the user may specify include
\code{"Gaussian"}, \code{"Beta"}, \code{"Triangle"}, \code{"Cosinus"},
and \code{"Optcosinus"}.  Further numeric arguments relating to the
chosen kernel include \code{kernl.g} to specify the shape parameter for
when \code{kern.l = "Beta"} and \code{kernl.h} to specify the bandwidth
which controls the size of the window used in the local estimation of
the mixing proportions. See the corresponding help file for additional
details.

For the GNP and emissions dataset, Figure \ref{co2EM} indicates that the
assumption of constant weights for the component regressions across all
values of the covariate space may not be appropriate.  The countries
with higher GNP values appear to have a greater probability of belonging
to the first component (i.e., the red line in Figure \ref{co2EM}).  We
will therefore apply the IGLE algorithm to this dataset.

We will use the triweight kernel in equation (\ref{lambdax}), which is
given by setting $\gamma=3$ in
\begin{equation}\label{beta}
  K_{h}(x)=\frac{1}{hB(1/2,\gamma+1)}\left(1-\frac{x^2}{h^2}\right)^{\gamma}_{+},
\end{equation}
where $B(x,y)=\Gamma(x)\Gamma(y)/\Gamma(x+y)$ is the beta function.  For
the triweight, $B(1/2, 4)$ is exactly $32/35$.  This kernel may be
specified in \code{regmixEM.loc} with \code{kern.l = "Beta"} and
\code{kernl.g = 3}. The bandwidth we selected was $h=20$, which we
specify with \code{kernl.h = 20}.

For this implementation of the IGLE algorithm, we set the parameter
estimates and posterior probability estimates 
obtained from the mixture of regressions EM algorithm as
starting values for $\hat{\vec{\beta}}_{1}$, $\hat{\vec{\beta}}_{2}$,
$\hat{\sigma}_{1}$, $\hat{\sigma}_{2}$, and $\lambda(x_{i})$.
<<CO2igle>>=
CO2igle <- regmixEM.loc(CO2, GNP, beta = CO2reg$beta, sigma = CO2reg$sigma,
                        lambda = CO2reg$posterior, kern.l = "Beta",
                        kernl.h = 20, kernl.g = 3)
@

We can view the estimates for $\hat{\vec{\beta}}_{1}$,
$\hat{\vec{\beta}}_{2}$, $\hat{\sigma}_{1}$, and $\hat{\sigma}_{2}$.
Notice that the estimates are comparable to those obtained for the
mixture of regressions EM output and the log-likelihood value is
slightly higher.
<<CO2iglesummary>>=
summary(CO2igle)
@

Next, we can plot the estimates of $\lambda(x_{i})$ from the IGLE
algorithm.

<<lamplot, eval=FALSE>>=
plot(GNP, CO2igle$post[,1], xlab = "GNP", cex.axis = 1.4, cex.lab = 1.5,
  ylab = "Final posterior probabilities")
lines(sort(GNP), CO2igle$lambda[order(GNP), 1], col=2)
abline(h = CO2igle$lambda[1], lty = 2)
@

<<truelamplot, echo=FALSE, results=hide>>=
pdf("lamplot.pdf")
plot(GNP, CO2igle$post[,1], xlab = "GNP", cex.axis = 1.4, cex.lab = 1.5,
  ylab = "Final posterior probabilities")
lines(sort(GNP), CO2igle$lambda[order(GNP), 1], col=2, lwd=2)
abline(h = CO2igle$lambda[1], lty = 2, lwd=2)
dev.off()
@

This plot is given in Figure \ref{lamplot}.  Notice the curvature
provided by the estimates from the IGLE fit.  These fits indicate an
upward trend in the posteriors.  The predictor-dependent mixing
proportions model provides a viable way to reveal this trend since
the regular mixture of regressions fit simply provides the same
estimate of $\lambda$ for all $x_{i}$.

\begin{figure}[h]
  \centering
  \includegraphics[height=3in,width=3in]{lamplot.pdf}
  \caption{Posterior membership probabilities $p_{i1}$ for component
    one versus the predictor GNP along with estimates of
    $\lambda_1(x)$ from the IGLE algorithm (the solid red curve) and
    $\lambda_1$ from the mixture of linear regressions EM algorithm
    (the dashed black line).}
  \label{lamplot}
\end{figure}


\subsection{Parametric bootstrapping for standard errors}

With likelihood methods for estimation in mixture models, it is possible
to obtain standard error estimates by using the inverse of the observed
information matrix when implementing a Newton-type method. However, this
may be computationally burdensome. An alternative way to report standard
errors in the likelihood setting is by implementing a parametric
bootstrap.  \citet{eftib} claim that the parametric bootstrap should
provide similar standard error estimates to the traditional method
involving the information matrix.  In a mixture-of-regressions context,
a parametric bootstrap scheme may be outlined as follows:

\begin{enumerate}
\item Use \code{regmixEM} to find a local maximizer $\hat{\vec\theta}$
  of the likelihood.
\item For each $\textbf{x}_{i}$, simulate a response value $y_{i}^{*}$
  from the mixture density $g_{\hat{\vec\theta}}(\cdot|\textbf{x}_{i})$.
\item Find a parameter estimate $\tilde{\vec{\theta}}$ for the bootstrap
  sample using \code{regmixEM}.
\item Use some type of check to determine whether label-switching
  appears to have occurred, and if so, correct it.
\item Repeat steps 2 through 4 $B$ times to simulate the bootstrap
  sampling distribution of $\hat{\vec\theta}$.
\item Use the sample covariance matrix of the bootstrap sample as an
  approximation to the covariance matrix of $\hat{\vec\theta}$.
\end{enumerate}
Note that step 3, which is not part of a standard parametric bootstrap,
can be especially important in a mixture setting.

The \pkg{mixtools} package implements a parametric bootstrap algorithm
in the \code{boot.se} function. We may apply it to the regression
example of this section, which assumes the same estimate of $\lambda$
for all $x_{i}$, as follows:
<<CO2boot, results=hide>>=
set.seed(123)
CO2boot <- boot.se(CO2reg, B = 100)
@

This output consists of both the standard error estimates and the
parameter estimates obtained at each bootstrap replicate.  An
examination of the slope and intercept parameter estimates of the 500
bootstrap replicates reveals that no label-switching is likely to have
occurred.  For instance, the intercept terms of component one range from
4 to 11, whereas the intercept terms of component two are all tightly
clumped around 0:
<<bootresults>>=
rbind(range(CO2boot$beta[1,]), range(CO2boot$beta[2,]))
@

We may examine the bootstrap standard error estimates by themselves as follows:
<<CO2bootse>>=
CO2boot[c("lambda.se", "beta.se", "sigma.se")]
@

\section[Additional capabilities of mixtools]{Additional capabilities of
  \pkg{mixtools}}
\label{section:misc}

\subsection{Selecting the number of components}
\label{ss:nbcomp}

Determining the number of components $k$ is still a major contemporary
issue in mixture modeling.  Two commonly employed techniques are
information criterion and parametric bootstrapping of the likelihood
ratio test statistic values for testing
\begin{eqnarray}\label{mixturetest}
\nonumber
H_{0}&:& k = k_{0} \\
H_{1}&:& k = k_{0}+1
\end{eqnarray}
for some positive integer $k_{0}$ \citep{mclach}.

The \pkg{mixtools} package has functions to employ each of these methods
using EM output from various mixture models. The information criterion
functions calculate An Information Criterion (AIC) of \citet{aic}, the
Bayesian Information Criterion (BIC) of \citet{schw}, the Integrated
Completed Likelihood (ICL) of \citet{biern}, and the consistent AIC
(CAIC) of \citet{boz}. The functions for performing parametric
bootstrapping of the likelihood ratio test statistics sequentially test
$k=k_{0}$ versus $k=k_{0}+1$ for $k_0=1, 2, \ldots$, terminating after
the bootstrapped $p$-value for one of these tests exceeds a specified
significance level.

Currently, \pkg{mixtools} has functions for calculating information
criteria for mixtures of multinomials (\code{multmixmodel.sel}),
mixtures of multivariate normals under the conditionally i.i.d.\
assumption (\code{repnormmixmodel.sel}), and mixtures of regressions
(\code{regmixmodel.sel}). Output from various mixture model fits
available in \pkg{mixtools} can also be passed to the function
\code{boot.comp} for the parametric bootstrapping approach.  The
parameter estimates from these EM fits are used to simulate data from
the null distribution for the test given in (\ref{mixturetest}).  For
example, the following application of the \code{multmixmodel.sel}
function to the water-level multinomial data from
Section~\ref{section:cut} indicates that either 3 or 4 components seems
like the best option (no more than 4 are allowed here since there are
only 8 multinomial trials per observation and the mixture of
multinomials requires $2m \le r+1$ for identifiability):
<<modelsel>>=
<<cutpoint>>
set.seed(10)
multmixmodel.sel(watermult, comps = 1:4, epsilon = 0.001)
@

\citet{youngphd} gives more
applications of these functions to real datasets.



\subsection{Bayesian methods}

Currently, there are only two \pkg{mixtools} functions relating to
Bayesian methodology and they both pertain to analyzing mixtures of
regressions as described in \citet{hurn}.  The \code{regmixMH} function
performs a Metropolis-Hastings algorithm for fitting a mixture of
regressions model where a proper prior has been assumed.  The sampler
output from \code{regmixMH} can then be passed to \code{regcr} in order
to construct credible regions of the regression lines.  Type
\code{help("regmixMH")} and \code{help("regcr")} for details and an
illustrative example.



\section*{Acknowledgments}
This research is partially supported by NSF Award SES-0518772.  DRH
received additional funding from Le Studium, an agency of the Centre
National de la Recherche Scientifique of France.


\bibliography{mixtools}


\end{document}

\name{post.beta}
\title{Summary of Posterior Regression Coefficients in Mixtures of Random Effects Regressions}
\alias{post.beta}
\usage{
post.beta(y, x, p.beta, p.z)
}

\description{
  Returns a 2x2 matrix of plots summarizing the posterior intercept and slope terms in a mixture of
  random effects regression with arbitrarily many components.
}
\arguments{
  \item{y}{A list of N response trajectories with (possibly) varying dimensions of
  length \eqn{n_i}.}
  \item{x}{A list of N predictor values of dimension \eqn{n_i}.  Each trajectory in y has
  its own design vector.}
  \item{p.beta}{A list of N 2xk matrices giving the posterior intercept and slope values from the output of an
  EM algorithm.}
  \item{p.z}{An Nxk matrix of posterior membership probabilities from the output of an EM algorithm.}
}
\value{
  \code{post.beta} returns a 2x2 matrix of plots giving:
  \item{(1, 1)}{The data plotted on the x-y axes with all posterior regression lines.}
  \item{(1, 2)}{The data plotted on the x-y axes with most probable posterior regression lines.}
  \item{(2, 1)}{A beta-space plot of all posterior regression coefficients.}
  \item{(1, 1)}{A beta-space plot of most probable posterior regression coefficients.}
}
\seealso{
\code{\link{regmixEM.mixed}}, \code{\link{plot.mixEM}}
}
\references{
  Young, D. S. and Hunter, D. R. (2015) Random Effects Regression Mixtures for Analyzing Infant Habituation,
  \emph{Journal of Applied Statistics}, \bold{42(7)}, 1421--1441.
}
\examples{
## EM output for simulated data from 2-component mixture of random effects.

data(RanEffdata)
set.seed(100)
x <- lapply(1:length(RanEffdata), function(i) 
            matrix(RanEffdata[[i]][, 2:3], ncol = 2))
x <- x[1:20]
y <- lapply(1:length(RanEffdata), function(i) 
            matrix(RanEffdata[[i]][, 1], ncol = 1))
y <- y[1:20]
lambda <- c(0.45, 0.55)
mu <- matrix(c(0, 4, 100, 12), 2, 2)
sigma <- 2
R <- list(diag(1, 2), diag(1, 2))
em.out <- regmixEM.mixed(y, x, sigma = sigma, arb.sigma = FALSE,
                         lambda = lambda, mu = mu, R = R,
                         addintercept.random = FALSE,
                         epsilon = 1e-02, verb = TRUE)

## Obtaining the 2x2 matrix of plots.

x.ran <- lapply(1:length(x), function(i) x[[i]][, 2])
p.beta <- em.out$posterior.beta
p.z <- em.out$posterior.z
post.beta(y, x.ran, p.beta = p.beta, p.z = p.z)
}
\details{
This is primarily used for within \code{plot.mixEM}.
}

\keyword{internal}

\name{npMSL}
\title{Nonparametric EM-like Algorithm for Mixtures of Independent Repeated Measurements - Maximum Smoothed Likelihood version}
\alias{npMSL}
\usage{
npMSL(x, mu0, blockid = 1:ncol(x), 
      bw = bw.nrd0(as.vector(as.matrix(x))), samebw = TRUE, 
      bwmethod = "S", h = bw, eps = 1e-8, 
      maxiter=500, bwiter = maxiter, nbfold = NULL,
      ngrid=200, post=NULL, verb = TRUE)
}

\description{
  Returns nonparametric Smoothed Likelihood algorithm output 
  (Levine et al, 2011) for mixtures
  of multivariate (repeated measures) data where the coordinates of a row (case)
  in the data matrix are assumed to be independent, conditional on the mixture
  component (subpopulation) from which they are drawn.
}
\arguments{
  \item{x}{An \eqn{n\times r}{n x r} matrix of data.  Each of the \eqn{n} rows is a case,
  and each case has \eqn{r} repeated measurements.  These measurements are assumed
  to be conditionally independent, conditional on the mixture component (subpopulation) 
  from which the case is drawn.}
  \item{mu0}{Either an \eqn{m\times r}{m x r} matrix specifying the initial
      centers for the \link{kmeans} function, or an integer \eqn{m} specifying the 
      number of initial centers, which are then choosen randomly in
  \link{kmeans}}
  \item{blockid}{A vector of length \eqn{r} identifying coordinates 
  (columns of \code{x}) that are
  assumed to be identically distributed (i.e., in the same block).  
  For instance,
  the default has all distinct elements, indicating that no two coordinates 
  are assumed identically distributed and thus a separate set of \eqn{m} 
  density estimates is produced for each column of \eqn{x}.  On the other hand,
  if \code{blockid=rep(1,ncol(x))}, then the coordinates in each row 
  are assumed conditionally i.i.d.}
  
  \item{bw}{Bandwidth for density estimation, equal to the standard deviation 
  of the kernel density.  By default, a simplistic application of the 
  default \code{\link{bw.nrd0}} 
  bandwidth used by \code{\link{density}} to the entire dataset.}
  
  \item{samebw}{Logical:  If \code{TRUE}, use the same bandwidth for
  each iteration and for each component and block.  If \code{FALSE}, 
  use a separate bandwidth for each component and block, and update
  this bandwidth at each iteration of the algorithm 
  until \code{bwiter} is reached (see below). Two adaptation methods are provided, 
  see \code{bwmethod} below.}
  
  \item{bwmethod}{Define the adaptive bandwidth strategy when \code{samebw = FALSE}, in which case
  the bandwidth depends on each component, block, and iteration of the algorithm.
  If set to "S" (the default), adaptation is done using a suitably
  modified \code{\link{bw.nrd0}} method as described in 
  Benaglia et al (2011).
  If set to "CV", an adaptive \eqn{k}-fold Cross Validation method is applied,
  as described in Chauveau et al (2014), where \code{nbfold} is the number of subsamples.
  This corresponds to a Leave-\eqn{[n/nbfold]}-Out CV.
  }
  \item{h}{Alternative way to specify the bandwidth, to provide backward 
  compatibility.}
  \item{eps}{Tolerance limit for declaring algorithm convergence.  Convergence
  is declared whenever the maximum change in any coordinate of the 
  \code{lambda} vector (of mixing proportion estimates) does not exceed 
  \code{eps}.}

  \item{maxiter}{The maximum number of iterations allowed, convergence
  may be declared before \code{maxiter} iterations (see \code{eps} above).}
  
  \item{bwiter}{The maximum number of iterations allowed for adaptive bandwidth stage,
  when \code{samebw = FALSE}. If set to \code{0}, then the initial bandwidth matrix is used without adaptation.}

  \item{nbfold}{A parameter passed to the internal function \code{wbs.kCV}, which controls the weighted bandwidth selection by k-fold cross-validation.}

  
  \item{ngrid}{Number of points in the discretization of the intervals over which are approximated the (univariate) integrals for non linear smoothing of the log-densities, as required in the E step of the npMSL algorithm,
  see Levine et al (2011).}

  \item{post}{If non-NULL, an \eqn{n\times m}{n x m} matrix specifying the initial 
  posterior probability vectors for each of the observations, i.e., the
  initial values to start the EM-like algorithm.}
  \item{verb}{If TRUE, print updates for every iteration of the algorithm as
  it runs}
}
\value{
\code{npMSL} returns a list of class \code{npEM} with the following items:
  \item{data}{The raw data (an \eqn{n\times r}{n x r} matrix).}
  \item{posteriors}{An \eqn{n\times m}{n x m} matrix 
   of posterior probabilities for
   observation.}
  \item{bandwidth}{If \code{samebw==TRUE}, 
  same as the \code{bw} input argument; otherwise, value of \code{bw} matrix
  at final iteration.  This
  information is needed by any method that produces density estimates from the
  output.}
  \item{blockid}{Same as the \code{blockid} input argument, but recoded to have
  positive integer values.  Also needed by any method that produces density 
  estimates from the output.}
  \item{lambda}{The sequence of mixing proportions over iterations.}
  \item{lambdahat}{The final mixing proportions.}
  \item{loglik}{The sequence of log-likelihoods over iterations.}
  \item{f}{An array of size \eqn{ngrid \times m \times l}{ngrid x m x l},
  returning last values of density for component \eqn{j}
  and block \eqn{k} over \code{grid} points.}
  \item{meanNaN}{Average number of \code{NaN} that occured over iterations (for internal testing and control purpose).}
  \item{meanUdfl}{Average number of "underflow" that occured over iterations (for internal testing and control purpose).}
}
\seealso{
\code{\link{npEM}}, \code{\link{plot.npEM}}, 
\code{\link{normmixrm.sim}}, \code{\link{spEMsymloc}},
\code{\link{spEM}}, \code{\link{plotseq.npEM}}
}
\references{
   \itemize{   	   
   \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526.

   \item Benaglia, T., Chauveau, D. and Hunter, D.R. (2011),
   Bandwidth Selection in an EM-like algorithm for nonparametric multivariate mixtures.
	Nonparametric Statistics and Mixture Models: A Festschrift in
  	Honor of Thomas P. Hettmansperger. World Scientific Publishing Co., 
  	pages 15-27.

   
   \item Chauveau D., Hunter D. R. and Levine M. (2014),
  Semi-Parametric Estimation for Conditional Independence Multivariate Finite Mixture Models.
  Preprint (under revision).
    
  \item Levine, M., Hunter, D. and Chauveau, D. (2011),
   Maximum Smoothed Likelihood for Multivariate Mixtures,
   Biometrika 98(2): 403-416.
   }
}

\examples{
## Examine and plot water-level task data set.
## Block structure pairing clock angles that are directly opposite one
## another (1:00 with 7:00, 2:00 with 8:00, etc.)
set.seed(111) # Ensure that results are exactly reproducible
data(Waterdata)
blockid <- c(4,3,2,1,3,4,1,2) # see Benaglia et al (2009a)

\dontrun{
a <- npEM(Waterdata[,3:10], mu0=3, blockid=blockid, bw=4)  # npEM solution
b <- npMSL(Waterdata[,3:10], mu0=3, blockid=blockid, bw=4) # smoothed version

# Comparisons on the 4 default plots, one for each block
par(mfrow=c(2,2))
for (l in 1:4){
plot(a, blocks=l, breaks=5*(0:37)-92.5,
	xlim=c(-90,90), xaxt="n",ylim=c(0,.035), xlab="")
plot(b, blocks=l, hist=FALSE, newplot=FALSE, addlegend=FALSE, lty=2,
	dens.col=1)
axis(1, at=30*(1:7)-120, cex.axis=1)
legend("topleft",c("npMSL"),lty=2, lwd=2)}
}

}


\keyword{file}

\name{multmixmodel.sel}
\title{Model Selection Mixtures of Multinomials}
\alias{multmixmodel.sel}
\usage{
multmixmodel.sel(y, comps = NULL, ...)
}

\description{
  Assess the number of components in a mixture of multinomials model using the Akaike's information
  criterion (AIC), Schwartz's Bayesian information criterion (BIC), Bozdogan's consistent AIC (CAIC),
  and Integrated Completed Likelihood (ICL).
}
\arguments{
  \item{y}{Either An nxp matrix of data (multinomial counts), where n is the
  sample size and p is the number of multinomial bins, or the
  output of the \code{\link{makemultdata}} function.  It is not necessary
  that all of the rows contain the same number of multinomial trials (i.e.,
  the row sums of \code{y} need not be identical).}
  \item{comps}{Vector containing the numbers of components to consider.
    If NULL, this is set to be 1:(max possible), where (max possible) is
    floor((m+1)/2) and m is the minimum row sum of y.}
  \item{...}{Arguments passed to \code{multmixEM} that control convergence of the underlying EM algorithm.}
}
\value{
  \code{multmixmodel.sel} returns a table summarizing the AIC, BIC, CAIC, ICL, and log-likelihood
  values along with the winner (the number with the lowest aforementioned values).
}
 
\seealso{
\code{\link{compCDF}}, \code{\link{makemultdata}}, \code{\link{multmixEM}}
}
\examples{
##Data generated using the multinomial cutpoint method.

set.seed(100)
x <- matrix(rpois(70, 6), 10, 7) 
x.new <- makemultdata(x, cuts = 5)
multmixmodel.sel(x.new$y, comps = c(1,2), epsilon = 1e-03)

}

\keyword{file}

\name{plotly_spEMN01}
\alias{plotly_spEMN01}

\title{Plot mixture pdf for the semiparametric mixture model output by \code{spEMsymlocN01} using \code{plotly}.}

\description{This is an updated version of \code{plotlspEMN01} function by using \code{plotly}. For technical details, please refer to \code{\link{plot.spEMN01}}.}

\usage{
plotly_spEMN01(x, bw=x$bandwidth, knownpdf=dnorm, add.plot=FALSE,
               width = 3 , col.dens = NULL, col.hist =  '#1f77b4',
               title = NULL , title.size = 15 , 
               title.x = 0.5 , title.y = 0.95,
               xlab = "t" , xlab.size = 15 , xtick.size = 15,
               ylab = "Density" , ylab.size = 15 , ytick.size = 15,
               legend.text = "Densities" , legend.text.size = 15 , 
               legend.size = 15)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{An object of class "spEMN01" as returned by spEMsymlocN01}
  \item{bw}{Bandwidth for weighted kernel density estimation.}
  \item{knownpdf}{The known density of component 1, default to \code{dnorm}.}
  \item{add.plot}{Set to TRUE to add to an existing plot.}
  \item{width}{Line width.}
  \item{col.dens}{Color of density lines. Number of colors specified needs to be consistent with number of components.}
  \item{col.hist}{Color of histogram.}
  \item{title}{Text of the main title.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horsizontal position of the main title.}
  \item{title.y}{Vertical posotion of the main title.}
  \item{xlab}{Label of X-axis.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab}{Label of Y-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
  \item{legend.text}{Title of legend.}
  \item{legend.text.size}{Size of the legend title.}
  \item{legend.size}{Size of legend.}
}

\value{A plot of the density of the mixture}

\references{
	   \itemize{
	\item Chauveau, D., Saby, N., Orton, T. G., Lemercier B., Walter, C. and Arrouys, D.
	Large-scale simultaneous hypothesis testing in soil monitoring:
	A semi-parametric mixture approach, preprint (2013).
	}
}

\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{spEMsymlocN01}}, \code{\link{plot.spEMN01}}}
%
\examples{
## Probit transform of p-values
## from a Beta-Uniform mixture model
## comparion of parametric and semiparametric EM fit
## Note: in actual situations n=thousands
set.seed(50)
n=300 # nb of multiple tests
m=2 # 2 mixture components
a=c(1,0.1); b=c(1,1); lambda=c(0.6,0.4) # parameters
z=sample(1:m, n, rep=TRUE, prob = lambda)
p <- rbeta(n, shape1 = a[z], shape2 = b[z]) # p-values
o <- order(p)
cpd <- cbind(z,p)[o,] # sorted complete data, z=1 if H0, 2 if H1
p <- cpd[,2] # sorted p-values
y <- qnorm(p) # probit transform of the pvalues
# gaussian EM fit with component 1 constrained to N(0,1)
s1 <- normalmixEM(y, mu=c(0,-4),
                  mean.constr = c(0,NA), sd.constr = c(1,NA))
s2 <- spEMsymlocN01(y, mu0 = c(0,-3)) # spEM with N(0,1) fit
plotly_spEMN01(s2 , add.plot = FALSE)
%%\dontrun{
%%}
}
\keyword{file}

\name{regmixmodel.sel}
\title{Model Selection in Mixtures of Regressions}
\alias{regmixmodel.sel}
\usage{
regmixmodel.sel(x, y, w = NULL, k = 2, type = c("fixed", 
                "random", "mixed"), ...)
}
\description{
  Assess the number of components in a mixture of regressions model using the Akaike's information
  criterion (AIC), Schwartz's Bayesian information criterion (BIC), Bozdogan's consistent AIC (CAIC),
  and Integrated Completed Likelihood (ICL).
}
\arguments{
  \item{x}{An nxp matrix (or list) of predictors. If an intercept is required, then \code{x} must NOT include a column of 1's!
  Requiring an intercept may be controlled through arguments specified in \code{...}.}
  \item{y}{An n-vector (or list) of response values.}
  \item{w}{An optional list of fixed effects predictors for type "mixed" or "random".}
  \item{k}{The maximum number of components to assess.}
  \item{type}{The type of regression mixture to use.  If "fixed", then a mixture of regressions with fixed effects
  will be used.  If "random", then a mixture of regressions where the random effects regression coefficients are assumed
  to come from a mixture will be used.  If "mixed", the mixture structure used is the same as "random", except a coefficient
  of fixed effects is also assumed.}
  \item{...}{Additional arguments passed to the EM algorithm used for calculating the type of regression mixture specified
  in \code{type}.}
}
\value{
  \code{regmixmodel.sel} returns a matrix of the AIC, BIC, CAIC, and ICL values along with the winner (i.e., the highest
  value given by the model selection criterion) for various types of regression mixtures.
}
\seealso{
  \code{\link{regmixEM}}, \code{\link{regmixEM.mixed}}
}
\references{
  Biernacki, C., Celeux, G. and Govaert, G. (2000) Assessing a Mixture Model for Clustering with the
  Integrated Completed Likelihood, \emph{IEEE Transactions on Pattern Analysis and Machine Intelligence} \bold{22(7)}, 719--725.

  Bozdogan, H. (1987) Model Selection and Akaike's Information Criterion (AIC): The General Theory and its
  Analytical Extensions, \emph{Psychometrika} \bold{52}, 345--370.
}
\examples{
## Assessing the number of components for NOdata.

data(NOdata)
attach(NOdata)
set.seed(100)
regmixmodel.sel(x = NO, y = Equivalence, k = 3, type = "fixed")
}

\keyword{file}

\name{density.npEM}
\title{Normal kernel density estimate for nonparametric EM output}
\alias{density.npEM} 
\usage{
\method{density}{npEM}(x, u=NULL, component=1, block=1, scale=FALSE, \dots)
}

\description{
    Takes an object of class \code{npEM} and returns an object of class
    \code{\link{density}} giving the kernel density estimate for the selected
    component and, if applicable, the selected block.
} 
\arguments{
  \item{x}{An object of class \code{npEM} such as the output
  of the \code{\link{npEM}} or \code{\link{spEMsymloc}} functions.}
  \item{u}{Vector of points at which the density is to be evaluated}
  \item{component}{Mixture component number; should be an integer from 1 to the
  number of columns of \code{x$posteriors}.}
  \item{block}{Block of repeated measures.  Only applicable in repeated measures
  case, for which \code{x$blockid} exists; should be an integer from 1 to
  \code{max(x$blockid)}.}
  \item{scale}{Logical:  If TRUE, multiply the density values by the 
  corresponding mixing proportions found in \code{x$lambdahat}}
  \item{\dots}{Additional arguments; not used by this method.}
}
\details{
  The bandwidth is taken to be the same as that used to produce the \code{npEM}
  object, which is given by \code{x$bandwidth}.
}
\value{
  \code{density.npEM} returns a list of type \code{"density"}.  See 
  \code{\link{density}} for details.  In particular, the output of 
  \code{density.npEM} may be used directly by functions such as 
  \code{\link{plot}} or \code{\link{lines}}.
}

\seealso{ 
\code{\link{npEM}}, \code{\link{spEMsymloc}}, \code{\link{plot.npEM}}
} 

\examples{ 
## Look at histogram of Old Faithful waiting times
data(faithful)
Minutes <- faithful$waiting
hist(Minutes, freq=FALSE)

## Superimpose equal-variance normal mixture fit:
set.seed(100)
nm <- normalmixEM(Minutes, mu=c(50,80), sigma=5, arbvar=FALSE, fast=TRUE)
x <- seq(min(Minutes), max(Minutes), len=200)
for (j in 1:2) 
  lines(x, nm$lambda[j]*dnorm(x, mean=nm$mu[j], sd=nm$sigma), lwd=3, lty=2)
  
## Superimpose several semiparametric fits with different bandwidths:
bw <- c(1, 3, 5)
for (i in 1:3) {
  sp <- spEMsymloc(Minutes, c(50,80), bw=bw[i], eps=1e-3)
  for (j in 1:2) 
    lines(density(sp, component=j, scale=TRUE), col=1+i, lwd=2)    
}
legend("topleft", legend=paste("Bandwidth =",bw), fill=2:4)
}
\keyword{file}

\name{RTdata}
\docType{data}
\title{Reaction Time (RT) Data Set}
\alias{RTdata}
\usage{
data(RTdata)
}
\description{
  This data set involves normally developing children 9 years of age presented with two visual simuli on a computer monitor.
  The left image is the target stimuli and on the right is either an exact copy or a mirror image
  of the target stimuli.  The child must press one key if it is a copy or another key if it is a mirror image.  The
  data consists of the reaction times (RT) of the 197 children who provided correct responses for all 6 task trials. 
}
\format{This data frame consists of 197 children (the rows) and their 6 responses (the columns)
to the stimulus presented.  The response (RT) is recorded in milliseconds.
}
\references{
  Cruz-Medina, I. R., Hettmansperger, T. P. and Thomas, H. (2004) Semiparametric Mixture Models and Repeated
  Measures: The Multinomial Cut Point Model, \emph{Applied Statistics} \bold{53(3)}, 463--474.
  
  Miller, C. A., Kail, R., Leonard, L. B. and Tomblin, J. B. (2001) Speed of Processing in Children with
  Specific Language Impairment, \emph{Journal of Speech, Language, and Hearing Research} \bold{44(2)}, 416--433.
}
\seealso{
\code{\link{RTdata2}}
}
\keyword{datasets}

\name{Habituationdata}
\alias{Habituationdata}
\docType{data}
\title{Infant habituation data}
\author{Hoben Thomas}
\description{
From Thomas et al (2011):

"Habituation is a standard method of studying infant behaviors. Indeed,
much of what is known about infant memory and perception rests on 
habituation methods. Six-month infants (n = 51) were habituated to a 
checker-board pattern on two occasions, 
one week apart. On each occasion, the
infant was presented with the checkerboard pattern and the length of time
the infant viewed the pattern before disengaging was recorded; this denoted
the end of a trial. After disengagement, another trial was presented. The
procedure was implemented for eleven trials. The conventional index of 
habituation performance is the summed observed fixation to the checkerboard
pattern over the eleven trials. Thus, an index of reliability focuses on how
these fixation times, in seconds, on the two assessment occasions correlate:
\eqn{r = .29}."}
\usage{data(Habituationdata)}
\format{A data frame with two variables, \code{m1} and \code{m2}, and
51 cases.  The two variables are the summed observations times
for the two occasions described above.}
\source{
Original source: 
Thomas et al. (2011).  See references section.
}
\references{
Thomas, H., Lohaus, A., and Domsch, H. (2011), Extensions of
Reliability Theory, in
Nonparametric Statistics and Mixture Models:  A Festschrift in
Honor of Thomas Hettmansperger (Singapore:  World Scientific),
pp. 309-316.
}
\keyword{datasets}

\name{plotly_mixEM}
\alias{plotly_mixEM}

\title{Visualization of output of \code{mixEM} function using \code{plotly}}

\description{This is an updated version of \code{plot.mixEM}. For more technical details, please refer to \code{plot.mixEM}.}

\usage{
plotly_mixEM(x, 
             loglik = TRUE,
             density = FALSE,
             xlab1="Iteration", xlab1.size=15 , xtick1.size=15,
             ylab1="Log-Likelihood", ylab1.size=15 , ytick1.size=15,
             title1="Observed Data Log-Likelihood", title1.size=15,
             title1.x = 0.5,title1.y=0.95,
             col1="#1f77b4", lwd1=3, cex1=6,
             xlab2=NULL, xlab2.size=15 , xtick2.size=15,
             ylab2=NULL, ylab2.size=15 , ytick2.size=15,
             title2=NULL, title2.size=15,
             title2.x = 0.5,title2.y=0.95, col.hist = "#1f77b4",
             col2=NULL, lwd2=3, cex2=6,
             alpha = 0.05, marginal = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{An object of class \code{mixEM}.}
  \item{loglik}{If TRUE, a plot of the log-likelihood versus the EM iterations is given.}
  \item{density}{Graphics pertaining to certain mixture models. The details are given below.}
  \item{xlab1}{Label of x-axis to be passed to the loglikelihood plot. Trying to change these parameters using \code{xlab} result in an error.} 
  \item{xlab1.size}{Font of \code{xlab1}.}
  \item{xtick1.size}{Font of tick labels of x-axis to be passed to the loglikelihood plot.}
  \item{ylab1}{Label of y-axis to be passed to the loglikelihood plot. Trying to change these parameters using \code{ylab} result in an error.} 
  \item{ylab1.size}{Font of \code{ylab1}.}
  \item{ytick1.size}{Font of tick labels of y-axis to be passed to the loglikelihood plot.}
  \item{title1}{Title to be passed to the loglikelihood plot.} 
  \item{title1.size}{Tile size of the loglikelihood plot.}
  \item{title1.x}{Horizontal position of the loglikelihood plot.}
  \item{title1.y}{Verticle position of the loglikelihood plot.}
  \item{col1}{Color of the loglikelihood plot.}
  \item{lwd1}{Width of the density curve of the loglikelihood plot.}
  \item{cex1}{Dot size of the loglikelihood plot.}
  
  \item{xlab2}{Label of x-axis to be passed to the density plot. Trying to change these parameters using \code{xlab} result in an error.} 
  \item{xlab2.size}{Font of \code{xlab2}.}
  \item{xtick2.size}{Font of tick labels of x-axis to be passed to the density plot.}
  \item{ylab2}{Label of y-axis to be passed to the density plot. Trying to change these parameters using \code{ylab} result in an error.} 
  \item{ylab2.size}{Font of \code{ylab2}.}
  \item{ytick2.size}{Font of tick labels of y-axis to be passed to the density plot.}
  \item{title2}{Title to be passed to the density plot.} 
  \item{title2.size}{Tile size of the density plot.}
  \item{title2.x}{Horizontal position of the density plot.}
  \item{title2.y}{Verticle position of the density plot.}
  \item{col2}{Color of the density plot.}
  \item{lwd2}{Width of the density curve of the density plot.}
  \item{cex2}{Dot size of the density plot.}
  \item{col.hist}{Color of the histogram of the density plot}
  \item{alpha}{A vector of significance levels when constructing confidence ellipses and confidence bands for the mixture of multivariate normals and mixture of regressions
cases, respectively. The default is 0.05} 
  \item{marginal}{If \code{TRUE}, marginal density is presented on the side of the corresponding variable.}
 }

\value{A plot of the output of \code{mixEM} function is presented depends on output type.}

\seealso{
\code{\link{post.beta}}}

\examples{
\dontrun{
## EM output for data generated from a 2-component binary logistic regression model.
beta <- matrix(c(-10, .1, 20, -.1), 2, 2)
x <- runif(500, 50, 250)
x1 <- cbind(1, x)
xbeta <- x1%*%beta
w <- rbinom(500, 1, .3)
y <- w*rbinom(500, size = 1, prob = (1/(1+exp(-xbeta[, 1]))))+
  (1-w)*rbinom(500, size = 1, prob =
                 (1/(1+exp(-xbeta[, 2]))))
out.2 <- logisregmixEM(y, x, beta = beta, lambda = c(.3, .7),
                       verb = TRUE, epsilon = 1e-01)
plotly_mixEM(out.2 , col2 = c("red" , "green") , density = TRUE)

## Fitting randomly generated data with a 2-component location mixture of bivariate normals.
set.seed(100)
x.1 <- rmvnorm(40, c(0, 0))
x.2 <- rmvnorm(60, c(3, 4))
X.1 <- rbind(x.1, x.2)
mu <- list(c(0, 0), c(3, 4))
out.1 <- mvnormalmixEM(X.1, arbvar = FALSE, mu = mu,
                       epsilon = 1e-02)
plotly_mixEM(out.1 , col2 = c("brown" , "blue") ,
             alpha = c(0.01 , 0.05 , 0.1),
             density = TRUE , marginal = FALSE)

## Fitting randomly generated data with a 2-component scale mixture of bivariate normals.
x.3 <- rmvnorm(40, c(0, 0), sigma =
                 matrix(c(200, 1, 1, 150), 2, 2))
x.4 <- rmvnorm(60, c(0, 0))
X.2 <- rbind(x.3, x.4)
lambda <- c(0.40, 0.60)
sigma <- list(diag(1, 2), matrix(c(200, 1, 1, 150), 2, 2))
out.2 <- mvnormalmixEM(X.2, arbmean = FALSE,
                       sigma = sigma, lambda = lambda,
                       epsilon = 1e-02)
plotly_mixEM(out.1 , col2 = c("brown" , "blue") ,
             alpha = c(0.01 , 0.05 , 0.1),
             density = TRUE , marginal = TRUE)

## EM output for simulated data from 2-component mixture of random effects.
data(RanEffdata)
set.seed(100)
x <- lapply(1:length(RanEffdata), function(i)
  matrix(RanEffdata[[i]][, 2:3], ncol = 2))
x <- x[1:20]
y <- lapply(1:length(RanEffdata), function(i)
  matrix(RanEffdata[[i]][, 1], ncol = 1))
y <- y[1:20]
lambda <- c(0.45, 0.55)
mu <- matrix(c(0, 4, 100, 12), 2, 2)
sigma <- 2
R <- list(diag(1, 2), diag(1, 2))
em.out <- regmixEM.mixed(y, x, sigma = sigma, arb.sigma = FALSE,
                         lambda = lambda, mu = mu, R = R,
                         addintercept.random = FALSE,
                         epsilon = 1e-02, verb = TRUE)
plotly_mixEM(em.out , col2 = c("gold" , "purple") , 
             density = TRUE , lwd2 = 1 , cex2 =9)

## Analyzing the Old Faithful geyser data with a 2-component mixture of normals.
data(faithful)
attach(faithful)
set.seed(100)
out <- normalmixEM(waiting, arbvar = FALSE, verb = TRUE,
                   epsilon = 1e-04)
plotly_mixEM(out, density = TRUE , col2 = c("gold" , "purple"))

## EM output for the water-level task data set.
data(Waterdata)
set.seed(100)
water <- t(as.matrix(Waterdata[,3:10]))
em.out <- repnormmixEM(water, k = 2, verb = TRUE, epsilon = 1e-03)
plotly_mixEM(em.out, density = TRUE , col2 = c("gold" , "purple"))
}
}
%
\keyword{file}

\name{plot.spEMN01}
\alias{plot.spEMN01}

\title{Plot mixture pdf for the semiparametric mixture model output by spEMsymlocN01}

\description{Plot mixture density for the semiparametric mixture model output by spEMsymlocN01, with one component known and set to normal(0,1), and a symmetric nonparametric density with location parameter.}

\usage{
\method{plot}{spEMN01}(x, bw = x$bandwidth, knownpdf = dnorm, add.plot = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{An object of class "spEMN01" as returned by spEMsymlocN01}
  \item{bw}{Bandwidth for weighted kernel density estimation.}
  \item{knownpdf}{The known density of component 1, default to \code{dnorm}.}
  \item{add.plot}{Set to TRUE to add to an existing plot.}
  \item{...}{further arguments passed to \code{plot} if \code{add.plot = FALSE},
  	and to \code{lines} if \code{add.plot = TRUE}.}
}

\value{A plot of the density of the mixture}

\references{
	   \itemize{
	\item Chauveau, D., Saby, N., Orton, T. G., Lemercier B., Walter, C. and Arrouys, D.
	Large-scale simultaneous hypothesis testing in soil monitoring:
	A semi-parametric mixture approach, preprint (2013).
	}
}

\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{spEMsymlocN01}}}
%
\keyword{file}

\name{Waterdata}
\docType{data}
\title{Water-Level Task Data Set}
\alias{Waterdata}
\alias{WaterdataFull}
\usage{
data(Waterdata)
}
\description{
  This data set arises from the water-level task proposed by the Swiss 
  psychologist Jean Piaget to assess children's understanding of the physical 
  world.  This involves presenting a child with a rectangular vessel with a cap,
  affixed to a wall, that can be tilted (like the minute hand of a clock) to point
  in any direction.  A separate disk with a water line indicated on it, which
  can similarly be spun so that the water line may assume any desired angle with
  the horizontal, is positioned so that by spinning this disk, the child subject 
  may make the hypothetical surface of water inside the vessel assume any desired
  orientation.  For each of eight different orientations of the vessel, corresponding
  to the clock angles at 1:00, 2:00, 4:00, 5:00, 7:00, 8:00, 10:00, and 11:00,
  the child subject is asked to position the water level as it would appear in reality
  if water were in the vessel.  The measurement is the acute angle with 
  the horizontal, in degrees,
  assumed by the water line after it is positioned by the child.  A sign is attached
  to the measurement to indicate whether the line slopes up 
  (positive) or down (negative) from left to right.  Thus, each 
  child has 8 repeated measurements, one for each vessel angle, and the range of
  possible values are from -90 to 90.
  
  The setup of the experiment, along with a photograph of the testing apparatus,
  is given by Thomas and Jamison (1975).  A more detailed analysis using a 
  subset of 405 of the original 579 subjects is given
  by Thomas and Lohaus (1993); further analyses using the functions in
  \code{mixtools} are given by Benaglia et al (2008) and Levine et al (2011), 
  among others.
  
  There are two versions of the dataset included in \code{mixtools}.  The full 
  dataset, called \code{WaterdataFull}, has 579 individuals.  The dataset
  called \code{Waterdata} is a subset of 405 individuals, comprising all children
  aged 11 years or more and omitting any individuals with any observations equal
  to 100, which in this context indicates a missing value (since all of the degree
  measurements should be in the range from -90 to +90, 100 is not a possible value).
}
\format{These data frames consist of 405 or 579 rows, one row for each child. 
  There are ten columns:
  The age (in years) and sex (where 1=male and 0=female) are given for each 
  individual along with the 
  degree of deviation from the horizontal
  for 8 specified clock-hour orientations 
  (11, 4, 2, 7, 10, 5, 1, and 8 o'clock, in order).
}
\source{
  Benaglia, T., Chauveau, D., and Hunter, D.R. (2009), 
  An EM-Like Algorithm for Semi- and Non-Parametric Estimation in 
  Multivariate Mixtures, 
  \emph{Journal of Computational and Graphical Statistics}, 
  18: 505-526.

  Levine, M., Hunter, D.R., and Chauveau, D. (2011), 
  Maximum Smoothed Likelihood for Multivariate Mixtures, 
  \emph{Biometrika}, 
  98(2): 403-416.

  Thomas, H. and Jamison, W. (1975),
  On the Acquisition of Understanding that Still Water is Horizontal,
  \emph{Merrill-Palmer Quarterly of Behavior and Development},
  21(1): 31-44.

  Thomas, H. and Lohaus, A.  (1993),
  \emph{Modeling Growth and Individual Differences in Spatial Tasks},
  University of Chicago Press, Chicago, available on JSTOR.  
} 
\keyword{datasets}

\name{summary.spRMM}
\alias{summary.spRMM}
\title{Summarizing fits from Stochastic EM algorithm for semiparametric scaled mixture of censored data}
\usage{
\method{summary}{spRMM}(object, digits = 6, ...)
}
\arguments{
  \item{object}{an object of class \code{spRMM} such as a result of a call
  to \code{\link{spRMM_SEM}}}
  \item{digits}{Significant digits for printing values}
  \item{...}{Additional parameters passed to \code{print}.}
}
\description{
\code{\link[base]{summary}} method for class \code{spRMM}.
}
\details{
 \code{\link{summary.spRMM}} prints scalar parameter estimates for
 a fitted mixture model: each component weight and the scaling factor, see reference below.
 The functional (nonparametric) estimates of survival and hazard rate funcions can be obtained 
 using \code{\link{plotspRMM}}.
}
\value{
  The function \code{\link{summary.spRMM}} prints the final loglikelihood
  value at the solution as well as The estimated mixing weights and the scaling parameter.
}

\seealso{
Function for plotting functional (nonparametric) estimates:
  \code{\link{plotspRMM}}.
  
  Other models and algorithms for censored lifetime data
  (name convention is model_algorithm):
  \code{\link{expRMM_EM}},
  \code{\link{weibullRMM_SEM}}.
}

\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}

\examples{
# See example(spRMM_SEM)
}
\keyword{file}

\name{ellipse}
\title{Draw Two-Dimensional Ellipse Based on Mean and Covariance}
\alias{ellipse}
\usage{
ellipse(mu, sigma, alpha = .05, npoints = 250, newplot = FALSE,
        draw = TRUE, ...)
}

\description{
Draw a two-dimensional ellipse that traces a bivariate normal density
contour for a given mean vector, covariance matrix, and probability content.
}
\arguments{
  \item{mu}{A 2-vector giving the mean.}
  \item{sigma}{A 2x2 matrix giving the covariance matrix.}
  \item{alpha}{Probability to be excluded from the ellipse. The
default value is alpha = .05, which results in a 95\% ellipse.}
  \item{npoints}{Number of points comprising the border of the ellipse.}
  \item{newplot}{If newplot = TRUE and draw = TRUE, plot the ellipse on a new
plot.  If newplot = FALSE and draw = TRUE, add the ellipse to an
existing plot.}
  \item{draw}{If TRUE, draw the ellipse.}
  \item{...}{Graphical parameters passed to \code{lines} or \code{plot}
command.} 
}
\value{
  \code{ellipse} returns an \code{npoints}x2 matrix of the points forming the
border of the ellipse.
}
\references{
  Johnson, R. A. and Wichern, D. W. (2002) \emph{Applied Multivariate Statistical Analysis, Fifth Edition},
  Prentice Hall.
}
\seealso{
\code{\link{regcr}}
}
\examples{
## Produce a 95\% ellipse with the specified mean and covariance structure. 

mu <- c(1, 3)
sigma <- matrix(c(1, .3, .3, 1.5), 2, 2)

ellipse(mu, sigma, npoints = 200, newplot = TRUE)
 
}

\keyword{file}

\name{ddirichlet}
\alias{ddirichlet}

\title{Density Function for the Dirichlet Distribution}
\description{
  Density function for the Dirichlet distribution.
}
\usage{
ddirichlet(x, alpha)
}

\arguments{
  \item{x}{A k-dimensional vector of values that sum to 1 for which to calculate the density}
  \item{alpha}{A k-dimensional vector of the Dirichlet distribution parameters.}
}

\details{
  This is usually not to be called by the user.
}

\keyword{internal}

\name{lambda}
\title{Local Estimation for Lambda in Mixtures of Regressions}
\alias{lambda}
\usage{
lambda(z, x, xi, h = NULL, kernel = c("Gaussian", "Beta", 
       "Triangle", "Cosinus", "Optcosinus"), g = 0)
}

\description{
  Return local estimates of the mixing proportions from each component
  of a mixture of regressions model using output from an EM algorithm.   
}
\arguments{
  \item{z}{An nxk matrix of posterior probabilities obtained from the EM
    algorithm.}
  \item{x}{A vector of values for which the local estimation is calculated.}
  \item{xi}{An nx(p-1) matrix of the predictor values.}
  \item{h}{The bandwidth controlling the size of the window used for the
  local estimation.}
  \item{kernel}{The type of kernel to be used for the local estimation.}
  \item{g}{A shape parameter required for the symmetric beta kernel.  The default
  is \code{g} = 0 which yields the uniform kernel.  Some common values are \code{g} = 1 for the
  Epanechnikov kernel, \code{g} = 2 for the biweight kernel, and \code{g} = 3 for the triweight kernel.}
}
\value{
  \code{lambda} returns local estimates of the mixing proportions for the inputted
  \code{x} vector.
}
\seealso{
\code{\link{regmixEM.loc}}
}
\note{\code{lambda} is for use within \code{regmixEM.loc}.}

\keyword{internal}

\name{makemultdata}
\title{Produce Cutpoint Multinomial Data}
\alias{makemultdata}
\usage{
makemultdata(..., cuts)
}

\description{
  Change data into a matrix of multinomial counts using the
  cutpoint method and generate EM algorithm starting values for 
  a k-component mixture of multinomials.
}
\arguments{
  \item{...}{Either vectors (possibly of different lengths) of raw data 
    or an nxm matrix (or data frame) of data. If \code{...} are vectors of varying length, 
    then \code{makemultdata} will create a matrix of size nxm where n is the
    sample size and m is the length of the vector with maximum length.  Those 
    vectors with length less than m will have \code{NA}s to make the 
    corresponding row in the matrix of length m.  If \code{...} is a matrix (or data frame), then
    the rows must correspond to the sample and the columns the repeated measures.}
  \item{cuts}{A vector of cutpoints.  This vector is sorted by the algorithm.}
}
\value{
  \code{makemultdata} returns an object which is a list with components:
  \item{x}{An nxm matrix of the raw data.}
  \item{y}{An nxp matrix of the discretized data where p is one more than the
    number of cutpoints. Each row is a multinomial vector of counts.  In particular,
    each row should sum to the number of repeated measures for that sample.}
} 
\details{
  The (i, j)th entry of the matrix \code{y} (for j < p)
  is equal to the number of entries
  in the ith column of \code{x} that are less than or equal to \code{cuts}[j].
  The (i, p)th entry is equal to the number of entries greater than
  \code{cuts}[j].
}
\seealso{
\code{\link{compCDF}}, \code{\link{multmixmodel.sel}}, \code{\link{multmixEM}}
}
\references{
  Elmore, R. T., Hettmansperger, T. P. and Xuan, F. (2004) The Sign Statistic, One-Way Layouts
  and Mixture Models, \emph{Statistical Science} \bold{19(4)}, 579--587.
}
\examples{
## Randomly generated data.

set.seed(100)
y <- matrix(rpois(70, 6), 10, 7)
cuts <- c(2, 5, 7)
out1 <- makemultdata(y, cuts = cuts)
out1

## The sulfur content of the coal seams in Texas.

A <- c(1.51, 1.92, 1.08, 2.04, 2.14, 1.76, 1.17)
B <- c(1.69, 0.64, .9, 1.41, 1.01, .84, 1.28, 1.59)
C <- c(1.56, 1.22, 1.32, 1.39, 1.33, 1.54, 1.04, 2.25, 1.49)
D <- c(1.3, .75, 1.26, .69, .62, .9, 1.2, .32)
E <- c(.73, .8, .9, 1.24, .82, .72, .57, 1.18, .54, 1.3)

out2 <- makemultdata(A, B, C, D, E, 
                     cuts = median(c(A, B, C, D, E)))
out2

## The reaction time data.

data(RTdata)
out3 <- makemultdata(RTdata, cuts = 
                     100*c(5, 10, 12, 14, 16, 20, 25, 30, 40, 50))
dim(out3$y)
out3$y[1:10,]
}

\keyword{file}

\name{RTdata2}
\docType{data}
\title{Reaction Time (RT) Data Set \# 2} 
\alias{RTdata2}
\usage{
data(RTdata2)
}
\description{
  This data set involves normally developing children 9 years of age presented 
  visual simuli on a computer monitor.  There are three different experimental
  conditions, according to the length of the delay after which the stimulus 
  was displayed on the screen.  Each subject experienced each condition eight
  times, and these 24 trials were given in random order.
  These data give the 82 children for whom
  there are complete measurements among over 200 total subjects.
}
\format{This data frame consists of 82 children (the rows) and their 24 responses (the columns)
to the stimulus presented.  The response is recorded in milliseconds.  The columns are
not in the order in which the stimuli were presented to the children;  rather, they
are arranged into three blocks of eight columns each so that each eight-column
block contains only trials from one of the three conditions.
}
\references{
  Miller, C. A., Kail, R., Leonard, L. B. and Tomblin, J. B. (2001) Speed of Processing in Children with
  Specific Language Impairment, \emph{Journal of Speech, Language, and Hearing Research} \bold{44(2)}, 416--433.
}
\seealso{
\code{\link{RTdata}}
}
\keyword{datasets}

\name{compCDF}
\title{Plot the Component CDF}
\alias{compCDF}
\usage{
compCDF(data, weights, 
        x=seq(min(data, na.rm=TRUE), max(data, na.rm=TRUE), len=250), 
        comp=1:NCOL(weights), makeplot=TRUE, ...) 
}
\description{
Plot the components' CDF via the posterior probabilities.
}
\arguments{
  \item{data}{A matrix containing the raw data. Rows are subjects and columns
   are repeated measurements.}
  \item{weights}{The weights to compute the empirical CDF; however, most of
   time they are the posterior probabilities.}
  \item{x}{The points at which the CDFs are to be evaluated.}
  \item{comp}{The mixture components for which CDFs are desired.}
  \item{makeplot}{Logical:  Should a plot be produced as a side effect?}
  \item{...}{Additional arguments (other than \code{lty} and \code{type}, 
    which are already used)
    to be passed directly to \code{plot} and \code{lines} functions.}
}
\value{
  A matrix with \code{length(comp)} rows and \code{length(x)} columns 
  in which each row gives the CDF evaluated at each point of \code{x}.
}
\details{
  When \code{makeplot} is \code{TRUE}, a line plot is produced of the
  CDFs evaluated at \code{x}.  The plot is not a step function plot; 
  the points \eqn{(x, CDF(x))} are simply joined by line segments.
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
  
  Elmore, R. T., Hettmansperger, T. P. and Xuan, F. (2004) The Sign Statistic, One-Way Layouts
  and Mixture Models, \emph{Statistical Science} \bold{19(4)}, 579--587.
}
\seealso{
\code{\link{makemultdata}}, \code{\link{multmixmodel.sel}}, \code{\link{multmixEM}}.
}
\examples{
## The sulfur content of the coal seams in Texas

set.seed(100)

A <- c(1.51, 1.92, 1.08, 2.04, 2.14, 1.76, 1.17)
B <- c(1.69, 0.64, .9, 1.41, 1.01, .84, 1.28, 1.59) 
C <- c(1.56, 1.22, 1.32, 1.39, 1.33, 1.54, 1.04, 2.25, 1.49) 
D <- c(1.3, .75, 1.26, .69, .62, .9, 1.2, .32) 
E <- c(.73, .8, .9, 1.24, .82, .72, .57, 1.18, .54, 1.3)

dis.coal <- makemultdata(A, B, C, D, E, 
                         cuts = median(c(A, B, C, D, E)))
temp <- multmixEM(dis.coal)

## Now plot the components' CDF via the posterior probabilities

compCDF(dis.coal$x, temp$posterior, xlab="Sulfur", ylab="", main="empirical CDFs")
}

\keyword{file}

\name{density.spEM}
\title{Normal kernel density estimate for semiparametric EM output}
\alias{density.spEM} 
\usage{
\method{density}{spEM}(x, u=NULL, component=1, block=1, scale=FALSE, \dots)
}

\description{
    Takes an object of class \code{spEM} and returns an object of class
    \code{\link{density}} giving the kernel density estimate.
} 
\arguments{
  \item{x}{An object of class \code{npEM} such as the output
  of the \code{\link{npEM}} or \code{\link{spEMsymloc}} functions.}
  \item{u}{Vector of points at which the density is to be evaluated}
  \item{component}{Mixture component number; should be an integer from 1 to the
  number of columns of \code{x$posteriors}.}
  \item{block}{Block of repeated measures.  Only applicable in repeated measures
  case, for which \code{x$blockid} exists; should be an integer from 1 to
  \code{max(x$blockid)}.}  
  \item{scale}{Logical:  If TRUE, multiply the density values by the 
  corresponding mixing proportions found in \code{x$lambdahat}}
  \item{\dots}{Additional arguments; not used by this method.}
}
\details{
  The bandwidth is taken to be the same as that used to produce the \code{npEM}
  object, which is given by \code{x$bandwidth}.
}
\value{
  \code{density.spEM} returns a list of type \code{"density"}.  See 
  \code{\link{density}} for details.  In particular, the output of 
  \code{density.spEM} may be used directly by functions such as 
  \code{\link{plot}} or \code{\link{lines}}.
}

\seealso{ 
\code{\link{spEM}}, \code{\link{spEMsymloc}}, \code{\link{plot.spEM}}
} 

\examples{ 
set.seed(100)
mu <- matrix(c(0, 15), 2, 3)
sigma <- matrix(c(1, 5), 2, 3)
x <- rmvnormmix(300, lambda = c(.4,.6), mu = mu, sigma = sigma)

d <- spEM(x, mu0 = 2, blockid = rep(1,3), constbw = TRUE) 
plot(d, xlim=c(-10, 40), ylim = c(0, .16), xlab = "", breaks = 30, 
     cex.lab=1.5, cex.axis=1.5) # plot.spEM calls density.spEM here
}
\keyword{file}

\name{plotly_ellipse}
\title{Draw Two-Dimensional Ellipse Based on Mean and Covariance using \code{plotly}}
\alias{plotly_ellipse}
\usage{
plotly_ellipse(mu, sigma, alpha=.05, npoints=250,
               draw=TRUE, cex = 3, col = "#1f77b4", lwd = 3,
               title = "", title.x = 0.5, title.y = 0.95, title.size = 15,
               xlab = "X", xlab.size = 15, xtick.size = 15,
               ylab = "Y", ylab.size = 15, ytick.size = 15)
}

\description{
This is an updated version of \code{ellipse}. For more technical details, please refer to \code{ellipse}.
}
\arguments{
  \item{mu}{A 2-vector giving the mean.}
  \item{sigma}{A 2x2 matrix giving the covariance matrix.}
  \item{alpha}{Probability to be excluded from the ellipse. The
default value is alpha = .05, which results in a 95\% ellipse.}
  \item{npoints}{Number of points comprising the border of the ellipse.}
  \item{draw}{If TRUE, draw the ellipse.}
  \item{cex}{Size of markers.}
  \item{lwd}{Line width of the ellipse.}
  \item{col}{Color of both markers and lines.}
  \item{title}{Text of the main title.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horsizontal position of the main title.}
  \item{title.y}{Vertical posotion of the main title.}
  \item{xlab}{Label of X-axis.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab}{Label of Y-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
}
\value{
  \code{plotly_ellipse} returns an \code{npoints}x2 matrix of the points forming the
border of the ellipse.
}
\references{
  Johnson, R. A. and Wichern, D. W. (2002) \emph{Applied Multivariate Statistical Analysis, Fifth Edition},
  Prentice Hall.
}
\seealso{
\code{\link{regcr}}, \code{\link{ellipse}}
}
\examples{
## Produce a 95% ellipse with the specified mean and covariance structure.
mu <- c(1, 3)
sigma <- matrix(c(1, .3, .3, 1.5), 2, 2)
plotly_ellipse(mu, sigma, npoints = 200)
}

\keyword{file}

\name{NOdata}
\docType{data}
\alias{NOdata}
\title{Ethanol Fuel Data Set}
\description{
  This data set gives the equivalence ratios and peak nitrogen oxide emissions in a
  study using pure ethanol as a spark-ignition engine fuel.  
}
\usage{
data(NOdata)
}
\format{This data frame consists of:
\itemize{
  \item{\code{NO}}{The peak nitrogen oxide emission levels.}
  \item{\code{Equivalence}}{The equivalence ratios for the engine at compression ratios from 7.5 to 18.}
}
}
\source{Brinkman, N. D. (1981) Ethanol Fuel -- A Single-Cylinder Engine Study of Efficiency and
Exhaust Emissions, \emph{S.A.E. Transactions}, 68.
}
\references{
  Hurn, M., Justel, A. and Robert, C. P. (2003) Estimating Mixtures of Regressions, \emph{Journal 
  of Computational and Graphical Statistics} \bold{12(1)}, 55--79.
}
\keyword{datasets}

\name{plotly_post.beta}
\title{Visualization of Posterior Regression Coefficients in Mixtures of Random Effects Regressions using \code{plotly}}
\alias{plotly_post.beta}
\usage{
plotly_post.beta(y, x, p.beta, p.z,
                 cex = 6,lwd=1,
                 title.size = 15,
                 xlab.size = 15 , xtick.size = 15,
                 ylab.size = 15 , ytick.size = 15,
                 col.data = "#1f77b4",
                 col.comp = NULL)
}

\description{
  Returns a 2x2 matrix of plots summarizing the posterior intercept and slope terms in a mixture of random effects regression with arbitrarily many components.
}
\arguments{
  \item{y}{A list of N response trajectories with (possibly) varying dimensions of
  length \eqn{n_i}.}
  \item{x}{A list of N predictor values of dimension \eqn{n_i}.  Each trajectory in y has
  its own design vector.}
  \item{p.beta}{A list of N 2xk matrices giving the posterior intercept and slope values from the output of an
  EM algorithm.}
  \item{p.z}{An Nxk matrix of posterior membership probabilities from the output of an EM algorithm.}
  \item{cex}{Size of dots of posterior Coefficients.}
  \item{lwd}{Width of lines.}
  \item{title.size}{Size of the main title.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
  \item{col.data}{Color of original data points.}
  \item{col.comp}{Color of points and lines of components. Number of colors specified needs to be consistent with number of components.}
}
\value{
  Plots returned.
}
\seealso{
\code{\link{regmixEM.mixed}}, \code{\link{plot.mixEM}}, \code{\link{post.beta}}
}
\references{
  Young, D. S. and Hunter, D. R. (2015) Random Effects Regression Mixtures for Analyzing Infant Habituation,
  \emph{Journal of Applied Statistics}, \bold{42(7)}, 1421--1441.
}
\examples{
data(RanEffdata)
set.seed(100)
x <- lapply(1:length(RanEffdata), function(i)
  matrix(RanEffdata[[i]][, 2:3], ncol = 2))
x <- x[1:20]
y <- lapply(1:length(RanEffdata), function(i)
  matrix(RanEffdata[[i]][, 1], ncol = 1))
y <- y[1:20]
lambda <- c(0.45, 0.55)
mu <- matrix(c(0, 4, 100, 12), 2, 2)
sigma <- 2
R <- list(diag(1, 2), diag(1, 2))
em.out <- regmixEM.mixed(y, x, sigma = sigma, arb.sigma = FALSE,
                         lambda = lambda, mu = mu, R = R,
                         addintercept.random = FALSE,
                         epsilon = 1e-02, verb = TRUE)

x.1 = em.out$x
n = sum(sapply(x.1, nrow))
x.1.sum = sum(sapply(1:length(x.1), function(i) length(x.1[[i]][,1])))
if (x.1.sum == n) {
  x = lapply(1:length(x.1), function(i) matrix(x.1[[i]][,-1], ncol = 1))
} else {
  x = x.1
}

plotly_post.beta(x = x, y = em.out$y, p.beta = em.out$posterior.beta, 
                 p.z = em.out$posterior.z)
}
\details{
This is primarily used for within \code{plot.mixEM}.
}

\keyword{internal}

\name{expRMM_EM}
\title{EM algorithm for Reliability Mixture Models (RMM) with right Censoring}
\alias{expRMM_EM}
\usage{
  expRMM_EM(x, d=NULL, lambda = NULL, rate = NULL, k = 2, 
		    complete = "tdz", epsilon = 1e-08, maxit = 1000, verb = FALSE) }

\description{
  Parametric EM algorithm for univariate finite mixture of exponentials 
  distributions with randomly right censored data.
}
\arguments{
  \item{x}{A vector of \eqn{n}{n} real positive lifetime (possibly censored) durations.
  If \code{d} is not \code{NULL} then a vector of random censoring times
  \code{c} occurred,  
  so that \eqn{x= min(x,c)} and \eqn{d = I(x <= c)}.}
  
  \item{d}{The vector of censoring indication, where 1 means observed lifetime data, 
  and 0 means censored lifetime data.}
  
  \item{lambda}{Initial value of mixing proportions.
  If \code{NULL}, then \code{lambda} is set to \code{rep(1/k,k)}.}
  
  \item{rate}{Initial value of component exponential rates, 
    all set to 1 if  \code{NULL}.}

  \item{k}{Number of components of the mixture.}

  \item{complete}{Nature of complete data involved within the EM machinery,
  can be "tdz" for \code{(t,d,z)} (the default), or "xz" for \code{(x,z)}
  (see  Bordes L. and Chauveau D. (2016) reference below).}
  
  \item{epsilon}{Tolerance limit for declaring algorithm convergence based on
  the change between two consecutive iterations.}

  \item{maxit}{The maximum number of iterations allowed, convergence
  may be declared before \code{maxit} iterations (see \code{epsilon} above).}
  
  \item{verb}{If TRUE, print updates for every iteration of the algorithm as
  it runs}
}

\value{
\code{expRMM_EM} returns a list of class "mixEM" with the following items:
  \item{x}{The input data.}
  \item{d}{The input censoring indicator.}
  \item{lambda}{The estimates for the mixing proportions.}
  \item{rate}{The estimates for the component rates.}
  \item{loglik}{The log-likelihood value at convergence of the algorithm.}
  \item{posterior}{An \eqn{n\times k}{n x k} matrix of posterior probabilities for
   observation, after convergence of the algorithm.}
  
  \item{all.loglik}{The sequence of log-likelihoods over iterations.}
  \item{all.lambda}{The sequence of mixing proportions over iterations.}
  
  \item{all.rate}{The sequence of component rates over iterations.}

   \item{ft}{A character vector giving the name of the function.}
}
\seealso{
Related functions: 
  \code{\link{plotexpRMM}},
  \code{\link{summary.mixEM}}.
  
  Other models and algorithms for censored lifetime data:
  \code{\link{weibullRMM_SEM}}, 
  \code{\link{spRMM_SEM}}.
}
\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}

\examples{
n <- 300 # sample size
m <- 2   # number of mixture components
lambda <- c(1/3,1-1/3); rate <- c(1,1/10) # mixture parameters
set.seed(1234)
x <- rexpmix(n, lambda, rate) # iid ~ exponential mixture
cs <- runif(n,0,max(x)) # Censoring (uniform) and incomplete data
t <- apply(cbind(x,cs),1,min) # observed or censored data
d <- 1*(x <= cs)              # censoring indicator

###### EM for RMM, exponential lifetimes
l0 <- rep(1/m,m); r0 <- c(1, 0.5) # "arbitrary" initial values
a <- expRMM_EM(t, d, lambda = l0, rate = r0, k = m)
summary(a)                 # EM estimates etc
plotexpRMM(a, lwd=2) # default plot of EM sequences
plot(a, which=2) # or equivalently, S3 method for "mixEM" object
%%\dontrun{
%%}
}


\keyword{file}

\name{regcr}
\title{Add a Confidence Region or Bayesian Credible Region for Regression Lines to a Scatterplot}
\alias{regcr}
\usage{
regcr(beta, x, em.beta = NULL, em.sigma = NULL, alpha = .05, 
      nonparametric = FALSE, plot = FALSE, xyaxes = TRUE, ...)
}

\description{
Produce a confidence or credible region for regression lines based on
a sample of bootstrap beta values or posterior beta values.
The beta parameters are the intercept and slope
from a simple linear regression.
}
\arguments{
  \item{beta}{An nx2 matrix of regression parameters.  The first column
gives the intercepts and the second column gives the slopes.} 
  \item{x}{An n-vector of the predictor variable which is necessary when
  nonparametric = TRUE.}
  \item{em.beta}{The estimates for beta required when obtaining confidence
  regions. This is required for performing the standardization necessary when
  obtaining nonparametric confidence regions.}
  \item{em.sigma}{The estimates for the regression standard deviation required when obtaining confidence
  regions. This is required for performing the standardization necessary when
  obtaining nonparametric confidence regions.}
  \item{alpha}{The proportion of the beta sample to remove. In other
words, 1-alpha is the level of the credible region.}
  \item{nonparametric}{
If nonparametric = TRUE, then the region is based on the convex
hull of the remaining beta after trimming, which is accomplished
using a data depth technique.
If nonparametric = FALSE, then the region is based on the
asymptotic normal approximation.}
  \item{plot}{If plot = TRUE, lines are added to the existing plot.
  The type of plot created depends on the value of xyaxes.}
  \item{xyaxes}{If xyaxes = TRUE and plot = TRUE, then a confidence or credible region
    for the regression lines is plotted on the x-y axes, presumably
    overlaid on a scatterplot of the data.  If xyaxes = FALSE and
    plot = TRUE, the (convex) credible region for the regression line is
    plotted on the beta, or intercept-slope, axes, presumably overlaid
    on a scatterplot of beta.}
  \item{...}{Graphical parameters passed to \code{lines} or \code{plot}
    command.}   
}
\value{
  \code{regcr} returns a list containing the following items:
  \item{boundary}{A matrix of points in beta, or intercept-slope, space
    arrayed along the boundary of the confidence or credible region.}
  \item{upper}{A matrix of points in x-y space arrayed along the upper
    confidence or credible limit for the regression line.}
  \item{lower}{A matrix of points in x-y space arrayed along the lower
    confidence or credible limit for the regression line.}
}
 
\seealso{
\code{\link{regmixEM}}, \code{\link{regmixMH}}
}
\examples{
## Nonparametric credible regions fit to NOdata. 

data(NOdata)
attach(NOdata)
set.seed(100)
beta <- matrix(c(1.3, -0.1, 0.6, 0.1), 2, 2)
sigma <- c(.02, .05)
MH.out <- regmixMH(Equivalence, NO, beta = beta, s = sigma, 
                   sampsize = 2500, omega = .0013)
attach(data.frame(MH.out$theta))
beta.c1 <- cbind(beta0.1[2400:2499], beta1.1[2400:2499])
beta.c2 <- cbind(beta0.2[2400:2499], beta1.2[2400:2499])
plot(NO, Equivalence)
regcr(beta.c1, x = NO, nonparametric = TRUE, plot = TRUE, 
      col = 2)
regcr(beta.c2, x = NO, nonparametric = TRUE, plot = TRUE, 
      col = 3)

}


\keyword{file}

\name{weibullRMM_SEM}
\title{St-EM algorithm for Reliability Mixture Models (RMM) of Weibull with right Censoring}
\alias{weibullRMM_SEM}

\usage{
weibullRMM_SEM(x, d = NULL, lambda = NULL, shape = NULL, scale = NULL,
        k = 2, maxit = 200, maxit.survreg = 200, epsilon = 1e-03,
        averaged = TRUE, verb = FALSE) }
\description{
  Parametric Stochastic EM (St-EM) algorithm for univariate finite mixture of Weibull 
  distributions with randomly right censored data.
}
\arguments{
  \item{x}{A vector of \eqn{n}{n} real positive lifetime (possibly censored) durations.
  If \code{d} is not \code{NULL} then a vector of random censoring times
  \code{c} occured,  
  so that \eqn{x= min(x,c)} and \eqn{d = I(x <= c)}.}
  
  \item{d}{The vector of censoring indication, where 1 means observed lifetime data, 
  and 0 means censored lifetime data.}
  
  \item{lambda}{Initial value of mixing proportions.
  If \code{NULL}, then \code{lambda} is set to \code{rep(1/k,k)}.}
  
  \item{shape}{Initial value of Weibull component shapes, 
    all set to 1 if  \code{NULL}.}
    
  \item{scale}{Initial value of Weibull component scales, 
    all set to 1 if  \code{NULL}.}

  \item{k}{Number of components of the mixture.}

  \item{maxit}{The number of iterations allowed, since for St-EM algorithms convergence
  is not based on stabilization, exactly \code{maxit} iterations are performed
  (see  Bordes L. and Chauveau D. (2016) reference below).}
  
    \item{maxit.survreg}{The number of iterations allowed in the computations of the
    MLE for censored weibull data from the  \code{survival} package
  (see  Bordes L. and Chauveau D. (2016) reference below).}
  
    \item{epsilon}{Tolerance parameter used in the numerical computations of the
    MLE for censored weibull data by \code{survreg} from the  \code{survival} package
  (see  Bordes L. and Chauveau D. (2016) reference below).}


  \item{averaged}{The way of updating parameters at each iteration: if \code{TRUE}, current values of the parameters 
  are obtained by averaging the sequence
  (see Bordes L. and Chauveau D. (2016) reference below).}

  \item{verb}{If TRUE, print updates for every iteration of the algorithm as
  it runs}
}

\details{This St-EM algorithm calls functions from  the \code{survival} package to compute 
        parametric MLE for censored weibull data.}
\value{
\code{weibullRMM_SEM} returns a list of class "mixEM" with the following items:
  \item{x}{The input data.}
  \item{d}{The input censoring indicator.}
  \item{lambda}{The estimates for the  mixing proportions.}
  \item{scale}{The estimates for the Weibull component scales.}
  \item{shape}{The estimates for the Weibull component shapes.}
  \item{loglik}{The log-likelihood value at convergence of the algorithm.}
  \item{posterior}{An \eqn{n\times k}{n x k} matrix of posterior probabilities for
   observation, after convergence of the algorithm.}
  \item{all.loglik}{The sequence of log-likelihoods over iterations.}
  \item{all.lambda}{The sequence of mixing proportions over iterations.}
  \item{all.scale}{The sequence of component scales over iterations.}
  \item{all.shape}{The sequence of component shapes over iterations.}
   \item{ft}{A character vector giving the name of the function called.}
}
\seealso{
Related functions:
\code{\link{plotweibullRMM}},   \code{\link{summary.mixEM}}.

Other models and algorithms for censored lifetime data 
(name convention is model_algorithm):
\code{\link{expRMM_EM}},
\code{\link{spRMM_SEM}}.
}

\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}

\examples{
n = 500 # sample size
m = 2 # nb components
lambda=c(0.4, 0.6)
shape <- c(0.5,5); scale <- c(1,20) # model parameters
set.seed(321)
x <- rweibullmix(n, lambda, shape, scale) # iid ~ weibull mixture
cs=runif(n,0,max(x)+10) # iid censoring times
t <- apply(cbind(x,cs),1,min) # censored observations
d <- 1*(x <= cs)              # censoring indicator

## set arbitrary or "reasonable" (e.g., data-driven) initial values
l0 <- rep(1/m,m); sh0 <- c(1, 2); sc0 <- c(2,10)
# Stochastic EM algorithm 
a <- weibullRMM_SEM(t, d, lambda = l0, shape = sh0, scale = sc0, maxit = 200)

summary(a) # Parameters estimates etc
plotweibullRMM(a) # plot of St-EM sequences
plot(a, which=2) # or equivalently, S3 method for "mixEM" object
%%\dontrun{
%%}
}

\keyword{file}

\name{spEMsymlocN01}
\alias{spEMsymlocN01}

\title{semiparametric EM-like algorithm for univariate mixture in False Discovery Rate (FDR) estimation}

\description{Return semiparametric EM-like algorithm output for a 2-components 
mixture model with one component set to Normal(0,1), and the other component 
being a unspecified but symmetric density with a location parameter. 
This model is tailored to
FDR estimation on probit transform (\code{qnorm}) of p-values arising from multiple testing.}


\usage{
spEMsymlocN01(x, mu0 = 2, bw = bw.nrd0(x), h=bw, eps = 1e-8,
              maxiter = 100, verbose = FALSE, plotf = FALSE)}

\arguments{
  \item{x}{A vector of length n consisting of the data, probit transform of pvalues, preferably sorted.}

  \item{mu0}{Starting value of vector of component means. If not set then the initial value
   is randomly generated by a \code{kmeans} of the data in two bins. Since component 1 is 
   theoretically normal(0,1), \code{mu[1]} must be 0 and \code{mu[2]} some negative value (see details).}

  \item{bw}{Bandwidth for weighted kernel density estimation.}

  \item{h}{Alternative way to specify the bandwidth, to provide backward compatibility.}

  \item{eps}{Tolerance limit for declaring algorithm convergence. 
  	Convergence is declared before \code{maxiter} iterations 
  	whenever the maximum change in any coordinate of the 
    \code{lambda} (mixing proportion estimates) and \code{mu} (mean of the semiparametric component) 
    vector  does not exceed \code{eps}}

  \item{maxiter}{The maximum number of iterations allowed; convergence
  may be declared before \code{maxiter} iterations (see \code{eps} above).}

  \item{verbose}{If TRUE, print updates for every iteration of the algorithm as
  it runs.}

  \item{plotf}{If TRUE, plots successive updates of the nonparametric density estimate
  	over iterations. Mostly for testing purpose.}
}
\details{This algorithm is a specific version of semiparametric EM-like algorithm
	similar in spirit to \code{\link{spEMsymloc}}, but specialized for FDR estimation on
	probit transform (\code{qnorm}) of p-values in multiple testing framework.
	In this model, component 1 corresponds to the individuals under the null hypothesis,
	i.e. theoretically normal(0,1) distributed, whereas component 2 corresponds to individuals in the
	alternative hypothesis, with typically very small p-values and consequently
	negative values for probit(p) data. This model only assumes
	that these individuals come from an unspecified but symmetric density with a location parameter,
	as in Bordes and Vandekerkhove (2010) and Chauveau et al. (2014).}

\value{
	\code{spEMsymlocN01} returns a list of class \code{spEMN01} with the following items:
  \item{data}{The raw data (an \eqn{n\times r}{n x r} matrix).}
  \item{posteriors}{An \eqn{n\times 2}{n x 2} matrix of posterior probabilities for
   observations. This can be used in, e.g., \code{\link{plotFDR}} to plot
   False Discovery Rate estimates.}
  \item{bandwidth}{Same as the \code{bw} input argument, returned because this
  information is needed by any method that produces density estimates from the
  output.}
  \item{lambda}{The sequence of mixing proportions over iterations.}
  \item{lambdahat}{The final estimate for mixing proportions.}
  \item{mu}{the sequence of second component mean over iterations.}
  \item{muhat}{the final estimate of second component mean.}
  \item{symmetric}{Flag indicating that the kernel density estimate is
   using a symmetry assumption.}
}

\references{
	   \itemize{
   \item Bordes, L. and Vandekerkhove, P. (2010). 
   Semiparametric two-component mixture model with a known component:
   an asymptotically normal estimator.
   Mathematical Methods of Statistics, 19(1):22-41
	
	\item Chauveau, D., Saby, N., Orton, T. G., Lemercier B., 
		Walter, C. and Arrouys, D. (2014)
    Large-scale simultaneous hypothesis testing in monitoring carbon
    content from french soil database: A semi-parametric mixture approach.
    Geoderma 219-220 (2014): 117-124.
	}
}

\author{Didier Chauveau}

\seealso{
\code{\link{spEMsymloc}}, \code{\link{normalmixEM}},
\code{\link{npEM}}, \code{\link{plot.spEMN01}},
\code{\link{plotFDR}}
}
\examples{
## Probit transform of p-values
## from a Beta-Uniform mixture model
## comparion of parametric and semiparametric EM fit
## Note: in actual situations n=thousands 
set.seed(50)
n=300 # nb of multiple tests
m=2 # 2 mixture components
a=c(1,0.1); b=c(1,1); lambda=c(0.6,0.4) # parameters
z=sample(1:m, n, rep=TRUE, prob = lambda)
p <- rbeta(n, shape1 = a[z], shape2 = b[z]) # p-values
o <- order(p)
cpd <- cbind(z,p)[o,] # sorted complete data, z=1 if H0, 2 if H1
p <- cpd[,2] # sorted p-values

y <- qnorm(p) # probit transform of the pvalues
# gaussian EM fit with component 1 constrained to N(0,1)
s1 <- normalmixEM(y, mu=c(0,-4), 
				mean.constr = c(0,NA), sd.constr = c(1,NA)) 
s2 <- spEMsymlocN01(y, mu0 = c(0,-3)) # spEM with N(0,1) fit
hist(y, freq = FALSE, col = 8, main = "histogram of probit(pvalues)")
plot(s2, add.plot = TRUE, lwd = 2)

# Exemples of plot capabilities
# Note: posteriors must be ordered by p for plot.FDR
# plotFDR(s1$post) # when true complete data not observed
# plotFDR(s1$post, s2$post) # comparing 2 strategies
plotFDR(s1$post, s2$post, lg1 = "normalmixEM", lg2 = "spEMsymlocN01", 
		complete.data = cpd) # with true FDR computed from z
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{file}

\name{poisregmixEM}
\title{EM Algorithm for Mixtures of Poisson Regressions}
\alias{poisregmixEM}
\usage{
poisregmixEM(y, x, lambda = NULL, beta = NULL, k = 2,
             addintercept = TRUE, epsilon = 1e-08, 
             maxit = 10000, verb = FALSE)
}

\description{
  Returns EM algorithm output for mixtures of Poisson regressions with
  arbitrarily many components.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{beta}.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and k is number of components.
    If NULL, then \code{beta} is generated by binning the data into k bins and using \code{glm} on
    the values in each of the bins.  If both \code{lambda} and \code{beta} are NULL, then 
    number of components is determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless \code{lambda} and \code{beta} are both NULL.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{poisregmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The predictor values.}
  \item{y}{The response values.}
  \item{lambda}{The final mixing proportions.}
  \item{beta}{The final Poisson regression coefficients.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
    observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{logisregmixEM}}
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
  
  Wang, P., Puterman, M. L., Cockburn, I. and Le, N.  (1996)
  Mixed Poisson Regression Models with Covariate Dependent Rates, \emph{Biometrics},
  \bold{52(2)}, 381--400.
}
\examples{
## EM output for data generated from a 2-component model.

set.seed(100)
beta <- matrix(c(1, .5, .7, -.8), 2, 2)
x <- runif(50, 0, 10)
xbeta <- cbind(1, x)\%*\%beta
w <- rbinom(50, 1, .5)
y <- w*rpois(50, exp(xbeta[, 1]))+(1-w)*rpois(50, exp(xbeta[, 2]))
out <- poisregmixEM(y, x, verb = TRUE,  epsilon = 1e-03)
out
}


\keyword{file}

\name{plot.mvnpEM}
\title{Plots of Marginal Density Estimates from the mvnpEM Algorithm Output}
\alias{plot.mvnpEM} 
\usage{
\method{plot}{mvnpEM}(x, truenorm = FALSE, lambda = NULL, mu = NULL, v = NULL, 
            lgdcex = 1, \dots)
}
\description{
    Takes an object of class \code{mvnpEM}, as the one returned by the  \code{\link{mvnpEM}}
    algorithm, and returns a set of plots of the 
    density estimates for each coordinate within each multivariate block.
    All the components are displayed on each plot so it is possible
    to see the mixture structure for each coordinate and block. The final bandwidth values are also displayed,
    in a format depending on the bandwidth strategy .     
} 
\arguments{
  \item{x}{An object of class \code{mvnpEM} such as the output
  of the \code{\link{mvnpEM}} function}
  
  \item{truenorm}{Mostly for checking purpose, if the nonparametric model is to be compared
  with a multivariate Gaussian mixture as the true model.}
  
  \item{lambda}{true weight parameters, for Gaussian models only (see above)}
  \item{mu}{true mean parameters, for Gaussian models only (see above)}
  \item{v}{true covariance matrices, for Gaussian models only (see above)}

  \item{lgdcex}{Character expansion factor for \code{\link{legend}}.}
  \item{\dots}{Any remaining arguments are passed to \code{\link{hist}}.}
}
\value{
  \code{plot.mvnpEM} currently just plots the figure.
} 

\seealso{ 
\code{\link{mvnpEM}}, \code{\link{npEM}}, \code{\link{density.npEM}}
}
 

\examples{ 
# example as in Chauveau and Hoang (2015) with 6 coordinates
\dontrun{
m=2; r=6; blockid <-c(1,1,2,2,3,3) # 3 bivariate blocks 
# generate some data x ...
a <- mvnpEM(x, mu0=2, blockid, samebw=F) # adaptive bandwidth
plot(a) # this S3 method produces 6 plots of univariate marginals
summary(a)}
}

\keyword{file}

\name{plotseq.npEM}
\alias{plotseq.npEM}
\title{Plotting sequences of estimates from non- or semiparametric EM-like Algorithm}
\usage{
  \method{plotseq}{npEM}(x, \dots)
}
\description{
  Returns plots of the sequences of scalar parameter
  estimates along iterations from an object of class \code{npEM}.
}
\arguments{
  \item{x}{an object of class \code{npEM}, as output by \code{\link{npEM}} 
  or \code{\link{spEMsymloc}}}
  \item{\dots}{further parameters that are passed to \code{\link{plot}}}
}
\details{
\code{plotseq.npEM} returns a figure with one plot for each component 
 proportion, and, in the case of \code{\link{spEMsymloc}}, one plot for each 
component mean.
}
\seealso{
\code{\link{plot.npEM}}, \code{\link{rnormmix}},
\code{\link{npEM}}, \code{\link{spEMsymloc}}
}
\references{
   \itemize{
   \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics (to appear).

   \item Bordes, L., Chauveau, D., and Vandekerkhove, P. (2007),
   An EM algorithm for a semiparametric mixture model, 
   Computational Statistics and Data Analysis, 51: 5429-5443.   
   }
}
\examples{
## Example from a normal location mixture
n <- 200
set.seed(100)
lambda <- c(1/3,2/3)
mu <- c(0, 4); sigma<-rep(1, 2)
x <- rnormmix(n, lambda, mu, sigma)
b <- spEMsymloc(x, mu0=c(-1, 2), stochastic=FALSE)
plotseq(b)
bst <- spEMsymloc(x, mu0=c(-1, 2), stochastic=TRUE)
plotseq(bst)
}

\keyword{file}

\name{hmeEM}
\title{EM Algorithm for Mixtures-of-Experts}
\alias{hmeEM}
\usage{
hmeEM(y, x, lambda = NULL, beta = NULL, sigma = NULL, w = NULL,
      k = 2, addintercept = TRUE, epsilon = 1e-08, 
      maxit = 10000, verb = FALSE)
}

\description{
  Returns EM algorithm output for a mixture-of-experts model. Currently, this code
  only handles a 2-component mixture-of-experts, but will be extended to the
  general k-component hierarchical mixture-of-experts.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{lambda}{Initial value of mixing proportions, which are modeled as an inverse
    logit function of the predictors.  Entries should sum to 1.  
    If NULL, then \code{lambda} is taken as 1/\code{k} for each \code{x}.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and k is number of components.
    If NULL, then \code{beta} has standard normal entries according to a binning method done on the data.}
  \item{sigma}{A vector of standard deviations.  If NULL, then \eqn{1/\code{sigma}^2} has
    random standard exponential entries according to a binning method done on the data.}
  \item{w}{A p-vector of coefficients for the way the mixing proportions are modeled.  See \code{lambda}.}
  \item{k}{Number of components.  Currently, only \code{k}=2 is accepted.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{hmeEM} returns a list of class \code{mixEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's if \code{addintercept} = TRUE).}
  \item{y}{The response values.}
  \item{w}{The final coefficients for the functional form of the mixing proportions.}
  \item{lambda}{An nxk matrix of the final mixing proportions.}
  \item{beta}{The final regression coefficients.}
  \item{sigma}{The final standard deviations. If \code{arbmean} = FALSE, then only the smallest standard
   deviation is returned. See \code{scale} below.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{regmixEM}}
}
\references{
  Jacobs, R. A., Jordan, M. I., Nowlan, S. J. and Hinton, G. E. (1991) Adaptive Mixtures of Local
  Experts, \emph{Neural Computation} \bold{3(1)}, 79--87.
  
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
## EM output for NOdata.
 
data(NOdata)
attach(NOdata)
set.seed(100)
em.out <- regmixEM(Equivalence, NO)
hme.out <- hmeEM(Equivalence, NO, beta = em.out$beta)
hme.out[3:7]
}


\keyword{file}

\name{repnormmixmodel.sel}
\title{Model Selection in Mixtures of Normals with Repeated Measures}
\alias{repnormmixmodel.sel}
\usage{
repnormmixmodel.sel(x, k = 2, ...)
}
\description{
  Assess the number of components in a mixture model with normal components and repeated measures 
  using the Akaike's information criterion (AIC), Schwartz's Bayesian information criterion (BIC), 
  Bozdogan's consistent AIC (CAIC), and Integrated Completed Likelihood (ICL).
}
\arguments{
  \item{x}{An mxn matrix of observations. The rows correspond to the repeated measures and the columns correspond
  to the subject.}
  \item{k}{The maximum number of components to assess.}
  \item{...}{Additional arguments passed to \code{repnormmixEM}.}
}
\value{
  \code{repnormmixmodel.sel} returns a matrix of the AIC, BIC, CAIC, and ICL values along with the winner (i.e., the highest
  value given by the model selection criterion) for a mixture of normals with repeated measures.
}
\seealso{
  \code{\link{repnormmixEM}}
}
\references{
  Biernacki, C., Celeux, G., and Govaert, G. (2000). Assessing a Mixture Model for Clustering with the
  Integrated Completed Likelihood. \emph{IEEE Transactions on Pattern Analysis and Machine Intelligence}, 22(7):719-725.
  
  Bozdogan, H. (1987). Model Selection and Akaike's Information Criterion (AIC): The General Theory and its
  Analytical Extensions. \emph{Psychometrika}, 52:345-370.
}
\examples{
## Assessing the number of components for the water-level task data set.

data(Waterdata)
water<-t(as.matrix(Waterdata[,3:10]))
set.seed(100)
out <- repnormmixmodel.sel(water, k = 3, epsilon = 5e-01)
out
}

\keyword{file}

\name{parse.constraints}
\title{Constraint Function}
\alias{parse.constraints}
\usage{
parse.constraints(constr, k = 2, allsame = FALSE)
}
\description{
Constraint function for some mixture EM algorithms.
}
\arguments{
  \item{constr}{Vector indicating constrained/unconstrained means.}
  \item{k}{Number of components.}
  \item{allsame}{Logical indicating for processing the constraints.}
}
\details{
This function is not intended to be called by the user.
}


\keyword{internal}

\name{test.equality}
\title{Performs Chi-Square Tests for Scale and Location Mixtures}
\alias{test.equality}
\usage{
test.equality(y, x = NULL, arbmean = TRUE, arbvar = FALSE, 
              mu = NULL, sigma = NULL, beta = NULL, 
              lambda = NULL, ...)
}

\description{
  Performs a likelihood ratio test of a location (or scale) normal or regression mixture versus the more
  general model.  For a normal mixture, the alternative hypothesis is that each
  component has its own mean and variance, whereas the null is that all means
  (in the case of a scale mixture) or all variances (in the case of a location
  mixture) are equal.
  This test is asymptotically chi-square with degrees of freedom equal to k-1,
  where k is
  the number of components.
}
\arguments{
  \item{y}{The responses for \code{regmixEM} or the data for \code{normalmixEM}.}
  \item{x}{The predictors for \code{regmixEM}.}
  \item{arbmean}{If FALSE, then a scale mixture analysis is performed for \code{normalmixEM} or \code{regmixEM}.}
  \item{arbvar}{If FALSE, then a location mixture analysis is performed for \code{normalmixEM} or \code{regmixEM}.}
  \item{mu}{An optional vector for starting values (under the null hypothesis) for \code{mu} in \code{normalmixEM}.}
  \item{sigma}{An optional vector for starting values (under the null hypothesis) for \code{sigma} in \code{normalmixEM} or \code{regmixEM}.}
  \item{beta}{An optional matrix for starting values (under the null hypothesis) for \code{beta} in \code{regmixEM}.}
  \item{lambda}{An otional vector for starting values (under the null hypothesis) for \code{lambda} in \code{normalmixEM} or \code{regmixEM}.}
  \item{...}{Additional arguments passed to the various EM algorithms for the mixture of interest.} 
}
\value{
  \code{test.equality} returns a list with the following items:
  \item{chi.sq}{The chi-squared test statistic.}
  \item{df}{The degrees of freedom for the chi-squared test statistic.}
  \item{p.value}{The p-value corresponding to this likelihood ratio test.}
}
\seealso{
\code{\link{test.equality.mixed}}
}
\examples{
## Should a location mixture be used for the Old Faithful data?

data(faithful)
attach(faithful)
set.seed(100)
test.equality(y = waiting, arbmean = FALSE, arbvar = TRUE)

}


\keyword{file}

\name{logisregmixEM}
\title{EM Algorithm for Mixtures of Logistic Regressions}
\alias{logisregmixEM}
\usage{
logisregmixEM(y, x, N = NULL, lambda = NULL, beta = NULL, k = 2,
              addintercept = TRUE, epsilon = 1e-08, 
              maxit = 10000, verb = FALSE)
}

\description{
  Returns EM algorithm output for mixtures of logistic regressions with
  arbitrarily many components.
}
\arguments{
  \item{y}{An n-vector of successes out of N trials.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{N}{An n-vector of number of trials for the logistic regression.  
  If NULL, then \code{N} is an n-vector of 1s for binary logistic regression.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{beta}.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and k is number of components.
    If NULL, then \code{beta} is generated by binning the data into k bins and using \code{glm} on
    the values in each of the bins.  If both \code{lambda} and \code{beta} are NULL, then 
    number of components is determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless \code{lambda} and \code{beta} are both NULL.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{logisregmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The predictor values.}
  \item{y}{The response values.}
  \item{lambda}{The final mixing proportions.}
  \item{beta}{The final logistic regression coefficients.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
    observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}   
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{poisregmixEM}}
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
## EM output for data generated from a 2-component logistic regression model.

set.seed(100)
beta <- matrix(c(1, .5, 2, -.8), 2, 2)
x <- runif(50, 0, 10)
x1 <- cbind(1, x)
xbeta <- x1\%*\%beta
N <- ceiling(runif(50, 50, 75))
w <- rbinom(50, 1, .3)
y <- w*rbinom(50, size = N, prob = (1/(1+exp(-xbeta[, 1]))))+
              (1-w)*rbinom(50, size = N, prob = 
              (1/(1+exp(-xbeta[, 2]))))
out.1 <- logisregmixEM(y, x, N, verb = TRUE, epsilon = 1e-01)
out.1

## EM output for data generated from a 2-component binary logistic regression model.

beta <- matrix(c(-10, .1, 20, -.1), 2, 2)
x <- runif(500, 50, 250)
x1 <- cbind(1, x)
xbeta <- x1\%*\%beta
w <- rbinom(500, 1, .3)
y <- w*rbinom(500, size = 1, prob = (1/(1+exp(-xbeta[, 1]))))+
              (1-w)*rbinom(500, size = 1, prob = 
              (1/(1+exp(-xbeta[, 2]))))
out.2 <- logisregmixEM(y, x, beta = beta, lambda = c(.3, .7), 
                       verb = TRUE, epsilon = 1e-01)
out.2
}


\keyword{file}

\name{ise.npEM}
\title{Integrated Squared Error for a selected density from npEM output}
\alias{ise.npEM}
\usage{
ise.npEM(npEMout, component=1, block=1, truepdf, lower=-Inf, 
         upper=Inf, plots = TRUE, ...)
}
\description{
Computes the integrated squared error for a selected estimated density 
from \code{\link{npEM}} output (selected by specifying the component 
and block number), 
relative to a true pdf that must be specified by the user.
The range for the numerical integration must be specified.  This function
also returns (by default) a plot of the
true and estimated densities.
}
\arguments{
  \item{npEMout}{An object of class \code{npEM} such as the output
  of the \code{\link{npEM}} function}
  \item{component, block}{Component and block of particular density to analyze
  from \code{npEMout}.}
  \item{truepdf}{an \R function taking a numeric first argument and 
  returning a numeric vector of the same length. Returning a 
  non-finite element will generate an error.}
  \item{lower, upper}{the limits of integration.  Can be infinite.}
  \item{plots}{logical:  Should plots be produced?}
  \item{...}{additional arguments to be passed to \code{truepdf}
  	(and that may be mandatory like, e.g., the \code{df = } argument of \code{dt}). 
  Remember to use argument names not matching those of \code{ise.npRM}.}
}
\value{
  Just as for the \code{\link{integrate}} function,
  a list of class \code{"integrate"} with components
  \item{value}{the final estimate of the integral.}
  \item{abs.error}{estimate of the modulus of the absolute error.}
  \item{subdivisions}{the number of subintervals produced in the
    subdivision process.}
  \item{message}{\code{"OK"} or a character string giving the error message.}
  \item{call}{the matched call.}
}
\details{This function calls the \code{\link{wkde}} (weighted kernel
density estimate) function.
}
\seealso{
\code{\link{npEM}}, \code{\link{wkde}}, \code{\link{integrate}}
}
\references{
   \itemize{
   
  \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526.

   \item Benaglia, T., Chauveau, D., Hunter, D. R., and Young, D. (2009),
  mixtools: An R package for analyzing finite mixture models.
	Journal of Statistical Software, 32(6):1-29.


   }
}
\examples{
# Mixture with mv gaussian model
set.seed(100)
m <- 2 # no. of components
r <- 3 # no. of repeated measures (coordinates)
lambda <- c(0.4, 0.6)
# Note:  Need first 2 coordinates conditionally iid due to block structure
mu <- matrix(c(0, 0, 0, 3, 3, 5), m, r, byrow=TRUE) # means 
sigma <- matrix(rep(1, 6), m, r, byrow=TRUE) # stdevs
blockid = c(1,1,2) # block structure of coordinates
n <- 200
x <- rmvnormmix(n, lambda, mu, sigma) # simulated data

# fit the model with "arbitrary" initial centers
centers <- matrix(c(0, 0, 0, 4, 4, 4), 2, 3, byrow=TRUE) 
a <- npEM(x, centers, blockid, eps=1e-8, verb=FALSE)

# Calculate integrated squared error for j=2, b=1:
j <- 2 # component
b <- 1 # block
coords <- a$blockid == b
ise.npEM(a, j, b, dnorm, lower=0, upper=10, plots=TRUE,
         mean=mu[j,coords][1], sd=sigma[j, coords][1])


# The following (lengthy) example recreates the normal multivariate 
# mixture model simulation from Benaglia et al (2009).  
mu <- matrix(c(0, 0, 0, 3, 4, 5), m, r, byrow=TRUE) 
nbrep <- 5  # Benaglia et al use 300 replications

# matrix for storing sums of Integrated Squared Errors 
ISE <- matrix(0,m,r,dimnames=list(Components=1:m, Blocks=1:r)) 

nblabsw <- 0 # no. of label switches
for (mc in 1:nbrep) {
  print(paste("REPETITION", mc))
	x <- rmvnormmix(n,lambda,mu,sigma) # simulated data
  a <- npEM(x, centers, verb=FALSE) #default:
	if (a$lambda[1] > a$lambda[2]) nblabsw <- nblabsw + 1
	for (j in 1:m) {  # for each component
		for (k in 1:r) { # for each coordinate; not assuming iid!
      # dnorm with correct mean, sd is the true density:
      ISE[j,k] <- ISE[j,k] + ise.npEM(a, j, k, dnorm, lower=mu[j,k]-5, 
               upper=mu[j,k]+5, plots=FALSE, mean=mu[j,k], 
               sd=sigma[j,k])$value
    }
  }
	MISE <- ISE/nbrep # Mean ISE
	sqMISE <- sqrt(MISE) # root-mean-integrated-squared error
}
sqMISE
}


\keyword{file}

\name{plotly_FDR}
\alias{plotly_FDR}

\title{Plot False Discovery Rate (FDR) estimates from output by EM-like strategies using \code{plotly}}

\description{This is an updated version of \code{plotFDR}. For more technical details, please refer to \code{plotFDR}.}

\usage{
plotly_FDR(post1, post2=NULL, lg1="FDR 1", lg2=NULL, 
          compH0=1, alpha=0.1, complete.data =NULL, pctfdr=0.3,
          col = NULL, width = 3 ,
          title = NULL , title.size = 15 , title.x = 0.5 , title.y = 0.95,
          xlab = "Index" , xlab.size = 15 , xtick.size = 15,
          ylab = "Probability" , ylab.size = 15 , ytick.size = 15,
          legend.text = "" , legend.text.size = 15 , legend.size = 15)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{post1}{The matrix of posterior probabilities from objects such as the output
  	from \code{\link{spEMsymlocN01}}. The rows need to be sorted by increasing pvalues.}
  \item{post2}{A second object like \code{post1} if comparison is desired, also sorted by increasing pvalues.}
  \item{lg1}{Text describing the FDR estimate in \code{post1}.}
  \item{lg2}{Text describing the FDR estimate in \code{post2} if provided.}
  \item{compH0}{The component indicator associated to the null hypothesis H0,
  	normally 1 since it is defined in this way in \code{\link{spEMsymlocN01}}, but in case of label switching in other algorithms it can be set to \code{2}.}
  \item{alpha}{The target FDR level; the index at which the FDR estimate crosses the horizontal line for level \code{alpha} gives the maximum number of cases to reject.}
  \item{complete.data}{An array with \eqn{n} lines and 2 columns, with the component indicator in column 1 and the p-values in column 2, sorted by p-values.}
  \item{pctfdr}{The level up to which the FDR is plotted, i.e. 
  	the scale of the vertical axis.}
  \item{col}{Color of traces.}
  \item{width}{Width of traces.}
  \item{title}{Text of the main title.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horsizontal position of the main title.}
  \item{title.y}{Vertical posotion of the main title.}
  \item{xlab}{Label of X-axis.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab}{Label of Y-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
  \item{legend.text}{Title of legend.}
  \item{legend.text.size}{Size of the legend title.}
  \item{legend.size}{Size of legend.}
 }

\value{A plot of one or two FDR estimates, with the true FDR if available}

\references{
	   \itemize{
	\item Chauveau, D., Saby, N., Orton, T. G., Lemercier B., Walter, C. and Arrouys, D.
	Large-scale simultaneous hypothesis testing in monitoring carbon content 
  from French soil database -- A semi-parametric mixture approach, Geoderma 219-220 (2014), 117-124.
	}
}

\author{Didier Chauveau}

\seealso{
\code{\link{spEMsymlocN01}}, \code{\link{plotFDR}}}

\examples{
## Probit transform of p-values
## from a Beta-Uniform mixture model
## comparion of parametric and semiparametric EM fit
## Note: in actual situations n=thousands
set.seed(50)
n=300 # nb of multiple tests
m=2 # 2 mixture components
a=c(1,0.1); b=c(1,1); lambda=c(0.6,0.4) # parameters
z=sample(1:m, n, rep=TRUE, prob = lambda)
p <- rbeta(n, shape1 = a[z], shape2 = b[z]) # p-values
o <- order(p)
cpd <- cbind(z,p)[o,] # sorted complete data, z=1 if H0, 2 if H1
p <- cpd[,2] # sorted p-values
y <- qnorm(p) # probit transform of the pvalues
# gaussian EM fit with component 1 constrained to N(0,1)
s1 <- normalmixEM(y, mu=c(0,-4),
                  mean.constr = c(0,NA), sd.constr = c(1,NA))
s2 <- spEMsymlocN01(y, mu0 = c(0,-3)) # spEM with N(0,1) fit
plotly_FDR(s1$post, s2$post, lg1 = "normalmixEM", lg2 = "spEMsymlocN01",
           complete.data = cpd) # with true FDR computed from z
}
%
\keyword{file}

\name{depth}
\alias{depth}
\title{Elliptical and Spherical Depth}
\description{
  Computation of spherical or elliptical depth.
}
\usage{
 depth(pts, x, Cx = var(x))

}
\arguments{
  \item{pts}{A kxd matrix containing the k points that one wants to compute the depth. Each row is a point. }
  \item{x}{A nxd matrix containing the reference data. Each row is an observation.}
  \item{Cx}{A dxd scatter matrix for the data x where the default is var(x). When Cx = I(d), it returns the sphercial depth.}

}
\value{
  \code{depth} returns a k-vector where each entry is the elliptical depth of a point in \code{pts}.

}
\references{
  Elmore, R. T., Hettmansperger, T. P. and Xuan, F. (2000) Spherical Data Depth and a Multivariate Median,
  \emph{Proceedings of Data Depth: Robust Multivariate Statistical Analysis, Computational Geometry and Applications}.
}
\seealso{\code{\link{regcr}}
}
\examples{
  set.seed(100)
  x <- matrix(rnorm(200),nc = 2)
  depth(x[1:3, ], x)
}
\note{\code{depth} is used in \code{regcr}.}

\keyword{file}

\name{spregmix}
\title{EM-like Algorithm for Semiparametric Mixtures of Regressions}
\alias{spregmix}
\usage{
spregmix(lmformula, bw = NULL, constbw = FALSE,
         bwmult = 0.9, z.hat = NULL, symm = TRUE, betamethod = "LS",
         m = ifelse(is.null(z.hat), 2, ncol(z.hat)),
         epsilon = 1e-04, maxit = 1000, verbose = FALSE, 
         \dots) 
}
\description{
  Returns parameter estimates for finite mixtures of linear
  regressions with unspecified error structure.  Based on 
  Hunter and Young (2012).
}
\arguments{
  \item{lmformula}{Formula for a linear model, in the same format used by
  \code{\link{lm}}.  Additional parameters may be passed to \code{\link{lm}}
  via the \code{...} argument.}
  \item{bw}{Initial bandwidth value.  If NULL, this will be chosen automatically
  by the algorithm.}
  \item{constbw}{Logical:  If TRUE, the bandwidth is held constant throughout the
  algorithm; if FALSE, it adapts at each iteration according to the rules
  given in Hunter and Young (2012).}
  \item{bwmult}{
  Whenever it is updated automatically,
  the bandwidth is equal to \code{bwmult} divided by 
  the fifth root of \eqn{n} times the smaller of s and IQR/1.34, 
  where s and IQR are estimates of the standard deviation and interquartile 
  range of the residuals, as explained in Hunter and Young (2012).
  The value of 0.9 gives the rule of Silverman (1986) and
  the value of 1.06 gives the rule of Scott (1992).
  Larger values lead to greater smoothing, whereas smaller values lead to less 
  smoothing.  }
  \item{z.hat}{Initial nxm matrix of posterior probabilities.  If NULL, this
  is initialized randomly.  As long as a parametric estimation method like least
  squares is used to estimate \code{beta} in each M-step, the \code{z.hat}
  values are the only values necessary to begin the EM iterations.}
  \item{symm}{Logical:  If TRUE, the error density is assumed symmetric
  about zero.  If FALSE, it is not.  WARNING:  If FALSE, the intercept parameter
  is not uniquely identifiable if it is included in the linear model.}
  \item{betamethod}{Method of calculating beta coefficients in the
  M-step.  Current possible values are "LS" for least-squares; 
  "L1" for least absolute deviation; "NP" for fully nonparametric; 
  and "transition" for a transition
  from least squares to fully nonparametric. If something other than
  these four possibilities is used, then "NP" is assumed. For details
  of these methods, see Hunter and Young (2012).}
  \item{m}{Number of components in the mixture.}
  \item{epsilon}{Convergence is declared if the largest change in any lambda or
  beta coordinate is smaller than \code{epsilon}.}
  \item{maxit}{The maximum number of iterations; if convergence is never declared
  based on comparison with \code{epsilon}, then the algorithm stops after
  \code{maxit} iterations.}
  \item{verbose}{Logical: If TRUE, then various updates are printed during 
  each iteration of the algorithm.} 
  \item{\dots}{Additional parameters passed to the 
  \code{\link{model.frame}} and \code{\link{model.matrix}} functions,
  which are used to obtain the response and predictor of the regression.}
}
\value{
  \code{regmixEM} returns a list of class \code{npEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's if \code{addintercept} = TRUE).}
  \item{y}{The response values.}
  \item{lambda}{The mixing proportions for every iteration in the form of a 
              matrix with m columns and (#iterations) rows}
  \item{beta}{The final regression coefficients.}
  \item{posterior}{An nxm matrix of posterior probabilities for
   observations.}
  \item{np.stdev}{Nonparametric estimate of the standard deviation, as given
  in Hunter and Young (2012)}
  \item{bandwidth}{Final value of the bandwidth}
  \item{density.x}{Points at which the error density is estimated}
  \item{density.y}{Values of the error density at the points \code{density.x}}
  \item{symmetric}{Logical:  Was the error density assumed symmetric?}
  \item{loglik}{A quantity similar to a log-likelihood, computed just like 
  a standard loglikelihood would be, conditional on the component density 
  functions being equal to the final density estimates.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{regmixEM}}, \code{\link{spEMsymloc}}, \code{\link{lm}}
}
\references{
  Hunter, D. R. and Young, D. S. (2012) Semi-parametric Mixtures of Regressions,
  Journal of Nonparametric Statistics 24(1): 19-38.

  Scott, D. W. (1992) \emph{Multivariate Density Estimation},
  John Wiley & Sons Inc., New York.

  Silverman, B. W. (1986). \emph{Density Estimation for Statistics and Data 
  Analysis}, Chapman & Hall, London.
}
\examples{
data(tonedata)
## By default, the bandwidth will adapt and the error density is assumed symmetric
set.seed(100)
a=spregmix(tuned~stretchratio, bw=.2, data=tonedata, verb=TRUE)

## Look at the sp mixreg solution:
plot(tonedata)
abline(a=a$beta[1,1],b=a$beta[2,1], col=2)
abline(a=a$beta[1,2],b=a$beta[2,2], col=3)

## Look at the nonparametric KD-based estimate of the error density, 
## constrained to be zero-symmetric:
plot(xx<-a$density.x, yy<-a$density.y, type="l")
## Compare to a normal density with mean 0 and NP-estimated stdev:
z <- seq(min(xx), max(xx), len=200)
lines(z, dnorm(z, sd=sqrt((a$np.stdev)^2+a$bandwidth^2)), col=2, lty=2)
# Add bandwidth^2 to variance estimate to get estimated var of KDE

## Now add the sp mixreg estimate without assuming symmetric errors:
b=spregmix(tuned~stretchratio, bw=.2, , symm=FALSE, data=tonedata, verb=TRUE)
lines(b$density.x, b$density.y, col=3)
}


\keyword{file}

\name{ldmult}
\title{Log-Density for Multinomial Distribution}
\alias{ldmult}
\usage{
ldmult(y, theta)
}
\description{
Return the logarithm of the multinomial density function.
}
\arguments{
  \item{y}{A vector of multinomial counts.}
  \item{theta}{A vector of multinomial probabilities.  May have same number of
    components as or one fewer component than \code{y}.  In the latter case, 
    an extra component is appended so that theta sums to one.}
}
\value{
  \code{ldmult} returns the logarithm of the multinomial density
  with parameter \code{theta}, evaluated at \code{y}.  
}
\details{
This function is called by \code{multmixEM}.
}
\seealso{
\code{\link{multmixEM}}
}
\examples{
y <- c(2, 2, 10)
theta <- c(0.2, 0.3, 0.5)
ldmult(y, theta)

}

\keyword{internal}

\name{plotly_spRMM}
\alias{plotly_spRMM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Plot output from Stochastic EM algorithm for semiparametric scaled mixture of censored data using \code{plotly}.
}

\description{This is an updated version of \code{plotspRMM} function. For technical details, please refer to \code{\link{plotspRMM}.
}}

\usage{
  plotly_spRMM(sem, tmax = NULL,
               width = 3 , col = '#1f77b4', cex = 3,
               title.size = 15 , 
               title.x = 0.5 , title.y = 0.95,
               xlab.size = 15 , xtick.size=15 ,
               ylab.size = 15 , ytick.size=15)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{sem}{An object returned by \code{\link{spRMM_SEM}}.}
  \item{tmax}{The max time for \eqn{x} axis, set to some default value if \code{NULL}.}
  \item{width}{Width of lines.}
  \item{col}{Color of lines.}
  \item{cex}{Size of dots.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horizontal position of the main title.}
  \item{title.y}{Vertical position of the main title.}
  \item{xlab.size}{Size of the label of X-axis.}
  \item{xtick.size}{Size of the tick of X-axis.}
  \item{ylab.size}{Size of the label of Y-axis.}
  \item{ytick.size}{Size of the tick of Y-axis.}
  }
  


\value{The four plots returned.}


\seealso{
Related functions: \code{\link{spRMM_SEM}} , \code{\link{plotspRMM}}.

Other models and algorithms for censored lifetime data 
(name convention is model_algorithm):
\code{\link{expRMM_EM}},
\code{\link{weibullRMM_SEM}}.
}

\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{https://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
\dontrun{
n=500 # sample size
m=2 # nb components
lambda=c(0.4, 0.6) # parameters
meanlog=3; sdlog=0.5; scale=0.1
set.seed(12)
# simulate a scaled mixture of lognormals
x <- rlnormscalemix(n, lambda, meanlog, sdlog, scale)
cs=runif(n,20,max(x)+400) # Censoring (uniform) and incomplete data
t <- apply(cbind(x,cs),1,min)
d <- 1*(x <= cs)
tauxc <- 100*round( 1-mean(d),3)
cat(tauxc, "percents of data censored.\n")

c0 <- c(25, 180) # data-driven initial centers (visible modes)
sc0 <- 25/180    # and scaling
s <- spRMM_SEM(t, d, scaling = sc0, centers = c0, bw = 15, maxit = 100)

plotly_spRMM(s) # default
summary(s)   # S3 method for class "spRMM"
}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{file}

\name{aug.x}
\title{Augmented Predictor Function}
\alias{aug.x}
\usage{
aug.x(X, cp.locs, cp, delta = NULL)
}
\description{
Creates the augmented predictor matrix based on an appropriately defined changepoint structure.
}
\arguments{
  \item{X}{The raw matrix of predictor values.  Note that the raw data matrix should not include a columns of 1's.}
  \item{cp.locs}{The locations of the changepoints.  The length of this vector must be equal to the sum of the entries of \code{cp}.}
  \item{cp}{A vector having length equal to the number of predictors.}
  \item{delta}{A vector to accommodate discontinuities.  If NULL, then no discontinuities are included.  Otherwise, this must be a vector of the same length as \code{cp.locs}.}
}
\value{
  \code{aug.x} returns a matrix of the original matrix \code{X} with the predictor adjusted for changepoints and (optional) discontinuities.
}
\details{
This function is called by \code{segregmixEM} and the associated internal functions.
}
\seealso{
\code{\link{segregmixEM}}
}
\examples{
x <- matrix(1:30, nrow = 10)
cp <- c(1, 3, 0)
cp.locs <- c(3, 12, 14, 16)
d <- rep(0, 4)
x1 <- aug.x(x, cp.locs, cp, delta = NULL)
x1
x2 <- aug.x(x, cp.locs, cp, delta = d)
x2
}


\keyword{internal}

\name{plotly_mixMCMC}
\title{Various Plots Pertaining to Mixture Model Output Using MCMC Methods using \code{plotly}}
\alias{plotly_mixMCMC} 
\usage{ 
plotly_mixMCMC(x, trace.plot = TRUE, summary.plot = FALSE, burnin = 2000, 
               credit.region = 0.95, col.cr = NULL,
               cex.trace = 3, width.trace = 3, 
               cex.summary = 3, width.summary = 1,
               title.trace = "", title.trace.x = 0.5, 
               title.trace.y = 0.95, title.trace.size = 15,
               xlab.trace = "Index", xlab.trace.size = 15, xtick.trace.size = 15,
               ylab.trace = NULL, ylab.trace.size = 15, ytick.trace.size = 15,
               title.summary = "Credible Regions", title.summary.x = 0.5, 
               title.summary.y = 0.95, title.summary.size = 15,
               xlab.summary = "Predictor", xlab.summary.size = 15, 
               xtick.summary.size = 15,
               ylab.summary = "Response", ylab.summary.size = 15, 
               ytick.summary.size = 15
) 
}

\description{
    This is an updated version of \code{plot.mixMCMC}. For technical details, please refer to \code{plot.mixMCMC}.
} 
\arguments{
  \item{x}{An object of class \code{mixMCMC}.}
  \item{trace.plot}{If TRUE, trace plots of the various parameters estimated by the MCMC methods is given.}
  \item{summary.plot}{Graphics pertaining to certain mixture models.  The details are given below.}
  \item{burnin}{The values 1 to \code{burnin} are dropped when producing the plots in \code{summary.plots}.}
  \item{credit.region}{Confidence level of credit region.}
  \item{col.cr}{Color of credit region. Number of color specified needs to be consistent with number of components.}
  \item{cex.trace}{Dot size of trace plots.}
  \item{width.trace}{Line width of trace plots.}
  \item{cex.summary}{Dot size of summary plots.}
  \item{width.summary}{Line width of summary plots.}
  \item{title.trace}{Text of the main title of trace plots.}
  \item{title.trace.x}{Horizontal position of main title of trace plots.}
  \item{title.trace.y}{Vertical position of main title of trace plots.}
  \item{title.trace.size}{Text sise of main title of trace plots.}
  \item{xlab.trace}{Label of X-axis of trace plots.}
  \item{xlab.trace.size}{Size of the lable of X-axis of trace plots.}
  \item{xtick.trace.size}{Size of tick lables of X-axis of trace plots.}
  \item{ylab.trace}{Label of Y-axis of trace plots.}
  \item{ylab.trace.size}{Size of the lable of Y-axis of trace plots.}
  \item{ytick.trace.size}{Size of tick lables of Y-axis of trace plots.}
  \item{title.summary}{Text of the main title of summar plot.}
  \item{title.summary.x}{Horizontal position of main title of summary plot.}
  \item{title.summary.y}{Vertical position of main title of summary plot.}
  \item{title.summary.size}{Text sise of main title of summary plot.}
  \item{xlab.summary}{Label of X-axis of summary plot.}
  \item{xlab.summary.size}{Size of the lable of X-axis of summary plot.}
  \item{xtick.summary.size}{Size of tick lables of X-axis of summary plot.}
  \item{ylab.summary}{Label of Y-axis of summary plot.}
  \item{ylab.summary.size}{Size of the lable of Y-axis of summary plot.}
  \item{ytick.summary.size}{Size of tick lables of Y-axis of summary plot.}

}
\value{
  \code{plotly_mixMCMC} returns trace plots of the various parameters estimated by the MCMC methods for all objects of class
  \code{mixMCMC}.  In addition, other plots may be produced for the following k-component mixture model functions:
  \item{regmixMH}{Credible bands for the regression lines in a mixture of linear regressions.  See \code{regcr} for more details.}
} 

\seealso{ 
\code{\link{regcr}}, \code{plot.mixMCMC} 
} 

\examples{ 
\dontrun{
data(NOdata)
attach(NOdata)
set.seed(100)
beta <- matrix(c(1.3, -0.1, 0.6, 0.1), 2, 2)
sigma <- c(.02, .05)
MH.out <- regmixMH(Equivalence, NO, beta = beta, s = sigma,
                   sampsize = 2500, omega = .0013)
plotly_mixMCMC(x = MH.out, summary.plot = TRUE, col.cr = c("red", "green"))
}
}

\keyword{file}

\name{normalmixMMlc}
\title{EC-MM Algorithm for Mixtures of Univariate Normals 
	with linear constraints}
\alias{normalmixMMlc}
\usage{
normalmixMMlc(x, lambda = NULL, mu = NULL, sigma = NULL, k = 2,
              mean.constr = NULL, mean.lincstr = NULL, 
              mean.constant = NULL, var.lincstr = NULL, 
              gparam = NULL, epsilon = 1e-08, maxit = 1000, 
              maxrestarts=20, verb = FALSE) 
}
\description{
  Return EC-MM (see below) algorithm output for mixtures of normal distributions
  with linear constraints on the means and variances parameters,
  as in Chauveau and Hunter (2013).
  The linear constraint for the means is of the form
  \eqn{\mu = M \beta + C}, where \eqn{M} and \eqn{C} are matrix 
  and vector specified as parameters.
  The linear constraints for the variances are actually specified on
  the inverse variances, by \eqn{\pi = A \gamma}, where \eqn{\pi}
  is the vector of inverse variances, and \eqn{A} is a matrix
  specified as a parameter (see below).
}
\details{
  This is a specific "EC-MM" algorithm for normal mixtures 
  with linear constraints on the means and variances parameters.
  EC-MM here means that this algorithm is similar to 
  an ECM algorithm as in Meng and Rubin (1993), 
  except that it uses conditional MM 
  (Minorization-Maximization)-steps
  instead of simple M-steps. Conditional means that it 
  alternates between maximizing with respect to the \code{mu}
  and \code{lambda} while holding \code{sigma} fixed, and maximizing with
  respect to \code{sigma} and \code{lambda} while holding \code{mu}
  fixed.  This ECM generalization of EM is forced in the case of linear constraints because there is no closed-form EM algorithm.
}
\arguments{
  \item{x}{A vector of length n consisting of the data.}

  \item{lambda}{Initial value of mixing proportions.  Automatically 
  repeated as necessary 
  to produce a vector of length \code{k}, then normalized to sum to 1.
  If \code{NULL}, then \code{lambda} is random from a uniform Dirichlet
  distribution (i.e., its entries are uniform random and then it is 
  normalized to sum to 1).}

  \item{mu}{Starting value of vector of component means.
  	  If non-NULL and a vector,
  \code{k} is set to \code{length(mu)}.  If NULL, then the initial value
  is randomly generated from a normal distribution with center(s) determined
  by binning the data.}

  \item{sigma}{Starting value of vector of component standard deviations 
  for algorithm.  
  Obsolete for linear constraints on the inverse variances;
  use \code{gparam} instead to specify a starting value.}

  \item{k}{Number of components.  Initial value ignored unless \code{mu} 
  and \code{sigma} are both NULL.}

  \item{mean.constr}{First, simplest way to define 
  	  equality constraints on the mean parameters, given as
  a vector of length \code{k}, as in \code{\link{normalmixEM}}.  
  Each vector entry specifies the constraints,
  if any, on the corresponding mean parameter:  If \code{NA}, the corresponding
  parameter is unconstrained.  If numeric, the corresponding
  parameter is fixed at that value.  If a character string consisting of
  a single character preceded by a coefficient, such as \code{"0.5a"}
  or \code{"-b"}, all parameters using the same single character in their
  constraints will fix these parameters equal to the coefficient times
  some the same free parameter.  For instance, if 
  \code{mean.constr = c(NA, 0, "a", "-a")}, then the first mean parameter
  is unconstrained, the second is fixed at zero, and the third and forth
  are constrained to be equal and opposite in sign.
  Note: if there are no linear constraints for the means, it is
  more efficient to use directly \code{\link{normalmixEM}}.}
  
  \item{mean.lincstr}{Matrix \eqn{M} \eqn{(k,p)} in the linear constraint for the means
  	equation \eqn{\mu = M \beta + C}, with \eqn{p \le k}.}

  \item{mean.constant}{Vector of \eqn{k} constants \eqn{C} 
  			in the linear constraint for the means
  			equation \eqn{\mu = M \beta + C}.}

  \item{var.lincstr}{Matrix \eqn{A} \eqn{(k,q)} in the linear constraint for the
  	  inverse variances equation \eqn{\pi = A \gamma}, with \eqn{q \le k}.}
  
  	\item{gparam}{Vector of \eqn{q} starting values
  		for the \eqn{\gamma} parameter in the
  	  linear constraint for the inverse variances;
  	  see \code{var.lincstr}.  If NULL, a vector of randomly generated
    	standard exponential variables is used.}
  
  \item{epsilon}{The convergence criterion.  
  	  Convergence is declared when the change in 
 	 the observed data log-likelihood increases by less than epsilon.}

  \item{maxit}{The maximum allowed number of iterations.}

  \item{maxrestarts}{The maximum number of restarts allowed in case of a problem
  with the particular starting values chosen due to one of the variance
  estimates getting too small
  (each restart uses randomly chosen
  starting values).  It is well-known that when each component of a normal
  mixture may have its own mean and variance, the likelihood has no maximizer;
  in such cases, we hope to find a "nice" local maximum with this algorithm
  instead, but occasionally the algorithm finds a "not nice" solution and
  one of the variances goes to zero, driving the likelihood to infinity.}

  \item{verb}{If TRUE, then various updates are printed during each 
  iteration of the algorithm.} 

}
\value{
  \code{normalmixMMlc} returns a list of class \code{mixEM} with items:
  \item{x}{The raw data.}
  \item{lambda}{The final mixing proportions.}
  \item{mu}{The final mean parameters.}
  \item{sigma}{The final standard deviation(s)}
  \item{scale}{Scale factor for the component standard deviations, if applicable.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.  This vector
  includes both the initial and the final values; thus, the number of iterations 
  is one less than its length.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{beta}{The final \eqn{\beta} parameter estimate.}
  \item{gamma}{The final \eqn{\gamma} parameter estimate.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
 \code{\link{normalmixEM}}, \code{\link{mvnormalmixEM}}, 
 \code{\link{normalmixEM2comp}}, \code{\link{tauequivnormalmixEM}}
}
\references{
  \itemize{
  \item McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, 
  John Wiley & Sons, Inc.
  \item Meng, X.-L. and Rubin, D. B. (1993) Maximum Likelihood Estimation
  Via the ECM Algorithm:  A General Framework, \emph{Biometrika} 80(2):
  267-278.
  \item Chauveau, D. and Hunter, D.R. (2013)
  	ECM and MM algorithms for mixtures with constrained parameters,
  	\emph{preprint \url{http://hal.archives-ouvertes.fr/hal-00625285}}.
  \item Thomas, H., Lohaus, A., and Domsch, H. (2011) Stable Unstable Reliability
        Theory, \emph{British Journal of Mathematical and Statistical Psychology}
        65(2): 201-221.
  }
}


\author{Didier Chauveau}


\examples{
## Analyzing synthetic data as in the tau equivalent model  
## From Thomas et al (2011), see also Chauveau and Hunter (2013)
## a 3-component mixture of normals with linear constraints.
lbd <- c(0.6,0.3,0.1); m <- length(lbd)
sigma <- sig0 <- sqrt(c(1,9,9))
# means constaints mu = M beta
M <- matrix(c(1,1,1,0,-1,1), 3, 2)
beta <- c(1,5) # unknown constrained mean
mu0 <- mu <- as.vector(M \%*\% beta)
# linear constraint on the inverse variances pi = A.g
A <- matrix(c(1,1,1,0,1,0), m, 2, byrow=TRUE)
iv0 <- 1/(sig0^2)
g0 <- c(iv0[2],iv0[1] - iv0[2]) # gamma^0 init 

# simulation and EM fits
set.seed(50); n=100; x <- rnormmix(n,lbd,mu,sigma)
s <- normalmixEM(x,mu=mu0,sigma=sig0,maxit=2000) # plain EM
# EM with var and mean linear constraints
sc <- normalmixMMlc(x, lambda=lbd, mu=mu0, sigma=sig0,
					mean.lincstr=M, var.lincstr=A, gparam=g0)
# plot and compare both estimates
dnormmixt <- function(t, lam, mu, sig){
	m <- length(lam); f <- 0
	for (j in 1:m) f <- f + lam[j]*dnorm(t,mean=mu[j],sd=sig[j])
	f}
t <- seq(min(x)-2, max(x)+2, len=200)
hist(x, freq=FALSE, col="lightgrey", 
		ylim=c(0,0.3), ylab="density",main="")
lines(t, dnormmixt(t, lbd, mu, sigma), col="darkgrey", lwd=2) # true
lines(t, dnormmixt(t, s$lambda, s$mu, s$sigma), lty=2) 
lines(t, dnormmixt(t, sc$lambda, sc$mu, sc$sigma), col=1, lty=3)
legend("topleft", c("true","plain EM","constr EM"), 
	col=c("darkgrey",1,1), lty=c(1,2,3), lwd=c(2,1,1))
}

\keyword{file}

\name{boot.comp}
\title{Performs Parametric Bootstrap for Sequentially Testing
the Number of Components in Various Mixture Models}
\alias{boot.comp}
\usage{
boot.comp(y, x = NULL, N = NULL, max.comp = 2, B = 100,
          sig = 0.05, arbmean = TRUE, arbvar = TRUE,
          mix.type = c("logisregmix", "multmix", "mvnormalmix",
          "normalmix", "poisregmix", "regmix", "regmix.mixed", 
          "repnormmix"), hist = TRUE, ...)
}

\description{
  Performs a parametric bootstrap by producing B bootstrap realizations of the likelihood ratio
  statistic for testing the null hypothesis of a k-component fit versus the alternative
  hypothesis of a (k+1)-component fit to various mixture models. This is performed for up to a
  specified number of maximum components, k. A p-value is calculated for each test and once the p-value
  is above a specified significance level, the testing terminates.  An optional histogram showing
  the distribution of the likelihood ratio statistic along with the observed statistic can also be
  produced.
}
\arguments{
  \item{y}{The raw data for \code{multmix}, \code{mvnormalmix}, \code{normalmix}, and \code{repnormmix} and
    the response values for \code{logisregmix}, \code{poisregmix}, and \code{regmix}.  See the documentation
    concerning their respective EM algorithms for specific structure of the raw data.}
  \item{x}{The predictor values required only for the regression mixtures \code{logisregmix}, 
  \code{poisregmix}, and \code{regmix}. A column of 1s for the intercept term must not be included! See the
    documentation concerning their respective EM algorithms for specific structure of the predictor values.}
  \item{N}{An n-vector of number of trials for the logistic regression type \code{logisregmix}.  
  If NULL, then \code{N} is an n-vector of 1s for binary logistic regression.}
  \item{max.comp}{The maximum number of components to test for.  The default is 2.  This function will 
    perform a test of k-components versus (k+1)-components sequentially until we fail to reject the null hypothesis.
    This decision rule is governed by the calculated p-value and \code{sig}.}
  \item{B}{The number of bootstrap realizations of the likelihood ratio statistic to produce.  The default is 100,
    but ideally, values of 1000 or more would be more acceptable.}
  \item{sig}{The significance level for which to compare the p-value against when performing the test of k-components
    versus (k+1)-components.}
  \item{arbmean}{If FALSE, then a scale mixture analysis can be performed for \code{mvnormalmix}, \code{normalmix}, \code{regmix},
    or \code{repnormmix}. The default is TRUE.}
  \item{arbvar}{If FALSE, then a location mixture analysis can be performed for \code{mvnormalmix}, \code{normalmix}, \code{regmix},
    or \code{repnormmix}. The default is TRUE.}
  \item{mix.type}{The type of mixture analysis you wish to perform. The data inputted for \code{y} and \code{x} depend on
    which type of mixture is selected.  \code{logisregmix} corresponds to a mixture of logistic regressions.  \code{multmix} corresponds
    to a mixture of multinomials with data determined by the cut-point method.  \code{mvnormalmix} corresponds to a mixture of
    multivariate normals.  \code{normalmix} corresponds to a mixture of univariate normals. \code{poisregmix} corresponds to a mixture of
    Poisson regressions.  \code{regmix} corresponds to a mixture of regressions with normal components.  \code{regmix.mixed}
    corresponds to a mixture of regressions with random or mixed effects.  \code{repnormmix} corresponds to a mixture
    of normals with repeated measurements.}
  \item{hist}{An argument to provide a matrix plot of histograms for the boostrapped likelihood ratio statistic.}
  \item{...}{Additional arguments passed to the various EM algorithms for the mixture of interest.} 
}
\value{
  \code{boot.comp} returns a list with items:
  \item{p.values}{The p-values for each test of k-components versus (k+1)-components.}
  \item{log.lik}{The B bootstrap realizations of the likelihood ratio statistic.}
  \item{obs.log.lik}{The observed likelihood ratio statistic for each test which is used in determining
    the p-values.}
}
\seealso{
\code{\link{logisregmixEM}}, \code{\link{multmixEM}}, \code{\link{mvnormalmixEM}}, \code{\link{normalmixEM}},
\code{\link{poisregmixEM}}, \code{\link{regmixEM}}, \code{\link{regmixEM.mixed}}, \code{\link{repnormmixEM}}
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
## Bootstrapping to test the number of components on the RTdata.

data(RTdata)
set.seed(100)
x <- as.matrix(RTdata[, 1:3])
y <- makemultdata(x, cuts = quantile(x, (1:9)/10))$y
a <- boot.comp(y = y, max.comp = 1, B = 5, mix.type = "multmix", 
               epsilon = 1e-3)
a$p.values

}


\keyword{file}

\name{plotweibullRMM}
\alias{plotweibullRMM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Plot sequences from the Stochastic EM algorithm for mixture of Weibull
}

\description{Function for plotting sequences of estimates along iterations, from an object returned by \code{\link{weibullRMM_SEM}}, a Stochastic EM algorithm for mixture of Weibull 
distributions with randomly right censored data (see reference below).
}

\usage{
  plotweibullRMM(a, title = NULL, rowstyle = TRUE, subtitle = NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{An object returned by \code{\link{weibullRMM_SEM}}.}
  \item{title}{The title of the plot, set to some default value if \code{NULL}.}
  \item{rowstyle}{Window organization, for plots in rows (the default) or columns.}
  \item{subtitle}{A subtitle for the plot, set to some default value if \code{NULL}.}
  \item{...}{Other parameters (such as \code{lwd}) passed to \code{plot}, \code{lines}, and  
          \code{legend} commands.}
  }


\value{The plot returned}

\seealso{
Related functions:
\code{\link{weibullRMM_SEM}},   \code{\link{summary.mixEM}}.

Other models and algorithms for censored lifetime data 
(name convention is model_algorithm):
\code{\link{expRMM_EM}},
\code{\link{spRMM_SEM}}
.
}

\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
n = 500 # sample size
m = 2 # nb components
lambda=c(0.4, 0.6)
shape <- c(0.5,5); scale <- c(1,20) # model parameters
set.seed(321)
x <- rweibullmix(n, lambda, shape, scale) # iid ~ weibull mixture
cs=runif(n,0,max(x)+10) # iid censoring times
t <- apply(cbind(x,cs),1,min) # censored observations
d <- 1*(x <= cs)              # censoring indicator

## set arbitrary or "reasonable" (e.g., data-driven) initial values
l0 <- rep(1/m,m); sh0 <- c(1, 2); sc0 <- c(2,10)
# Stochastic EM algorithm 
a <- weibullRMM_SEM(t, d, lambda = l0, shape = sh0, scale = sc0, maxit = 200)

summary(a) # Parameters estimates etc
plotweibullRMM(a) # default plot of St-EM sequences
%%\dontrun{
%%}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{file}

\name{spRMM_SEM}
\title{Stochastic EM algorithm for semiparametric scaled mixture of censored data}
\alias{spRMM_SEM}
\usage{
  spRMM_SEM(t, d = NULL, lambda = NULL, scaling = NULL, 
          centers = 2, kernelft = triang_wkde, 
          bw = rep(bw.nrd0(t),length(t)), averaged = TRUE,
          epsilon = 1e-08, maxit = 100, batchsize = 1, verb = FALSE) }

\description{
  Stochastic EM algorithm for semiparametric scaled mixture for randomly right censored data.
}

\arguments{
  \item{t}{A vector of \eqn{n}{n} real positive lifetime (possibly censored) durations.
  If \code{d} is not \code{NULL} then a vector of random censoring times
  \code{c} occurred,  
  so that \eqn{x= min(x,c)} and \eqn{d = I(x <= c)}.}
  
  \item{d}{The vector of censoring indication, where 1 means observed lifetime data, 
  and 0 means censored lifetime data.}
  
  \item{lambda}{Initial value of mixing proportions.
  If \code{NULL}, then \code{lambda} is set to \code{rep(1/k,k)}.}
  
  \item{scaling}{Initial value of scaling between components, 
        set to 1 if \code{NULL}.}

  \item{centers}{initial centers for initial call to kmeans for initialization.}

  \item{kernelft}{.}
  
  \item{bw}{Bandwidth in the kernel hazard estimates.}
  
  \item{averaged}{averaged.}
  
  \item{epsilon}{Tolerance limit.}

  \item{maxit}{The number of iterations allowed.}
  
  \item{batchsize}{The batchsize (see reference below).}

  \item{verb}{If TRUE, print updates for every iteration of the algorithm as
  it runs}
}

\value{
\code{spRMM_SEM} returns a list of class \code{"spRMM"} with the following items:
  \item{t}{The input data.}
  \item{d}{The input censoring indicator.}
  \item{lambda}{The estimates for the mixing proportions.}
  \item{scaling}{The estimates for the components scaling.}
  \item{posterior}{An \eqn{n\times k}{n x 2} matrix of posterior probabilities for
   observation, after convergence of the algorithm.}
  
  \item{loglik}{The (pseudo) log-likelihood value at convergence of the algorithm.}

  \item{all.loglik}{The sequence of log-likelihood values over iterations.}

  \item{all.lambda}{The sequence of mixing proportions over iterations.}
  
  \item{all.scaling}{The sequence of scaling parameter over iterations.}
  
  \item{meanpost}{Posterior probabilities averaged over iterations.}
  
  \item{survival}{Kaplan-Meier last iteration estimate (a \code{stepfun} object).}
  
  \item{hazard}{Hazard rate last iteration estimate evaluated at \code{final.t}.}
  
  \item{final.t}{Last iteration unscaled sample (see reference).}
  
  \item{s.hat}{Kaplan-Meier average estimate.}
  
  \item{t.hat}{Ordered unscaled sample, for testing purpose.}
  
  \item{avg.od}{For testing purpose only.}
  
  \item{hazard.hat}{Hazard rate average estimate on \code{t.hat}.}
  
  \item{batch.t}{Batch sample (not ordered), see reference.}
  
  \item{batch.d}{Associated event indicators just \code{rep(d,batchsize)}, for testing purpose.}
  
  \item{sumNaNs}{Internal control of numerical stability.}

   \item{ft}{A character vector giving the name of the function.}
}
\seealso{
Related functions: 
  \code{\link{plotspRMM}},
  \code{\link{summary.spRMM}}.
  
  Other models and algorithms for censored lifetime data
  (name convention is model_algorithm):
  \code{\link{expRMM_EM}},
  \code{\link{weibullRMM_SEM}}.
}
\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}

\examples{
\dontrun{
n=500 # sample size
m=2 # nb components
lambda=c(0.4, 0.6) # parameters
meanlog=3; sdlog=0.5; scale=0.1
set.seed(12)
# simulate a scaled mixture of lognormals
x <- rlnormscalemix(n, lambda, meanlog, sdlog, scale)
cs=runif(n,20,max(x)+400) # Censoring (uniform) and incomplete data
t <- apply(cbind(x,cs),1,min)
d <- 1*(x <= cs)
tauxc <- 100*round( 1-mean(d),3)
cat(tauxc, "percents of data censored.\n")

c0 <- c(25, 180) # data-driven initial centers (visible modes)
sc0 <- 25/180    # and scaling
s <- spRMM_SEM(t, d, scaling = sc0, centers = c0, bw = 15, maxit = 100)

plotspRMM(s) # default
summary(s)   # S3 method for class "spRMM"
}
}


\keyword{file}

\name{tauequivnormalmixEM}
\title{Special EM Algorithm for three-component tau equivalence model}
\alias{tauequivnormalmixEM}
\usage{
tauequivnormalmixEM (x, lambda = NULL, mu = NULL, sigma = NULL, k = 3, 
          mean.constr = NULL, sd.constr = NULL, gparam = NULL,
          epsilon = 1e-08, maxit = 10000, maxrestarts=20, 
          verb = FALSE, fast=FALSE, ECM = TRUE,
          arbmean = TRUE, arbvar = TRUE) 
}
\description{
  Return ECM algorithm output for a specific case of a three-component tau equivalence model
}
\details{
  The \code{tauequivnormalmixEM} function is merely a wrapper for the
  \code{\link{normalmixMMlc}} function.
 # This is the standard EM algorithm for normal mixtures that maximizes
 # the conditional expected complete-data
 # log-likelihood at each M-step of the algorithm.
#  If desired, the
#  EM algorithm may be replaced by an ECM algorithm (see \code{ECM} argument)
#  that alternates between maximizing with respect to the \code{mu}
#  and \code{lambda} while holding \code{sigma} fixed, and maximizing with
#  respect to \code{sigma} and \code{lambda} while holding \code{mu}
#  fixed.  In the case where \code{arbmean} is \code{FALSE}
#  and \code{arbvar} is \code{TRUE}, there is no closed-form EM algorithm,
#  so the ECM option is forced in this case.
}
\arguments{
  \item{x}{A vector of length n consisting of the data,
  passed directly to \code{\link{normalmixMMlc}}.}

  \item{lambda}{Initial value of mixing proportions, 
  passed directly to \code{\link{normalmixMMlc}}.  Automatically 
  repeated as necessary 
  to produce a vector of length \code{k}, then normalized to sum to 1.
  If \code{NULL}, then \code{lambda} is random from a uniform Dirichlet
  distribution (i.e., its entries are uniform random and then it is 
  normalized to sum to 1).}

  \item{mu}{Starting value of vector of component means for algorithm,
  passed directly to \code{\link{normalmixMMlc}}. 
  If non-NULL and a vector,
  \code{k} is set to \code{length(mu)}.  If NULL, then the initial value
  is randomly generated from a normal distribution with center(s) determined
  by binning the data.}

  \item{sigma}{Starting value of vector of component standard deviations 
  for algorithm, passed directly to \code{\link{normalmixMMlc}}.  
  Obsolete for linear constraint on the inverse variances,
  use \code{gparam} instead to specify a starting value.
  Note: This needs more precision}
  
  \item{k}{Number of components, passed directly to \code{\link{normalmixMMlc}}.  
  Initial value ignored unless \code{mu} and \code{sigma} are both NULL.
  Also, initial value is ignored if \code{mean.constr} is NULL, since in that
  case we presume \code{k=3}.}

  \item{mean.constr}{If non-NULL, this parameter is
  	  passed directly to \code{\link{normalmixMMlc}} and both 
  	  \code{mean.lincstr} and \code{var.lincstr} are passed as NULL to
  	  \code{\link{normalmixMMlc}}.  If NULL, then 
  	  it is assumed that \code{k=3} and the means must take the form
  	  \eqn{\alpha}, \eqn{\alpha-\delta}, and \eqn{\alpha+\delta} for unknown parameters
      \eqn{\alpha} and \eqn{\delta}.  Furthermore, the reciprocal variances are
      assumed to be \eqn{\gamma_1+\gamma_2}, \eqn{\gamma_1}, and \eqn{\gamma_1}
      for unknown positive parameters \eqn{\gamma_1} and \eqn{\gamma_2}.
      These constraints are passed to the
  	  \code{\link{normalmixMMlc}} function using the \code{mean.lincstr} and
  	  \code{var.lincstr} arguments
  as shown in the examples for the \code{\link{normalmixMMlc}} help file.}
  
  \item{sd.constr}{Deprecated.}

  \item{gparam}{This argument is passed directly to \code{\link{normalmixMMlc}.}
  }

  \item{epsilon}{The convergence criterion.  Convergence is declared when the change in 
  the observed data log-likelihood increases by less than epsilon.}

  \item{maxit}{The maximum number of iterations.}

  \item{maxrestarts}{The maximum number of restarts allowed in case of a problem
  with the particular starting values chosen due to one of the variance
  estimates getting too small
  (each restart uses randomly chosen
  starting values).  It is well-known that when each component of a normal
  mixture may have its own mean and variance, the likelihood has no maximizer;
  in such cases, we hope to find a "nice" local maximum with this algorithm
  instead, but occasionally the algorithm finds a "not nice" solution and
  one of the variances goes to zero, driving the likelihood to infinity.}

  \item{verb}{If TRUE, then various updates are printed during each 
  iteration of the algorithm.} 

  \item{fast}{If TRUE and k==2 and arbmean==TRUE, then use 
  \code{\link{normalmixEM2comp}}, which is a much faster version of the EM 
  algorithm for this case.
  This version is less protected against certain kinds of underflow
  that can cause numerical problems and it does not permit any restarts.  If
  k>2, \code{fast} is ignored.}
  
  \item{ECM}{logical:  Should this algorithm be an ECM algorithm in the sense
  of Meng and Rubin (1993)?  If FALSE, the algorithm is a true EM algorithm;
  if TRUE, then every half-iteration alternately updates the means conditional
  on the variances or the variances conditional on the means, with an extra
  E-step in between these updates.  For \code{tauequivnormalmixEM}, it must
  be TRUE.}

  \item{arbmean}{Deprecated.}

  \item{arbvar}{Deprecated.}
}
\value{
  \code{normalmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The raw data.}
  \item{lambda}{The final mixing proportions.}
  \item{mu}{The final mean parameters.}
  \item{sigma}{The final standard deviation(s)}
  \item{scale}{Scale factor for the component standard deviations, if applicable.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.  This vector
  includes both the initial and the final values; thus, the number of iterations 
  is one less than its length.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
  \code{\link{normalmixMMlc}}, \code{\link{normalmixEM}}, \code{\link{mvnormalmixEM}}, \code{\link{normalmixEM2comp}}
}
\references{
  \itemize{
  \item Thomas, H., Lohaus, A., and Domsch, H. (2011) Stable Unstable Reliability
        Theory, \emph{British Journal of Mathematical and Statistical Psychology}
        65(2): 201-221.
  \item Meng, X.-L. and Rubin, D. B. (1993) Maximum Likelihood Estimation
  Via the ECM Algorithm:  A General Framework, \emph{Biometrika} 80(2):
  267-278.
  }
}
\examples{
## Analyzing synthetic data as in the tau equivalent model  
## From Thomas et al (2011), see also Chauveau and Hunter (2013)
## a 3-component mixture of normals with linear constraints.
lbd <- c(0.6,0.3,0.1); m <- length(lbd)
sigma <- sig0 <- sqrt(c(1,9,9))
# means constaints mu = M beta
M <- matrix(c(1,1,1,0,1,-1), 3, 2)
beta <- c(1,5) # unknown constained mean
mu0 <- mu <- as.vector(M \%*\% beta)
# linear constraint on the inverse variances pi = A.g
A <- matrix(c(1,1,1,0,1,0), m, 2, byrow=TRUE)
iv0 <- 1/(sig0^2)
g0 <- c(iv0[2],iv0[1] - iv0[2]) # gamma^0 init 

# simulation and EM fits
set.seed(40); n=100; x <- rnormmix(n,lbd,mu,sigma)
s <- normalmixEM(x,mu=mu0,sigma=sig0,maxit=2000) # plain EM
# EM with var and mean linear constraints
sc <- normalmixMMlc(x, lambda=lbd, mu=mu0, sigma=sig0,
					mean.lincstr=M, var.lincstr=A, gparam=g0)
# Using tauequivnormalmixEM function to call normalmixMMlc					
tau <- tauequivnormalmixEM (x, lambda=lbd, mu=mu0, gparam=g0)
# plot and compare both estimates
dnormmixt <- function(t, lam, mu, sig){
	m <- length(lam); f <- 0
	for (j in 1:m) f <- f + lam[j]*dnorm(t,mean=mu[j],sd=sig[j])
	f}
t <- seq(min(x)-2, max(x)+2, len=200)
hist(x, freq=FALSE, col="lightgrey", 
		ylim=c(0,0.3), ylab="density",main="")
lines(t, dnormmixt(t, lbd, mu, sigma), col="darkgrey", lwd=2) # true
lines(t, dnormmixt(t, s$lambda, s$mu, s$sigma), lty=2) 
lines(t, dnormmixt(t, sc$lambda, sc$mu, sc$sigma), col=1, lty=3)
lines(t, dnormmixt(t, tau$lambda, tau$mu, tau$sigma), col=2, lty=4)
legend("topleft", c("true","plain EM","constr EM", "Tau Equiv"), 
	col=c("darkgrey",1,1,2), lty=c(1,2,3,4), lwd=c(2,1,1,1))
}

\keyword{file}

\name{summary.mixEM}
\alias{summary.mixEM}
\title{Summarizing EM mixture model fits}
\usage{
\method{summary}{mixEM}(object, digits=6, \dots)
}
\arguments{
  \item{object}{an object of class \code{mixEM} such as a result of a call
  to \code{\link{normalmixEM}}}
  \item{digits}{Significant digits for printing values}
  \item{\dots}{further arguments passed to \code{print} method.}
}
\description{
\code{\link[base]{summary}} method for class \code{mixEM}.
}
\details{
 \code{\link{summary.mixEM}} prints parameter estimates for
 each component of a fitted mixture model.  
 The estimates printed vary with the type of model.
}
\value{
  The function \code{\link{summary.mixEM}} prints the final loglikelihood
  value at the solution as well as a matrix of values for each component
  that could include:
  \item{lambda}{The estimated mixing weights}
  \item{mu}{The estimated mean parameters}
  \item{sigma}{The estimated standard deviations}
  \item{theta}{The estimated multinomial parameters}
  \item{beta}{The estimated regression parameters}
}
\seealso{ 
\code{\link{normalmixEM}},
\code{\link{logisregmixEM}},
\code{\link{multmixEM}},
\code{\link{mvnormalmixEM}},
\code{\link{poisregmixEM}},
\code{\link{regmixEM}},
\code{\link{regmixEM.lambda}},
\code{\link{regmixEM.loc}},
\code{\link{regmixEM.mixed}},
\code{\link{regmixEM.chgpt}},
\code{\link{repnormmixEM}},
\code{\link{expRMM_EM}},
\code{\link{weibullRMM_SEM}}
}
\examples{
data(faithful)
attach(faithful)
set.seed(100)
out <- normalmixEM(waiting, mu=c(50,80), sigma=c(5,5), lambda=c(.5,.5))
summary(out)
}
\keyword{file}

\name{matsqrt}
\title{Calculates the Square Root of a Diagonalizable Matrix}
\alias{matsqrt}
\usage{
matsqrt(x)
}
\description{
Returns the square root of a diagonalizable matrix.
}
\arguments{
  \item{x}{An nxn diagonalizable matrix.}
}
\value{
  \code{matsqrt} returns the square root of \code{x}.  
}
\details{
This function is called by \code{regcr}.
}
\seealso{
\code{\link{regcr}}
}
\examples{
a <- matrix(c(1, -0.2, -0.2, 1), 2, 2)
matsqrt(a)

}

\keyword{internal}

\name{plot.mixEM}
\title{Various Plots Pertaining to Mixture Models}
\alias{plot.mixEM} 
\usage{
\method{plot}{mixEM}(x, whichplots = 1, 
     loglik = 1 \%in\% whichplots,
     density = 2 \%in\% whichplots,
     xlab1="Iteration", ylab1="Log-Likelihood",
     main1="Observed Data Log-Likelihood", col1=1, lwd1=2,
     xlab2=NULL, ylab2=NULL, main2=NULL, col2=NULL, 
     lwd2=2, alpha = 0.05, marginal = FALSE, ...)
}

\description{
    Takes an object of class \code{mixEM} and returns various graphical output for select mixture models.
} 
\arguments{
  \item{x}{An object of class \code{mixEM}.}
  \item{whichplots}{vector telling which plots to produce:  1 = loglikelihood
        plot, 2 = density plot.  Irrelevant if \code{loglik} and \code{density}
        are specified.}
  \item{loglik}{If TRUE, a plot of the log-likelihood versus the EM iterations is given.}
  \item{density}{Graphics pertaining to certain mixture models.  The details are given below.}
  \item{xlab1, ylab1, main1, col1, lwd1}{Graphical parameters \code{xlab}, ..., \code{lwd}
  to be passed to the loglikelihood plot.  Trying to change these parameters using
  \code{xlab}, ..., \code{lwd} will result in an error, but all other graphical parameters
  are passed directly to the plotting functions via ...}
  \item{xlab2, ylab2, main2, col2, lwd2}{Same as \code{xlab1} etc. but for the
  density plot}
  \item{alpha}{A vector of significance levels when constructing confidence ellipses and confidence bands for the mixture
  of multivariate normals and mixture of regressions cases, respectively.  The default is 0.05.}
  \item{marginal}{For the mixture of bivariate normals, should optional marginal histograms be included?}
  \item{...}{Graphical parameters passed to \code{plot} command.}
}
\value{
  \code{plot.mixEM} returns a plot of the log-likelihood versus the EM iterations by default for all objects of class
  \code{mixEM}.  In addition, other plots may be produced for the following k-component mixture model functions:
  \item{normalmixEM}{A histogram of the raw data is produced along with k density curves determined by \code{normalmixEM}.}
  \item{repnormmixEM}{A histogram of the raw data produced in a similar manner as for \code{normalmixEM}.}
  \item{mvnormalmixEM}{A 2-dimensional plot with each point color-coded to denote its most probable component membership. In
  addition, the estimated component means are plotted along with (1 - \code{alpha})\% bivariate normal density contours.  These ellipses are
  constructed by assigning each value to their component of most probable membership and then using normal theory. Optional marginal histograms
  may also be produced.}
  \item{regmixEM}{A plot of the response versus the predictor with each point color-coded to denote its most probable component
  membership.  In addition, the estimated component regression lines are plotted along with (1 - \code{alpha})\% Working-Hotelling 
  confidence bands. These bands are constructed by assigning each value to their component of most probable membership and then
  performing least squares estimation.}
  \item{logisregmixEM}{A plot of the binary response versus the predictor with each point color-coded to denote its most probable
  compopnent membership.  In addition, the estimate component logistic regression lines are plotted.}
  \item{regmixEM.mixed}{Provides a 2x2 matrix of plots summarizing the posterior slope and posterior intercept terms from a
  mixture of random effects regression.  See \code{post.beta} for a more detailed description.}
} 

\seealso{ 
\code{\link{post.beta}} 
} 

\examples{ 
##Analyzing the Old Faithful geyser data with a 2-component mixture of normals.

data(faithful)
attach(faithful)
set.seed(100)
out <- normalmixEM(waiting, arbvar = FALSE, verb = TRUE,
                   epsilon = 1e-04)
plot(out, density = TRUE, w = 1.1)

##Fitting randomly generated data with a 2-component location mixture of bivariate normals.

x.1 <- rmvnorm(40, c(0, 0))
x.2 <- rmvnorm(60, c(3, 4))
X.1 <- rbind(x.1, x.2)

out.1 <- mvnormalmixEM(X.1, arbvar = FALSE, verb = TRUE,
                       epsilon = 1e-03)
plot(out.1, density = TRUE, alpha = c(0.01, 0.05, 0.10), 
     marginal = TRUE)

}

\keyword{file}

\name{mvnpEM}
\title{EM-like Algorithm for Nonparametric Mixture Models 
with Conditionally Independent Multivariate Component Densities}
\alias{mvnpEM}
\usage{
mvnpEM(x, mu0, blockid = 1:ncol(x), samebw = TRUE, 
       bwdefault = apply(x,2,bw.nrd0), init = NULL,
       eps = 1e-8, maxiter = 500, verb = TRUE)
}
                    
                    
                    
                    

\description{
  An extension of the original \code{\link{npEM}} algorithm, for mixtures
  of multivariate data where the coordinates of a row (case)
  in the data matrix are assumed to be made of independent but multivariate blocks (instead of just coordinates), 
  conditional on the mixture
  component (subpopulation) from which they are drawn (Chauveau and Hoang 2015).
}
\arguments{
  \item{x}{An \eqn{n\times r}{n x r} matrix of data.  Each of the \eqn{n} rows is a case,
  and each case has \eqn{r} repeated measurements.  These measurements are assumed
  to be conditionally independent, conditional on the mixture component (subpopulation) 
  from which the case is drawn.}
  \item{mu0}{Either an \eqn{m\times r}{m x r} matrix specifying the initial
      centers for the \link{kmeans} function, or an integer \eqn{m} specifying the 
      number of initial centers, which are then chosen randomly in \link{kmeans}}
  \item{blockid}{A vector of length \eqn{r} identifying coordinates 
  (columns of \code{x}) that are in the same block.  The default has all distinct elements, 
  indicating that the model has \eqn{r} blocks of dimension 1, in which case the model is handled 
  directly by the \code{\link{npEM}} algorithm.
  See example below for actual multivariate blocks example.}
  
  \item{samebw}{Logical:  If \code{TRUE}, use the same bandwidth per coordinate for
  all iteration and all components.  If \code{FALSE}, 
  use a separate bandwidth for each component and coordinate, and update
  this bandwidth at each iteration of the algorithm using a suitably
  modified \code{\link{bw.nrd0}} method as described in 
  Benaglia et al (2011) and Chauveau and Hoang (2015).}


  \item{bwdefault}{Bandwidth default for density estimation,a simplistic application of the 
  default \code{\link{bw.nrd0}} for each coordinate (column) of the data.} 
  
  \item{init}{Initialization method, based on an initial \eqn{n\times m}{n x m}
  matrix for the posterior probabilities. If \code{NULL},
  a  \code{\link{kmeans}} clustering with \code{mu0} initial centers is applied to the data and the 
  initial matrix of posteriors is built from the result.}
  
  \item{eps}{Tolerance limit for declaring algorithm convergence.  Convergence
  is declared whenever the maximum change in any coordinate of the 
  \code{lambda} vector (of mixing proportion estimates) does not exceed 
  \code{eps}.}

  \item{maxiter}{The maximum number of iterations allowed; convergence
  may be declared before \code{maxiter} iterations (see \code{eps} above).}
  
  \item{verb}{Verbose mode; if TRUE, print updates for every iteration of the algorithm as
  it runs}
}
\value{
\code{mvnpEM} returns a list of class \code{mvnpEM} with the following items:
  \item{data}{The raw data (an \eqn{n\times r}{n x r} matrix).}
  
  \item{posteriors}{An \eqn{n\times m}{n x m} matrix of posterior probabilities for
   each observation (row).}
   
  \item{lambda}{The sequence of mixing proportions over iterations.}
  
  \item{blockid}{The \code{blockid} input argument. Needed by any method that produces density 
  estimates from the output, like \code{\link{plot.mvnpEM}}.}


  \item{samebw}{The \code{samebw} input argument. 
  Needed by any method that produces density estimates from the output, like \code{\link{plot.mvnpEM}}.}

  \item{bandwidth}{The final bandwidth matrix 
  after convergence of the algorithm.
  Its shape depends on the \code{samebw} input argument. If \code{samebw = TRUE}, a 
  vectors with the bandwidth value for each of the \code{r} coordinates (same for all components and iterations).
  If \code{samebw = FALSE}, a \eqn{m\times r}{m x r} matrix, where each row is associated to one component 
  and gives the \eqn{r} bandwidth values, one for each coordinate.
  Needed by any method that produces density estimates from the output, like \code{\link{plot.mvnpEM}}.}

  \item{lambdahat}{The final mixing proportions.}
  \item{loglik}{The sequence of pseudo log-likelihood values over iterations.}
}
\seealso{
\code{\link{plot.mvnpEM}}, \code{\link{npEM}}
}
\references{
   \itemize{
   \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526.
   
   \item Benaglia, T., Chauveau, D. and Hunter, D.R. (2011),
   Bandwidth Selection in an EM-like algorithm for nonparametric multivariate mixtures.
  Nonparametric Statistics and Mixture Models: A Festschrift in
  	Honor of Thomas P. Hettmansperger. World Scientific Publishing Co., 
  	pages 15-27.

   \item  Chauveau, D., and Hoang, V. T. L. (2015),
   Nonparametric mixture models with conditionally independent multivariate component densities, 
   Preprint under revision.
   \url{https://hal.archives-ouvertes.fr/hal-01094837}
   }
}

\examples{
# Example as in Chauveau and Hoang (2015) with 6 coordinates
\dontrun{
m=2; r=6; blockid <-c(1,1,2,2,3,3) # 3 bivariate blocks 
# generate some data x ...
a <- mvnpEM(x, mu0=2, blockid, samebw=F) # adaptive bandwidth
plot(a) # this S3 method produces 6 plots of univariate marginals
summary(a)}
}


\keyword{file}

\name{gammamixEM}
\title{EM Algorithm for Mixtures of Gamma Distributions}
\alias{gammamixEM}
\usage{
gammamixEM(x, lambda = NULL, alpha = NULL, beta = NULL, k = 2,
           mom.start = TRUE, fix.alpha = FALSE, epsilon = 1e-08, 
           maxit = 1000, maxrestarts = 20, verb = FALSE)
}
\description{
  Return EM algorithm output for mixtures of gamma distributions.
}
\arguments{
  \item{x}{A vector of length n consisting of the data.}
  \item{lambda}{Initial value of mixing proportions.  If \code{NULL}, 
  then \code{lambda} is random from a uniform Dirichlet
  distribution (i.e., its entries are uniform random and then it is 
  normalized to sum to 1).}
  \item{alpha}{Starting value of vector of component shape parameters.  If non-NULL, \code{alpha} must be of length \code{k} if allowing different component shape parameters, or a single value if \code{fix.alpha = TRUE}.  If NULL, then the initial value
  is estimated by partitioning the data into \code{k} regions (with \code{lambda} determining
  the proportion of values in each region) and then calculating the method of moments estimates.}
  \item{beta}{Starting value of vector of component scale parameters.  If non-NULL and a vector,
  \code{k} is set to \code{length(beta)}.  If NULL, then the initial value
  is estimated the same method described for \code{alpha}.}
  \item{k}{Number of components.  Initial value ignored unless \code{alpha} and \code{beta}
    are both NULL.}
  \item{mom.start}{Logical to indicate if a method of moments starting value strategy should be implemented.  If \code{TRUE}, then only unspecified starting values will be generated according to this strategy.}
  \item{epsilon}{The convergence criterion.  Convergence is declared when the change in 
  the observed data log-likelihood increases by less than epsilon.}
  \item{fix.alpha}{Logical to indicate if the components should have a common shape parameter \code{alpha} estimated.  The default is \code{FALSE}.}
  \item{maxit}{The maximum number of iterations.}
  \item{maxrestarts}{The maximum number of restarts allowed in case of a problem
  with the particular starting values chosen (each restart uses randomly chosen
  starting values).}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{gammamixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The raw data.}
  \item{lambda}{The final mixing proportions.}
  \item{gamma.pars}{A 2xk matrix where each column provides the component estimates of \code{alpha} and \code{beta}.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.  This vector
  includes both the initial and the final values; thus, the number of iterations 
  is one less than its length.}
  \item{ft}{A character vector giving the name of the function.}
}
\references{
  Dempster, A. P., Laird, N. M., and Rubin, D. B. (1977) Maximum Likelihood From Incomplete Data
  Via the EM Algorithm, \emph{Journal of the Royal Statistical Society, Series B}, \bold{39(1)}, 1--38.
  
  Young, D. S., Chen, X., Hewage, D., and Nilo-Poyanco, R. (2019) Finite Mixture-of-Gamma Distributions: Estimation, Inference, and Model-Based Clustering, \emph{Advances in Data Analysis and Classification}, \bold{13(4)}, 1053--1082.
}
\examples{
##Analyzing a 3-component mixture of gammas.

set.seed(100)
x <- c(rgamma(200, shape = 0.2, scale = 14), rgamma(200, 
     shape = 32, scale = 10), rgamma(200, shape = 5, scale = 6))
out <- gammamixEM(x, lambda = c(1, 1, 1)/3, verb = TRUE)
out[2:4]
}

\keyword{file}

\name{plot.npEM}
\title{Plot Nonparametric or Semiparametric EM Output}
\alias{plot.npEM} 
\alias{plot.spEM}
\usage{
\method{plot}{npEM}(x, blocks = NULL, hist=TRUE, addlegend = TRUE,
      scale=TRUE, title=NULL, breaks="Sturges", ylim=NULL, dens.col,
      newplot = TRUE, pos.legend = "topright", cex.legend = 1, \dots)         
\method{plot}{spEM}(x, blocks = NULL, hist=TRUE, addlegend = TRUE,
      scale=TRUE, title=NULL, breaks="Sturges", ylim=NULL, dens.col,
      newplot = TRUE, pos.legend = "topright", cex.legend = 1, \dots)
}
\description{
    Takes an object of class \code{npEM} and returns a set of plots of the 
    density estimates for each block and each component.  There is one plot
    per block, with all the components displayed on each block so it is possible
    to see the mixture structure for each block.      
} 
\arguments{
  \item{x}{An object of class \code{npEM} such as the output
  of the \code{\link{npEM}} function}
  \item{blocks}{Blocks (of repeated measures coordinates) to plot; not relevant
  for univariate case.  Default is to plot all blocks.}
  \item{hist}{If TRUE, superimpose density estimate plots on a histogram
    of the data}
  \item{addlegend}{If TRUE, adds legend to the plot.}
  \item{scale}{If TRUE, scale each density estimate by its corresponding estimated
  mixing proportion, so that the total area under all densities equals 1 and the 
  densities plotted may be added to produce an estimate of the mixture density.
  When FALSE, each density curve has area 1 in the plot.}
  \item{title}{Alternative vector of main titles for plots (recycled as many times
  as needed)}
  \item{breaks}{Passed directly to the \code{\link{hist}} function}
  \item{ylim}{\code{ylim} parameter to use for all plots, if desired.  If not
  given, each plot uses its own ylim that ensures that no part of the plot will
  go past the top of the plotting area.}
  \item{dens.col}{Color values to use for the individual component density
    functions, repeated as necessary.  Default value is \code{2:(m+1)}.}
  \item{newplot}{If TRUE, creates a new plot.}
  \item{pos.legend}{Single argument specifying the 
    position of the legend.  See `Details' section of
  \code{\link{legend}}.}
  \item{cex.legend}{Character expansion factor for \code{\link{legend}}.}
  \item{\dots}{Any remaining arguments are passed to the \code{\link{hist}} and 
  \code{\link{lines}} functions.}
}
\value{
  \code{plot.npEM} returns a list with two elements:
%  \item{means}{A \eqn{B\times m}{B x m} matrix of estimated population means,
%  where \eqn{B} is the number of blocks and \eqn{m} is the number of mixture components 
%  (subpopulations)}
%  \item{variances}{A \eqn{B\times m}{B x m} matrix of estimated population variances}
  \item{x}{List of matrices.  The \eqn{j}th column of the \eqn{i}th matrix is the vector of 
  \eqn{x}-values for the \eqn{j}th density in the \eqn{i}th plot.}
  \item{y}{\eqn{y}-values, given in the same form as the \eqn{x}-values.}
} 

\seealso{ 
\code{\link{npEM}}, \code{\link{density.npEM}}, \code{\link{spEMsymloc}},
\code{\link{plotseq.npEM}}
} 

\examples{ 
## Examine and plot water-level task data set.

## First, try a 3-component solution where no two coordinates are
## assumed i.d.
data(Waterdata)
set.seed(100)
\dontrun{
a <- npEM(Waterdata[,3:10], 3, bw=4)
par(mfrow=c(2,4))
plot(a) # This produces 8 plots, one for each coordinate
}

\dontrun{
## Next, same thing but pairing clock angles that are directly opposite one
## another (1:00 with 7:00, 2:00 with 8:00, etc.)
b <- npEM(Waterdata[,3:10], 3, blockid=c(4,3,2,1,3,4,1,2), bw=4)
par(mfrow=c(2,2))
plot(b) # Now only 4 plots, one for each block
}

}

\keyword{file}

\name{spEM}
\title{Semiparametric EM-like Algorithm for Mixtures of Independent Repeated Measurements}
\alias{spEM}
\usage{
spEM(x, mu0, blockid = 1:ncol(x), 
     bw = bw.nrd0(as.vector(as.matrix(x))), constbw = TRUE, 
     h = bw, eps = 1e-8, 
     maxiter = 500, stochastic = FALSE, verb = TRUE)
}

\description{
  Returns semiparametric EM algorithm output (Benaglia et al, 2009) for mixtures
  of multivariate (repeated measures) data where the coordinates of a row (case)
  in the data matrix are assumed to be independent, conditional on the mixture
  component (subpopulation) from which they are drawn.
  For now, this algorithm only implements model (4.7) in 
  Benaglia et al, in which each component and block has exactly
  the same (nonparametric) shape and they differ only by location and scale.
}
\arguments{
  \item{x}{An \eqn{n\times r}{n x r} matrix of data.  Each of the \eqn{n} rows is a case,
  and each case has \eqn{r} repeated measurements.  These measurements are assumed
  to be conditionally independent, conditional on the mixture component (subpopulation) 
  from which the case is drawn.}
  \item{mu0}{Either an \eqn{m\times r}{m x r} matrix specifying the initial
      centers for the \link{kmeans} function, or an integer \eqn{m} specifying the 
      number of initial centers, which are then choosen randomly in
  \link{kmeans}}
  \item{blockid}{A vector of length \eqn{r} identifying coordinates 
  (columns of \code{x}) that are
  assumed to be identically distributed (i.e., in the same block).  For instance,
  the default has all distinct elements, indicating that no two coordinates 
  are assumed identically distributed and thus a separate set of \eqn{m} 
  density estimates is produced for each column of \eqn{x}.  On the other hand,
  if \code{blockid=rep(1,ncol(x))}, then the coordinates in each row 
  are assumed conditionally i.i.d.}
  \item{bw}{Bandwidth for density estimation, equal to the standard deviation 
  of the kernel density.  By default, a simplistic application of the 
  default \code{\link{bw.nrd0}} 
  bandwidth used by \code{\link{density}} to the entire dataset.}
  \item{constbw}{Logical:  If \code{TRUE}, use the same bandwidth for
  each iteration and for each component and block.  If \code{FALSE}, 
  use a separate bandwidth for each component and block, and update
  this bandwidth at each iteration of the algorithm using a suitably
  modified \code{\link{bw.nrd0}} method as described in 
  Benaglia et al (2011).}
  \item{h}{Alternative way to specify the bandwidth, to provide backward 
  compatibility.}
  \item{eps}{Tolerance limit for declaring algorithm convergence.  Convergence
  is declared whenever the maximum change in any coordinate of the 
  \code{lambda} vector (of mixing proportion estimates) does not exceed 
  \code{eps}.}

  \item{maxiter}{The maximum number of iterations allowed, for both 
  stochastic and non-stochastic versions; 
  for non-stochastic algorithms (\code{stochastic = FALSE}), convergence
  may be declared before \code{maxiter} iterations (see \code{eps} above).}
  
  \item{stochastic}{Flag, if FALSE (the default), runs the non-stochastic version
      of the npEM algorithm, as in Benaglia et al (2009). Set to TRUE to
      run a stochastic version which simulates the posteriors at each
      iteration, and runs for \code{maxiter} iterations.}

  \item{verb}{If TRUE, print updates for every iteration of the algorithm as
  it runs}
}
\value{
\code{spEM} returns a list of class \code{spEM} with the following items:
  \item{data}{The raw data (an \eqn{n\times r}{n x r} matrix).}
  \item{posteriors}{An \eqn{n\times m}{n x m} matrix of posterior probabilities for
   observation. If \code{stochastic = TRUE}, this matrix is computed 
   from an average over the \code{maxiter} iterations.}
  \item{bandwidth}{If \code{constbw==TRUE}, 
  same as the \code{bw} input argument; otherwise, value of \code{bw} matrix
  at final iteration (since for now this algorithm only implements 
  model (4.7) in Benaglia et al, the bandwidth matrix is reduced to a single
  bandwith scalar).  
  This
  information is needed by any method that produces density estimates from the
  output.}
  \item{blockid}{Same as the \code{blockid} input argument, but recoded to have
  positive integer values.  Also needed by any method that produces density 
  estimates from the output.}
  \item{lambda}{The sequence of mixing proportions over iterations.}
  \item{lambdahat}{The final mixing proportions if \code{stochastic = FALSE}, 
  or the average mixing proportions if \code{stochastic = TRUE}.}
  \item{mu}{The sequence of location parameters over iterations.}
  \item{muhat}{The final location parameters if \code{stochastic = FALSE}, 
  or the average location parameters if \code{stochastic = TRUE}.}
  \item{sigma}{The sequence of scale parameters over iterations.}
  \item{sigmahat}{The final scale parameters if \code{stochastic = FALSE}, 
  or the average scale parameters if \code{stochastic = TRUE}.}
  \item{loglik}{The sequence of log-likelihoods over iterations.}
}
\seealso{
\code{\link{plot.spEM}}, \code{\link{normmixrm.sim}}, \code{\link{spEMsymloc}},
\code{\link{npEM}}, \code{\link{plotseq.npEM}}
}
\references{
   \itemize{
   \item Benaglia, T., Chauveau, D., and Hunter, D. R., An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526, 2009.
   
   \item Benaglia, T., Chauveau, D. and Hunter, D.R.
 	Bandwidth Selection in an EM-like algorithm for nonparametric multivariate mixtures.
	Nonparametric Statistics and Mixture Models: A Festschrift in
  	Honor of Thomas P. Hettmansperger. World Scientific Publishing Co., 
  	pages 15-27, 2011.
   
   \item Bordes, L., Chauveau, D., and Vandekerkhove, P.,
   An EM algorithm for a semiparametric mixture model, 
   Computational Statistics and Data Analysis, 51: 5429-5443, 2007.
   }
}
\examples{\dontrun{
## simulate a 2-component gaussian mixture with 3 iid repeated measures
set.seed(100)
mu <- matrix(c(0, 15), 2, 3)
sigma <- matrix(c(1, 5), 2, 3)
x <- rmvnormmix(300, lambda = c(.4,.6), mu = mu, sigma = sigma)

## apply spEM with or without an iterative bandwidth selection
d <- spEM(x, mu0 = 2, blockid = rep(1,3), constbw = FALSE)
d2 <- spEM(x, mu0 = 2, blockid = rep(1,3), constbw = TRUE)
plot(d, xlim=c(-10, 40), ylim = c(0, .16), xlab = "", breaks = 30,
     cex.lab=1.5, cex.axis=1.5, addlegend=FALSE)
plot(d2, newplot=FALSE, addlegend=FALSE, lty=2)}
}


\keyword{file}

\name{regmixEM.mixed}
\title{EM Algorithm for Mixtures of Regressions with Random Effects}
\alias{regmixEM.mixed}
\usage{
regmixEM.mixed(y, x, w = NULL, sigma = NULL, arb.sigma = TRUE,
               alpha = NULL, lambda = NULL, mu = NULL, 
               rho = NULL, R = NULL, arb.R = TRUE, k = 2, 
               ar.1 = FALSE, addintercept.fixed = FALSE, 
               addintercept.random = TRUE, epsilon = 1e-08, 
               maxit = 10000, verb = FALSE)
}

\description{
  Returns EM algorithm output for mixtures of multiple regressions with random effects
  and an option to incorporate fixed effects and/or AR(1) errors.
}
\arguments{
  \item{y}{A list of N response trajectories with (possibly) varying dimensions of
  length \eqn{n_i}.}
  \item{x}{A list of N design matrices of dimensions \eqn{(n_i)\times p}{(n_i) x p}.  
  Each trajectory in y has
  its own design matrix.}
  \item{w}{A list of N known explanatory variables having dimensions \eqn{(n_i)\times q}{(n-1) x q}. 
  If \code{mixed} = FALSE,
  then \code{w} is replaced by a list of N zeros.}
  \item{sigma}{A vector of standard deviations.  If NULL, then \eqn{1/s^2} has
    random standard exponential entries according to a binning method done on the data.}
  \item{arb.sigma}{If TRUE, then \code{sigma} is k-dimensional. Else a common standard deviation is assumed.}
  \item{alpha}{A q-vector of unknown regression parameters for the fixed effects.  If NULL and \code{mixed} = TRUE, then \code{alpha} is 
  random from a normal distribution with mean and variance according to a binning method done
  on the data. If \code{mixed} = FALSE, then \code{alpha} = 0.}
  \item{lambda}{Initial value of mixing proportions for the assumed mixture structure on the regression coefficients.
  Entries should sum to 1.  This determines number of components.  If NULL, then \code{lambda} is
   random from uniform Dirichlet and the number of components is determined by \code{mu}.}
  \item{mu}{A pxk matrix of the mean for the mixture components of the random regression coefficients. If NULL, then the columns
  of \code{mu} are random from a multivariate normal distribution with mean and variance determined by a binning method
  done on the data.}
  \item{rho}{An Nxk matrix giving initial values for the correlation term in an AR(1) process.  If NULL, then these values
  are simulated from a uniform distribution on the interval (-1, 1).}
  \item{R}{A list of N pxp covariance matrices for the mixture components of the random regression coefficients. If NULL, then
  each matrix is random from a standard Wishart distribution according to a binning method done on the data.}
  \item{arb.R}{If TRUE, then \code{R} is a list of N pxp covariance matrices.  Else, one common covariance matrix is assumed.}
  \item{k}{Number of components.  Ignored unless \code{lambda} is NULL.}
  \item{ar.1}{If TRUE, then an AR(1) process on the error terms is included.  The default is FALSE.}
  \item{addintercept.fixed}{If TRUE, a column of ones is appended to the matrices in w.}
  \item{addintercept.random}{If TRUE, a column of ones is appended to the matrices in x before p is calculated.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{regmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The predictor values corresponding to the random effects.}
  \item{y}{The response values.}
  \item{w}{The predictor values corresponding to the (optional) fixed effects.}
  \item{lambda}{The final mixing proportions.}
  \item{mu}{The final mean vectors.}
  \item{R}{The final covariance matrices.}
  \item{sigma}{The final component error standard deviations.}
  \item{alpha}{The final regression coefficients for the fixed effects.}
  \item{rho}{The final error correlation values if an AR(1) process is included.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior.z}{An Nxk matrix of posterior membership probabilities.}
  \item{posterior.beta}{A list of N pxk matrices giving the posterior regression coefficient values.} 
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{regmixEM}}, \code{\link{post.beta}}
}
\references{
  Xu, W. and Hedeker, D. (2001) A Random-Effects Mixture Model for Classifying Treatment Response in
  Longitudinal Clinical Trials, \emph{Journal of Biopharmaceutical Statistics}, \bold{11(4)}, 253--273.

  Young, D. S. and Hunter, D. R. (2015) Random Effects Regression Mixtures for Analyzing Infant Habituation,
  \emph{Journal of Applied Statistics}, \bold{42(7)}, 1421--1441.
}
\examples{
## EM output for simulated data from 2-component mixture of random effects.

data(RanEffdata)
set.seed(100)
x <- lapply(1:length(RanEffdata), function(i) 
            matrix(RanEffdata[[i]][, 2:3], ncol = 2))
x <- x[1:20]
y <- lapply(1:length(RanEffdata), function(i) 
            matrix(RanEffdata[[i]][, 1], ncol = 1))
y <- y[1:20]
lambda <- c(0.45, 0.55)
mu <- matrix(c(0, 4, 100, 12), 2, 2)
sigma <- 2
R <- list(diag(1, 2), diag(1, 2))
em.out <- regmixEM.mixed(y, x, sigma = sigma, arb.sigma = FALSE,
                         lambda = lambda, mu = mu, R = R,
                         addintercept.random = FALSE,
                         epsilon = 1e-02, verb = TRUE)
em.out[4:10]

}

\keyword{file}

\name{plot.mixMCMC}
\title{Various Plots Pertaining to Mixture Model Output Using MCMC Methods}
\alias{plot.mixMCMC} 
\usage{ 
\method{plot}{mixMCMC}(x, trace.plots = TRUE, 
     summary.plots = FALSE, burnin = 2000, \dots) 
}

\description{
    Takes an object of class \code{mixMCMC} and returns various graphical output for select mixture models.
} 
\arguments{
  \item{x}{An object of class \code{mixMCMC}.}
  \item{trace.plots}{If TRUE, trace plots of the various parameters estimated by the MCMC methods is given.}
  \item{summary.plots}{Graphics pertaining to certain mixture models.  The details are given below.}
  \item{burnin}{The values 1 to \code{burnin} are dropped when producing the plots in \code{summary.plots}.}
  \item{...}{Graphical parameters passed to \code{regcr} function.}
}
\value{
  \code{plot.mixMCMC} returns trace plots of the various parameters estimated by the MCMC methods for all objects of class
  \code{mixMCMC}.  In addition, other plots may be produced for the following k-component mixture model functions:
  \item{regmixMH}{Credible bands for the regression lines in a mixture of linear regressions.  See \code{regcr} for more details.}
} 

\seealso{ 
\code{\link{regcr}} 
} 

\examples{ 
## M-H algorithm for NOdata with acceptance rate about 40\%.

data(NOdata)
attach(NOdata)
set.seed(100)
beta <- matrix(c(1.3, -0.1, 0.6, 0.1), 2, 2)
sigma <- c(.02, .05)
MH.out <- regmixMH(Equivalence, NO, beta = beta, s = sigma, 
                   sampsize = 2500, omega = .0013)
plot(MH.out, summary.plots = TRUE, burnin = 2450, 
     alpha = 0.01)
}

\keyword{file}

\name{tonedata}
\alias{tonedata}
\docType{data}
\title{Tone perception data}
\author{Christian Hennig}
\description{
  The tone perception data stem
  from an experiment of Cohen (1980) and have been analyzed in de Veaux
  (1989) and Viele and Tong (2002).  The dataset and this documentation file
  were copied from the fpc package by Christian Hennig.
  A pure fundamental tone was played to a
  trained musician. Electronically generated overtones were added, determined 
  by a stretching ratio of \code{stretchratio}. \code{stretchratio=2.0}
  corresponds to the harmonic pattern
  usually heard in traditional definite pitched instruments. The musician was
  asked to tune an adjustable tone to the octave above the fundamental tone.
  \code{tuned} gives the ratio of the adjusted tone to the fundamental,
  i.e. \code{tuned=2.0} would be the correct tuning for all
  \code{stretchratio}-values.
  The data analyzed here belong to 150 trials 
  with the same musician. In the original study, there were four further
  musicians. 
}
\usage{data(tonedata)}
\format{A data frame with 2 variables, \code{stretchratio} and
  \code{tuned}, and 150 cases.}
\source{
Original source: Cohen, E. A. (1980), \emph{Inharmonic tone
    perception}. Unpublished Ph.D. dissertation, Stanford University

R source:  Hennig, Christian (2010), fpc: Flexible procedures 
   for clustering, R package version 2.0-2.  
   \url{https://cran.r-project.org/package=fpc}
}
\references{
de Veaux, R. D. (1989), Mixtures of Linear Regressions,
  \emph{Computational Statistics and Data Analysis} 8, 227-245.
  
Viele, K. and Tong, B. (2002), Modeling with Mixtures
of Linear Regressions, \emph{Statistics and Computing} 12, 315-330.
}
\keyword{datasets}

\name{regmixMH}
\title{Metropolis-Hastings Algorithm for Mixtures of Regressions}
\alias{regmixMH}
\usage{
regmixMH(y, x, lambda = NULL, beta = NULL, s = NULL, k = 2,
         addintercept = TRUE, mu = NULL, sig = NULL, lam.hyp = NULL,
         sampsize = 1000, omega = 0.01, thin = 1)
}

\description{
  Return Metropolis-Hastings (M-H) algorithm output for mixtures of multiple regressions with
  arbitrarily many components.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{beta}.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and k is number of components.
    If NULL, then \code{beta} has uniform standard normal entries.  If both
    \code{lambda} and \code{beta} are NULL, then number of components is determined by \code{s}.}
  \item{s}{k-vector of standard deviations.  If NULL, then \eqn{1/\code{s}^2} has
    random standard exponential entries.  If \code{lambda}, \code{beta}, and \code{s} are
    NULL, then number of components determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless all of \code{lambda}, \code{beta},
    and \code{s} are NULL.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{mu}{The prior hyperparameter of same size as \code{beta};
    the means of \code{beta} components.  If NULL,
    these are set to zero.}
  \item{sig}{The prior hyperparameter of same size as \code{beta};
    the standard deviations of \code{beta} components.  If NULL, these are 
    all set to five times the overall standard deviation of y.}
  \item{lam.hyp}{The prior hyperparameter of length \code{k} for the mixing proportions (i.e.,
	these are hyperparameters for the Dirichlet distribution).  If NULL, these are generated from a standard uniform
	distribution and then scaled to sum to 1.}
  \item{sampsize}{Size of posterior sample returned.}
  \item{omega}{Multiplier of step size to control M-H acceptance rate.
    Values closer to zero result in higher acceptance rates, generally.}
  \item{thin}{Lag between parameter vectors that will be kept.}
}
\value{
  \code{regmixMH} returns a list of class \code{mixMCMC} with items:
  \item{x}{A nxp matrix of the predictors.}
  \item{y}{A vector of the responses.}
  \item{theta}{A (\code{sampsize}/\code{thin}) x q matrix of MCMC-sampled
  q-vectors, where q is the total number of parameters in \code{beta}, \code{s}, and
  \code{lambda}.}
  \item{k}{The number of components.}
}
\seealso{
\code{\link{regcr}}
}
\references{
  Hurn, M., Justel, A. and Robert, C. P. (2003) Estimating Mixtures of Regressions, \emph{Journal 
  of Computational and Graphical Statistics} \bold{12(1)}, 55--79.
}
\examples{
## M-H algorithm for NOdata with acceptance rate about 40\%.

data(NOdata)
attach(NOdata)
set.seed(100)
beta <- matrix(c(1.3, -0.1, 0.6, 0.1), 2, 2)
sigma <- c(.02, .05)
MH.out <- regmixMH(Equivalence, NO, beta = beta, s = sigma, 
                   sampsize = 2500, omega = .0013)
MH.out$theta[2400:2499,]
}

\keyword{file}

\name{dmvnorm}
\alias{dmvnorm}
\alias{logdmvnorm}
\title{The Multivariate Normal Density}
\description{
  Density and log-density for the multivariate normal distribution 
  with mean equal to \code{mu}
  and variance matrix equal to \code{sigma}.
}
\usage{
dmvnorm(y, mu=NULL, sigma=NULL)
logdmvnorm(y, mu=NULL, sigma=NULL)
}
\arguments{
  \item{y}{Either a \eqn{d} - vector or an \eqn{n\times d}{n x d} matrix, where \eqn{d}
  is the dimension of the normal distribution and \eqn{n} is the number of
  points at which the density is to be evaluated.}
  \item{mu}{\eqn{d} - vector:  Mean of the normal distribution (or NULL uses
  the origin as default)}
  \item{sigma}{This \eqn{d\times d}{d x d} matrix is the variance matrix of the normal
  distribution (or NULL uses the identity matrix as default)}
}
\value{
  \code{dmvnorm} gives the densities, while
  \code{logdmvnorm} gives the logarithm of the densities.
}
\details{
This code is written to be efficient, using the qr-decomposition of the 
covariance matrix (and using it only once, rather than recalculating it
for both the determinant and the inverse of \code{sigma}).
}
\seealso{
  \code{\link{qr}}, \code{\link{qr.solve}}, \code{\link{dnorm}}, 
  \code{\link{rmvnorm}}
}
\keyword{distribution}

\name{print.npEM}
\alias{print.npEM}
\title{Printing non- and semi-parametric multivariate mixture model fits}
\usage{
\method{print}{npEM}(x, \dots)
}
\arguments{
  \item{x}{an object of class \code{npEM} such as a result of a call
  to \code{\link{npEM}}}
  \item{\dots}{Additional arguments to \code{\link{print}}}
}
\description{
\code{\link[base]{print}} method for class \code{npEM}.
}
\details{
 \code{print.npEM} prints the elements of an \code{npEM} object
 without printing the data or the posterior probabilities.
 (These may still be accessed as \code{x$data} and \code{x$posteriors}.)
}
\value{
  \code{print.npEM} returns (invisibly) the full value of \code{x} itself,
  including the \code{data} and \code{posteriors} elements.
}
\seealso{ 
\code{\link{npEM}},
\code{\link{plot.npEM}}
\code{\link{summary.npEM}}
}
\examples{
data(Waterdata)
set.seed(100)
\dontrun{npEM(Waterdata[,3:10], 3, bw=4, verb=FALSE) # Assume indep but not iid}
}
\keyword{file}

\name{plotly_ise.npEM}
\title{Visualization of Integrated Squared Error for a selected density from npEM output using \code{plotly}}
\alias{plotly_ise.npEM}
\usage{
plotly_ise.npEM(npEMout, component=1, block=1, truepdf=dnorm, lower=-Inf,
                upper=Inf, plots = TRUE ,
                col = NULL , width = 3,
                title = NULL , title.size = 15 , title.x = 0.5 , title.y = 0.95,
                xlab = "t" , xlab.size = 15 , xtick.size = 15,
                ylab = "" , ylab.size = 15 , ytick.size = 15,
                legend.text = "" , legend.text.size = 15 , legend.size = 15, ...)
}
\description{
This is an updated visualization function for \code{ise.npEM}. For more technical details, please refer to \code{ise.npEM}.
}
\arguments{
  \item{npEMout}{An object of class \code{npEM} such as the output
  of the \code{\link{npEM}} function}
  \item{component, block}{Component and block of particular density to analyze
  from \code{npEMout}.}
  \item{truepdf}{an \R function taking a numeric first argument and 
  returning a numeric vector of the same length. Returning a 
  non-finite element will generate an error.}
  \item{lower, upper}{the limits of integration.  Can be infinite.}
  \item{plots}{logical:  Should plots be produced?}
  \item{...}{additional arguments to be passed to \code{truepdf}
  	(and that may be mandatory like, e.g., the \code{df = } argument of \code{dt}). 
  Remember to use argument names not matching those of \code{ise.npRM}.}
  \item{col}{Color of traces.}
  \item{width}{Line width of traces.}
  \item{title}{Text of the main title.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horsizontal position of the main title.}
  \item{title.y}{Vertical posotion of the main title.}
  \item{xlab}{Label of X-axis.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab}{Label of Y-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
  \item{legend.text}{Title of legend.}
  \item{legend.text.size}{Size of the legend title.}
  \item{legend.size}{Size of legend.}
}
\value{
  Just as for the \code{\link{integrate}} function,
  a list of class \code{"integrate"} with components
  \item{value}{the final estimate of the integral.}
  \item{abs.error}{estimate of the modulus of the absolute error.}
  \item{subdivisions}{the number of subintervals produced in the
    subdivision process.}
  \item{message}{\code{"OK"} or a character string giving the error message.}
  \item{call}{the matched call.}
}
\details{This function calls the \code{\link{wkde}} (weighted kernel
density estimate) function.
}
\seealso{
\code{\link{npEM}}, \code{\link{wkde}}, \code{\link{integrate}}, \code{\link{ise.npEM}}
}
\references{
   \itemize{
   
  \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526.

   \item Benaglia, T., Chauveau, D., Hunter, D. R., and Young, D. (2009),
  mixtools: An R package for analyzing finite mixture models.
	Journal of Statistical Software, 32(6):1-29.


   }
}
\examples{
\dontrun{
data(Waterdata)
set.seed(100)
a <- npEM(Waterdata[,3:10], mu0=3, bw=4) # Assume indep but not iid
plotly_ise.npEM(a , plots = TRUE)
}
}


\keyword{file}

\name{rmvnormmix}
\title{Simulate from Multivariate (repeated measures) Mixtures of Normals}
\alias{normmixrm.sim}
\alias{rmvnormmix}
\usage{
rmvnormmix(n, lambda=1, mu=0, sigma=1)
}

\description{
Simulate from a mixture of multivariate zero-correlation normal distributions
}
\arguments{
  \item{n}{Number of cases to simulate.}
  \item{lambda}{Vector of mixture probabilities with length equal to \eqn{m}, the
    desired number of components.  This is assumed to sum
    to 1; if not, it is normalized.}
  \item{mu}{Matrix of means of dimensions \eqn{m\times r}{m x r}, where
  \eqn{m} is the number of components (subpopulations) and \eqn{r} is
  the number of coordinates (repeated measurements) per case.  Note:  \code{mu}
  is automatically coerced to a matrix with \eqn{m} rows even if it is not
  given in this form, which can lead to unexpected behavior in some cases.}
  \item{sigma}{Matrix of standard deviations, same dimensions as \code{mu}.
  The coordinates within a case are independent, conditional on the mixture
  component. (There is marginal correlation among the coordinates, but
  this is due to the mixture structure only.)  Note:  \code{sigma} is
  automatically coerced to a matrix with \eqn{m} rows even if it is not
  given in this form, which can lead to unexpected behavior in some cases.}
}
\details{
It is possible to generate univariate standard normal random variables using
the default values (but why bother?).  The case of conditionally iid
coordinates is covered by the situation in which
all columns in mu and sigma are identical.
}
\value{
  \code{rmvnormmix} returns an \eqn{n\times r}{n x r} matrix in which each row is
  a sample from one of the components of a mixture of zero-correlation 
  multivariate normals.  The mixture structure
  induces nonzero correlations among the coordinates.
}
 
\seealso{
\code{\link{rnormmix}}
}
\examples{
##Generate data from a 2-component mixture of trivariate normals.

set.seed(100)
n <- 200
lambda <- rep(1, 2)/2
mu <- matrix(2*(1:6), 2, 3)
sigma <- matrix(1,2,3)
mydata<-rmvnormmix(n, lambda, mu, sigma)

## Now check to see if we can estimate mixture densities well:
title <- paste("Does this resemble N(", mu[1,], ",1) and N(", mu[2,],",1)?",
                sep="")
plot(npEM(mydata, 2), title=title)
}

\keyword{file}

\name{CO2data}
\docType{data}
\title{GNP and CO2 Data Set}
\alias{CO2data}
\usage{
data(CO2data)
}
\description{
  This data set gives the gross national product (GNP) per capita in 1996 for various countries as
  well as their estimated carbon dioxide (CO2) emission per capita for the same year. 
}
\format{This data frame consists of 28 countries and the following columns:
\itemize{
  \item{\code{GNP}}{The gross national product per capita in 1996.}
  \item{\code{CO2}}{The estimated carbon dioxide emission per capita in 1996.}
  \item{\code{country}}{An abbreviation pertaining to the country measured 
  (e.g., "GRC" = Greece and "CH" = Switzerland).}
}
}
\references{
  Hurn, M., Justel, A. and Robert, C. P. (2003) Estimating Mixtures of Regressions, \emph{Journal 
  of Computational and Graphical Statistics} \bold{12(1)}, 55--79.
}
\keyword{datasets}

\name{spEMsymloc}
\title{Semiparametric EM-like Algorithm for univariate symmetric location mixture}
\alias{spEMsymloc}
\usage{
spEMsymloc(x, mu0, bw = bw.nrd0(x), h=bw, eps = 1e-8, maxiter = 100, 
           stochastic = FALSE, verbose = FALSE)
}

\description{
  Returns semiparametric EM algorithm output (Bordes et al, 2007,
  and Benaglia et al, 2009)  for location mixtures
  of univariate data and symmetric component density.
}
\arguments{
  \item{x}{A vector of length \eqn{n}{n} consisting of the data.}
  \item{mu0}{Either a vector specifying the initial
      centers for the \link{kmeans} function, and from which the number of 
   component is obtained,  or an integer \eqn{m} specifying the 
      number of initial centers, which are then choosen randomly in
   \link{kmeans}.}

  \item{bw}{Bandwidth for density estimation, equal to the standard deviation 
  of the kernel density.}
  \item{h}{Alternative way to specify the bandwidth, to provide backward 
  compatibility.}

  \item{eps}{Tolerance limit for declaring algorithm convergence.  Convergence
  is declared before \code{maxiter} iterations
  whenever the maximum change in any coordinate of the 
  \code{lambda} (mixing proportion estimates) and \code{mu} (means) 
  vector  does not exceed \code{eps}.}

  \item{maxiter}{The maximum number of iterations allowed, for both 
  stochastic and non-stochastic versions; 
  for non-stochastic algorithms (\code{stochastic = FALSE}), convergence
  may be declared before \code{maxiter} iterations (see \code{eps} above).}

  \item{stochastic}{Flag, if FALSE (the default), runs the non-stochastic version
      of the algorithm, as in Benaglia et al (2009). Set to TRUE to
      run a stochastic version which simulates the posteriors at each
      iteration (as in Bordes et al, 2007),
   and runs for \code{maxiter} iterations.}


  \item{verbose}{If TRUE, print updates for every iteration of the algorithm as
  it runs}
}
\value{
\code{spEMsymloc} returns a list of class \code{npEM} with the following items:
  \item{data}{The raw data (an \eqn{n\times r}{n x r} matrix).}
  \item{posteriors}{An \eqn{n\times m}{n x m} matrix of posterior probabilities for
   observations. If \code{stochastic = TRUE}, this matrix is computed 
   from an average over the \code{maxiter} iterations.}
  \item{bandwidth}{Same as the \code{bw} input argument, returned because this
  information is needed by any method that produces density estimates from the
  output.}
  \item{lambda}{The sequence of mixing proportions over iterations.}
  \item{lambdahat}{The final estimate for mixing proportions if \code{stochastic = 
  FALSE}, the average over the sequence if \code{stochastic = TRUE}.}
  \item{mu}{the sequence of component means over iterations.}
  \item{muhat}{the final estimate of component means if \code{stochastic = 
   FALSE}, the average over the sequence if \code{stochastic = TRUE}.}
  \item{symmetric}{Flag indicating that the kernel density estimate is
   using a symmetry assumption.}
}
\seealso{
\code{\link{plot.npEM}}, \code{\link{rnormmix}},
\code{\link{npEM}}, \code{\link{spEMsymlocN01}}, \code{\link{plotseq.npEM}}
}
\references{
   \itemize{
   \item Benaglia, T., Chauveau, D., and Hunter, D. R., An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526, 2009.
   
   \item Benaglia, T., Chauveau, D., Hunter, D. R., and Young, D.
	mixtools: An R package for analyzing finite mixture models.
	Journal of Statistical Software, 32(6):1-29, 2009.


   \item Bordes, L., Chauveau, D., and Vandekerkhove, P. (2007),
   An EM algorithm for a semiparametric mixture model, 
   Computational Statistics and Data Analysis, 51: 5429-5443.
   }
}
\examples{
## Example from a normal location mixture
set.seed(100)
n <- 200
lambda <- c(1/3,2/3)
mu <- c(0, 4); sigma<-rep(1, 2)
x <- rnormmix(n, lambda, mu, sigma)
out.stoc <- spEMsymloc(x, mu0=c(-1, 2), stochastic=TRUE)
out.nonstoc <- spEMsymloc(x, mu0=c(-1, 2))
}

\keyword{file}

\name{print.mvnpEM}
\alias{print.mvnpEM}
\title{Printing of Results from the mvnpEM Algorithm Output}
\usage{
\method{print}{mvnpEM}(x, \dots)
}
\arguments{
  \item{x}{an object of class \code{mvnpEM} such as a result of a call
  to \code{\link{mvnpEM}}}
  \item{\dots}{Additional arguments to \code{\link{print}}}
}
\description{
\code{\link[base]{print}} method for class \code{mvnpEM}.
}
\details{
 \code{print.mvnpEM} prints the elements of an \code{mvnpEM} object
 without printing the data or the posterior probabilities.
 (These may still be accessed as \code{x$data} and \code{x$posteriors}.)
}
\value{
  \code{print.mvnpEM} returns (invisibly) the full value of \code{x} itself,
  including the \code{data} and \code{posteriors} elements.
}
\seealso{ 
\code{\link{mvnpEM}},
\code{\link{plot.mvnpEM}}
\code{\link{summary.mvnpEM}}
}
\examples{
# Example as in Chauveau and Hoang (2015) with 6 coordinates
\dontrun{
m=2; r=6; blockid <-c(1,1,2,2,3,3) # 3 bivariate blocks 
# generate some data x ...
a <- mvnpEM(x, mu0=2, blockid, samebw=F) # adaptive bandwidth
print(a)}
}
\keyword{file}

\name{boot.se}
\title{Performs Parametric Bootstrap for Standard Error 
Approximation}
\alias{boot.se}
\usage{
boot.se(em.fit, B = 100, arbmean = TRUE, arbvar = TRUE, 
        N = NULL, ...)
}

\description{
  Performs a parametric bootstrap by producing B bootstrap samples for the parameters in the specified
  mixture model.
}
\arguments{
  \item{em.fit}{An object of class \code{mixEM}.  The estimates produced in \code{em.fit} will be used as the 
    parameters for the distribution from which we generate the bootstrap data.}
  \item{B}{The number of bootstrap samples to produce.  The default is 100,
    but ideally, values of 1000 or more would be more acceptable.}
  \item{arbmean}{If FALSE, then a scale mixture analysis can be performed for \code{mvnormalmix}, \code{normalmix}, \code{regmix},
    or \code{repnormmix}. The default is TRUE.}
  \item{arbvar}{If FALSE, then a location mixture analysis can be performed for \code{mvnormalmix}, \code{normalmix}, \code{regmix},
    or \code{repnormmix}. The default is TRUE.}
  \item{N}{An n-vector of number of trials for the logistic regression type \code{logisregmix}.  
  If NULL, then \code{N} is an n-vector of 1s for binary logistic regression.}
  \item{...}{Additional arguments passed to the various EM algorithms for the mixture of interest.} 
}
\value{
  \code{boot.se} returns a list with the bootstrap samples and standard errors for the mixture of interest.
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
## Bootstrapping standard errors for a regression mixture case.

data(NOdata)
attach(NOdata)
set.seed(100)
em.out <- regmixEM(Equivalence, NO, arbvar = FALSE)
out.bs <- boot.se(em.out, B = 10, arbvar = FALSE)
out.bs

}


\keyword{file}

\name{multmixEM}
\title{EM Algorithm for Mixtures of Multinomials}
\alias{multmixEM}
\usage{
multmixEM(y, lambda = NULL, theta = NULL, k = 2,
          maxit = 10000, epsilon = 1e-08, verb = FALSE)
}
\description{
  Return EM algorithm output for mixtures of multinomial distributions.
}
\arguments{
  \item{y}{Either An nxp matrix of data (multinomial counts), where n is the
  sample size and p is the number of multinomial bins, or the
  output of the \code{\link{makemultdata}} function.  It is not necessary
  that all of the rows contain the same number of multinomial trials (i.e.,
  the row sums of \code{y} need not be identical).}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{theta}.}
  \item{theta}{Initial value of \code{theta} parameters.  Should be a kxp matrix,
    where p is the number of columns of y and k is number of components.
    Each row of \code{theta} should sum to 1.
    If NULL, then each row is random from uniform Dirichlet.
    If both \code{lambda} and \code{theta} are NULL, then number of components 
    is determined by k.}
  \item{k}{Number of components.  Ignored unless \code{lambda} and \code{theta}
    are NULL.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{multmixEM} returns a list of class \code{mixEM} with items:
  \item{y}{The raw data.}
  \item{lambda}{The final mixing proportions.}
  \item{theta}{The final multinomial parameters.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
    observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.} 
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{compCDF}}, \code{\link{makemultdata}}, \code{\link{multmixmodel.sel}}
}
\references{
  \itemize{
  \item McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
  
  \item Elmore, R. T., Hettmansperger, T. P. and Xuan, F. (2004) The Sign Statistic, One-Way Layouts
  and Mixture Models, \emph{Statistical Science} \bold{19(4)}, 579--587.
  }
}
\examples{
## The sulfur content of the coal seams in Texas

set.seed(100)
A <- c(1.51, 1.92, 1.08, 2.04, 2.14, 1.76, 1.17)
B <- c(1.69, 0.64, .9, 1.41, 1.01, .84, 1.28, 1.59) 
C <- c(1.56, 1.22, 1.32, 1.39, 1.33, 1.54, 1.04, 2.25, 1.49) 
D <- c(1.3, .75, 1.26, .69, .62, .9, 1.2, .32) 
E <- c(.73, .8, .9, 1.24, .82, .72, .57, 1.18, .54, 1.3)

dis.coal <- makemultdata(A, B, C, D, E, 
                         cuts = median(c(A, B, C, D, E)))
em.out <- multmixEM(dis.coal)
em.out[1:4]
}

\keyword{file}

\name{try.flare}
\title{Mixtures of Regressions with Flare MM Algorithm}
\alias{try.flare}
\usage{
try.flare(y, x, lambda = NULL, beta = NULL, sigma = NULL, 
          alpha = NULL, nu = 1, epsilon = 1e-04, 
          maxit = 10000, verb = FALSE, restart = 50)
}

\description{
  The function which \code{flaremixEM} actually calls.  This only allows
  one barrier constant to be inputted at a time.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An n-vector of predictor values.  An intercept term will be added by default.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to 1.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a 2x2 matrix where the columns
    corresond to the component.}
  \item{sigma}{A vector of standard deviations.}
  \item{alpha}{A scalar for the exponential component's rate.}
  \item{nu}{A scalar specifying the barrier constant to use.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
  \item{restart}{The number of times to restart the algorithm in case convergence is not attained.
  The default is 50.}
}
\value{
  \code{try.flare} returns a list of class \code{mixEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's).}
  \item{y}{The response values.}
  \item{posterior}{An nx2 matrix of posterior probabilities for
   observations.}
  \item{lambda}{The final mixing proportions.}
  \item{beta}{The final regression coefficients.}
  \item{sigma}{The final standard deviations.}
  \item{alpha}{The final exponential rate.}
  \item{loglik}{The final log-likelihood.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{flaremixEM}}
}

\details{
  This usually is not called by the user.  The user will likely want \code{flaremixEM}, which also
  has an example to demonstrate this algorithm.
}
\keyword{internal}

\name{regmixEM.lambda}
\title{EM Algorithm for Mixtures of Regressions with Local Lambda Estimates}
\alias{regmixEM.lambda}
\usage{
regmixEM.lambda(y, x, lambda = NULL, beta = NULL, sigma = NULL, 
                k = 2, addintercept = TRUE, arbmean = TRUE,
                arbvar = TRUE, epsilon = 1e-8, maxit = 10000,
                verb = FALSE)
}

\description{
  Returns output for one step of an EM algorithm output for mixtures of multiple regressions 
  where the mixing proportions are estimated locally.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{lambda}{An nxk matrix of initial local values of mixing proportions.  
    Entries should sum to 1.  This determines number of components.  
    If NULL, then \code{lambda} is simply one over the number of components.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and k is number of components.
    If NULL, then \code{beta} has uniform standard normal entries.  If both
    \code{lambda} and \code{beta} are NULL, then number of components is determined by \code{sigma}.}
  \item{sigma}{k-vector of initial global values of standard deviations.  
    If NULL, then \eqn{1/\code{sigma}^2} has random standard exponential entries.  
    If \code{lambda}, \code{beta}, and \code{sigma} are NULL, then number of components is determined by \code{k}.}
  \item{k}{The number of components.  Ignored unless all of \code{lambda}, \code{beta},
    and \code{sigma} are NULL.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{arbmean}{If TRUE, each mixture component is assumed to have a different set of regression coefficients
  (i.e., the \code{beta}s).}
  \item{arbvar}{If TRUE, each mixture component is assumed to have a different \code{sigma}.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{regmixEM.lambda} returns a list of class \code{mixEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's if \code{addintercept} = TRUE).}
  \item{y}{The response values.}
  \item{lambda}{The inputted mixing proportions.}
  \item{beta}{The final regression coefficients.}
  \item{sigma}{The final standard deviations. If \code{arbmean} = FALSE, then only the smallest standard
   deviation is returned. See \code{scale} below.}
  \item{scale}{If \code{arbmean} = FALSE, then the scale factor for the component standard deviations is returned.
   Otherwise, this is omitted from the output.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\details{
    Primarily used within \code{regmixEM.loc}.
}
\seealso{
\code{\link{regmixEM.loc}}
}
\examples{
## Compare a 2-component and 3-component fit to NOdata.

data(NOdata)
attach(NOdata)
set.seed(100)
out1 <- regmixEM.lambda(Equivalence, NO)
out2 <- regmixEM.lambda(Equivalence, NO, k = 3)
c(out1$loglik, out2$loglik)

}

\keyword{file}

\name{test.equality.mixed}
\title{Performs Chi-Square Test for Mixed Effects Mixtures}
\alias{test.equality.mixed}
\usage{
test.equality.mixed(y, x, w=NULL, arb.R = TRUE, 
                    arb.sigma = FALSE, lambda = NULL, 
                    mu = NULL, sigma = NULL, R = NULL, 
                    alpha = NULL, ...)
}

\description{
  Performs a likelihood ratio test of either common variance terms between the response trajectories in a mixture
  of random (or mixed) effects regressions or for common variance-covariance matrices for the random effects mixture distribution.}
\arguments{
  \item{y}{The responses for \code{regmixEM.mixed}.}
  \item{x}{The predictors for the random effects in \code{regmixEM.mixed}.}
  \item{w}{The predictors for the (optional) fixed effects in \code{regmixEM.mixed}.}
  \item{arb.R}{If FALSE, then a test for different variance-covariance matrices for the random effects mixture is performed.}
  \item{arb.sigma}{If FALSE, then a test for different variance terms between the response trajectories is performed.}
  \item{lambda}{A vector of mixing proportions (under the null hypothesis) with same purpose as outlined in \code{regmixEM.mixed}.}
  \item{mu}{A matrix of the means (under the null hypothesis) with same purpose as outlined in \code{regmixEM.mixed}.}
  \item{sigma}{A vector of standard deviations (under the null hypothesis) with same purpose as outlined in \code{regmixEM.mixed}.}
  \item{R}{A list of covariance matrices  (under the null hypothesis) with same purpose as outlined in \code{regmixEM.mixed}.}
  \item{alpha}{An optional vector of fixed effects regression coefficients  (under the null hypothesis) with same purpose as outlined
  in \code{regmixEM.mixed}.}
  \item{...}{Additional arguments passed to \code{regmixEM.mixed}.} 
}
\value{
  \code{test.equality.mixed} returns a list with the following items:
  \item{chi.sq}{The chi-squared test statistic.}
  \item{df}{The degrees of freedom for the chi-squared test statistic.}
  \item{p.value}{The p-value corresponding to this likelihood ratio test.}
}
\seealso{
\code{\link{test.equality}}
}
\examples{
##Test of equal variances in the simulated data set.

data(RanEffdata)
set.seed(100)
x<-lapply(1:length(RanEffdata), function(i) 
          matrix(RanEffdata[[i]][, 2:3], ncol = 2))
x<-x[1:15]
y<-lapply(1:length(RanEffdata), function(i) 
          matrix(RanEffdata[[i]][, 1], ncol = 1))
y<-y[1:15]

out<-test.equality.mixed(y, x, arb.R = TRUE, arb.sigma = FALSE,
                         epsilon = 1e-1,  verb = TRUE,
                         maxit = 50,
                         addintercept.random = FALSE)
out
}

\keyword{file}

\name{summary.mvnpEM}
\alias{summary.mvnpEM}
\alias{print.summary.mvnpEM}
\title{Summarizing Fits for Nonparametric Mixture Models 
with Conditionally Independent Multivariate Component Densities}
\usage{
\method{summary}{mvnpEM}(object, \dots)
\method{print}{summary.mvnpEM}(x, digits=3, \dots)
}
\arguments{
  \item{object,x}{an object of class \code{mvnpEM} such as a result of a call
  to \code{\link{mvnpEM}}}
  \item{digits}{Significant digits for printing values}
  \item{\dots}{further arguments passed to or from other methods.}
}
\description{
\code{\link[base]{summary}} method for class \code{mvnpEM}.
}
\details{
 \code{\link{summary.mvnpEM}} prints means and variances of each block for
 each component.  These quantities might not be part of the model, but they
 are estimated nonparametrically based on the posterior probabilities and the
 data.
}
\value{
  The function \code{\link{summary.mvnpEM}} returns a list of type \code{summary.mvnpEM}
  with the following components:
  \item{n}{The number of observations}
  \item{m}{The number of mixture components}
  \item{B}{The number of blocks}
  \item{blockid}{The block ID (from 1 through B) for each of the coordinates
  of the multivariate observations.  The \code{blockid} component is of length
  \eqn{r}, the dimension of each observation.}
  \item{means}{A \eqn{B\times m}{B x m} matrix giving the estimated mean of each
  block in each component.}
  \item{variances}{Same as \code{means} but giving the estimated variances instead.}
}
\references{
 Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   \emph{Journal of Computational and Graphical Statistics}, \bold{18(2)}, 505--526.
   
Chauveau, D., and Hoang, V. T. L. (2015),
   Nonparametric mixture models with conditionally independent multivariate component densities, 
   Preprint under revision.
   \url{https://hal.archives-ouvertes.fr/hal-01094837}
}
\seealso{ 
\code{\link{mvnpEM}},
\code{\link{plot.mvnpEM}}
}
\examples{
# Example as in Chauveau and Hoang (2015) with 6 coordinates
\dontrun{
m=2; r=6; blockid <-c(1,1,2,2,3,3) # 3 bivariate blocks 
# generate some data x ...
a <- mvnpEM(x, mu0=2, blockid, samebw=F) # adaptive bandwidth
plot(a) # this S3 method produces 6 plots of univariate marginals
summary(a)}
}
\keyword{file}

\name{rweibullmix}
\title{Simulate from Mixtures of Weibull distributions}
\alias{rweibullmix}
\usage{
rweibullmix(n, lambda = 1, shape = 1, scale = 1)
}

\description{
Simulate from a mixture of univariate Weibull distributions.
}
\arguments{
  \item{n}{Number of cases to simulate.}
  \item{lambda}{Vector of mixture probabilities, with length equal to \eqn{m}, 
    the desired number of components (subpopulations).  This is assumed to sum
    to 1.}
  \item{shape}{Vector of component shapes.}
  \item{scale}{Vector of component scales.}
}
\value{
  \code{rexpmix} returns an \eqn{n}-vector sampled from an \eqn{m}-component 
  mixture of univariate Weibull distributions.
}
%%\details{This function simply calls \code{\link{sample}} and \code{\link{rexp}}.} 
\seealso{
\code{\link{rnormmix}} and \code{\link{rmvnormmix}} for Gaussian mixtures,
\code{\link{rexpmix}} for mixture of exponentials.
}
\examples{
n = 500 # sample size
m = 2 # nb components
lambda=c(0.4, 0.6)
shape <- c(0.5,5); scale <- c(1,20) # model parameters
set.seed(321)
x <- rweibullmix(n, lambda, shape, scale) # iid ~ weibull mixture

## histogram of the simulated data.
hist(x, col=8)
}

\keyword{file}

\name{plotexpRMM}
\alias{plotexpRMM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Plot sequences from the EM algorithm for censored mixture of exponentials
}
\description{Function for plotting sequences of estimates along iterations, from an object returned by the \code{\link{expRMM_EM}}, an EM algorithm for mixture of exponential 
distributions with randomly right censored data (see reference below).
}
\usage{
  plotexpRMM(a, title=NULL, rowstyle=TRUE, subtitle=NULL, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{An object returned by \code{\link{expRMM_EM}}.}
  \item{title}{The title of the plot, set to some default value if \code{NULL}.}
  \item{rowstyle}{Window organization, for plots in rows (the default) or columns.}
  \item{subtitle}{A subtitle for the plot, set to some default value if \code{NULL}.}
  \item{...}{Other parameters (such as \code{lwd}) passed to \code{plot}, \code{lines}, and  
          \code{legend} commands.}
  }


\value{The plot returned}


\seealso{
  Related functions: 
  \code{\link{expRMM_EM}},   \code{\link{summary.mixEM}}, \code{\link{plot.mixEM}}.

  Other models and algorithms for censored lifetime data 
(name convention is model_algorithm):
  \code{\link{weibullRMM_SEM}}, \code{\link{spRMM_SEM}}.
}

\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
n=300 # sample size
m=2   # number of mixture components
lambda <- c(1/3,1-1/3); rate <- c(1,1/10) # mixture parameters
set.seed(1234)
x <- rexpmix(n, lambda, rate) # iid ~ exponential mixture
cs=runif(n,0,max(x)) # Censoring (uniform) and incomplete data
t <- apply(cbind(x,cs),1,min) # observed or censored data
d <- 1*(x <= cs)              # censoring indicator

###### EM for RMM, exponential lifetimes
l0 <- rep(1/m,m); r0 <- c(1, 0.5) # "arbitrary" initial values
a <- expRMM_EM(t, d, lambda=l0, rate=r0, k = m)
summary(a)             # EM estimates etc
plotexpRMM(a, lwd=2) # plot of EM sequences
%%\dontrun{
%%}
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{file}

\name{regmixEM}
\title{EM Algorithm for Mixtures of Regressions}
\alias{regmixEM}
\usage{
regmixEM(y, x, lambda = NULL, beta = NULL, sigma = NULL, k = 2,
         addintercept = TRUE, arbmean = TRUE, arbvar = TRUE, 
         epsilon = 1e-08, maxit = 10000, verb = FALSE)
}

\description{
  Returns EM algorithm output for mixtures of multiple regressions with
  arbitrarily many components.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{beta}.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and k is number of components.
    If NULL, then \code{beta} has standard normal entries according to a binning method done on the data.  If both
    \code{lambda} and \code{beta} are NULL, then number of components is determined by \code{sigma}.}
  \item{sigma}{A vector of standard deviations.  If NULL, then 1/\code{sigma}^2 has
    random standard exponential entries according to a binning method done on the data.
    If \code{lambda}, \code{beta}, and \code{sigma} are
    NULL, then number of components is determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless all of \code{lambda}, \code{beta}, 
  and \code{sigma} are NULL.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{arbmean}{If TRUE, each mixture component is assumed to have a different set of regression coefficients
  (i.e., the \code{beta}s).}
  \item{arbvar}{If TRUE, each mixture component is assumed to have a different \code{sigma}.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{regmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's if \code{addintercept} = TRUE).}
  \item{y}{The response values.}
  \item{lambda}{The final mixing proportions.}
  \item{beta}{The final regression coefficients.}
  \item{sigma}{The final standard deviations. If \code{arbmean} = FALSE, then only the smallest standard
   deviation is returned. See \code{scale} below.}
  \item{scale}{If \code{arbmean} = FALSE, then the scale factor for the component standard deviations is returned.
   Otherwise, this is omitted from the output.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{regcr}}, \code{\link{regmixMH}}
}
\references{
de Veaux, R. D. (1989), Mixtures of Linear Regressions,
  \emph{Computational Statistics and Data Analysis} 8, 227-245.

  Hurn, M., Justel, A. and Robert, C. P. (2003) Estimating Mixtures of Regressions, \emph{Journal 
  of Computational and Graphical Statistics} \bold{12(1)}, 55--79.
  
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
## EM output for NOdata.
 
data(NOdata)
attach(NOdata)
set.seed(100)
em.out <- regmixEM(Equivalence, NO, verb = TRUE, epsilon = 1e-04)
em.out[3:6]
}


\keyword{file}

\name{wquantile}
\alias{wIQR}
\alias{wquantile}
\title{Weighted quantiles}
\description{
  Functions to compute weighted quantiles and the weighted interquartile range.
  }
\usage{
wquantile(wt = rep(1,length(x)), x, probs, already.sorted = FALSE, 
          already.normalized = FALSE)
wIQR(wt = rep(1,length(x)), x, already.sorted = FALSE, 
     already.normalized = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{wt}{ Vector of weights }
  \item{x}{ Vector of data, same length as \code{wt} }
  \item{probs}{ Numeric vector of probabilities with values in [0,1].}
  \item{already.sorted}{ If FALSE, sort \code{wt} and \code{x} in increasing
  order of \code{x}.  If TRUE, it is assumed that \code{wt} and \code{x}
  are already sorted.}
  \item{already.normalized}{ If FALSE, normalize \code{wt} by diving each
  entry by the sum of all entries.  If TRUE, it is assumed that \code{sum(wt)==1}}
}
\details{
  \code{wquantile} uses the \code{\link{findInterval}} function.  \code{wIQR}
  calls the \code{wquantile} function.
  }
\value{
  Returns the sample quantiles or interquartile range of a discrete distribution with
  support points \code{x} and corresponding probability masses \code{wt}
}
\seealso{ \code{\link{npEM}}}
\examples{
IQR(1:10)
wIQR(x=1:10) # Note:  Different algorithm than IQR function
wIQR(1:10,1:10) # Weighted quartiles are now 4 and 8
}

\keyword{robust}

\name{plotly_npEM}
\title{Plot Nonparametric or Semiparametric EM Output}
\alias{plotly_npEM} 
\alias{plotly_spEM}
\usage{
plotly_npEM(x, blocks = NULL, hist=TRUE, addlegend=TRUE,
            scale = TRUE, title=NULL, breaks="Sturges", 
            dens.col = NULL, newplot=TRUE, ylim = NULL ,
            col.hist = "#1f77b4",
            width = 3, title.x = 0.5 , title.y = 0.95, title.size = 15,
            xlab = "X" , xlab.size = 15 , xtick.size = 15,
            ylab = "Density" , ylab.size = 15 , ytick.size = 15,
            legend.text = "Posteriors",
            legend.text.size = 15,
            legend.size = 15)         
plotly_spEM(x, blocks = NULL, hist=TRUE, addlegend=TRUE,
            scale = TRUE, title=NULL, breaks="Sturges", 
            dens.col = NULL, newplot=TRUE, ylim = NULL ,
            col.hist = "#1f77b4",
            width = 3, title.x = 0.5 , title.y = 0.95, title.size = 15,
            xlab = "X" , xlab.size = 15 , xtick.size = 15,
            ylab = "Density" , ylab.size = 15 , ytick.size = 15,
            legend.text = "Posteriors",
            legend.text.size = 15,
            legend.size = 15)
}
\description{
  This is an updater version of \code{plot.npEM} function by using \code{plotly}. For technical details, please refer to \code{plot.npEM}.     
} 
\arguments{
  \item{x}{An object of class \code{npEM} such as the output
  of the \code{\link{npEM}} function}
  \item{blocks}{Blocks (of repeated measures coordinates) to plot; not relevant
  for univariate case.  Default is to plot all blocks.}
  \item{hist}{If TRUE, superimpose density estimate plots on a histogram
    of the data}
  \item{addlegend}{If TRUE, adds legend to the plot.}
  \item{scale}{If TRUE, scale each density estimate by its corresponding estimated
  mixing proportion, so that the total area under all densities equals 1 and the 
  densities plotted may be added to produce an estimate of the mixture density.
  When FALSE, each density curve has area 1 in the plot.}
  \item{title}{Alternative vector of main titles for plots (recycled as many times
  as needed)}
  \item{breaks}{Passed directly to the \code{\link{hist}} function}
  \item{ylim}{\code{ylim} parameter to use for all plots, if desired.  If not
  given, each plot uses its own ylim that ensures that no part of the plot will
  go past the top of the plotting area.}
  \item{dens.col}{Color values to use for the individual component density
    functions, repeated as necessary.  Default value is \code{2:(m+1)}.}
  \item{newplot}{If TRUE, creates a new plot.}
  \item{col.hist}{Color of the histogram to plot.}
  \item{width}{Line width.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horsizontal position of the main title.}
  \item{title.y}{Vertical posotion of the main title.}
  \item{xlab}{Label of X-axis.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab}{Label of Y-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
  \item{legend.text}{Title of legend.}
  \item{legend.text.size}{Size of the legend title.}
  \item{legend.size}{Size of legend.}
}
\value{
  \code{plotly_npEM} returns a list with two elements:
  \item{x}{List of matrices.  The \eqn{j}th column of the \eqn{i}th matrix is the vector of 
  \eqn{x}-values for the \eqn{j}th density in the \eqn{i}th plot.}
  \item{y}{\eqn{y}-values, given in the same form as the \eqn{x}-values.}
} 

\seealso{ 
\code{\link{npEM}}, \code{\link{density.npEM}}, \code{\link{spEMsymloc}},
\code{\link{plotseq.npEM}}, \code{plot.npEM}
} 

\examples{ 
\dontrun{
## Examine and plot water-level task data set.

## First, try a 3-component solution where no two coordinates are
## assumed i.d.
data(Waterdata)
set.seed(100)
a <- npEM(Waterdata[,3:10], 3, bw=4)
plotly_npEM(a , newplot = FALSE)

## Next, same thing but pairing clock angles that are directly opposite one
## another (1:00 with 7:00, 2:00 with 8:00, etc.)
b <- npEM(Waterdata[,3:10], 3, blockid=c(4,3,2,1,3,4,1,2), bw=4)
plotly_npEM(b , newplot = FALSE)
}

}

\keyword{file}

\name{regmixEM.loc}
\title{Iterative Algorithm Using EM Algorithm for Mixtures of Regressions with 
    Local Lambda Estimates}
\alias{regmixEM.loc}
\usage{
regmixEM.loc(y, x, lambda = NULL, beta = NULL, sigma = NULL, 
             k = 2, addintercept = TRUE, kern.l = c("Gaussian",
             "Beta", "Triangle", "Cosinus", "Optcosinus"), 
             epsilon = 1e-08, maxit = 10000, kernl.g = 0, 
             kernl.h = 1, verb = FALSE) 
}

\description{
  Iterative algorithm returning EM algorithm output for mixtures of multiple regressions where the mixing proportions
  are estimated locally.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An nxp matrix of predictors.  See \code{addintercept} below.}
  \item{lambda}{An nxk matrix of initial local values of mixing proportions.  
    Entries should sum to 1.  This determines number of components.  
    If NULL, then \code{lambda} is simply one over the number of components.}
  \item{beta}{Initial global values of \code{beta} parameters.  Should be a pxk matrix,
    where p is the number of columns of x and \code{k} is number of components.
    If NULL, then \code{beta} has uniform standard normal entries.  If both
    \code{lambda} and \code{beta} are NULL, then number of components is determined by \code{sigma}.}
  \item{sigma}{A k-vector of initial global values of standard deviations.
    If NULL, then \eqn{1/\code{sigma}^2} has random standard exponential entries.  
    If \code{lambda}, \code{beta}, and \code{sigma} are NULL, then number of components determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless all of \code{lambda}, \code{beta},
    and \code{sigma} are NULL.}
  \item{addintercept}{If TRUE, a column of ones is appended to the x
    matrix before the value of p is calculated.}
  \item{kern.l}{The type of kernel to use in the local estimation of \code{lambda}.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{kernl.g}{A shape parameter required for the symmetric beta kernel for local estimation of \code{lambda}.  
  The default is g = 0 which yields the uniform kernel.  Some common values are g = 1 for the
  Epanechnikov kernel, g = 2 for the biweight kernel, and g = 3 for the triweight kernel.}
  \item{kernl.h}{The bandwidth controlling the size of the window used in the
  local estimation of lambda around x.}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{regmixEM.loc} returns a list of class \code{mixEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's if \code{addintercept} = TRUE).}
  \item{y}{The response values.}
  \item{lambda.x}{The final local mixing proportions.}
  \item{beta}{The final global regression coefficients.}
  \item{sigma}{The final global standard deviations.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{regmixEM.lambda}}
}
\examples{
## Compare a 2-component and 3-component fit to NOdata.

data(NOdata)
attach(NOdata)
set.seed(100)
out1 <- regmixEM.loc(Equivalence, NO, kernl.h = 2, 
                     epsilon = 1e-02, verb = TRUE)
out2 <- regmixEM.loc(Equivalence, NO, kernl.h = 2, k = 3,
                     epsilon = 1e-02, verb = TRUE)
c(out1$loglik, out2$loglik)

}

\keyword{file}

%gammamix things temporarily commented out by DRH on 8-29-2008 
%gammamix things uncommented by DSY on 10-2-2009 
\name{mixtools initializations}
\alias{flaremix.init}
\alias{gammamix.init}
\alias{logisregmix.init}
\alias{multmix.init}
\alias{mvnormalmix.init}
\alias{normalmix.init}
\alias{poisregmix.init}
%\alias{regmix.chgpt.init}
\alias{regmix.init}
\alias{regmix.lambda.init}
\alias{regmix.mixed.init}
\alias{repnormmix.init}
\alias{segregmix.init}

\title{Initializations for Various EM Algorithms in 'mixtools'}
\description{
  Internal intialization functions for EM algorithms in the package \code{mixtools}.
}
\usage{
flaremix.init(y, x, lambda = NULL, beta = NULL, sigma = NULL,
              alpha = NULL)
gammamix.init(x, lambda = NULL, alpha = NULL, beta = NULL, 
              k = 2)
logisregmix.init(y, x, N, lambda = NULL, beta = NULL, k = 2)
multmix.init(y, lambda = NULL, theta = NULL, k = 2)
mvnormalmix.init(x, lambda = NULL, mu = NULL, sigma = NULL, 
                 k = 2, arbmean = TRUE, arbvar = TRUE)
normalmix.init(x, lambda = NULL, mu = NULL, s = NULL, k = 2, 
               arbmean = TRUE, arbvar = TRUE)
poisregmix.init(y, x, lambda = NULL, beta = NULL, k = 2)
%regmix.chgpt.init(y, x, lambda = NULL, beta = NULL,
%                  gamma = NULL, sigma = NULL, t = NULL, k = 2)
regmix.init(y, x, lambda = NULL, beta = NULL, s = NULL, k = 2, 
            addintercept = TRUE, arbmean = TRUE, arbvar=TRUE)
regmix.lambda.init(y, x, lambda = NULL, beta = NULL, s = NULL,
                   k = 2, addintercept = TRUE, arbmean = TRUE,
                   arbvar = TRUE)
regmix.mixed.init(y, x, w = NULL, sigma = NULL, 
                  arb.sigma = TRUE, alpha = NULL, lambda = NULL, 
                  mu = NULL, R = NULL, arb.R = TRUE, k = 2, 
                  mixed = FALSE, addintercept.fixed = FALSE,
                  addintercept.random = TRUE)
repnormmix.init(x, lambda = NULL, mu = NULL, s = NULL, k = 2, 
                arbmean = TRUE, arbvar = TRUE)
segregmix.init(y, x, lambda = NULL, beta = NULL, s = NULL, k = 2,
               seg.Z, psi, psi.locs = NULL)
}

\details{
  These are usually not to be called by the user. Definitions of the arguments appear in the respective EM algorithms.
}

\keyword{internal}

\name{rmvnorm}
\alias{rmvnorm}
\title{Simulate from a Multivariate Normal Distribution}
\description{
  Simulate from a multiviate normal distribution
}
\usage{
rmvnorm(n, mu=NULL, sigma=NULL)
}
\arguments{
  \item{n}{Number of vectors to simulate}
  \item{mu}{mean vector}
  \item{sigma}{covariance matrix, assumed symmetric and nonnegative definite}
}
\value{
  An \eqn{n \times d}{n x d} matrix in which each row is an independently
  generated realization from the desired multivariate normal distribution
}
\details{
This function uses an \code{\link{eigen}} decomposition assuming \code{sigma} is symmetric.
In particular, the upper triangle of \code{sigma} is ignored.
}
\seealso{
  \code{\link{eigen}}, \code{\link{dnorm}}, \code{\link{dmvnorm}}
}
\keyword{distribution}

\name{plotly_seq.npEM}
\alias{plotly_seq.npEM}
\title{Plotting sequences of estimates from non- or semiparametric EM-like Algorithm using \code{plotly}}

\description{This is an updated version of \code{\link{plotseq.npEM}}. For technical details, please refer to \code{\link{plotseq.npEM}}.
}

\usage{
  plotly_seq.npEM (x, col = '#1f77b4' , width = 6,
                   xlab = "Iteration" , xlab.size = 15 , xtick.size = 15,
                   ylab.size = 15 , ytick.size = 15,
                   title.size = 15 , title.x = 0.5 , title.y = 0.95)
}
\arguments{
  \item{x}{an object of class \code{npEM}, as output by \code{\link{npEM}} 
  or \code{\link{spEMsymloc}}}
  \item{col}{Line color.}
  \item{width}{Line width.}
  \item{title}{Text of the main title.}
  \item{title.size}{Size of the main title.}
  \item{title.x}{Horsizontal position of the main title.}
  \item{title.y}{Vertical posotion of the main title.}
  \item{xlab}{Label of X-axis.}
  \item{xlab.size}{Size of the lable of X-axis.}
  \item{xtick.size}{Size of tick lables of X-axis.}
  \item{ylab.size}{Size of the lable of Y-axis.}
  \item{ytick.size}{Size of tick lables of Y-axis.}
  }


\value{\code{plotly_seq.npEM} returns a figure with one plot for each component 
 proportion, and, in the case of \code{\link{spEMsymloc}}, one plot for each 
component mean.}

\seealso{
\code{\link{plot.npEM}}, \code{\link{rnormmix}},
\code{\link{npEM}}, \code{\link{spEMsymloc}}, \code{\link{plotly_seq.npEM}}
}


\references{
   \itemize{
   \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics (to appear).

   \item Bordes, L., Chauveau, D., and Vandekerkhove, P. (2007),
   An EM algorithm for a semiparametric mixture model, 
   Computational Statistics and Data Analysis, 51: 5429-5443.   
   }
}
\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
\dontrun{
## Examine and plot water-level task data set.
## First, try a 3-component solution where no two coordinates are
## assumed i.d.
data(Waterdata)
set.seed(100)
## Not run:
a <- npEM(Waterdata[,3:10], mu0=3, bw=4) # Assume indep but not iid
plotly_seq.npEM(a)
}
}
\keyword{file}

\name{flaremixEM}
\title{EM Algorithm for Mixtures of Regressions with Flare}
\alias{flaremixEM}
\usage{
flaremixEM(y, x, lambda = NULL, beta = NULL, sigma = NULL, 
           alpha = NULL, nu = NULL, epsilon = 1e-04, 
           maxit = 10000, verb = FALSE, restart = 50)
}

\description{
  Returns output for 2-component mixture of regressions with flaring
  using an EM algorithm with one step of Newton-Raphson requiring
  an adaptive barrier for maximization of the objective function. A mixture of regressions
  with flare occurs when there appears to be a common regression relationship for the data,
  but the error terms have a mixture structure of one normal component and one exponential
  component.
}
\arguments{
  \item{y}{An n-vector of response values.}
  \item{x}{An n-vector of predictor values.  An intercept term will be added by default.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to 1.}
  \item{beta}{Initial value of \code{beta} parameters.  Should be a 2x2 matrix where the columns
    correspond to the component.}
  \item{sigma}{A vector of standard deviations.}
  \item{alpha}{A scalar for the exponential component's rate.}
  \item{nu}{A vector specifying the barrier constants to use.  The first barrier constant where the algorithm
  converges is used.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
  \item{restart}{The number of times to restart the algorithm in case convergence is not attained.
  The default is 50.}
}
\value{
  \code{flaremixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The set of predictors (which includes a column of 1's).}
  \item{y}{The response values.}
  \item{posterior}{An nx2 matrix of posterior probabilities for
   observations.}
  \item{lambda}{The final mixing proportions.}
  \item{beta}{The final regression coefficients.}
  \item{sigma}{The final standard deviations.}
  \item{alpha}{The final exponential rate.}
  \item{loglik}{The final log-likelihood.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{regmixEM}}
}

\examples{
## Simulation output.

set.seed(100)
j=1
while(j == 1){
    x1 <- runif(30, 0, 10)
    x2 <- runif(20, 10, 20)
    x3 <- runif(30, 20, 30)
    y1 <- 3+4*x1+rnorm(30, sd = 1)
    y2 <- 3+4*x2+rexp(20, rate = .05)
    y3 <- 3+4*x3+rnorm(30, sd = 1)
    x <- c(x1, x2, x3)
    y <- c(y1, y2, y3)
    nu <- (1:30)/2

    out <- try(flaremixEM(y, x, beta = c(3, 4), nu = nu,
               lambda = c(.75, .25), sigma = 1), silent = TRUE)
    if(any(class(out) == "try-error")){
        j <- 1
    } else j <- 2
}

out[4:7]
plot(x, y, pch = 19)
abline(out$beta)



}


\keyword{file}

\name{repnormmixEM}
\title{EM Algorithm for Mixtures of Normals with Repeated Measurements}
\alias{repnormmixEM}
\usage{
repnormmixEM(x, lambda = NULL, mu = NULL, sigma = NULL, k = 2, 
             arbmean = TRUE, arbvar = TRUE, epsilon = 1e-08, 
             maxit = 10000, verb = FALSE)
}

\description{
  Returns EM algorithm output for mixtures of normals with repeated
  measurements and arbitrarily many components.
}
\arguments{
  \item{x}{An mxn matrix of data.  The columns correspond to the subjects and
  the rows correspond to the repeated measurements.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{mu}.}
  \item{mu}{A k-vector of component means.  If NULL, then \code{mu} is determined by a
    normal distribution according to a binning method done on the data.  If both
    \code{lambda} and \code{mu} are NULL, then number of components is determined by \code{sigma}.}
  \item{sigma}{A vector of standard deviations.  If NULL, then \eqn{1/\code{sigma}^2} has
    random standard exponential entries according to a binning method done on the data.
    If \code{lambda}, \code{mu}, and \code{sigma} are NULL, then number of components is determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless all of \code{lambda}, \code{mu}, 
    and \code{sigma} are NULL.}
  \item{arbmean}{If TRUE, then the component densities are allowed to have different \code{mu}s. If FALSE, then
  a scale mixture will be fit.}
  \item{arbvar}{If TRUE, then the component densities are allowed to have different \code{sigma}s. If FALSE, then
  a location mixture will be fit.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.} 
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{repnormmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The raw data.}
  \item{lambda}{The final mixing proportions.}
  \item{mu}{The final mean parameters.}
  \item{sigma}{The final standard deviations. If \code{arbmean} = FALSE, then only the smallest standard
   deviation is returned. See \code{scale} below.}
  \item{scale}{If \code{arbmean} = FALSE, then the scale factor for the component standard deviations is returned.
   Otherwise, this is omitted from the output.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
\code{\link{normalmixEM}}
}
\references{
  Hettmansperger, T. P. and Thomas, H.  (2000)
  Almost Nonparametric Inference for Repeated Measures in Mixture Models,
  \emph{Journal of the Royals Statistical Society, Series B} \bold{62(4)} 811--825.
}
\examples{
## EM output for the water-level task data set.

data(Waterdata)
set.seed(100)
water <- t(as.matrix(Waterdata[,3:10]))
em.out <- repnormmixEM(water, k = 2, verb = TRUE, epsilon = 1e-03)
em.out
}


\keyword{file}

\name{perm}
\title{Permutation Function}
\alias{perm}
\usage{
perm(n, r, v = 1:n)
}
\description{
Enumerates the possible permutations of a specified size from the elements of a vector having the same size.
}
\arguments{
  \item{n}{Size of the source vector.}
  \item{r}{Size of the target vector.}
  \item{v}{Source vector.  Must be a vector of length \code{n}.}
}
\value{
  \code{perm} returns a matrix where each row contains one of the permutations of length \code{r}.
}
\details{
This function is called by \code{segregmixEM} and the associated internal functions.  This is also a simplified version of the function \code{permutations} found in the package \code{gtools}.
}
\seealso{
\code{\link{segregmixEM}}
}
\examples{
perm(3, 3, 2:4)
}


\keyword{internal}

\name{rexpmix}
\title{Simulate from Mixtures of Exponentials}
\alias{rexpmix}
\usage{
rexpmix(n, lambda = 1, rate = 1)
}

\description{
Simulate from a mixture of univariate exponential distributions.
}
\arguments{
  \item{n}{Number of cases to simulate.}
  \item{lambda}{Vector of mixture probabilities, with length equal to \eqn{m}, 
    the desired number of components (subpopulations).  This is assumed to sum
    to 1.}
  \item{rate}{Vector of component rates.}
}
\value{
  \code{rexpmix} returns an \eqn{n}-vector sampled from an \eqn{m}-component 
  mixture of univariate exponential distributions.
}
%%\details{This function simply calls \code{\link{sample}} and \code{\link{rexp}}.} 
\seealso{
\code{\link{rnormmix}}, \code{\link{rmvnormmix}} for Gaussian mixtures,
\code{\link{rweibullmix}} for mixture of Weibull distributions.
}
\examples{
## Generate data from a 2-component mixture of exponentials.
n=300 # sample size
m=2 # nb components
lambda=c(1/3, 2/3); rate = c(1,1/10) # parameters
set.seed(1234)
x <- rexpmix(n, lambda, rate) # iid ~ exp mixture

## histogram of the simulated data.
hist(x, col=8)
}

\keyword{file}

\name{mvnormalmixEM}
\title{EM Algorithm for Mixtures of Multivariate Normals}
\alias{mvnormalmixEM}
\usage{
mvnormalmixEM(x, lambda = NULL, mu = NULL, sigma = NULL, k = 2,
              arbmean = TRUE, arbvar = TRUE, epsilon = 1e-08, 
              maxit = 10000, verb = FALSE)
}
\description{
  Return EM algorithm output for mixtures of multivariate normal distributions.
}
\arguments{
  \item{x}{A matrix of size nxp consisting of the data.}
  \item{lambda}{Initial value of mixing proportions.  Entries should sum to
    1.  This determines number of components.  If NULL, then \code{lambda} is
    random from uniform Dirichlet and number of
    components is determined by \code{mu}.}
  \item{mu}{A list of size k consisting of initial values for the p-vector mean parameters.  
    If NULL, then the vectors are generated from a normal distribution with
    mean and standard deviation according to a binning method done on the data.
    If both \code{lambda} and \code{mu} are NULL, then number of components is determined by \code{sigma}.}
  \item{sigma}{A list of size k consisting of initial values for the pxp variance-covariance matrices.  
    If NULL, then \code{sigma} is generated using the data.  
    If \code{lambda}, \code{mu}, and \code{sigma} are
    NULL, then number of components is determined by \code{k}.}
  \item{k}{Number of components.  Ignored unless \code{lambda}, \code{mu}, and \code{sigma}
    are all NULL.}
  \item{arbmean}{If TRUE, then the component densities are allowed to have different \code{mu}s. If FALSE, then
   a scale mixture will be fit.}
  \item{arbvar}{If TRUE, then the component densities are allowed to have different \code{sigma}s. If FALSE, then
   a location mixture will be fit.}
  \item{epsilon}{The convergence criterion.}
  \item{maxit}{The maximum number of iterations.}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\value{
  \code{normalmixEM} returns a list of class \code{mixEM} with items:
  \item{x}{The raw data.}
  \item{lambda}{The final mixing proportions.}
  \item{mu}{A list of with the final mean vectors.}
  \item{sigma}{A list with the final variance-covariance matrices.}
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nxk matrix of posterior probabilities for
    observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.}
  \item{restarts}{The number of times the algorithm restarted due to unacceptable choice of initial values.}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
  \code{\link{normalmixEM}}
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
##Fitting randomly generated data with a 2-component location mixture of bivariate normals.

set.seed(100)
x.1 <- rmvnorm(40, c(0, 0))
x.2 <- rmvnorm(60, c(3, 4))
X.1 <- rbind(x.1, x.2)
mu <- list(c(0, 0), c(3, 4))

out.1 <- mvnormalmixEM(X.1, arbvar = FALSE, mu = mu,
                       epsilon = 1e-02)
out.1[2:5]

##Fitting randomly generated data with a 2-component scale mixture of bivariate normals.

x.3 <- rmvnorm(40, c(0, 0), sigma = 
               matrix(c(200, 1, 1, 150), 2, 2))
x.4 <- rmvnorm(60, c(0, 0))
X.2 <- rbind(x.3, x.4)
lambda <- c(0.40, 0.60)
sigma <- list(diag(1, 2), matrix(c(200, 1, 1, 150), 2, 2))
 
out.2 <- mvnormalmixEM(X.2, arbmean = FALSE,
                       sigma = sigma, lambda = lambda,
                       epsilon = 1e-02)
out.2[2:5]
}

\keyword{file}

\name{wkde}
\title{Weighted Univariate (Normal) Kernel Density Estimate}
\alias{wkde}
\alias{wkde.symm}
\usage{
wkde(x, u=x, w=rep(1, length(x)), bw=bw.nrd0(as.vector(x)), sym=FALSE) 
}
\description{
Evaluates a weighted kernel density estimate, using a Gaussian kernel, 
at a specified vector of points.
}
\arguments{
  \item{x}{Data}
  \item{u}{Points at which density is to be estimated}
  \item{w}{Weights (same length as \code{x})}
  \item{bw}{Bandwidth}
  \item{sym}{Logical:  Symmetrize about zero?}
}
\value{A vector of the same length as \code{u}
}
\seealso{
\code{\link{npEM}}, \code{\link{ise.npEM}}
}
\references{
   \itemize{
   \item Benaglia, T., Chauveau, D., and Hunter, D. R. (2009), An EM-like algorithm
   for semi- and non-parametric estimation in multivariate mixtures, 
   Journal of Computational and Graphical Statistics, 18, 505-526.
   
   \item Benaglia, T., Chauveau, D., Hunter, D. R., and Young, D. (2009),
  mixtools: An R package for analyzing finite mixture models.
	Journal of Statistical Software, 32(6):1-29.
   }
}
\examples{
# Mixture with mv gaussian model
set.seed(100)
m <- 2 # no. of components
r <- 3 # no. of repeated measures (coordinates)
lambda <- c(0.4, 0.6)
mu <- matrix(c(0, 0, 0, 4, 4, 6), m, r, byrow=TRUE) # means 
sigma <- matrix(rep(1, 6), m, r, byrow=TRUE) # stdevs
centers <- matrix(c(0, 0, 0, 4, 4, 4), 2, 3, byrow=TRUE) # initial centers for est

blockid = c(1,1,2) # block structure of coordinates
n = 100
x <- rmvnormmix(n, lambda, mu, sigma) # simulated data
a <- npEM(x, centers, blockid, eps=1e-8, verb=FALSE)

par(mfrow=c(2,2))
u <- seq(min(x), max(x), len=200)
for(j in 1:2) { 
  for(b in 1:2) {
    xx <- as.vector(x[,a$blockid==b])
    wts <- rep(a$post[,j], length.out=length(xx))
    bw <- a$bandwidth
    title <- paste("j =", j, "and b =", b)
    plot(u, wkde(xx, u, wts, bw), type="l", main=title)
  }
}

}


\keyword{file}

\name{RanEffdata}
\docType{data}
\title{Simulated Data from 2-Component Mixture of Regressions with Random Effects}
\alias{RanEffdata}
\usage{
data(RanEffdata)
}
\description{
  This data set was generated from a 2-component mixture of regressions with random effects. 
} 
\format{This data set consists of a list with 100 25x3 matrices. 
    The first column is the response variable, the second column is a column of 1's and the last column is the
    predictor variable.
}
\seealso{
\code{\link{regmixEM.mixed}}
}
\keyword{datasets}

\name{plotspRMM}
\alias{plotspRMM}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Plot output from Stochastic EM algorithm for semiparametric scaled mixture of censored data
}

\description{Function for plotting various results from an object returned by \code{\link{spRMM_SEM}}, a Stochastic EM algorithm for semiparametric scaled 
mixture of randomly right censored lifetime data.
Four plots of sequences of estimates along iterations, survival and density estimates
(see reference below).
}

\usage{
  plotspRMM(sem, tmax = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{sem}{An object returned by \code{\link{spRMM_SEM}}.}
  \item{tmax}{The max time for \eqn{x} axis, set to some default value if \code{NULL}.}
  }


\value{The four plots returned}


\seealso{
Related functions: \code{\link{spRMM_SEM}}.

Other models and algorithms for censored lifetime data 
(name convention is model_algorithm):
\code{\link{expRMM_EM}},
\code{\link{weibullRMM_SEM}}.
}

\references{
   \itemize{
   \item Bordes, L., and Chauveau, D. (2016),
   Stochastic EM algorithms for parametric and semiparametric mixture models 
   for right-censored lifetime data, 
   Computational Statistics, Volume 31, Issue 4, pages 1513-1538.
   \url{http://link.springer.com/article/10.1007/s00180-016-0661-7}
   }
}
\author{Didier Chauveau}


%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
# See example(spRMM_SEM)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{file}

\name{mixtools-internal}
\alias{inv.logit}
\alias{dexpmixt}
\alias{HRkde}
\alias{kern.B}
\alias{kern.C}
\alias{kern.G}
\alias{kern.O}
\alias{kern.T}
\alias{kfoldCV}
\alias{KMintegrate}
\alias{KMod}
\alias{ldc}
\alias{logit}
\alias{npMSL_old}
\alias{plotseq}
\alias{rlnormscalemix}
\alias{splitsample}
\alias{triang_wkde}
\alias{wbw.kCV}

\title{Internal 'mixtools' Functions}
\description{
  Internal kernel, semiparametric-related, and miscellaneous functions for the package \code{mixtools}.
}
\usage{
dexpmixt(t, lam, rate)
HRkde(cpd, u = cpd[,1], kernelft = triang_wkde, 
      bw = rep(bw.nrd0(as.vector(cpd[,1])), length(cpd[,1])))
inv.logit(eta)
kern.B(x, xi, h, g = 0)
kern.C(x, xi, h)
kern.G(x, xi, h)
kern.O(x, xi, h)
kern.T(x, xi, h)
kfoldCV(h, x, nbsets = 2, w = rep(1, length(x)), 
        lower = mean(x) - 5*sd(x), upper = mean(x) + 5*sd(x))
KMintegrate(s) 
KMod(cpd, already.ordered = TRUE)
ldc(data, class, score)
logit(mu)
npMSL_old(x, mu0, blockid = 1:ncol(x),
          bw=bw.nrd0(as.vector(as.matrix(x))), samebw = TRUE,
          h=bw, eps=1e-8, maxiter=500, bwiter = maxiter,
          ngrid = 200, post = NULL, verb = TRUE)
plotseq(x, ...)
rlnormscalemix(n, lambda=1, meanlog=1, sdlog=1, scale=0.1)
splitsample(n, nbsets = 2)
triang_wkde(t, u=t, w=rep(1/length(t),length(t)), bw=rep(bw.nrd0(t), length(t)))
wbw.kCV(x, nbfold = 5, w = rep(1, length(x)), 
        hmin = 0.1*hmax, hmax = NULL)
}

\arguments{
  \item{x}{A vector of values to which local modeling techniques are applied.}
  \item{xi}{An n-vector of data values.}
  \item{h}{The bandwidth controlling the size of the window used for the
  local estimation around \code{x}.}
  \item{g}{A shape parameter required for the symmetric beta kernel.  The default
  is \code{g} = 0 which yields the uniform kernel.  Some common values are \code{g} = 1 for the
  Epanechnikov kernel, \code{g} = 2 for the biweight kernel, and \code{g} = 3 for the triweight kernel.}
  \item{mu0}{See updated arguments in the \code{npMSL} function.}
  \item{blockid}{See updated arguments in the \code{npMSL} function.}
  \item{bw}{See updated arguments in the \code{npMSL} function.}
  \item{samebw}{See updated arguments in the \code{npMSL} function.}
  \item{h}{See updated arguments in the \code{npMSL} function.}
  \item{eps}{See updated arguments in the \code{npMSL} function.}
  \item{maxiter}{See updated arguments in the \code{npMSL} function.}
  \item{bwiter}{See updated arguments in the \code{npMSL} function.}
  \item{ngrid}{See updated arguments in the \code{npMSL} function.}
  \item{post}{See updated arguments in the \code{npMSL} function.}
  \item{verb}{See updated arguments in the \code{npMSL} function.}
  \item{n}{See updated arguments in the \code{npMSL} function.}
  \item{nbsets}{See updated arguments in the \code{npMSL} function.}
  \item{w}{See updated arguments in the \code{npMSL} function.}
  \item{lower}{See updated arguments in the \code{npMSL} function.}
  \item{upper}{See updated arguments in the \code{npMSL} function.}
  \item{nbfold}{See updated arguments in the \code{npMSL} function.}
  \item{hmin}{See updated arguments in the \code{npMSL} function.}
  \item{hmax}{See updated arguments in the \code{npMSL} function.}
  \item{data}{Data, possibly multivariate, fed to the \code{mixturegram} function.}
  \item{class}{The number of classes, inputted based on number of components in the \code{mixturegram} function.}
  \item{score}{The score vector from LDA used in constructing a mixturegram.}
  \item{lam}{A vector of mixture proportions, should sum to one.}
  \item{rate}{A vector of mixture component rates.}
  \item{t}{Argument for \code{dexpmixt}.}
  \item{mu}{A proportion for which to calculate the logit function; i.e., \code{log(mu / (1 - mu))}.}
  \item{eta}{Any real value for which to calculate the inverse logit function; 
        i.e., \code{1 / (1 + exp(eta))}.}  
  \item{cpd}{Argument for \code{HRkde}.}
  \item{kernelft}{Argument for \code{HRkde}.} 
  \item{s}{Argument for \code{KMintegrate}.}
  \item{meanlog}{Argument for \code{rlnormscalemix}.}
  \item{sdlog}{Argument for \code{rlnormscalemix}.}
}

\details{
  These are usually not to be called by the user.
}

\seealso{
  \code{\link{npMSL}}
}


\keyword{internal}

\name{normalmixEM2comp}
\title{Fast EM Algorithm for 2-Component Mixtures of Univariate Normals}
\alias{normalmixEM2comp}
\usage{
normalmixEM2comp(x, lambda, mu, sigsqrd, eps= 1e-8, maxit = 1000, verb=FALSE)
}
\description{
  Return EM algorithm output for mixtures of univariate normal distributions 
  for the special case of 2 components, exploiting the simple structure of the
  problem to speed up the code.
}
\arguments{
  \item{x}{A vector of length \eqn{n} consisting of the data.}
  \item{lambda}{Initial value of first-component mixing proportion.}
  \item{mu}{A 2-vector of initial values for the mean parameters.}
  \item{sigsqrd}{Either a scalar or a 2-vector with initial value(s) for
  the variance parameters.  If a scalar, the algorithm assumes that the
  two components have equal variances; if a 2-vector, it assumes that the
  two components do not have equal variances.}
  \item{eps}{The convergence criterion.  Convergence is declared when the change in 
  the observed data log-likelihood increases by less than epsilon.}
  \item{maxit}{The maximum possible number of iterations.}
  \item{verb}{If TRUE, then various updates are printed during each iteration of the algorithm.} 
}
\details{
  This code is written to be very fast, sometimes more than an order of magnitude
  faster than \code{\link{normalmixEM}} for the same problem.  It is less numerically
  stable that \code{\link{normalmixEM}} in the sense that it does not safeguard 
  against underflow as carefully.
  
  Note that when the two components are assumed to have unequal variances,
  the loglikelihood is unbounded.  However, in practice this is rarely a problem
  and quite often the algorithm converges to a "nice" local maximum.
}
\value{
  \code{normalmixEM2comp} returns a list of class \code{mixEM} with items:
  \item{x}{The raw data.}
  \item{lambda}{The final mixing proportions (lambda and 1-lambda).}
  \item{mu}{The final two mean parameters.}
  \item{sigma}{The final one or two standard deviations. }
  \item{loglik}{The final log-likelihood.}
  \item{posterior}{An nx2 matrix of posterior probabilities for
   observations.}
  \item{all.loglik}{A vector of each iteration's log-likelihood.  This vector
  includes both the initial and the final values; thus, the number of iterations 
  is one less than its length.}
  \item{restarts}{The number of times the algorithm restarted due to 
  unacceptable choice of initial values (always zero).}
  \item{ft}{A character vector giving the name of the function.}
}
\seealso{
  \code{\link{mvnormalmixEM}}, \code{\link{normalmixEM}}
}
\references{
  McLachlan, G. J. and Peel, D. (2000) \emph{Finite Mixture Models}, John Wiley \& Sons, Inc.
}
\examples{
##Analyzing the Old Faithful geyser data with a 2-component mixture of normals.

data(faithful)
attach(faithful)
set.seed(100)
system.time(out <- normalmixEM2comp(waiting, lambda=.5, 
            mu=c(50,80), sigsqrd=100))
out$all.loglik # Note:  must be monotone increasing

# Compare elapsed time with more general version
system.time(out2 <- normalmixEM(waiting, lambda=c(.5,.5), 
            mu=c(50,80), sigma=c(10,10), arbvar=FALSE))
out2$all.loglik # Values should be identical to above
}

\keyword{file}

\name{lambda.pert}
\title{Perturbation of Mixing Proportions}
\alias{lambda.pert}
\usage{
lambda.pert(lambda, pert)
}
\description{
Perturbs a set of mixing proportions by first scaling the
mixing proportions, then taking the logit of the scaled values,
perturbing them, and inverting back to produce a set of
new mixing proportions.
}
\arguments{
  \item{lambda}{A vector of length k giving the mixing proportions which
  are to be perturbed.}
  \item{pert}{A vector (possibly of length k-1) for which to perturb \code{lambda}.
  If the length is less than k, then values of the vector are recycled.  If length
  is greater than k, then only the first k values are used.}
}
\value{
  \code{lambda.pert} returns new \code{lambda} values perturbed by \code{pert}.  
}
\details{
This function is called by \code{regmixMH}.
}
\seealso{
\code{\link{regmixMH}}
}
\examples{
set.seed(100)
x <- c(0.5, 0.2, 0.3)
lambda.pert(x, rcauchy(3))

}

\keyword{internal}

\name{rnormmix}
\title{Simulate from Mixtures of Normals}
\alias{normmix.sim}
\alias{rnormmix}
\usage{
rnormmix(n, lambda=1, mu=0, sigma=1)
}

\description{
Simulate from a mixture of univariate normal distributions.
}
\arguments{
  \item{n}{Number of cases to simulate.}
  \item{lambda}{Vector of mixture probabilities, with length equal to \eqn{m},
    the desired number of components (subpopulations).  This is assumed to sum
    to 1; if not, it is normalized.}
  \item{mu}{Vector of means.}
  \item{sigma}{Vector of standard deviations.}
}
\value{
  \code{rnormmix} returns an \eqn{n}-vector sampled from an \eqn{m}-component 
  mixture of univariate normal distributions.
}
\details{This function simply calls \code{\link{rmvnormmix}}.} 
\seealso{
\code{\link{makemultdata}}, \code{\link{rmvnormmix}}
}
\examples{
##Generate data from a 2-component mixture of normals.

set.seed(100)
n <- 500
lambda <- rep(1, 2)/2
mu <- c(0, 5)
sigma <- rep(1, 2)
mixnorm.data <- rnormmix(n, lambda, mu, sigma)

##A histogram of the simulated data.

hist(mixnorm.data)
}

\keyword{file}
